// Copyright (c) 2010 Motorola, Inc. All Rights Reserved.

#include "TApplication.h"

#include "interface/ToiObjectNames.h"

using namespace std;

TApplication::TApplication(TIpcClient& ipcClient,
                           IIpcEventHandler& ipcEventHandler,
                           IToiNameService& nameService,
                           IToiApplicationService::TToiApplicationId
                           applicationId) :
  Tracker(TTracker::GetProcessTracker()),
  ApplicationAddress(ipcEventHandler.CreateObject()),
  Dispatcher(&ipcEventHandler, ApplicationAddress, this),
  ApplicationId(applicationId)
{
  string applicationServiceAddress;
  nameService.LookupObject(TOI_APPLICATION_SERVICE_NAME,
                           applicationServiceAddress);
  ApplicationService =
    new TToiApplicationServiceCaller(&ipcClient, applicationServiceAddress);

  Tracker->Debug("Reporting started to the application service");
  ApplicationService->ReportStarted(applicationId, ApplicationAddress);
}

TApplication::~TApplication() throw ()
{
  delete ApplicationService;
}

void TApplication::Ping() throw ()
{
  // Report to the Application Service that we have received a ping.
  try {
    ApplicationService->ReportPing(ApplicationId);
  }
  catch (TException& e) {
    // Ignore
  }
}

void
TApplication::ChangeState(const IToiApplicationService::TToiApplicationState&
                          /*state*/) throw ()
{
  // Ignore
}

void TApplication::LoadUri(const string& /*uri*/,
                           const string& /*mimeType*/) throw ()
{
  // Ignore
}

void TApplication::ExecuteCommand(const string& /*command*/) throw ()
{
  // Ignore
}
