/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *--------------------------------------------------------------------
 *
 * TCaApp.h
 * This class is responsible for observing all media sessions and creating
 * CA core objects for each media player instance, to enable decrypion of all
 * played media. The class also implements all the required functionality to
 * act as a KreaTV application.
 *
 *
 * Copyright (c) 2010 Motorola inc. All Rights Reserved.
 *
 *--------------------------------------------------------------------
 */

#ifndef TCAAPP_H
#define TCAAPP_H

#include "debug/TTracker.h"
#include "interface/TNameServiceCaller.h"
#include "interface/TToiApplicationDispatcher.h"
#include "interface/TToiApplicationServiceCaller.h"

#include "interface/InternalObjectNames.h"
#include "interface/TToiMediaObserverDispatcher.h"
#include "interface/IToiTypes.h"

#include "TCaCore.h"
#include <map>

class TCaApp :public IToiApplication,
              public IToiMediaObserver

{
private:
  TTracker* Tracker;
  int32_t ApplicationId;

  // IPC variables
  TIpcServer IpcServer;
  TIpcClient IpcClient;

  // Handlers of services
  TToiNameServiceCaller* NameServiceCaller;
  TToiApplicationServiceCaller* ApplicationServiceCaller;
  TToiMediaServiceCaller* MediaServiceCaller;

  std::string ApplicationAddress;
  std::string NameServiceAddress;
  std::string MediaServiceAddress;
  std::string MediaObserverAddress;

  TToiApplicationDispatcher* ApplicationDispatcher;
  TToiMediaObserverDispatcher* MediaObserverDispatcher;

  // One TCaCore instance for each media player session.
  typedef std::map<int, TCaCore*> TCaCoreMap;
  TCaCoreMap CaCores;

public:
  TCaApp() throw ();
  virtual ~TCaApp();

  void Run() throw ();

  // IToiApplication
  virtual void Ping() throw ();
  virtual void ChangeState(const IToiApplicationService::
                           TToiApplicationState& state)
    throw ();
  virtual void LoadUri(const std::string& uri,
                       const std::string& mimeType) throw ();
  virtual void ExecuteCommand(const std::string& command) throw ();

  // IToiMediaObserver
  virtual void OnSessionStateChanged(const IToiMediaService::
                                     TToiSessionInfo& sessionInfo) throw ();
  virtual void OnSessionStreamInfoChanged(const IToiMediaService::
                                          TToiSessionInfo& sessionInfo,
                                          const IToiMediaSession::
                                          TToiStreamInfo& streamInfo) throw ();
};

#endif // TCAAPP_H
