/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *-----------------------------------------------------------------------------
 *
 * TCaCore.h
 * This class is responsible for enabling decryption for a media player
 * session. It sets the cipher and keys needed in the session.
 *
 * Copyright (c) 2010 Motorola, Inc. All Rights Reserved.
 * Copyright (c) 2011 Motorola Mobility, Inc. All rights reserved.
 *
 *-----------------------------------------------------------------------------
 */

#ifndef TCACORE_H
#define TCACORE_H
#include <stdint.h>
#include "debug/TTracker.h"

#include "ipc/TIpcServer.h"
#include "ipc/TIpcClient.h"

#include "interface/TToiMediaServiceCaller.h"
#include "interface/TToiMediaSessionCaller.h"
#include "event/IIOTimeoutHandler.h"


class TCaCore : public IIOTimeoutHandler
{
private:
  // IPC variables
  TIpcServer& IpcServer;
  TIpcClient IpcClient;

  // Handles of media service and media session.
  TToiMediaServiceCaller* MediaServiceCaller;
  TToiMediaSessionCaller* MediaSessionCaller;

  int32_t ApplicationId;

  std::string MediaSessionObserverAddress;

  TTracker* Tracker;

  // Info about the session
  int SessionId;
  int SessionType;
  std::string SessionUrl;

  // The list of pids, in which the media data is encrypted.
  IToiMediaSession::TToiStreamIdSequence EsPids;

  // Timer identifier, used to demonstrate key changes at regular intervals.
  int UpdateKeyTimerId;

  // Use the functions of media session to enable decryption.
  bool DoSetDecryptKey() throw ();

  // Info of decryption
  uint8_t OddControlWord[32];
  uint8_t EvenControlWord[32];
public:
  TCaCore(TIpcServer& ipcServer,
          TToiMediaServiceCaller* mediaService,
          int32_t applicationId,
          long sessionId,
          long sessionType,
          std::string sessionUrl) throw();

  virtual ~TCaCore();

  // IIOTimeoutHandler
  virtual void HandleTimeout(int timerId) throw ();
};

#endif //TCACORE_H
