// Copyright (c) 2010 Motorola, Inc. All Rights Reserved.

#include "TMedia.h"

#include "interface/ToiObjectNames.h"

#include <iostream>

using namespace std;

TMedia::TMedia(TIpcClient& ipcClient, IIpcEventHandler& ipcEventHandler,
               IToiNameService& nameService,
               IToiApplicationService::TToiApplicationId applicationId,
               const string& socketName) :
  IpcClient(ipcClient),
  ApplicationId(applicationId),
  ObserverAddress(ipcEventHandler.CreateObject()),
  Dispatcher(&ipcEventHandler, ObserverAddress, this),
  MediaSession(NULL),
  RelayId(0),
  SocketName(socketName)
{
  // Connect to media service
  string mediaAddress;
  nameService.LookupObject(TOI_MEDIA_SERVICE_NAME, mediaAddress);
  MediaService = new TToiMediaServiceCaller(&IpcClient, mediaAddress);
  MediaService->RegisterObserver(ApplicationId, ObserverAddress);
}

TMedia::~TMedia() throw ()
{
  RemoveSession();
  MediaService->ReleaseObserver(ObserverAddress);
  delete MediaService;
}

void TMedia::RemoveSession()
{
  if (MediaSession != NULL) {
    MediaSession->RemoveDataRelay(RelayId);
    MediaSession->ReleaseInstance();
    delete MediaSession;
    MediaSession = NULL;
  }
}

void TMedia::OnSessionStateChanged(const IToiMediaService::TToiSessionInfo&
                                   sessionInfo) throw ()
{
  if (sessionInfo.Type != IToiMediaSession::TYPE_PLAYER) {
    return;
  }
  if (sessionInfo.State != IToiMediaSession::STATE_RUNNING) {
    return;
  }

  // Subscribe to the PAT PID (0)
  RemoveSession();
  string sessionAddress;
  MediaService->OpenSession(ApplicationId, sessionInfo.Id, sessionAddress);
  MediaSession = new TToiMediaSessionCaller(&IpcClient, sessionAddress);
  MediaSession->AddPacketDataRelay(0, SocketName, RelayId);
}
