/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *---------------------------------------------------------------------------
 *
 * TRecorder.cpp
 * This file is the definition of the TRecorder class.
 *
 * Copyright (c) 2010 Motorola. All Rights Reserved.
 *
 *---------------------------------------------------------------------------
 */

#include "TRecorder.h"

#include "debug/TTracker.h"
#include "interface/TToiMediaRecorderCaller.h"
#include "interface/IToiTypes.h"
#include "ipc/TIpcClient.h"

TRecorder::TRecorder(TIpcClient* ipcClient,
                     const std::string& recorderAddress)
  : Tracker(TTracker::GetProcessTracker()),
    IpcClient(ipcClient),
    Recorder(NULL)
{
  Recorder = new TToiMediaRecorderCaller(ipcClient, recorderAddress);
}

TRecorder::~TRecorder()
{
  delete Recorder;
}

void TRecorder::Open(const std::string& url, const std::string& assetId)
{
  try {
    if (Recorder == NULL) {
      return;
    }
    Recorder->Open(url, assetId);
  }
  catch (std::exception& e) {
    Tracker->Error("<- %s(): exception \"%s\"", __func__, e.what());
    throw;
  }
  catch (...) {
    Tracker->Error("<- %s(): unknown exception", __func__);
    throw;
  }
}

void TRecorder::Close()
{
  try {
    if (Recorder == NULL) {
      return;
    }
    Recorder->Close();
  }
  catch (std::exception& e) {
    Tracker->Error("<- %s(): exception \"%s\"", __func__, e.what());
    throw;
  }
  catch (...) {
    Tracker->Error("<- %s(): unknown exception", __func__);
    throw;
  }
}

void TRecorder::Record()
{
  try {
    if (Recorder == NULL) {
      return;
    }
    Recorder->Record();
  }
  catch (std::exception& e) {
    Tracker->Error("<- %s(): exception \"%s\"", __func__, e.what());
    throw;
  }
  catch (...) {
    Tracker->Error("<- %s(): unknown exception", __func__);
    throw;
  }
}

void TRecorder::ReleaseInstance()
{
  try {
    if (Recorder == NULL) {
      return;
    }
    Recorder->ReleaseInstance();
  }
  catch (std::exception& e) {
    Tracker->Error("<- %s(): exception \"%s\"", __func__, e.what());
    throw;
  }
  catch (...) {
    Tracker->Error("<- %s(): unknown exception", __func__);
    throw;
  }
}

std::string TRecorder::GetAssetId()
{
  std::string assetId;
  if (Recorder != NULL) {
    Recorder->GetAssetId(assetId);
  }
  return assetId;
}
