/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *---------------------------------------------------------------------------
 *
 * TRecorder.h
 * This file declares a class whose name is "TRecorder" to handle all the
 * possible operations related to channel recording, such as
 * Open()/Record()/Close().
 *
 * Copyright (c) 2010 Motorola. All Rights Reserved.
 *
 *---------------------------------------------------------------------------
 */

#ifndef TRECORDER_H
#define TRECORDER_H

#include <string>

class TToiMediaRecorderCaller;
class TIpcClient;
class TTracker;

class TRecorder
{
private:
  // A handler to print out logs.
  TTracker* Tracker;

  // IPC variables.
  TIpcClient* IpcClient;

  // A handler to control recording.
  TToiMediaRecorderCaller* Recorder;

public:
  TRecorder(TIpcClient* ipcClient,
            const std::string& recorderAddress);

  virtual ~TRecorder();

  // This function starts a new recording session by trying to establish a
  // connection to a media source.
  void Open(const std::string& url, const std::string& assetId);

  // This function ends the current session by stopping any recording and
  // disconnecting from the media source.
  void Close();

  // This function starts recording of the stream.
  void Record();

  // This function releases the resources used by this instance.
  void ReleaseInstance();

  // Returns the asset used for recording.
  std::string GetAssetId();

};


#endif
