// Copyright (c) 2010 Motorola, Inc. All Rights Reserved.

#include "TSocket.h"

#include "base/TException.h"
#include "media/libH222/TTsPatHandle.h"

#include <cstring>

#include <sys/socket.h>
#include <sys/un.h>
#include <unistd.h>

using namespace std;

TSocket::TSocket(TMedia& media, const string& path) :
  Media(media),
  Path(path)
{
  // Create a UNIX datagram socket for server
  Socket = socket(PF_UNIX, SOCK_DGRAM, 0);
  if (Socket < 0) {
    throw TException("Could not create socket");
  }

  // Set up address structure for server socket
  sockaddr_un address;
  address.sun_family = AF_UNIX;
  strncpy(address.sun_path, Path.c_str(), sizeof(address.sun_path));

  // Bind to socket
  unlink(Path.c_str());
  if (bind(Socket, (sockaddr*)&address, sizeof(address)) < 0) {
    close(Socket);
    throw TException("Could not bind to socket");
  }
}

TSocket::~TSocket()
{
  unlink(Path.c_str());
  close(Socket);
}

int TSocket::GetSocket() const
{
  return Socket;
}

void TSocket::HandleEvent(int fd, uint32_t /*event*/) throw ()
{
  // As we only have subscribed to one fd and one event we do not need to
  // sanity check.
  uint8_t buffer[128];
  read(fd, buffer, sizeof(buffer));
  TTsPatHandle(buffer).PrintInfo(cerr);
  Media.RemoveSession();
}
