#!/usr/bin/perl

use FindBin;
use lib $FindBin::Bin;

use strict;
use warnings FATAL => 'all';
use KreaTV;

# This script adds a local portal to the infobroker_default_objects.xml file
#
# If the cfg.portal.whitelisturls object was missing, it is created.
# If a certain local portal URL did not exist, it is added.
# Duplicate portal URLs are not added.
# If the 3rd parameter is "Startup", it means the url is used by startup
# portal, and the url will be added to the head of portal url list. Any other
# value will have the url to be added to the end of the list.

my $rootdisk = shift;
my $portal_url = shift;
my $portal_option = shift;

my $configfile = "$rootdisk/etc/infobroker_default_objects.xml";
my $filedata = KreaTV::backticks("cat $configfile");

# Uncomment for increased verbosity during debug
#print "Before: $filedata\n";

# Create portal URL object if it was missing
if (!($filedata =~ m!<Name>cfg.portal.whitelisturls</Name>!)) {
    $filedata =~ s!</Objects>!  <Object>\n    <Name>cfg.portal.whitelisturls</Name>\n    <Value>&lt;?xml version=\"1.0\"?>&lt;PortalURLs>&lt;/PortalURLs></Value>\n  </Object>\n</Objects>!;
}

# Add local portal URL if it was missing
if (!($filedata =~ m!&lt;LocalURL>$portal_url&lt;/LocalURL>!)) {
    print "Adding local portal URL\n";
    if (defined $portal_option && $portal_option eq "Startup") {
        $filedata =~ s!&lt;PortalURLs>!&lt;PortalURLs>&lt;LocalURL>$portal_url&lt;/LocalURL>!;
    }
    else {
        $filedata =~ s!&lt;/PortalURLs>!&lt;LocalURL>$portal_url&lt;/LocalURL>&lt;/PortalURLs>!;
    }
}

# Uncommend for increased verbosity during debug
#print "After: $filedata";

open(FILE, "> $configfile") or die "Error: Could not open $configfile: $!";
print FILE "$filedata";
close FILE;
