/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *---------------------------------------------------------------------------
 *
 * MulticastModule.cpp
 *
 * This file contains the access point for the loadable module which
 * contains the code for the file source element and the multicast
 * source element.
 *
 * Copyright (c) 2004 Kreatel Communications AB. All Rights Reserved.
 *
 *---------------------------------------------------------------------------
 */

#include "TMulticastSourceDescriptor.h"
#include "platform/ILoadableModule.h"

extern "C" {
  // The class factory for the file source descriptor class:
  static const char* GetMulticastClassName();
  static uint32_t GetMulticastClassVersion();
  static void* CreateMulticastObject();

  static const char* GetInterfaceName();
  static const char* GetBinding();
  static void DestroyObject(void* object);

  // The entry function for the module:
  ILoadableModule* GetModule();
}


// ----------------------------------------------------------------------------
// Multicast

static const char* GetMulticastClassName()
{
  return "Multicast Source Descriptor";
}

static uint32_t GetMulticastClassVersion()
{
  return 1;
}

static void* CreateMulticastObject()
{
  static TMulticastSourceDescriptor Descriptor;
  IElementDescriptor* object = &Descriptor;
  return object;
}


// ----------------------------------------------------------------------------
// Common

static const char* GetInterfaceName()
{
  return "IElementDescriptor";
}

static const char* GetBinding()
{
  return "C++";
}

static void DestroyObject(void* /*object*/)
{
  // Returned static object, no need to destroy!
}


// ----------------------------------------------------------------------------
// Factory structs

static IModuleClassFactory MulticastSourceDescriptorFactory = {
  GetMulticastClassName,
  GetMulticastClassVersion,
  GetInterfaceName,
  GetBinding,
  CreateMulticastObject,
  DestroyObject
};


// ----------------------------------------------------------------------------
// Module

static const char* GetModuleName()
{
  return "Multicast Source Element";
}

static uint32_t GetClassCount()
{
  return 1;
}

static const IModuleClassFactory* GetFactory(uint32_t index)
{
  const IModuleClassFactory* result = NULL;
  switch (index) {
  case 0:
    result = &MulticastSourceDescriptorFactory;
    break;
  }
  return result;
}

static ILoadableModule Module = {
  GetModuleName,
  GetClassCount,
  GetFactory,
  NULL,
  NULL
};

ILoadableModule* GetModule()
{
  return &Module;
}
