/*
 *--------------------------------------------------------------------
 *
 * TBasicSourceElement.h
 *
 * Base class for basic source elements. This can be used for simple
 * source elements, i.e. file / unicast / multicast etc.
 * 
 * Copyright (c) 2005 Kreatel Communications AB. All Rights Reserved.
 *
 *--------------------------------------------------------------------
 */

#ifndef TBASICSOURCEELEMENT_H
#define TBASICSOURCEELEMENT_H

#include "media/streamer/IBlackboard.h"
#include "media/streamer/IDebug.h"
#include "media/streamer/IElement.h"
#include "media/streamer/IRoot.h"
#include "media/streamer/ISource.h"
#include "media/streamer/IEvent.h"
#include <cassert>

class TBasicSourceElement : public IElement,
                            public IRoot,
                            public ISource,
                            public IEventCallback
{
private:
  // The descriptor registered in IEvent or -1.
  int RegisteredDescriptor;

protected:
  // The descriptor we read data from, e.g. a multicast socket.
  int FileDescriptor;
  bool DataFlowFlag;

  // Pointer to the allotted buffer.
  uint8_t* Buffer;
  // Size of the allotted buffer.
  uint32_t BufferSize;
  // Number of bytes read into the buffer.
  uint32_t ReadCount;

  IPump* Pump;
  IDebug* Debug;
  IOutputPad* OutputPad;
  IEvent* EventEngine;
  ISourceCallback* SourceCallback;
  IBlackboard* Blackboard;

  virtual bool Connect(const std::string& uri) throw () = 0;
  virtual void Cleanup() throw () = 0;
  virtual int Read(uint8_t* buffer, uint32_t size) throw () = 0;
  virtual bool IsBufferFilled() throw () = 0;
  virtual const char* GetConnectionType() throw () = 0;

  void RegisterDescriptor(int descriptor) throw ();
  void UnregisterDescriptor() throw ();

  bool DoProcess() throw ();
  void DoFlush() throw ();
  void CommitDataBuffer() throw ();
  void SetPlaybackMode() throw ();
  void CommitConnectionType() throw ();
  void Failure() throw ();

public:
  enum { PACE_PAUSE = 0};
  enum { PACE_PLAY = 1000};
  enum { PACE_FASTFORWARD = 5000};
  enum { PACE_REWIND = -5000};

  TBasicSourceElement() throw ();
  virtual ~TBasicSourceElement() throw ();

  // IElement interface.
  virtual void Destroy() throw ();
  virtual void Initialize(IPump* pump, const char* dataFormat) throw ();
  virtual void* QueryInterface(const char* interfaceName) throw ();

  // IEventCallback interface.
  virtual void OnEvent(int fd, uint32_t eventMask) throw ();
  virtual void OnTimeout(int timerId) throw ();

  // IRoot interface.
  virtual void Process(uint8_t* buffer, uint32_t size) throw ();
  virtual void Flush() throw ();

  // ISource interface.
  virtual void Open(const char* uri) throw ();
  virtual void Close() throw ();
  virtual void Play(int32_t pace) throw ();
  virtual void PlayFromPosition(uint32_t position, int32_t pace) throw ();
};


#endif
