/*
 *--------------------------------------------------------------------
 *
 * TMulticastSourceDescriptor.cpp
 *
 * This class describes the multicast source element.
 *
 * Copyright (c) 2003 Kreatel Communications AB. All Rights Reserved.
 *
 *--------------------------------------------------------------------
 */

#include "TMulticastSourceDescriptor.h"
#include "TMulticastSourceElement.h"
#include "media/streamer/ParameterNames.h"

#include <cstdio>

namespace {
  // An anonymous namespace forces local linkage, which enables the
  // linker to create a smaller executable.

  const IElementDescriptor::TMetadataInfo MetadataInfo[] = {
    { PARAM_CONNECTION_TYPE,
      IElementDescriptor::TMetadataInfo::INPUT_PRODUCED,
      IElementDescriptor::TMetadataInfo::OUTPUT_COMMITTED,
      IElementDescriptor::TMetadataInfo::TYPE_CONTINUAL },
  };

  const IElementDescriptor::TMetadataInfo* const MetadataInfoVector[] = {
    &MetadataInfo[0],
    NULL
  };

  const char* OutputFormats[] = {
    "Raw",
    NULL
  };

  const IElementDescriptor::TProperties Properties = {
    "Multicast Source",
    500,
    INPUT_NONE,
    "Raw",
    IElementDescriptor::TProperties::ELEMENT_SOURCE,
    MetadataInfoVector,
    NULL
  };
}


void TMulticastSourceDescriptor::GetProperties(TProperties* properties)
  throw ()
{
  *properties = Properties;
}

bool TMulticastSourceDescriptor::MatchUri(const std::string& uri) throw ()
{
  int schemeLength = 0;
  if (uri.find("igmp://") == 0) {
    schemeLength = 7;
  }
  uint32_t b1, b2, b3, b4;
  uint32_t port;
  if (::sscanf(uri.c_str() + schemeLength, "%u.%u.%u.%u:%u",
               &b1, &b2, &b3, &b4, &port) != 5) {
    return false;
  }
  return (b1 >= 224 && b1 <= 239 &&
          b2 <= 255 && b3 <= 255 && b4 <= 255 &&
          port > 0);
}

bool TMulticastSourceDescriptor::HasTrait(const std::string& /*uri*/) throw ()
{
  return false;
}

IElement* TMulticastSourceDescriptor::CreateElement() throw ()
{
  return new TMulticastSourceElement;
}
