/*
 *--------------------------------------------------------------------
 *
 * TMulticastSourceElement.h
 *
 * A Streamer Source Element reading from a multicast socket.
 * 
 * Copyright (c) 2005 Kreatel Communications AB. All Rights Reserved.
 *
 *--------------------------------------------------------------------
 */

#ifndef TMULTICASTSOURCEELEMENT_H
#define TMULTICASTSOURCEELEMENT_H

#include "TBasicSourceElement.h"

#include <netinet/in.h>

class TMulticastSourceElement : public TBasicSourceElement
{
private:
  enum { MAX_UDP_SIZE        = 64 * 1024 };
  // Size of the buffer space received from the Streamer in each call
  // to Process().
  enum { PROCESS_BUFFER_SIZE = 2 * MAX_UDP_SIZE };
  // Size of socket receive buffer.
  enum { SOCKET_BUFFER_SIZE  = 512 * 1024 };
  
  bool JoinedFlag;
  ip_mreq MulticastRequest;

  // Socket helper functions
  void OpenSocket() throw ();
  bool CloseSocket() throw ();
  bool JoinChannel(const char* address, uint16_t port) throw ();
  bool LeaveChannel() throw ();

  // Interface from TBasicSourceElement.
  virtual bool Connect(const std::string& uri) throw ();
  virtual void Cleanup() throw ();
  virtual int Read(uint8_t* buffer, uint32_t size) throw ();
  virtual bool IsBufferFilled() throw ();
  virtual const char* GetConnectionType() throw ();

public:
  TMulticastSourceElement() throw ();
  virtual ~TMulticastSourceElement() throw ();

  // Parts of the IRoot interface.
  virtual uint32_t GetBufferRequirement() throw ();

  // Parts of the ISource interface.
  virtual bool IsFeatureSupported(const char* feature) throw ();
  virtual void Play(int32_t pace) throw ();
};

#endif
