/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *--------------------------------------------------------------------
 *
 * ExampleInternal.h
 *
 * This header contains common definitions for runtime plugin
 *
 * Copyright (c) 2010 Motorola Inc. All Rights Reserved.
 * Copyright (c) 2011 Motorola Mobility, Inc. All rights reserved.
 *
 * This program is confidential and proprietary to Motorola Mobility, Inc and
 * may not be copied, reproduced, disclosed to others, published or used, in
 * whole or in part, without the expressed prior written permission of Motorola
 * Mobility, Inc.
 *
 *--------------------------------------------------------------------
 */
#ifndef EXAMPLEINTERNAL_H
#define EXAMPLEINTERNAL_H

#include <boost/shared_ptr.hpp>
#include <boost/intrusive_ptr.hpp>
#include "npapi/npapi.h"
#include "npapi/npfunctions.h"
#include "npapi/npruntime.h"

#ifdef FORCE_DEBUG
#include "debug/TDebug.h"
#define TRACE(f, args...) TTracker::GetProcessTracker()-> \
  Debug("%s " f "", __PRETTY_FUNCTION__, ## args)
#else
#define TRACE(...)
#endif

extern NPNetscapeFuncs NetscapeFunctions;

#ifndef HIBYTE
#define HIBYTE(x) ((((uint32_t)(x)) & 0xff00) >> 8)
#endif

#ifndef LOBYTE
#define LOBYTE(W) ((W) & 0xFF)
#endif

// fix for inconsistence in npapi.h
extern "C" int32_t NPP_Write_Fixed(NPP, NPStream*, int32_t, int32_t, void* );

// functions created for shared_ptr<NPVariant> handling
typedef boost::shared_ptr<NPVariant> TSharedVariant;

TSharedVariant ObjectInvoke(NPP npp, NPObject *obj, NPIdentifier methodName,
                            const NPVariant *args, unsigned argCount);
TSharedVariant ObjectInvokeDefault(NPP npp, NPObject *obj,
                                   const NPVariant *args, unsigned argCount);
TSharedVariant ObjectEvaluate(NPP npp, NPObject *obj, NPString *script);
TSharedVariant ObjectGetProperty(NPP npp, NPObject *obj,
                                 NPIdentifier propertyName);

class TExamplePlugin;
TExamplePlugin* GetPluginClass(NPP instance);

const double PAGE_EVENT_DELTA = 0.2;
const double FLASH_EVENT_DELTA = 2;

// do not call directly
void intrusive_ptr_add_ref(NPObject*);
void intrusive_ptr_release(NPObject*);

typedef boost::intrusive_ptr<NPObject> TNPObject;

std::string ToString(NPIdentifier pointer);

#endif //EXAMPLEINTERNAL_H
