/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *--------------------------------------------------------------------
 *
 * TExamplePlugin.h
 *
 * This file contains example plugin class declaration
 *
 * Copyright (c) 2010 Motorola Inc. All Rights Reserved.
 * Copyright (c) 2011 Motorola Mobility, Inc. All rights reserved.
 *
 * This program is confidential and proprietary to Motorola Mobility, Inc and
 * may not be copied, reproduced, disclosed to others, published or used, in
 * whole or in part, without the expressed prior written permission of Motorola
 * Mobility, Inc.
 *
 *--------------------------------------------------------------------
 */

#ifndef TEXAMPLEPLUGIN_H
#define TEXAMPLEPLUGIN_H
#include <string>
#include <map>
#include <boost/shared_ptr.hpp>
#include <boost/scoped_ptr.hpp>
#include <Plugins/EKPluginEvent.h>
#include <base/NonCopyable.h>
#include <event/TUnixEventLoop.h>
#include <event/IIOEventHandler.h>
#include <event/IIOTimeoutHandler.h>
#include <event/TDeferredMethodCaller.h>
#include <ipc/TIpcClient.h>
#include <ipc/TIpcEventHandler.h>
#include <interface/TToiInformationObserverDispatcher.h>
#include <interface/IToiInformationObserver.h>
#include <interface/TToiInformationServiceCaller.h>
#include "ExampleInternal.h"
//#include "TExampleRenderer.h"
#include "TTestRenderer.h"

class TExamplePlugin : private IIOEventHandler,
  private IIOTimeoutHandler,
  private IToiInformationObserver
// IIOEventHandler needed for event-based model of the behaviour
// TThreadEntry needed to run event loop in a separate thread
// TDeferredMethodCaller is needed as we receive external events from the
// browser thread and need to execute them in the example event loop
// IToiInformationObserver needed to implement is variable changes
{
private:
  // no copies or default constructor
  NON_COPYABLE(TExamplePlugin);
  TExamplePlugin();

  static void* ThreadEntry(void* data);
  // event handlers
  virtual void HandleEvent(int fd, uint32_t event) throw ();
  virtual void HandleTimeout(int timerId) throw ();
  // this is from IToiInfomationObserver
  virtual void OnObjectsChanged(const
                                IToiInformationService::TToiObjectNameSequence&
                               ) throw (TIpcException);
  virtual void OnObjectsRemoved(const
                                IToiInformationService::TToiObjectNameSequence&
                               ) throw (TIpcException) {/*empty*/};

  void StartExample();
  void StopExample();
  void IdleAction();
  void MakePageUpdate();
  void PrepareError(const std::string&);
  const NPP parent_;
  NPWindow* window_;

  //class TExampleRenderer renderer_;
  //boost::scoped_ptr<DvbExampleHandler> example_;
  class TTestRenderer renderer_;
  TUnixEventLoop eventLoop_;
  pthread_t thread;
  boost::scoped_ptr<TDeferredMethodCaller> methodCaller_;
  TIpcClient ipcClient_;
  TIpcEventHandler ipcEventHandler_;
  std::string observerName_;
  TToiInformationObserverDispatcher informationObserver_;
  int exampleEventHandle_;
  boost::scoped_ptr<TToiInformationServiceCaller> informationCaller_;
  time_t baseTime_;
  double lastPageUpdate_;
  double lastFlashUpdate_;
  TNPObject defaultObject_;
#ifndef SVG_MODE
  TNPObject style_;
#endif
  double transparency_;
  bool animate;

public:
  TExamplePlugin(NPP instance);
  virtual ~TExamplePlugin();
  NPError SetWindow(NPWindow* window);
  int16_t HandleEvent(void* event);
  NPError GetValue(NPPVariable variable, void *ret_value);
  NPError SetValue(NPNVariable variable, void *value);

  // scriptable plugin object methods
  bool InvokeDefault(const NPVariant *args,
                     uint32_t argCount, NPVariant *result);
  bool HasMethod(const std::string& name);
  bool Invoke(const std::string& name,
              const NPVariant *args, uint32_t argCount, NPVariant *result);
  bool HasProperty(const std::string& name);
  bool GetProperty(const std::string& name, NPVariant *result);
  bool SetProperty(const std::string& name, const NPVariant *value);
  bool RemoveProperty(const std::string& name);

  static void Animate(void* data);
};

#endif // TEXAMPLEPLUGIN_H
