/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *--------------------------------------------------------------------
 *
 * np_entry.cpp
 *
 * This file contains plugin entry points
 *
 * Copyright (c) 2010 Motorola Inc. All Rights Reserved.
 *
 *--------------------------------------------------------------------
 */

#include <boost/utility/value_init.hpp>
#include <cassert>
#include "ExampleInternal.h"

NPNetscapeFuncs NetscapeFunctions = boost::value_initialized<NPNetscapeFuncs>(); 

char *NPP_GetMIMEDescription();

extern "C" NPError NP_GetEntryPoints(NPPluginFuncs* pFuncs)
{
  TRACE("Entering");
  if(pFuncs == NULL)
    return NPERR_INVALID_FUNCTABLE_ERROR;

  if(pFuncs->size < sizeof(NPPluginFuncs))
    return NPERR_INVALID_FUNCTABLE_ERROR;

  pFuncs->version       = (NP_VERSION_MAJOR << 8) | NP_VERSION_MINOR;
  pFuncs->newp          = NPP_New;
  pFuncs->destroy       = NPP_Destroy;
  pFuncs->setwindow     = NPP_SetWindow;
  pFuncs->newstream     = NPP_NewStream;
  pFuncs->destroystream = NPP_DestroyStream;
  pFuncs->asfile        = NPP_StreamAsFile;
  pFuncs->writeready    = NPP_WriteReady;
  pFuncs->write         = NPP_Write_Fixed;
  pFuncs->print         = NPP_Print;
  pFuncs->event         = NPP_HandleEvent;
  pFuncs->urlnotify     = NPP_URLNotify;
  pFuncs->getvalue      = NPP_GetValue;
  pFuncs->setvalue      = NPP_SetValue;
  pFuncs->javaClass     = NULL;

  return NPERR_NO_ERROR;
}

extern "C" char* NP_GetMIMEDescription()
{
  TRACE("Entering");
  return NPP_GetMIMEDescription();
}

extern "C" NPError NP_GetValue(void* future, NPPVariable variable, void *value)
{
  TRACE("Entering");
  return NPP_GetValue(static_cast<NPP_t *>(future), variable, value);
}

extern "C" NPError NP_Initialize(NPNetscapeFuncs* pFuncs
              , NPPluginFuncs* pluginFuncs
              )
{
  TRACE("Entering");
  if(pFuncs == NULL)
    return NPERR_INVALID_FUNCTABLE_ERROR;

  if(HIBYTE(pFuncs->version) > NP_VERSION_MAJOR)
    return NPERR_INCOMPATIBLE_VERSION_ERROR;
  TRACE("pFuncs->size=%d,sizeof=%d",pFuncs->size,sizeof(NPNetscapeFuncs));
  int copySize = sizeof(NPNetscapeFuncs);
  if (pFuncs->size < copySize) {
    copySize = pFuncs->size;
  }
  memcpy(&NetscapeFunctions,pFuncs,copySize);

  NPError errorCode = NP_GetEntryPoints(pluginFuncs);
  if (NPERR_NO_ERROR != errorCode)
  {
      return errorCode;
  }

  return NPERR_NO_ERROR;
}

extern "C" NPError NP_Shutdown()
{
  TRACE("Entering");
  return NPERR_NO_ERROR;
}
