/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *--------------------------------------------------------------------
 *
 * npn_gate.cpp
 *
 * This file contains all browser functions which can be called by plugin
 *
 * Copyright (c) 2010 Motorola Inc. All Rights Reserved.
 * Copyright (c) 2011 Motorola Mobility, Inc. All rights reserved.
 *
 * This program is confidential and proprietary to Motorola Mobility, Inc and
 * may not be copied, reproduced, disclosed to others, published or used, in
 * whole or in part, without the expressed prior written permission of Motorola
 * Mobility, Inc.
 *
 *--------------------------------------------------------------------
 */

#include "ExampleInternal.h"

#include <cassert>

extern NPNetscapeFuncs NetscapeFunctions;

//these functions mimics nppapi.h functions, so they are C exports
extern "C" {

void NPN_Version(int* plugin_major, int* plugin_minor, int* netscape_major, int* netscape_minor)
{
  TRACE("plugin_major=%p, plugin_minor=%p, netscape_major=%p, netscape_minor=%p",plugin_major,plugin_minor,netscape_major,netscape_minor);
  *plugin_major   = NP_VERSION_MAJOR;
  *plugin_minor   = NP_VERSION_MINOR;
  *netscape_major = HIBYTE(NetscapeFunctions.version);
  *netscape_minor = LOBYTE(NetscapeFunctions.version);
}

NPError NPN_GetURL(NPP instance, const char* URL, const char* window)
{
  if ( NULL == NetscapeFunctions.geturl ) {
    TRACE( "pointer for function geturl is NULL. Doing nothing" );
    return NPERR_GENERIC_ERROR;
  }
  return NetscapeFunctions.geturl(instance,URL,window);
}

NPError NPN_PostURL(NPP instance, const char* URL, const char* window,
                    uint32_t len, const char* buf, NPBool file)
{
  if ( NULL == NetscapeFunctions.posturl ) {
    TRACE( "pointer for function posturl is NULL. Doing nothing" );
    return NPERR_GENERIC_ERROR;
  }
  return NetscapeFunctions.posturl(instance,URL,window,len,buf,file);
}

NPError NPN_RequestRead(NPStream* stream, NPByteRange* rangeList)
{
  if ( NULL == NetscapeFunctions.requestread ) {
    TRACE( "pointer for function requestread is NULL. Doing nothing" );
    return NPERR_GENERIC_ERROR;
  }
  return NetscapeFunctions.requestread(stream,rangeList);
}

NPError NPN_NewStream(NPP instance, NPMIMEType type, const char* window, NPStream** stream)
{
  if ( NULL == NetscapeFunctions.newstream ) {
    TRACE( "pointer for function newstream is NULL. Doing nothing" );
    return NPERR_GENERIC_ERROR;
  }
  return NetscapeFunctions.newstream(instance,type,window,stream);
}

int32_t NPN_Write(NPP instance, NPStream* stream, int32_t len, void* buffer)
{
  if ( NULL == NetscapeFunctions.write ) {
    TRACE( "pointer for function write is NULL. Doing nothing" );
    return -1;
  }
  return NetscapeFunctions.write(instance,stream,len,buffer);
}

NPError NPN_DestroyStream(NPP instance, NPStream* stream, NPReason reason)
{
  if ( NULL == NetscapeFunctions.destroystream ) {
    TRACE( "pointer for function destroystream is NULL. Doing nothing" );
    return NPERR_GENERIC_ERROR;
  }
  return NetscapeFunctions.destroystream(instance,stream,reason);
}

void NPN_Status(NPP instance, const char* message)
{
  if ( NULL == NetscapeFunctions.status ) {
    TRACE( "pointer for function status is NULL. Doing nothing" );
    return ;
  }
  return NetscapeFunctions.status(instance,message);
}

const char* NPN_UserAgent(NPP instance)
{
  if ( NULL == NetscapeFunctions.uagent ) {
    TRACE( "pointer for function uagent is NULL. Doing nothing" );
    return NULL;
  }
  return NetscapeFunctions.uagent(instance);
}

void* NPN_MemAlloc(uint32_t size)
{
  if ( NULL == NetscapeFunctions.memalloc ) {
    TRACE( "pointer for function memalloc is NULL. Doing nothing" );
    return NULL;
  }
  return NetscapeFunctions.memalloc(size);
}

void NPN_MemFree(void* ptr)
{
  if ( NULL == NetscapeFunctions.memfree ) {
    TRACE( "pointer for function memfree is NULL. Doing nothing" );
    return ;
  }
  return NetscapeFunctions.memfree(ptr);
}

uint32_t NPN_MemFlush(uint32_t size)
{
  if ( NULL == NetscapeFunctions.memflush ) {
    TRACE( "pointer for function memflush is NULL. Doing nothing" );
    return 0;
  }
  return NetscapeFunctions.memflush(size);
}

void NPN_ReloadPlugins(NPBool reloadPages)
{
  if ( NULL == NetscapeFunctions.reloadplugins ) {
    TRACE( "pointer for function reloadplugins is NULL. Doing nothing" );
    return ;
  }
  return NetscapeFunctions.reloadplugins(reloadPages);
}

void* NPN_GetJavaEnv(void)
{
  TRACE();
  if ( NULL == NetscapeFunctions.getJavaEnv ) {
    TRACE( "pointer for function getJavaEnv is NULL. Doing nothing" );
    return NULL;
  }
  return NetscapeFunctions.getJavaEnv();
}

void* NPN_GetJavaPeer(NPP instance)
{
  if ( NULL == NetscapeFunctions.getJavaPeer ) {
    TRACE( "pointer for function getJavaPeer is NULL. Doing nothing" );
    return NULL;
  }
  return NetscapeFunctions.getJavaPeer(instance);
}

NPError NPN_GetURLNotify(NPP instance, const char* URL, const char* window, void* notifyData)
{
  if ( NULL == NetscapeFunctions.geturlnotify ) {
    TRACE( "pointer for function geturlnotify is NULL. Doing nothing" );
    return NPERR_GENERIC_ERROR;
  }
  return NetscapeFunctions.geturlnotify(instance,URL,window,notifyData);
}

NPError NPN_PostURLNotify(NPP instance, const char* URL, const char* window,
                          uint32_t len, const char* buf, NPBool file,
                          void* notifyData)
{
  if ( NULL == NetscapeFunctions.posturlnotify ) {
    TRACE( "pointer for function posturlnotify is NULL. Doing nothing" );
    return NPERR_GENERIC_ERROR;
  }
  return NetscapeFunctions.posturlnotify(instance,URL,window,len,buf,file,notifyData);
}

NPError NPN_GetValue(NPP instance, NPNVariable variable, void *ret_value)
{
  if ( NULL == NetscapeFunctions.getvalue ) {
    TRACE( "pointer for function getvalue is NULL. Doing nothing" );
    return NPERR_GENERIC_ERROR;
  }
  return NetscapeFunctions.getvalue(instance,variable,ret_value);
}

NPError NPN_SetValue(NPP instance, NPPVariable variable, void *value)
{
  if ( NULL == NetscapeFunctions.setvalue ) {
    TRACE( "pointer for function setvalue is NULL. Doing nothing" );
    return NPERR_GENERIC_ERROR;
  }
  return NetscapeFunctions.setvalue(instance,variable,value);
}

void NPN_InvalidateRect(NPP instance, NPRect *rect)
{
  if ( NULL == NetscapeFunctions.invalidaterect ) {
    TRACE( "pointer for function invalidaterect is NULL. Doing nothing" );
    return ;
  }
  return NetscapeFunctions.invalidaterect(instance,rect);
}

void NPN_InvalidateRegion(NPP instance, NPRegion region)
{
  if ( NULL == NetscapeFunctions.invalidateregion ) {
    TRACE( "pointer for function invalidateregion is NULL. Doing nothing" );
    return ;
  }
  return NetscapeFunctions.invalidateregion(instance,region);
}

void NPN_ForceRedraw(NPP instance)
{
  if ( NULL == NetscapeFunctions.forceredraw ) {
    TRACE( "pointer for function forceredraw is NULL. Doing nothing" );
    return ;
  }
  return NetscapeFunctions.forceredraw(instance);
}

NPIdentifier NPN_GetStringIdentifier(const NPUTF8 *name)
{
  if ( NULL == NetscapeFunctions.getstringidentifier ) {
    TRACE( "pointer for function getstringidentifier is NULL. Doing nothing" );
    return NULL;
  }
  return NetscapeFunctions.getstringidentifier(name);
}

void NPN_GetStringIdentifiers(const NPUTF8 **names, int32_t nameCount, NPIdentifier *identifiers)
{
  if ( NULL == NetscapeFunctions.getstringidentifiers ) {
    TRACE( "pointer for function getstringidentifiers is NULL. Doing nothing" );
    return ;
  }
  return NetscapeFunctions.getstringidentifiers(names,nameCount,identifiers);
}

NPIdentifier NPN_GetIntIdentifier(int32_t intid)
{
  if ( NULL == NetscapeFunctions.getintidentifier ) {
    TRACE( "pointer for function getintidentifier is NULL. Doing nothing" );
    return NULL;
  }
  return NetscapeFunctions.getintidentifier(intid);
}

bool NPN_IdentifierIsString(NPIdentifier identifier)
{
  if ( NULL == NetscapeFunctions.identifierisstring ) {
    TRACE( "pointer for function identifierisstring is NULL. Doing nothing" );
    return false;
  }
  return NetscapeFunctions.identifierisstring(identifier);
}

NPUTF8 * NPN_UTF8FromIdentifier(NPIdentifier identifier)
{
  if ( NULL == NetscapeFunctions.utf8fromidentifier ) {
    TRACE( "pointer for function utf8fromidentifier is NULL. Doing nothing" );
    return NULL;
  }
  return NetscapeFunctions.utf8fromidentifier(identifier);
}

int32_t NPN_IntFromIdentifier(NPIdentifier identifier)
{
  if ( NULL == NetscapeFunctions.intfromidentifier ) {
    TRACE( "pointer for function intfromidentifier is NULL. Doing nothing" );
    return -1;
  }
  return NetscapeFunctions.intfromidentifier(identifier);
}

NPObject* NPN_CreateObject(NPP npp, NPClass *aClass)
{
  if ( NULL == NetscapeFunctions.createobject ) {
    TRACE( "pointer for function createobject is NULL. Doing nothing" );
    return NULL;
  }
  return NetscapeFunctions.createobject(npp,aClass);
}

NPObject* NPN_RetainObject(NPObject *obj)
{
  if ( NULL == NetscapeFunctions.retainobject ) {
    TRACE( "pointer for function retainobject is NULL. Doing nothing" );
    return NULL;
  }
  return NetscapeFunctions.retainobject(obj);
}

void NPN_ReleaseObject(NPObject *obj)
{
  if ( NULL == NetscapeFunctions.releaseobject ) {
    TRACE( "pointer for function releaseobject is NULL. Doing nothing" );
    return ;
  }
  return NetscapeFunctions.releaseobject(obj);
}


bool NPN_Invoke(NPP npp, NPObject *obj, NPIdentifier methodName, const NPVariant *args, unsigned argCount, NPVariant *result)
{
  if ( NULL == NetscapeFunctions.invoke ) {
    TRACE( "pointer for function invoke is NULL. Doing nothing" );
    return false;
  }
  return NetscapeFunctions.invoke(npp,obj,methodName,args,argCount,result);
}

bool NPN_InvokeDefault(NPP npp, NPObject *obj, const NPVariant *args, unsigned argCount, NPVariant *result)
{
  if ( NULL == NetscapeFunctions.invokeDefault ) {
    TRACE( "pointer for function invokeDefault is NULL. Doing nothing" );
    return false;
  }
  return NetscapeFunctions.invokeDefault(npp,obj,args,argCount,result);
}

bool NPN_Evaluate(NPP npp, NPObject *obj, NPString *script, NPVariant *result)
{
  if ( NULL == NetscapeFunctions.evaluate ) {
    TRACE( "pointer for function evaluate is NULL. Doing nothing" );
    return false;
  }
  return NetscapeFunctions.evaluate(npp,obj,script,result);
}

bool NPN_GetProperty(NPP npp, NPObject *obj, NPIdentifier  propertyName, NPVariant *result)
{
  if ( NULL == NetscapeFunctions.getproperty ) {
    TRACE( "pointer for function getproperty is NULL. Doing nothing" );
    return false;
  }
  return NetscapeFunctions.getproperty(npp,obj,propertyName,result);
}

bool NPN_SetProperty(NPP npp, NPObject *obj, NPIdentifier  propertyName, const NPVariant *value)
{
  if ( NULL == NetscapeFunctions.setproperty ) {
    TRACE( "pointer for function setproperty is NULL. Doing nothing" );
    return false;
  }
  return NetscapeFunctions.setproperty(npp,obj,propertyName,value);
}

bool NPN_RemoveProperty(NPP npp, NPObject *obj, NPIdentifier propertyName)
{
  if ( NULL == NetscapeFunctions.removeproperty ) {
    TRACE( "pointer for function removeproperty is NULL. Doing nothing" );
    return false;
  }
  return NetscapeFunctions.removeproperty(npp,obj,propertyName);
}

bool NPN_HasProperty(NPP npp, NPObject *npobj, NPIdentifier propertyName)
{
  if ( NULL == NetscapeFunctions.hasproperty ) {
    TRACE( "pointer for function hasproperty is NULL. Doing nothing" );
    return false;
  }
  return NetscapeFunctions.hasproperty(npp,npobj,propertyName);
}

bool NPN_HasMethod(NPP npp, NPObject *npobj, NPIdentifier methodName)
{
  if ( NULL == NetscapeFunctions.hasmethod ) {
    TRACE( "pointer for function hasmethod is NULL. Doing nothing" );
    return false;
  }
  return NetscapeFunctions.hasmethod(npp,npobj,methodName);
}

void NPN_ReleaseVariantValue(NPVariant *variant)
{
  if ( NULL == NetscapeFunctions.releasevariantvalue ) {
    TRACE( "pointer for function releasevariantvalue is NULL. Doing nothing" );
    return ;
  }
  return NetscapeFunctions.releasevariantvalue(variant);
}

void NPN_SetException(NPObject *obj, const NPUTF8 *message)
{
  if ( NULL == NetscapeFunctions.setexception ) {
    TRACE( "pointer for function setexception is NULL. Doing nothing" );
    return ;
  }
  return NetscapeFunctions.setexception(obj,message);
}

void NPN_PushPopupsEnabledState(NPP instance, NPBool enabled)
{
  if ( NULL == NetscapeFunctions.pushpopupsenabledstate ) {
    TRACE( "pointer for function pushpopupsenabledstate is NULL. Doing nothing" );
    return ;
  }
  return NetscapeFunctions.pushpopupsenabledstate(instance,enabled);
}

void NPN_PopPopupsEnabledState(NPP instance)
{
  if ( NULL == NetscapeFunctions.poppopupsenabledstate ) {
    TRACE( "pointer for function poppopupsenabledstate is NULL. Doing nothing" );
    return ;
  }
  return NetscapeFunctions.poppopupsenabledstate(instance);
}

bool NPN_Enumerate(NPP npp, NPObject *npobj, NPIdentifier **identifier,
                   uint32_t *count)
{
  if ( NULL == NetscapeFunctions.enumerate ) {
    TRACE( "pointer for function enumerate is NULL. Doing nothing" );
    return false;
  }
  return NetscapeFunctions.enumerate(npp,npobj,identifier,count);
}

void NPN_PluginThreadAsyncCall(NPP npp, void (*func)(void *), void *userData)
{
  if ( NULL == NetscapeFunctions.pluginthreadasynccall ) {
    TRACE( "pointer for function pluginthreadasynccall is NULL. Doing nothing" );
    return ;
  }
  return NetscapeFunctions.pluginthreadasynccall(npp,func,userData);
}

bool NPN_Construct(NPP npp, NPObject* obj, const NPVariant *args, uint32_t argCount, NPVariant *result)
{
  if ( NULL == NetscapeFunctions.construct ) {
    TRACE( "pointer for function construct is NULL. Doing nothing" );
    return false;
  }
  return NetscapeFunctions.construct(npp,obj,args,argCount,result);
}

NPError NPN_GetValueForURL(NPP npp, NPNURLVariable variable, const char* url, char** value, uint32_t* len)
{
  if ( NULL == NetscapeFunctions.getvalueforurl ) {
    TRACE( "pointer for function getvalueforurl is NULL. Doing nothing" );
    return NPERR_GENERIC_ERROR;
  }
  return NetscapeFunctions.getvalueforurl(npp,variable,url,value,len);
}

NPError NPN_SetValueForURL(NPP npp, NPNURLVariable variable, const char* url,
                           const char* value, uint32_t len)
{
  if ( NULL == NetscapeFunctions.setvalueforurl ) {
    TRACE( "pointer for function setvalueforurl is NULL. Doing nothing" );
    return NPERR_GENERIC_ERROR;
  }
  return NetscapeFunctions.setvalueforurl(npp,variable,url,value,len);
}

NPError NPN_GetAuthentication(NPP npp, const char* protocol, const char* host, int32_t port, const char* scheme, const char *realm, char** username, uint32_t* ulen, char** password, uint32_t* plen)
{
  if ( NULL == NetscapeFunctions.getauthenticationinfo ) {
    TRACE( "pointer for function getauthenticationinfo is NULL. Doing nothing" );
    return NPERR_GENERIC_ERROR;
  }
  return NetscapeFunctions.getauthenticationinfo(npp,protocol,host,port,scheme,realm,username,ulen,password,plen);
}

uint32_t NPN_ScheduleTimer(NPP npp, uint32_t interval, NPBool repeat,
                           void (*timerFunc)(NPP npp, uint32_t timerID))
{
  if ( NULL == NetscapeFunctions.scheduletimer ) {
    TRACE( "pointer for function scheduletimer is NULL. Doing nothing" );
    return 0;
  }
  return NetscapeFunctions.scheduletimer(npp,interval,repeat,timerFunc);
}

void NPN_UnscheduleTimer(NPP npp, uint32_t timerID)
{
  if ( NULL == NetscapeFunctions.unscheduletimer ) {
    TRACE( "pointer for function unscheduletimer is NULL. Doing nothing" );
    return ;
  }
  return NetscapeFunctions.unscheduletimer(npp,timerID);
}

NPError NPN_PopUpContextMenu(NPP instance, NPMenu* menu)
{
  if ( NULL == NetscapeFunctions.popupcontextmenu ) {
    TRACE( "pointer for function popupcontextmenu is NULL. Doing nothing" );
    return NPERR_GENERIC_ERROR;
  }
  return NetscapeFunctions.popupcontextmenu(instance,menu);
}

// closing extern "C"
}

// reworked C++ functions

void ReleaseVariantValue(NPVariant *value)
{
  if ( NULL == value ) {
    return;
  }
  if ( value->type == NPVariantType_String
       || value->type == NPVariantType_Object ) {
    NPN_ReleaseVariantValue(value);
  }
  delete value;
}

TSharedVariant ObjectInvoke(NPP npp, NPObject *obj, NPIdentifier methodName,
                            const NPVariant *args, unsigned argCount)
{
  NPVariant result;
  if ( NULL == NetscapeFunctions.invoke ||
       !NetscapeFunctions.invoke(npp,
                                 obj,
                                 methodName,
                                 args,
                                 argCount,
                                 &result)) {
    TRACE( "pointer for function invoke is NULL or no result. Doing nothing" );
    return TSharedVariant();
  }
  return TSharedVariant(new NPVariant(result),ReleaseVariantValue);
}

TSharedVariant ObjectInvokeDefault(NPP npp, NPObject *obj,
                                   const NPVariant *args, unsigned argCount)
{
  NPVariant result;
  if ( NULL == NetscapeFunctions.invokeDefault ||
       !NetscapeFunctions.invokeDefault(npp,obj,args,argCount,&result) ) {
    TRACE( "pointer for function invokeDefault is NULL or no result."
           " Doing nothing" );
    return TSharedVariant();
  }
  return TSharedVariant(new NPVariant(result),ReleaseVariantValue);
}

TSharedVariant ObjectEvaluate(NPP npp, NPObject *obj, NPString *script)
{
  NPVariant result;
  if ( NULL == NetscapeFunctions.evaluate ||
       !NetscapeFunctions.evaluate(npp,obj,script,&result) ) {
    TRACE( "pointer for function evaluate is NULL or no result."
           " Doing nothing" );
    return TSharedVariant();
  }
  return TSharedVariant(new NPVariant(result),ReleaseVariantValue);
}

TSharedVariant ObjectGetProperty(NPP npp, NPObject *obj,
                                 NPIdentifier propertyName)
{
  NPVariant result;
  if ( NULL == NetscapeFunctions.getproperty ||
       !NetscapeFunctions.getproperty(npp,obj,propertyName,&result) ) {
    TRACE( "pointer for function getproperty is NULL or no result."
           " Doing nothing" );
    return TSharedVariant();
  }
  return TSharedVariant(new NPVariant(result),ReleaseVariantValue);
}

// boost::intrusive_ptr<NPObject> functionality
void intrusive_ptr_add_ref(NPObject *obj)
{
  NPN_RetainObject(obj);
}

void intrusive_ptr_release(NPObject *obj)
{
  NPN_ReleaseObject(obj);
}

