/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *--------------------------------------------------------------------
 *
 * npp_gate.cpp
 *
 * This file contains redirectord for exported plugin methods
 *
 * Copyright (c) 2010 Motorola Inc. All Rights Reserved.
 * Copyright (c) 2011 Motorola Mobility, Inc. All rights reserved.
 *
 * This program is confidential and proprietary to Motorola Mobility, Inc and
 * may not be copied, reproduced, disclosed to others, published or used, in
 * whole or in part, without the expressed prior written permission of Motorola
 * Mobility, Inc.
 *
 *--------------------------------------------------------------------
 */

#include <set>
#include <memory>
#include "ExampleInternal.h"
#include "TExamplePlugin.h"

typedef std::set<TExamplePlugin*> TExamplePluginPointerSet;
TExamplePluginPointerSet livingPlugins;

TExamplePlugin* GetPluginClass(NPP instance)
{
  if (NULL == instance || NULL == instance->pdata
      || livingPlugins.find(static_cast<TExamplePlugin*>(instance->pdata))
      == livingPlugins.end() )
  {
    return NULL;
  }
  return static_cast<TExamplePlugin*>(instance->pdata);
}

char* NPP_GetMIMEDescription(void)
{
  return const_cast<char *>("application/motorola-example-plugin:"
                            ".xxx:Motorola Example Plugin");
}

extern "C" {

void NPP_Shutdown(void)
{
  TRACE();
}

// here the plugin creates an instance of our CPlugin object which 
// will be associated with this newly created plugin instance and 
// will do all the neccessary job
NPError NPP_New(NPMIMEType type __attribute__ ((unused)),
                NPP instance __attribute__ ((unused)),
                uint16_t mode __attribute__ ((unused)),
                int16_t argc __attribute__ ((unused)),
                char* argn[] __attribute__ ((unused)),
                char* argv[] __attribute__ ((unused)),
                NPSavedData* saved __attribute__ ((unused)))
{
  TRACE("type=%s,instance=%p,mode=%hd,argc=%hd,saved=%p",type,instance,mode,argc,saved);
  for (int i=0; i<argc; ++i) {
    TRACE("\t%d - %s = %s",i,argn[i],argv[i]);
  }
  if(instance == NULL)
    return NPERR_INVALID_INSTANCE_ERROR;
  TExamplePlugin* pointer = GetPluginClass(instance);
  if ( NULL != pointer ) {
    TRACE("\tprevious plugin destroyed");
    delete pointer;
  }

  std::auto_ptr<TExamplePlugin> plugin( new TExamplePlugin(instance) );
  if(NULL == plugin.get())
    return NPERR_OUT_OF_MEMORY_ERROR;
  livingPlugins.insert(plugin.get());
  instance->pdata = static_cast<void *>(plugin.release());
  return NPERR_NO_ERROR;
}

NPError NPP_Destroy (NPP instance, NPSavedData** /* data */ )
{
  TRACE("instance=%p",instance);
  TExamplePlugin* pointer = GetPluginClass(instance);
  if ( NULL == pointer ) {
    return NPERR_INVALID_INSTANCE_ERROR;
  }
  delete pointer;
  instance->pdata = NULL;
  livingPlugins.erase( pointer );
  TRACE("instance=%p destroy completed",instance);
  return NPERR_NO_ERROR;
}

NPError NPP_SetWindow(NPP instance, NPWindow* window)
{
  TRACE("instance=%p,window=%p",instance,window);
  TExamplePlugin* p = GetPluginClass(instance);
  if ( NULL == p ) {
    return NPERR_INVALID_INSTANCE_ERROR;
  }
  return p->SetWindow(window);
}

NPError NPP_NewStream(NPP instance,
                      NPMIMEType type __attribute__ ((unused)),
                      NPStream* stream __attribute__ ((unused)),
                      NPBool seekable __attribute__ ((unused)),
                      uint16_t* stype __attribute__ ((unused)))
{
  TRACE("instance=%p,type=%s,stream=%p,seekable=%hhd,stype=%p",instance,type,stream,seekable,stype);
  TExamplePlugin* p = GetPluginClass(instance);
  if ( NULL == p ) {
    return NPERR_INVALID_INSTANCE_ERROR;
  }
  //return p->NewStream(type,stream,seekable,stype);
  return NPERR_GENERIC_ERROR;
}

NPError NPP_DestroyStream(NPP instance,
                          NPStream* stream __attribute__ ((unused)),
                          NPReason reason __attribute__ ((unused)))
{
  TRACE("instance=%p,stream=%p,reason=%hd",instance,stream,reason);
  TExamplePlugin* p = GetPluginClass(instance);
  if ( NULL == p ) {
    return NPERR_INVALID_INSTANCE_ERROR;
  }
  //return p->DestroyStream(stream,reason);
  return NPERR_GENERIC_ERROR;
}

void NPP_StreamAsFile(NPP instance,
                      NPStream* stream __attribute__ ((unused)),
                      const char* fname __attribute__ ((unused)))
{
  TRACE("instance=%p,stream=%p,fname=%s",instance,stream,fname);
  TExamplePlugin* p = GetPluginClass(instance);
  if ( NULL == p ) {
    return;
  }
  //p->StreamAsFile(stream,fname);
  return;
}

int32_t NPP_WriteReady(NPP instance, NPStream* stream __attribute__ ((unused)))
{
  TRACE("instance=%p,stream=%p",instance,stream);
  TExamplePlugin* p = GetPluginClass(instance);
  if ( NULL == p ) {
    return 0;
  }
  //return p->WriteReady(stream);
  return 0;
}


//int32_t NPP_Write(NPP instance, NPStream* stream, int32_t offset,
//                  int32_t len, void* buffer)
int32_t NPP_Write_Fixed(NPP instance,
                        NPStream* stream __attribute__ ((unused)),
                        int32_t offset __attribute__ ((unused)),
                        int32_t len __attribute__ ((unused)),
                        void* buffer __attribute__ ((unused)))
{
  TRACE("instance=%p,stream=%p,offset=%d,len=%d,buffer=%p",instance,stream,offset,len,buffer);
  TExamplePlugin* p = GetPluginClass(instance);
  if ( NULL == p ) {
    return 0;
  }
  //return p->Write(stream,offset,len,buffer);
  return 0;
}

void NPP_Print(NPP instance, NPPrint* platformPrint __attribute__ ((unused)))
{
  TRACE("instance=%p,platformPrint=%p",instance,platformPrint);
  TExamplePlugin* p = GetPluginClass(instance);
  if ( NULL == p ) {
    return;
  }
  //p->Print(platformPrint);
  return;
}

int16_t NPP_HandleEvent(NPP instance, void* event)
{
  TRACE("instance=%p,event=%p",instance,event);
  TExamplePlugin* p = GetPluginClass(instance);
  if ( NULL == p ) {
    return 0;
  }
  return p->HandleEvent(event);
}

void NPP_URLNotify(NPP instance,
                   const char* URL __attribute__ ((unused)),
                   NPReason reason __attribute__ ((unused)),
                   void* notifyData __attribute__ ((unused)))
{
  TRACE("instance=%p,URL=%s,reason=%hd,notifyData=%p",instance,URL,reason,notifyData);
  TExamplePlugin* p = GetPluginClass(instance);
  if ( NULL == p ) {
    return;
  }
  //p->URLNotify(URL,reason,notifyData);
  return;
}

NPError NPP_GetValue(NPP instance, NPPVariable variable, void *ret_value)
{
  TRACE("instance=%p,variable=%d,ret_value=%p",instance,variable,ret_value);
  TExamplePlugin* p = GetPluginClass(instance);
  if ( NULL == p ) {
    return NPERR_INVALID_INSTANCE_ERROR;
  }
  return p->GetValue(variable,ret_value);
}

NPError NPP_SetValue(NPP instance, NPNVariable variable, void *value)
{
  TRACE("instance=%p,variable=%d,value=%p",instance,variable,value);
  TExamplePlugin* p = GetPluginClass(instance);
  if ( NULL == p ) {
    return NPERR_INVALID_INSTANCE_ERROR;
  }
  return p->SetValue(variable,value);
}

// closing export "C"
}
