/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *---------------------------------------------------------------------------
 *
 * SimpleModule.cpp
 *
 * This file contains the access point for the loadable module which
 * contains the code for the "Simple Element".
 * 
 * Copyright (c) 2004 Kreatel Communications AB. All Rights Reserved.
 *
 *---------------------------------------------------------------------------
 */

#include "TSimpleDescriptor.h"

#include "platform/ILoadableModule.h"

extern "C" {
  // The class factory for the simple descriptor class:
  static const char* GetClassName();
  static uint32_t GetClassVersion();
  static const char* GetInterfaceName();
  static const char* GetBinding();
  static void* CreateObject();
  static void DestroyObject(void* object);

  // The module holding the class factories:
  static const char* GetModuleName();
  static uint32_t GetClassCount();
  static const IModuleClassFactory* GetFactory(uint32_t index);

  // The entry function for the module. This is the only symbol that
  // must be visible outside the DSO:
  ILoadableModule* GetModule();
}

static const char* GetClassName()
{
  return "Simple Descriptor";
}

static uint32_t GetClassVersion()
{
  return 1;
}

static const char* GetInterfaceName()
{
  return "IElementDescriptor";
}

static const char* GetBinding()
{
  return "C++";
}

static void* CreateObject()
{
  static TSimpleDescriptor Descriptor;
  IElementDescriptor* object = &Descriptor;
  return object;
}

static void DestroyObject(void* /*object*/)
{
  // Returned static object, no need to destroy!
}

static IModuleClassFactory SimpleDescriptorFactory = {
  GetClassName,
  GetClassVersion,
  GetInterfaceName,
  GetBinding,
  CreateObject,
  DestroyObject
};

static const char* GetModuleName()
{
  return "Simple Element";
}

static uint32_t GetClassCount()
{
  return 1;
}

static const IModuleClassFactory* GetFactory(uint32_t index)
{
  const IModuleClassFactory* result = NULL;
  switch (index) {
  case 0:
    result = &SimpleDescriptorFactory;
    break;
  }
  return result;
}

static ILoadableModule Module = {
  GetModuleName,
  GetClassCount,
  GetFactory
};

ILoadableModule* GetModule()
{
  return &Module;
}
