/*
 *--------------------------------------------------------------------
 *
 * TSimpleElement.cpp
 *
 * A simple example pipeline element. Write to the log the key-value
 * pairs of all metadata passing by in the pipeline.
 *
 * Copyright (c) 2004 Kreatel Communications AB. All Rights Reserved.
 *
 *--------------------------------------------------------------------
 */

#include "TSimpleElement.h"

#include "media/streamer/IPump.h"
#include "media/streamer/IOutputPad.h"

#include <cstring>


TSimpleElement::TSimpleElement() throw ()
  : Debug(NULL),
    OutputPad(NULL)
{
  // Empty.
}

TSimpleElement::~TSimpleElement() throw ()
{
  // Empty.
}

void TSimpleElement::Initialize(IPump* pump,
                                const char* dataFormat) throw ()
{
  // The IPump interface is generally a valuable asset that we need to
  // keep around, but in this simple example we will be able to cope
  // without it.

  // It is generally a good idea to get hold of the IDebug interface
  // right away so that we can report any irregularity without hassle.
  Debug = static_cast<IDebug*>(pump->QueryInterface("IDebug"));

  // We open the output pad right away with the same format we get on
  // our input pad.
  OutputPad = pump->OpenPad(dataFormat);

  // Just for fun, tell the log we're done.
  Debug->Debug("TSimpleElement initialization complete");
}

void* TSimpleElement::QueryInterface(const char* name) throw ()
{
  // Since this is not a source element, it is required to provide an
  // IInputPad interface.
  if (strcmp(name, "IInputPad") == 0) {
    return static_cast<IInputPad*>(this);
  }
  return NULL;
}

void TSimpleElement::Destroy() throw ()
{
  // This delete undoes the memory allocation performed by the
  // TSimpleDescriptor::CreateElement() function.
  delete this;
}

void TSimpleElement::Process(const TSegmentList& segments) throw ()
{
  for (TSegmentList::TConstIterator iter = segments.Begin();
       !iter.AtEnd(); ++iter) {
    const TMetadata* md = iter->GetFirstMetadata();
    while (md != NULL) {
      Debug->Debug("Metadata: %s=%s",
                   md->GetKey().c_str(), md->GetValue().c_str());
      OutputPad->CommitMetadata(md);
      md = md->GetNext();
    }

    if (iter->GetFrameCount() != 0) {
      OutputPad->CommitSegment(iter->GetPosition(),
                               iter->GetFrameCount(), iter->GetFrameSize(),
                               iter->GetFocusOffset(), iter->GetFocusSize());
    }
  }
}

void TSimpleElement::Flush(const TSegmentList& segments) throw ()
{
  Process(segments);
}
