/*
 * The EKLibrary library is proprietary and copyright Ekioh Ltd.
 * This EKLibrary file is licensed as follows:
 *
 * Copyright (c) 2008-2010 Ekioh Ltd.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to
 * deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

#ifndef EK_EKPLUGINMANAGERINTERFACE_H
#define EK_EKPLUGINMANAGERINTERFACE_H

#include "EKTypes.h"

namespace ekioh {

class EKPlugin;
class EKPluginClient;
class EKPluginInstance;
class EKString;
class EKUrl;

/** Plugin manager interface.
 *
 * EKPluginManager is the top-level application facing interface to the plugin
 * system.
 */
class EKPluginManagerInterface
{
public:
    /** Get the plugin manager.
     *
     * @return plugin manager
     */
    static EKPluginManagerInterface* instance();

    /** Find a plugin to handle specified mime type.
     *
     * @param[in] mimeType mime type
     * @param[in] url url of data to load into plugin (currently unused)
     *
     * @return EKPlugin or 0 if no plugin found
     */
    virtual EKPlugin* findPlugin(
            const EKString& mimeType, const EKUrl& url) = 0;

    /** Create application independent plugin instance.
     *
     * @param[in] client application specific plugin instance
     * @param[in] plugin plugin typically obtained with findPlugin
     * @param[in] mimeType mime type
     * @param[in] argc number of arguments
     * @param[in] argn argument names
     * @param[in] argv argument values
     *
     * @return application independent plugin instance or 0 on failure
     */
    virtual EKPluginInstance* createPluginInstance(
            EKPluginClient& client,
            EKPlugin& plugin,
            const EKString& mimeType, int16_t argc, char* argn[], char* argv[]) = 0;

    /** Destroy application independent plugin instance.
     *
     * @param[in] pluginInstance application independent plugin instance
     */
    virtual void destroyPluginInstance(const EKPluginInstance& pluginInstance) = 0;

protected:
    EKPluginManagerInterface();
    virtual ~EKPluginManagerInterface();
};

} // namespace

#endif
