# Copyright (c) 2011 Motorola Mobility, Inc. All rights reserved.

from kreatvipcobserver import KreaTvIpcObserver
import collections
import sys

class ToiAssetManagerObserver(KreaTvIpcObserver):
    OnAssetsAddedArguments = collections.namedtuple('OnAssetsAddedArguments', 'assetIds ')
    OnAssetsRemovedArguments = collections.namedtuple('OnAssetsRemovedArguments', 'assetIds ')
    OnAssetsChangedArguments = collections.namedtuple('OnAssetsChangedArguments', 'assetIds ')
    OnDirectoriesAddedArguments = collections.namedtuple('OnDirectoriesAddedArguments', 'dirIds ')
    OnDirectoriesRemovedArguments = collections.namedtuple('OnDirectoriesRemovedArguments', 'dirIds ')
    OnDirectoriesChangedArguments = collections.namedtuple('OnDirectoriesChangedArguments', 'dirIds ')

    def __init__(self, ipcproxy, service=None, queue=None):
        KreaTvIpcObserver.__init__(self, ipcproxy, service, queue)

    def handle_callback(self, interface_name, method_name, stack):
        """
        Pushes the call details onto the log queue.
        Called on the worker thread.
        """
        call = KreaTvIpcObserver.Call(address=self.observer_address,
                                      interface=interface_name,
                                      method=method_name)
        if method_name == 'OnAssetsAdded':
            args = {}
            temp_assetIds = []
            for o in range(stack.pop_int32()):
              temp_assetIds.append(stack.pop_string())
            args['assetIds'] = temp_assetIds
            call.arguments = ToiAssetManagerObserver.OnAssetsAddedArguments(**args)
        if method_name == 'OnAssetsRemoved':
            args = {}
            temp_assetIds = []
            for o in range(stack.pop_int32()):
              temp_assetIds.append(stack.pop_string())
            args['assetIds'] = temp_assetIds
            call.arguments = ToiAssetManagerObserver.OnAssetsRemovedArguments(**args)
        if method_name == 'OnAssetsChanged':
            args = {}
            temp_assetIds = []
            for o in range(stack.pop_int32()):
              temp_assetIds.append(stack.pop_string())
            args['assetIds'] = temp_assetIds
            call.arguments = ToiAssetManagerObserver.OnAssetsChangedArguments(**args)
        if method_name == 'OnDirectoriesAdded':
            args = {}
            temp_dirIds = []
            for o in range(stack.pop_int32()):
              temp_dirIds.append(stack.pop_int32())
            args['dirIds'] = temp_dirIds
            call.arguments = ToiAssetManagerObserver.OnDirectoriesAddedArguments(**args)
        if method_name == 'OnDirectoriesRemoved':
            args = {}
            temp_dirIds = []
            for o in range(stack.pop_int32()):
              temp_dirIds.append(stack.pop_int32())
            args['dirIds'] = temp_dirIds
            call.arguments = ToiAssetManagerObserver.OnDirectoriesRemovedArguments(**args)
        if method_name == 'OnDirectoriesChanged':
            args = {}
            temp_dirIds = []
            for o in range(stack.pop_int32()):
              temp_dirIds.append(stack.pop_int32())
            args['dirIds'] = temp_dirIds
            call.arguments = ToiAssetManagerObserver.OnDirectoriesChangedArguments(**args)
        # Thread-safe operation:
        self._queue.append(call)

    def wait_OnAssetsAdded(self, discard_non_matching_events=False,
                            timeout_sec=10, **argument_filters):
        """
        Convenience function to wait for callbacks to OnAssetsAdded.

        Keyword arguments can be given to specify parameter values to wait for.
        If a received event does not match the filter, a KreaTvIpcException
        will be raised, unless discard_non_matching_events is true in which
        case the event will be silently ignored (and the function will continue
        to wait for a matching event).
        """
        call = self.wait_callback(self.observer_address,
                                  'IToiAssetManagerObserver',
                                  'OnAssetsAdded',
                                  discard_non_matching_events,
                                  timeout_sec,
                                  argument_filters)
        return call.arguments.assetIds

    def wait_OnAssetsRemoved(self, discard_non_matching_events=False,
                            timeout_sec=10, **argument_filters):
        """
        Convenience function to wait for callbacks to OnAssetsRemoved.

        Keyword arguments can be given to specify parameter values to wait for.
        If a received event does not match the filter, a KreaTvIpcException
        will be raised, unless discard_non_matching_events is true in which
        case the event will be silently ignored (and the function will continue
        to wait for a matching event).
        """
        call = self.wait_callback(self.observer_address,
                                  'IToiAssetManagerObserver',
                                  'OnAssetsRemoved',
                                  discard_non_matching_events,
                                  timeout_sec,
                                  argument_filters)
        return call.arguments.assetIds

    def wait_OnAssetsChanged(self, discard_non_matching_events=False,
                            timeout_sec=10, **argument_filters):
        """
        Convenience function to wait for callbacks to OnAssetsChanged.

        Keyword arguments can be given to specify parameter values to wait for.
        If a received event does not match the filter, a KreaTvIpcException
        will be raised, unless discard_non_matching_events is true in which
        case the event will be silently ignored (and the function will continue
        to wait for a matching event).
        """
        call = self.wait_callback(self.observer_address,
                                  'IToiAssetManagerObserver',
                                  'OnAssetsChanged',
                                  discard_non_matching_events,
                                  timeout_sec,
                                  argument_filters)
        return call.arguments.assetIds

    def wait_OnDirectoriesAdded(self, discard_non_matching_events=False,
                            timeout_sec=10, **argument_filters):
        """
        Convenience function to wait for callbacks to OnDirectoriesAdded.

        Keyword arguments can be given to specify parameter values to wait for.
        If a received event does not match the filter, a KreaTvIpcException
        will be raised, unless discard_non_matching_events is true in which
        case the event will be silently ignored (and the function will continue
        to wait for a matching event).
        """
        call = self.wait_callback(self.observer_address,
                                  'IToiAssetManagerObserver',
                                  'OnDirectoriesAdded',
                                  discard_non_matching_events,
                                  timeout_sec,
                                  argument_filters)
        return call.arguments.dirIds

    def wait_OnDirectoriesRemoved(self, discard_non_matching_events=False,
                            timeout_sec=10, **argument_filters):
        """
        Convenience function to wait for callbacks to OnDirectoriesRemoved.

        Keyword arguments can be given to specify parameter values to wait for.
        If a received event does not match the filter, a KreaTvIpcException
        will be raised, unless discard_non_matching_events is true in which
        case the event will be silently ignored (and the function will continue
        to wait for a matching event).
        """
        call = self.wait_callback(self.observer_address,
                                  'IToiAssetManagerObserver',
                                  'OnDirectoriesRemoved',
                                  discard_non_matching_events,
                                  timeout_sec,
                                  argument_filters)
        return call.arguments.dirIds

    def wait_OnDirectoriesChanged(self, discard_non_matching_events=False,
                            timeout_sec=10, **argument_filters):
        """
        Convenience function to wait for callbacks to OnDirectoriesChanged.

        Keyword arguments can be given to specify parameter values to wait for.
        If a received event does not match the filter, a KreaTvIpcException
        will be raised, unless discard_non_matching_events is true in which
        case the event will be silently ignored (and the function will continue
        to wait for a matching event).
        """
        call = self.wait_callback(self.observer_address,
                                  'IToiAssetManagerObserver',
                                  'OnDirectoriesChanged',
                                  discard_non_matching_events,
                                  timeout_sec,
                                  argument_filters)
        return call.arguments.dirIds

