# Copyright (c) 2011 Motorola Mobility, Inc. All rights reserved.

from kreatvipccaller import KreaTvIpcCaller
from binipcparameterstack import BinIpcParameterStack
from ToiAssetManagerObserver import ToiAssetManagerObserver


class ToiAssetManagerService(KreaTvIpcCaller):

  # Enum TToiConditionOperation
  EQUALS = 0
  NOT_EQUAL = 1
  GREATER = 2
  LESS = 3
  GREATER_OR_EQUAL = 4
  LESS_OR_EQUAL = 5
  CONTAINS = 6

  # Enum TToiPropertyType
  TYPE_BOOLEAN = 0
  TYPE_STRING = 1
  TYPE_INTEGER = 2
  TYPE_DATE = 3
  TYPE_ASSETLINK = 4

  # Enum TToiAssetType
  ASSET_PVR = 0
  ASSET_IMAGE = 1
  ASSET_AUDIO = 2
  ASSET_VIDEO = 3

  # Enum TToiAssetIntegrityStatus
  INTEGRITY_UNKNOWN = 0
  INTEGRITY_OK = 1
  INTEGRITY_COULD_NOT_UPGRADE = 2
  INTEGRITY_COULD_NOT_REPAIR = 3

  # Enum TToiImportPolicy
  IMPORT_POLICY_COPY = 0
  IMPORT_POLICY_MOVE = 1

  # Const OP_RESULT_ASSET_INFO
  OP_RESULT_ASSET_INFO = 1

  # Const OP_RESULT_LONG
  OP_RESULT_LONG = 2

  # Const ROOT_DIRECTORY
  ROOT_DIRECTORY = -1

  # Const PROPERTY_SYSTEM_DURATION
  PROPERTY_SYSTEM_DURATION = "system.duration"

  # Const PROPERTY_SYSTEM_FILESIZE
  PROPERTY_SYSTEM_FILESIZE = "system.filesize"

  # Const PROPERTY_SYSTEM_PLAYBACKURI
  PROPERTY_SYSTEM_PLAYBACKURI = "system.playbackuri"

  # Const PROPERTY_SYSTEM_FILENAME
  PROPERTY_SYSTEM_FILENAME = "system.filename"

  # Const PROPERTY_SYSTEM_TYPE
  PROPERTY_SYSTEM_TYPE = "system.type"

  # Const PROPERTY_SYSTEM_INTEGRITY_STATUS
  PROPERTY_SYSTEM_INTEGRITY_STATUS = "system.integritystatus"

  # Const PROPERTY_INFO_TITLE
  PROPERTY_INFO_TITLE = "info.title"

  # Const PROPERTY_INFO_DATE
  PROPERTY_INFO_DATE = "info.date"

  # Const PROPERTY_INFO_DESCRIPTION
  PROPERTY_INFO_DESCRIPTION = "info.description"

  # Const PROPERTY_INFO_LONGDESCRIPTION
  PROPERTY_INFO_LONGDESCRIPTION = "info.longdescription"

  # Const PROPERTY_INFO_CHANNELNAME
  PROPERTY_INFO_CHANNELNAME = "info.channelname"

  # Const PROPERTY_INFO_RECORDINGSTATUS
  PROPERTY_INFO_RECORDINGSTATUS = "info.recordingstatus"

  # Const PROPERTY_INFO_MATURITYRATING
  PROPERTY_INFO_MATURITYRATING = "info.maturityrating"

  # Const PROPERTY_INFO_SHARED
  PROPERTY_INFO_SHARED = "info.shared"

  # Const PROPERTY_INFO_COVER
  PROPERTY_INFO_COVER = "info.cover"

  # Const PROPERTY_INFO_CHILDREN
  PROPERTY_INFO_CHILDREN = "info.children"

  # Const PROPERTY_INFO_LOST_AND_FOUND
  PROPERTY_INFO_LOST_AND_FOUND = "info.lostandfound"

  def __init__(self, ipcproxy):
    KreaTvIpcCaller.__init__(self, ipcproxy.ip_address, ipcproxy.port)
    self._ipcproxy = ipcproxy
    self.lookup("AssetManagerService")

  def GetAssetOperationManager(self):
    """Returns the operation manager that handles asynchronous operations for
    this asset manager. The procedure of executing asynchronous operations
    is generic and looks like this:
    <ul>
    <li>Get a handle to the operation manager.</li>
    <li>Register an observer to get operation events from the operation
    manager.</li>
    <li>Create an operation in order to get an operation identifier.</li>
    <li>Call a asynchronous operation using the operation identifier.</li>
    <li>Receive progress events for the asynchronous operation.</li>
    <li>When the operation is completed, get result data (if any).</li>
    <li>Release the operation or reuse the identifier for another
    asynchronous call.</li>
    </ul>
    
    \retval address     The address to the \ref IToiOperationManager.
    """
    __request = BinIpcParameterStack()
    __response = self.call(__request, "IToiAssetManagerService", "GetAssetOperationManager")
    __result = []
    # Read address
    __result.append(__response.pop_string())
    return __result[0]

  def CreateDirectory(self, parentId, title):
    """
    * Creates a new directory.
    *
    * \param parentId The identifier of the parent directory. Use
    *                 \ref ROOT_DIRECTORY to create the top directories.
    *
    * \param title The title to use for the directory.
    *
    * \retval dirId The identifier of the created directory.
    *
    * \exception TToiInvalidArgumentException
    *            Raised if the title is invalid or already taken, or if the
    *            parent identifier is invalid.
    * \exception TToiNotEnoughResourcesException
    *            Raised if the directory could not be created due to missing
    *            system resources.
    """
    __request = BinIpcParameterStack()
    # Write parentId
    __request.push_int32(parentId)
    # Write title
    __request.push_string(title)
    __response = self.call(__request, "IToiAssetManagerService", "CreateDirectory")
    __result = []
    # Read dirId
    __result.append(__response.pop_int32())
    return __result[0]

  def MoveDirectory(self, dirId, newParentId):
    """
    * Moves the directory to a new parent directory.
    *
    * \param dirId The directory identifier.
    *
    * \param newParentId The new parent directory identifier.
    *
    * \exception TToiInvalidArgumentException
    *            Raised if either the directory identifier or the parent
    *            identifier is invalid.
    """
    __request = BinIpcParameterStack()
    # Write dirId
    __request.push_int32(dirId)
    # Write newParentId
    __request.push_int32(newParentId)
    self.call(__request, "IToiAssetManagerService", "MoveDirectory")

  def SetDirectoryProperties(self, dirId, properties):
    """
    * Sets the properties for a directory.
    *
    * \param dirId The directory identifier.
    *
    * \param properties The list of properties.
    *
    * \exception TToiInvalidArgumentException
    *            Raised if the directory identifier is invalid.
    """
    __request = BinIpcParameterStack()
    # Write dirId
    __request.push_int32(dirId)
    # Write properties
    for iter_k in properties[::-1]:
      __request.push_string(iter_k["Id"])
      __request.push_string(iter_k["Value"])
    __request.push_int32(len(properties))
    self.call(__request, "IToiAssetManagerService", "SetDirectoryProperties")

  def RemoveDirectoryProperties(self, dirId, properties):
    """
    * Unsets the properties for a directory.
    *
    * \param dirId The directory identifier.
    *
    * \param properties The list of properties.
    
    * \exception TToiInvalidArgumentException
    *            Raised if the directory identifier is invalid.
    """
    __request = BinIpcParameterStack()
    # Write dirId
    __request.push_int32(dirId)
    # Write properties
    for iter_k in properties[::-1]:
      __request.push_string(iter_k)
    __request.push_int32(len(properties))
    self.call(__request, "IToiAssetManagerService", "RemoveDirectoryProperties")

  def RemoveDirectory(self, dirId, recursiveFlag, removeAssetsFlag):
    """
    * Remove a directory. The directory can only be removed if it is
    * empty and does not contain any assets or directories. If the
    * recursive flag is set all assets and directories will also be
    * removed.
    *
    * \param dirId The directory identifier.
    *
    * \param recursiveFlag If true all child directories are also
    *                      removed.
    *
    * \param removeAssetsFlag If true all assets in the directories are also
    *                         removed.
    *                         This completely removes the asset and
    *                         thus it will be removed from all
    *                         directories it is linked to.
    *
    * \exception TToiInvalidArgumentException
    *            Raised if the directory identifier is invalid.
    * \exception TToiPermissionDeniedException
    *            Raised if the directory contains directories/assets and the
    *            corresponding override flag (recursive/remove) is not set.
    """
    __request = BinIpcParameterStack()
    # Write dirId
    __request.push_int32(dirId)
    # Write recursiveFlag
    __request.push_bool(recursiveFlag)
    # Write removeAssetsFlag
    __request.push_bool(removeAssetsFlag)
    self.call(__request, "IToiAssetManagerService", "RemoveDirectory")

  def GetDirectoryInfo(self, dirId):
    """
    * Returns directory information.
    *
    * \param dirId The directory identifier.
    *
    * \retval info The directory and its properties.
    *
    * \exception TToiInvalidArgumentException
    *            Raised if the asset identifier is invalid.
    """
    __request = BinIpcParameterStack()
    # Write dirId
    __request.push_int32(dirId)
    __response = self.call(__request, "IToiAssetManagerService", "GetDirectoryInfo")
    __result = []
    # Read info
    info = {}
    info["Properties"] = []
    for k in range(__response.pop_int32()):
      temp_l = {}
      temp_l["Value"] = __response.pop_string()
      temp_l["Id"] = __response.pop_string()
      info["Properties"].append(temp_l)
    info["ParentDirectoryId"] = __response.pop_int32()
    info["DirectoryId"] = __response.pop_int32()
    __result.append(info)
    return __result[0]

  def GetDirectories(self, dirId):
    """Returns all direct directory children of given directory.
    *
    * \param dirId The directory identifier. If it equals to ROOT all top
    *              level directory identifiers will be returned.
    *
    * \retval childDirIds The list of directory identifiers.
    *
    * \exception TToiInvalidArgumentException
    *            Raised if the directory identifier is invalid.
    """
    __request = BinIpcParameterStack()
    # Write dirId
    __request.push_int32(dirId)
    __response = self.call(__request, "IToiAssetManagerService", "GetDirectories")
    __result = []
    # Read childDirIds
    childDirIds = []
    for k in range(__response.pop_int32()):
      childDirIds.append(__response.pop_int32())
    __result.append(childDirIds)
    return __result[0]

  def GetDefinedProperties(self):
    """
    * Returns the property identifiers for all defined properties
    * also including all properties that are currently used in the
    * database. This lists usually more properties than only those that
    * have been created by \ref CreatePropertyDefinition().
    *
    * \retval properties All defined property identifiers.
    *
    * \see CreatePropertyDefinition()
    """
    __request = BinIpcParameterStack()
    __response = self.call(__request, "IToiAssetManagerService", "GetDefinedProperties")
    __result = []
    # Read properties
    properties = []
    for k in range(__response.pop_int32()):
      properties.append(__response.pop_string())
    __result.append(properties)
    return __result[0]

  def GetPropertyDefinition(self, propertyId):
    """
    * Returns the property definition for a given property identifier.
    *
    * \param propertyId The property identifier.
    *
    * \retval definition The property definition.
    *
    * \exception TToiInvalidArgumentException
    *            Raised if the property identifier is invalid.
    """
    __request = BinIpcParameterStack()
    # Write propertyId
    __request.push_string(propertyId)
    __response = self.call(__request, "IToiAssetManagerService", "GetPropertyDefinition")
    __result = []
    # Read definition
    definition = {}
    definition["IsSystemDefined"] = __response.pop_bool()
    definition["HasMultipleValueSupport"] = __response.pop_bool()
    definition["Type"] = __response.pop_int32()
    definition["PropertyId"] = __response.pop_string()
    __result.append(definition)
    return __result[0]

  def CreatePropertyDefinition(self, propertyId, type, hasMultipleValueSupport):
    """
    * Creates a new property definition.
    * This method is needed for creating properties that can have
    * multiple values.
    *
    * \param propertyId The identifier of the new property, a unique name.
    *
    * \param type The property type.
    *
    * \param hasMultipleValueSupport Whether the property can have
    *                                multiple values.
    *
    * \exception TToiInvalidArgumentException
    *            Raised if the name is already in use by a property.
    """
    __request = BinIpcParameterStack()
    # Write propertyId
    __request.push_string(propertyId)
    # Write type
    __request.push_int32(type)
    # Write hasMultipleValueSupport
    __request.push_bool(hasMultipleValueSupport)
    self.call(__request, "IToiAssetManagerService", "CreatePropertyDefinition")

  def RemovePropertyDefinition(self, propertyId):
    """
    * Removes a user defined property, created using
    * \ref CreatePropertyDefinition().
    *
    * \param propertyId The identifier of the property to remove.
    *
    * \exception TToiInvalidArgumentException
    *            Raised if the property identifier is invalid or not a user
    *            defined property.
    """
    __request = BinIpcParameterStack()
    # Write propertyId
    __request.push_string(propertyId)
    self.call(__request, "IToiAssetManagerService", "RemovePropertyDefinition")

  def CreateAsset(self, assetType):
    """
    * Creates an asset. If the type is \ref ASSET_PVR, the identifier of the
    * asset can be used with a media recorder to save a media stream to disk.
    *
    * \param assetType The type of asset
    *
    * \retval assetId The identifier of the created asset.
    *
    * \exception TToiInvalidArgumentException
    *            Raised if the type is not supported.
    * \exception TToiNotEnoughResourcesException
    *            Raised if the asset could not be created due to missing
    *            system resources.
    *
    * \see IToiMediaRecorder::Open()
    """
    __request = BinIpcParameterStack()
    # Write assetType
    __request.push_int32(assetType)
    __response = self.call(__request, "IToiAssetManagerService", "CreateAsset")
    __result = []
    # Read assetId
    __result.append(__response.pop_string())
    return __result[0]

  def ImportAsset(self, operationId, assetId, uri, importPolicy):
    """
    * Import an asset from an external source. This is an asynchronous operation
    * and completion of the operation will not provide any result values, i.e.
    * the result will always be \ref IToiOperationManager::OP_RESULT_NONE.
    *
    * \param operationId The operation identifier created by
    *                    IToiOperationManager.
    *
    * \param assetId The identifier of the asset. Currently, only assets of type
    *                \ref ASSET_IMAGE are supported.
    *
    * \param uri URI to the source to be imported. Currently, only
    *            <code>file://</code> URIs are supported.
    *
    * \param importPolicy Policy to be applied when importing source.
    *                     For file URIs only
    *                     \ref IMPORT_POLICY_COPY and
    *                     \ref IMPORT_POLICY_MOVE are valid.
    *
    * \exception TToiInvalidArgumentException
    *            Raised if the asset type, uri, or import policy is invalid.
    *
    * \exception TToiNoDataException
    *            Raised if there is a problem reading from uri or updating the
    *            new asset.
    """
    __request = BinIpcParameterStack()
    # Write operationId
    __request.push_int32(operationId)
    # Write assetId
    __request.push_string(assetId)
    # Write uri
    __request.push_string(uri)
    # Write importPolicy
    __request.push_int32(importPolicy)
    self.call(__request, "IToiAssetManagerService", "ImportAsset")

  def RemoveAsset(self, operationId, assetId):
    """
    * Removes an asset. An asset can only be erased if the asset is not
    * locked for playback or recording. This is an asynchronous
    * operation and completion of the operation will not provide any
    * result values, i.e. the result will always be
    * \ref IToiOperationManager::OP_RESULT_NONE.
    *
    * \param operationId The operation identifier created by
    *                    IToiOperationManager
    *
    * \param assetId The asset identifier.
    *
    * \exception TToiInvalidArgumentException
    *            Raised if the asset identifier is invalid.
    * \exception TToiNoDataException
    *            Raised if there is no data to remove.
    * \exception TToiPermissionDeniedException
    *            Raised if the asset is locked.
    """
    __request = BinIpcParameterStack()
    # Write operationId
    __request.push_int32(operationId)
    # Write assetId
    __request.push_string(assetId)
    self.call(__request, "IToiAssetManagerService", "RemoveAsset")

  def CopyAsset(self, operationId, sourceAssetId, startOffset, duration, destinationAssetId):
    """
    * \techpreview This method is a TECHNICAL PREVIEW, it may change.
    * Copies data from an existing source asset to a destination asset.
    *
    * \param operationId The operation identifier created by
    *                    IToiOperationManager
    * \param sourceAssetId The asset identifier for the source asset.
    * \param startOffset Offset (in milliseconds) relative to the
    *                    source asset beginning to start the copy
    *                    operation from.
    * \param duration The duration (in milliseconds) to be copied.
    * \param destinationAssetId The asset identifier for the
    *                           destination asset.
    *
    * \exception TToiInvalidArgumentException Raised if
    *   - the operation identifier is invalid
    *   - one or more of the asset identifiers are invalid
    *   - one or more of the asset types are not \ref ASSET_PVR
    *   - the destination asset is not empty
    *   - duration is equal to zero or startOffset is not less than
    *     the source asset duration
    * \exception TToiPermissionDeniedException
    *            Raised if the destination asset is locked.
    """
    __request = BinIpcParameterStack()
    # Write operationId
    __request.push_int32(operationId)
    # Write sourceAssetId
    __request.push_string(sourceAssetId)
    # Write startOffset
    __request.push_uint32(startOffset)
    # Write duration
    __request.push_uint32(duration)
    # Write destinationAssetId
    __request.push_string(destinationAssetId)
    self.call(__request, "IToiAssetManagerService", "CopyAsset")

  def LinkAssets(self, dirId, assetIds):
    """
    * Links some assets to a directory, i.e. makes the assets visible
    * as a member of the directory.
    *
    * \param dirId The directory to link the assets to.
    *
    * \param assetIds The assets to link.
    *
    * \exception TToiInvalidArgumentException
    *            Raised if the directory identifier or one of the asset ids is
    *            invalid.
    """
    __request = BinIpcParameterStack()
    # Write dirId
    __request.push_int32(dirId)
    # Write assetIds
    for iter_k in assetIds[::-1]:
      __request.push_string(iter_k)
    __request.push_int32(len(assetIds))
    self.call(__request, "IToiAssetManagerService", "LinkAssets")

  def UnlinkAssets(self, dirId, assetIds):
    """
    * Removes the links from some assets to a directory.
    *
    * \param dirId The directory to unlink the assets from.
    *
    * \param assetIds The assets to unlink.
    *
    * \exception TToiInvalidArgumentException
    *            Raised if the directory is identifier.
    """
    __request = BinIpcParameterStack()
    # Write dirId
    __request.push_int32(dirId)
    # Write assetIds
    for iter_k in assetIds[::-1]:
      __request.push_string(iter_k)
    __request.push_int32(len(assetIds))
    self.call(__request, "IToiAssetManagerService", "UnlinkAssets")

  def RunQuery(self, operationId, queryConditions, requestedProperties, sortConditions):
    """
    * Starts a query that finds assets that match given conditions.
    * This is an asynchronous operation. When the application receives
    * a \ref IToiOperationObserver::OnOperationResult() call with the
    * operation result \ref OP_RESULT_ASSET_INFO, then the application
    * should get the results by calling the \ref GetAssetInfoResult()
    * function. The resulting \ref TToiAssetInfo will only include property
    * values for the requested properties.
    *
    * \param operationId The operation identifier created by
    *                    \ref IToiOperationManager
    *
    * \param queryConditions The conditions that define the query to be
    *                        executed. Only assets with matching
    *                        properties will be in the result.
    *
    * \param requestedProperties The properties that will be part of the result.
    *
    * \param sortConditions Defines the sorting order for the result.
    *
    * \exception TToiInvalidArgumentException
    *            Raised if any of the conditions are invalid,
    *            or raised if the operation identifier is illegal.
    * \exception TToiNotEnoughResourcesException
    *            Raised if it was not enough system resources to perform
    *            the query.
    """
    __request = BinIpcParameterStack()
    # Write operationId
    __request.push_int32(operationId)
    # Write queryConditions
    for iter_k in queryConditions[::-1]:
      __request.push_string(iter_k["PropertyId"])
      __request.push_int32(iter_k["Operation"])
      __request.push_string(iter_k["Value"])
    __request.push_int32(len(queryConditions))
    # Write requestedProperties
    for iter_k in requestedProperties[::-1]:
      __request.push_string(iter_k)
    __request.push_int32(len(requestedProperties))
    # Write sortConditions
    for iter_k in sortConditions[::-1]:
      __request.push_string(iter_k["SortProperty"])
      __request.push_bool(iter_k["SortAscending"])
    __request.push_int32(len(sortConditions))
    self.call(__request, "IToiAssetManagerService", "RunQuery")

  def RunQueryForDirectory(self, operationId, directoryId, queryConditions, requestedProperties, sortConditions):
    """
    * Starts a query that finds assets in the specified directory that
    * match given conditions. This is an asynchronous operation. When
    * the application receives a
    * \ref IToiOperationObserver::OnOperationResult() call with the operation
    * result \ref OP_RESULT_ASSET_INFO, then the application should get
    * the results by calling the \ref GetAssetInfoResult()
    * function. The resulting \ref TToiAssetInfo will only include property
    * values for the requested properties.
    *
    * \param operationId The operation identifier created by \ref
    *                    IToiOperationManager
    *
    * \param directoryId The directory which assets will be matched.
    *
    * \param queryConditions The conditions that define the query to be
    *                        executed. Only assets with matching
    *                        properties will be in the result.
    *
    * \param requestedProperties The properties that will be part of the result.
    *
    * \param sortConditions Defines the sorting order for the result.
    *
    * \exception TToiInvalidArgumentException
    *            Raised if any of the conditions are invalid,
    *            or raised if the operation identifier is illegal.
    * \exception TToiNotEnoughResourcesException
    *            Raised if it was not enough system resources to perform
    *            the query.
    """
    __request = BinIpcParameterStack()
    # Write operationId
    __request.push_int32(operationId)
    # Write directoryId
    __request.push_int32(directoryId)
    # Write queryConditions
    for iter_k in queryConditions[::-1]:
      __request.push_string(iter_k["PropertyId"])
      __request.push_int32(iter_k["Operation"])
      __request.push_string(iter_k["Value"])
    __request.push_int32(len(queryConditions))
    # Write requestedProperties
    for iter_k in requestedProperties[::-1]:
      __request.push_string(iter_k)
    __request.push_int32(len(requestedProperties))
    # Write sortConditions
    for iter_k in sortConditions[::-1]:
      __request.push_string(iter_k["SortProperty"])
      __request.push_bool(iter_k["SortAscending"])
    __request.push_int32(len(sortConditions))
    self.call(__request, "IToiAssetManagerService", "RunQueryForDirectory")

  def RunCountQuery(self, operationId, queryConditions):
    """
    * Starts a query that finds the number of assets that match given
    * conditions. This is an asynchronous operation. When application
    * gets the \ref IToiOperationObserver::OnOperationResult() with the
    * operation result \ref OP_RESULT_LONG, then the application should
    * get the result by calling the \ref GetLongResult() function.
    *
    * \param operationId The operation identifier created by
    *                    \ref IToiOperationManager
    *
    * \param queryConditions The conditions that define the query to be
    *                        executed. Only assets with matching
    *                        properties will be counted.
    *
    * \exception TToiInvalidArgumentException
    *            Raised if any of the conditions are invalid,
    *            or raised if the operation identifier is illegal.
    * \exception TToiNotEnoughResourcesException
    *            Raised if it was not enough system resources to perform
    *            the query.
    """
    __request = BinIpcParameterStack()
    # Write operationId
    __request.push_int32(operationId)
    # Write queryConditions
    for iter_k in queryConditions[::-1]:
      __request.push_string(iter_k["PropertyId"])
      __request.push_int32(iter_k["Operation"])
      __request.push_string(iter_k["Value"])
    __request.push_int32(len(queryConditions))
    self.call(__request, "IToiAssetManagerService", "RunCountQuery")

  def GetAssetInfoResult(self, operationId, maxCount):
    """
    * Used to get the query results once the applications recieve
    * \ref IToiOperationObserver::OnOperationResult().
    *
    * \param operationId The operation identifier of \ref RunQuery().
    *
    * \param maxCount The maximum number of objects the user are
    *                 interested in, 0 if as many as possible shall be returned.
    *
    * \retval objects The resulting asset information.
    *
    * \retval hasMore When true, there are some assets info in the cache of
    *                 asset manager. Additional call to this method required.
    *
    * \exception TToiInvalidArgumentException
    *            Raised if operation is unknown or does not support
    *            capability results.
    *
    * \exception TToiNoDataException
    *            Raised if no result is available.
    """
    __request = BinIpcParameterStack()
    # Write operationId
    __request.push_int32(operationId)
    # Write maxCount
    __request.push_uint32(maxCount)
    __response = self.call(__request, "IToiAssetManagerService", "GetAssetInfoResult")
    __result = []
    # Read hasMore
    __result.append(__response.pop_bool())
    # Read objects
    objects = []
    for k in range(__response.pop_int32()):
      temp_l = {}
      temp_l["Properties"] = []
      for l in range(__response.pop_int32()):
        temp_m = {}
        temp_m["Value"] = __response.pop_string()
        temp_m["Id"] = __response.pop_string()
        temp_l["Properties"].append(temp_m)
      temp_l["AssetId"] = __response.pop_string()
      objects.append(temp_l)
    __result.append(objects)
    return tuple(__result)

  def GetLongResult(self, operationId):
    """
    * Used to get the count result once the applications recieve
    * \ref IToiOperationObserver::OnOperationResult().
    *
    * \param operationId The operation identifier.
    *
    * \retval value The resulting long value from the operation.
    *
    * \exception TToiInvalidArgumentException
    *            Raised if operation is unknown or does not have a long
    *            value as result.
    *
    * \exception TToiNoDataException
    *            Raised if no result is available.
    """
    __request = BinIpcParameterStack()
    # Write operationId
    __request.push_int32(operationId)
    __response = self.call(__request, "IToiAssetManagerService", "GetLongResult")
    __result = []
    # Read value
    __result.append(__response.pop_uint32())
    return __result[0]

  def GetProperties(self, assetId, requestedProperties):
    """
    * Returns metadata property values for an asset.
    *
    * \param assetId The asset identifier.
    *
    * \param requestedProperties The identifiers of the wanted properties.
    *
    * \retval properties The id/value pairs of the requested properties.
    *
    * \exception TToiInvalidArgumentException
    *            Raised if the asset identifier is invalid.
    * \exception TToiNoDataException
    *            Raised if no data can be read for the asset.
    * \exception TToiNotEnoughResourcesException
    *            Raised if the needed system resources are not accessible.
    """
    __request = BinIpcParameterStack()
    # Write assetId
    __request.push_string(assetId)
    # Write requestedProperties
    for iter_k in requestedProperties[::-1]:
      __request.push_string(iter_k)
    __request.push_int32(len(requestedProperties))
    __response = self.call(__request, "IToiAssetManagerService", "GetProperties")
    __result = []
    # Read properties
    properties = []
    for k in range(__response.pop_int32()):
      temp_l = {}
      temp_l["Value"] = __response.pop_string()
      temp_l["Id"] = __response.pop_string()
      properties.append(temp_l)
    __result.append(properties)
    return __result[0]

  def GetAllProperties(self, assetId):
    """
    * Returns all metadata properties for an asset.
    *
    * \param assetId The asset identifier.
    *
    * \retval properties The id/value pairs of all the asset's properties.
    *
    * \exception TToiInvalidArgumentException
    *            Raised if the asset identifier is invalid.
    * \exception TToiNoDataException
    *            Raised if no data can be read for the asset.
    * \exception TToiNotEnoughResourcesException
    *            Raised if the needed system resources are not accessible.
    """
    __request = BinIpcParameterStack()
    # Write assetId
    __request.push_string(assetId)
    __response = self.call(__request, "IToiAssetManagerService", "GetAllProperties")
    __result = []
    # Read properties
    properties = []
    for k in range(__response.pop_int32()):
      temp_l = {}
      temp_l["Value"] = __response.pop_string()
      temp_l["Id"] = __response.pop_string()
      properties.append(temp_l)
    __result.append(properties)
    return __result[0]

  def SetProperties(self, id, properties):
    """
    * Sets metadata property values for an asset. For properties that supports
    * multiple values, any values set with this method are added to existing
    * values.
    *
    * \param id The asset identifier.
    *
    * \param properties The properties to set data for.
    *
    * \exception TToiInvalidArgumentException
    *            Raised if the asset identifier is invalid, or if a property
    *            identifier is invalid.
    * \exception TToiNoDataException
    *            Raised if no data can be read for the asset.
    * \exception TToiNotEnoughResourcesException
    *            Raised if the needed system resources are not accessible.
    """
    __request = BinIpcParameterStack()
    # Write id
    __request.push_string(id)
    # Write properties
    for iter_k in properties[::-1]:
      __request.push_string(iter_k["Id"])
      __request.push_string(iter_k["Value"])
    __request.push_int32(len(properties))
    self.call(__request, "IToiAssetManagerService", "SetProperties")

  def RemoveProperties(self, id, properties):
    """
    * Removes metadata properties for an asset. All values are erased whether
    * a property has a single or multiple values, i.e., the property is
    * completely removed.
    *
    * \param id The asset identifier.
    *
    * \param properties The properties to unset.
    *
    * \exception TToiInvalidArgumentException
    *            Raised if the asset identifier is invalid, or if a
    *            property identifier is invalid.
    * \exception TToiNoDataException
    *            Raised if no data can be accessed for the asset.
    * \exception TToiNotEnoughResourcesException
    *            Raised if the needed system resources are not accessible.
    """
    __request = BinIpcParameterStack()
    # Write id
    __request.push_string(id)
    # Write properties
    for iter_k in properties[::-1]:
      __request.push_string(iter_k)
    __request.push_int32(len(properties))
    self.call(__request, "IToiAssetManagerService", "RemoveProperties")

  def RemoveMultiValuedPropertyValue(self, id, property):
    """
    * Removes a value from a property that is multi-valued.
    *
    * \param id The asset identifier.
    *
    * \param property The id/value pair of the property from which the
    *                 corresponding value should be removed.
    *
    * \exception TToiInvalidArgumentException
    *            Raised if the asset identifier is invalid, or if a
    *            property identifier is invalid.
    * \exception TToiNoDataException
    *            Raised if no data can be accessed for the asset.
    * \exception TToiNotEnoughResourcesException
    *            Raised if the needed system resources are not accessible.
    """
    __request = BinIpcParameterStack()
    # Write id
    __request.push_string(id)
    # Write property
    __request.push_string(property["Id"])
    __request.push_string(property["Value"])
    self.call(__request, "IToiAssetManagerService", "RemoveMultiValuedPropertyValue")

  def RegisterObserver(self, applicationId, address):
    """Register an observer of type IToiAssetManagerObserver.
    The observer will receive events sent by this IToiAssetManagerService instance.
    
    \param applicationId  The identifier of the observing application.
    \param address        The address where the IToiAssetManagerService can find an
    IToiAssetManagerObserver interface.
    \exception TToiInvalidArgumentException
    Raised if the specified address is invalid
    or if the application has already registered
    itself as an observer with the same address.
    \see IToiAssetManagerObserver
    """
    __request = BinIpcParameterStack()
    # Write applicationId
    __request.push_int32(applicationId)
    # Write address
    __request.push_string(address)
    self.call(__request, "IToiAssetManagerService", "RegisterObserver")

  def ReleaseObserver(self, address):
    """Release an observer previously registered with \ref RegisterObserver.
    
    \param address        The address where the IToiAssetManagerService
    can find the registered IToiAssetManagerObserver interface.
    \exception TToiInvalidArgumentException
    Raised if the specified address is invalid.
    \see IToiAssetManagerObserver
    """
    __request = BinIpcParameterStack()
    # Write address
    __request.push_string(address)
    self.call(__request, "IToiAssetManagerService", "ReleaseObserver")

  def CreateObserver(self):
    """Create and register an observer for the service.
       (This method is only available from Python.)
    """
    observer = ToiAssetManagerObserver(self._ipcproxy, self)
    request = BinIpcParameterStack()
    request.push_int32(self._ipcproxy.application_id)
    request.push_string(observer.observer_address)
    self.call(request, "IToiAssetManagerService", "RegisterObserver")
    return observer
