# Copyright (c) 2011 Motorola Mobility, Inc. All rights reserved.

from kreatvipccaller import KreaTvIpcCaller
from binipcparameterstack import BinIpcParameterStack


class ToiAudioOutputConfiguration(KreaTvIpcCaller):

  # Enum TToiAudioFormat
  AUDIO_FORMAT_NULL = 0
  AUDIO_FORMAT_MPEG_1 = 1
  AUDIO_FORMAT_MPEG_2 = 2
  AUDIO_FORMAT_AC3 = 3
  AUDIO_FORMAT_AC3_PLUS = 4
  AUDIO_FORMAT_AAC_HE = 5
  AUDIO_FORMAT_AAC_LC = 6
  AUDIO_FORMAT_DTS = 7
  AUDIO_FORMAT_DTS_HD = 8
  AUDIO_FORMAT_PCM_2CH = 9
  AUDIO_FORMAT_PCM_MULTI = 10

  # Enum TToiHdmiConnectionStatus
  HDMI_CONNECTION_STATUS_UNKNOWN = 0
  HDMI_CONNECTION_STATUS_CONNECTED = 1
  HDMI_CONNECTION_STATUS_CONNECTED_CAPABILITIES_UNKNOWN = 2
  HDMI_CONNECTION_STATUS_DISCONNECTED = 3

  def __init__(self, ipcproxy, object_address):
    KreaTvIpcCaller.__init__(self, ipcproxy.ip_address, ipcproxy.port)
    self._ipcproxy = ipcproxy
    self.set_object_address(object_address)

  def GetFormatMapping(self, connectionId, inputFormat):
    """Returns the audio format mapping for the specified output connection
    and input format.
    
    \param connectionId   The identifier of the output connection.
    
    \param inputFormat    The input format.
    
    \retval formatMapping  The requested format mapping.
    
    \exception TToiInvalidArgumentException
    Raised if the output connection or the input format
    is invalid.
    """
    __request = BinIpcParameterStack()
    # Write connectionId
    __request.push_int32(connectionId)
    # Write inputFormat
    __request.push_int32(inputFormat)
    __response = self.call(__request, "IToiAudioOutputConfiguration", "GetFormatMapping")
    __result = []
    # Read formatMapping
    formatMapping = {}
    formatMapping["ActiveMixFlag"] = __response.pop_bool()
    formatMapping["ActiveOutputFormat"] = __response.pop_int32()
    formatMapping["OutputFormatOptionList"] = []
    for k in range(__response.pop_int32()):
      temp_l = {}
      temp_l["MixFlag"] = __response.pop_bool()
      temp_l["Format"] = __response.pop_int32()
      formatMapping["OutputFormatOptionList"].append(temp_l)
    formatMapping["InputFormat"] = __response.pop_int32()
    __result.append(formatMapping)
    return __result[0]

  def GetFormatMappingList(self, connectionId):
    """Returns all audio format mappings for the specified output connection.
    
    \param connectionId        The identifier of the output connection.
    
    \retval formatMappingList  The list of audio format mappings.
    
    \exception TToiInvalidArgumentException
    Raised if the output connection is invalid.
    """
    __request = BinIpcParameterStack()
    # Write connectionId
    __request.push_int32(connectionId)
    __response = self.call(__request, "IToiAudioOutputConfiguration", "GetFormatMappingList")
    __result = []
    # Read formatMappingList
    formatMappingList = []
    for k in range(__response.pop_int32()):
      temp_l = {}
      temp_l["ActiveMixFlag"] = __response.pop_bool()
      temp_l["ActiveOutputFormat"] = __response.pop_int32()
      temp_l["OutputFormatOptionList"] = []
      for l in range(__response.pop_int32()):
        temp_m = {}
        temp_m["MixFlag"] = __response.pop_bool()
        temp_m["Format"] = __response.pop_int32()
        temp_l["OutputFormatOptionList"].append(temp_m)
      temp_l["InputFormat"] = __response.pop_int32()
      formatMappingList.append(temp_l)
    __result.append(formatMappingList)
    return __result[0]

  def GetHdmiSinkInfo(self, connectionId):
    """Returns the sink information for specified HDMI connection.
    
    \param connectionId      The identifier of the HDMI output connection.
    
    \retval sinkInfo         The sink information for the HDMI
    connection.
    
    \exception TToiInvalidArgumentException
    Raised if the output connection is invalid.
    """
    __request = BinIpcParameterStack()
    # Write connectionId
    __request.push_int32(connectionId)
    __response = self.call(__request, "IToiAudioOutputConfiguration", "GetHdmiSinkInfo")
    __result = []
    # Read sinkInfo
    sinkInfo = {}
    sinkInfo["ManufacturerInfo"] = {}
    sinkInfo["ManufacturerInfo"]["YearOfManufacture"] = __response.pop_int32()
    sinkInfo["ManufacturerInfo"]["WeekOfManufacture"] = __response.pop_int32()
    sinkInfo["ManufacturerInfo"]["SerialNumber"] = __response.pop_int32()
    sinkInfo["ManufacturerInfo"]["ProductId"] = __response.pop_int32()
    sinkInfo["ManufacturerInfo"]["Name"] = __response.pop_string()
    sinkInfo["AudioFormatList"] = []
    for k in range(__response.pop_int32()):
      sinkInfo["AudioFormatList"].append(__response.pop_int32())
    sinkInfo["Status"] = __response.pop_int32()
    __result.append(sinkInfo)
    return __result[0]

