# Copyright (c) 2011 Motorola Mobility, Inc. All rights reserved.

from kreatvipccaller import KreaTvIpcCaller
from binipcparameterstack import BinIpcParameterStack


from ToiAudioOutputConfiguration import ToiAudioOutputConfiguration

class ToiAudioOutputConfigurationSession(ToiAudioOutputConfiguration):

  def __init__(self, ipcproxy, object_address):
    ToiAudioOutputConfiguration.__init__(self, ipcproxy, object_address)

  def ResetFormatMapping(self, inputFormat):
    """Resets output format to 'no output' for the specified input format
    on all available outputs.
    
    \param inputFormat    The input format to reset.
    """
    __request = BinIpcParameterStack()
    # Write inputFormat
    __request.push_int32(inputFormat)
    self.call(__request, "IToiAudioOutputConfigurationSession", "ResetFormatMapping")

  def SetFormatMapping(self, connectionId, inputFormat, outputFormatOption):
    """Sets the format mapping on the specified audio output
    connection.
    
    \param connectionId        The identifier of the output connection.
    
    \param inputFormat         The input format.
    
    \param outputFormatOption  The output format option.
    
    \exception TToiInvalidArgumentException
    Raised if the output connection or the output format
    option is invalid.
    
    \exception TToiOperationNotSupportedException
    Raised if the format mapping can not be set..
    """
    __request = BinIpcParameterStack()
    # Write connectionId
    __request.push_int32(connectionId)
    # Write inputFormat
    __request.push_int32(inputFormat)
    # Write outputFormatOption
    __request.push_int32(outputFormatOption["Format"])
    __request.push_bool(outputFormatOption["MixFlag"])
    self.call(__request, "IToiAudioOutputConfigurationSession", "SetFormatMapping")

  def Apply(self):
    """Applies any changes made in this session. This method also releases
    the resources used by this object and removes all registered observers.
    Upon return this object will no longer be available and any subsequent
    method calls will result in a low-level run-time failure.
    """
    __request = BinIpcParameterStack()
    self.call(__request, "IToiAudioOutputConfigurationSession", "Apply")

  def Revert(self):
    """Reverts any changes made in this session. This method also releases
    the resources used by this object and removes all registered observers.
    Upon return this object will no longer be available and any subsequent
    method calls will result in a low-level run-time failure.
    """
    __request = BinIpcParameterStack()
    self.call(__request, "IToiAudioOutputConfigurationSession", "Revert")

