# Copyright (c) 2011 Motorola Mobility, Inc. All rights reserved.

from kreatvipcobserver import KreaTvIpcObserver
import collections
import sys

class ToiAudioOutputObserver(KreaTvIpcObserver):
    OnConfigurationChangedArguments = collections.namedtuple('OnConfigurationChangedArguments', '')
    OnVolumeChangedArguments = collections.namedtuple('OnVolumeChangedArguments', 'connectionId level ')
    OnMuteStateChangedArguments = collections.namedtuple('OnMuteStateChangedArguments', 'connectionId state ')
    OnHdmiConnectionStatusChangedArguments = collections.namedtuple('OnHdmiConnectionStatusChangedArguments', 'connectionId sinkInfo ')
    OnLatencyChangedArguments = collections.namedtuple('OnLatencyChangedArguments', 'connectionId latency ')

    def __init__(self, ipcproxy, service=None, queue=None):
        KreaTvIpcObserver.__init__(self, ipcproxy, service, queue)

    def handle_callback(self, interface_name, method_name, stack):
        """
        Pushes the call details onto the log queue.
        Called on the worker thread.
        """
        call = KreaTvIpcObserver.Call(address=self.observer_address,
                                      interface=interface_name,
                                      method=method_name)
        if method_name == 'OnConfigurationChanged':
            pass
        if method_name == 'OnVolumeChanged':
            args = {}
            args['level'] = stack.pop_uint32()
            args['connectionId'] = stack.pop_uint32()
            call.arguments = ToiAudioOutputObserver.OnVolumeChangedArguments(**args)
        if method_name == 'OnMuteStateChanged':
            args = {}
            args['state'] = stack.pop_bool()
            args['connectionId'] = stack.pop_uint32()
            call.arguments = ToiAudioOutputObserver.OnMuteStateChangedArguments(**args)
        if method_name == 'OnHdmiConnectionStatusChanged':
            args = {}
            dict_sinkInfo = {}
            dict_ManufacturerInfo = {}
            dict_ManufacturerInfo['YearOfManufacture'] = stack.pop_int32()
            dict_ManufacturerInfo['WeekOfManufacture'] = stack.pop_int32()
            dict_ManufacturerInfo['SerialNumber'] = stack.pop_int32()
            dict_ManufacturerInfo['ProductId'] = stack.pop_int32()
            dict_ManufacturerInfo['Name'] = stack.pop_string()
            dict_sinkInfo['ManufacturerInfo'] = dict_ManufacturerInfo
            temp_AudioFormatList = []
            for o in range(stack.pop_int32()):
              temp_AudioFormatList.append(stack.pop_int32())
            dict_sinkInfo['AudioFormatList'] = temp_AudioFormatList
            dict_sinkInfo['Status'] = stack.pop_int32()
            args['sinkInfo'] = dict_sinkInfo
            args['connectionId'] = stack.pop_uint32()
            call.arguments = ToiAudioOutputObserver.OnHdmiConnectionStatusChangedArguments(**args)
        if method_name == 'OnLatencyChanged':
            args = {}
            args['latency'] = stack.pop_int32()
            args['connectionId'] = stack.pop_uint32()
            call.arguments = ToiAudioOutputObserver.OnLatencyChangedArguments(**args)
        # Thread-safe operation:
        self._queue.append(call)

    def wait_OnConfigurationChanged(self, discard_non_matching_events=False,
                            timeout_sec=10, **argument_filters):
        """
        Convenience function to wait for callbacks to OnConfigurationChanged.

        Keyword arguments can be given to specify parameter values to wait for.
        If a received event does not match the filter, a KreaTvIpcException
        will be raised, unless discard_non_matching_events is true in which
        case the event will be silently ignored (and the function will continue
        to wait for a matching event).
        """
        call = self.wait_callback(self.observer_address,
                                  'IToiAudioOutputObserver',
                                  'OnConfigurationChanged',
                                  discard_non_matching_events,
                                  timeout_sec,
                                  argument_filters)
        return

    def wait_OnVolumeChanged(self, discard_non_matching_events=False,
                            timeout_sec=10, **argument_filters):
        """
        Convenience function to wait for callbacks to OnVolumeChanged.

        Keyword arguments can be given to specify parameter values to wait for.
        If a received event does not match the filter, a KreaTvIpcException
        will be raised, unless discard_non_matching_events is true in which
        case the event will be silently ignored (and the function will continue
        to wait for a matching event).
        """
        call = self.wait_callback(self.observer_address,
                                  'IToiAudioOutputObserver',
                                  'OnVolumeChanged',
                                  discard_non_matching_events,
                                  timeout_sec,
                                  argument_filters)
        return call.arguments

    def wait_OnMuteStateChanged(self, discard_non_matching_events=False,
                            timeout_sec=10, **argument_filters):
        """
        Convenience function to wait for callbacks to OnMuteStateChanged.

        Keyword arguments can be given to specify parameter values to wait for.
        If a received event does not match the filter, a KreaTvIpcException
        will be raised, unless discard_non_matching_events is true in which
        case the event will be silently ignored (and the function will continue
        to wait for a matching event).
        """
        call = self.wait_callback(self.observer_address,
                                  'IToiAudioOutputObserver',
                                  'OnMuteStateChanged',
                                  discard_non_matching_events,
                                  timeout_sec,
                                  argument_filters)
        return call.arguments

    def wait_OnHdmiConnectionStatusChanged(self, discard_non_matching_events=False,
                            timeout_sec=10, **argument_filters):
        """
        Convenience function to wait for callbacks to OnHdmiConnectionStatusChanged.

        Keyword arguments can be given to specify parameter values to wait for.
        If a received event does not match the filter, a KreaTvIpcException
        will be raised, unless discard_non_matching_events is true in which
        case the event will be silently ignored (and the function will continue
        to wait for a matching event).
        """
        call = self.wait_callback(self.observer_address,
                                  'IToiAudioOutputObserver',
                                  'OnHdmiConnectionStatusChanged',
                                  discard_non_matching_events,
                                  timeout_sec,
                                  argument_filters)
        return call.arguments

    def wait_OnLatencyChanged(self, discard_non_matching_events=False,
                            timeout_sec=10, **argument_filters):
        """
        Convenience function to wait for callbacks to OnLatencyChanged.

        Keyword arguments can be given to specify parameter values to wait for.
        If a received event does not match the filter, a KreaTvIpcException
        will be raised, unless discard_non_matching_events is true in which
        case the event will be silently ignored (and the function will continue
        to wait for a matching event).
        """
        call = self.wait_callback(self.observer_address,
                                  'IToiAudioOutputObserver',
                                  'OnLatencyChanged',
                                  discard_non_matching_events,
                                  timeout_sec,
                                  argument_filters)
        return call.arguments

