# Copyright (c) 2011 Motorola Mobility, Inc. All rights reserved.

from kreatvipcobserver import KreaTvIpcObserver
import collections
import sys

class ToiAudioOverlayObserver(KreaTvIpcObserver):
    OnStoppedArguments = collections.namedtuple('OnStoppedArguments', 'file ')

    def __init__(self, ipcproxy, service=None, queue=None):
        KreaTvIpcObserver.__init__(self, ipcproxy, service, queue)

    def handle_callback(self, interface_name, method_name, stack):
        """
        Pushes the call details onto the log queue.
        Called on the worker thread.
        """
        call = KreaTvIpcObserver.Call(address=self.observer_address,
                                      interface=interface_name,
                                      method=method_name)
        if method_name == 'OnStopped':
            args = {}
            args['file'] = stack.pop_string()
            call.arguments = ToiAudioOverlayObserver.OnStoppedArguments(**args)
        # Thread-safe operation:
        self._queue.append(call)

    def wait_OnStopped(self, discard_non_matching_events=False,
                            timeout_sec=10, **argument_filters):
        """
        Convenience function to wait for callbacks to OnStopped.

        Keyword arguments can be given to specify parameter values to wait for.
        If a received event does not match the filter, a KreaTvIpcException
        will be raised, unless discard_non_matching_events is true in which
        case the event will be silently ignored (and the function will continue
        to wait for a matching event).
        """
        call = self.wait_callback(self.observer_address,
                                  'IToiAudioOverlayObserver',
                                  'OnStopped',
                                  discard_non_matching_events,
                                  timeout_sec,
                                  argument_filters)
        return call.arguments.file

