# Copyright (c) 2011 Motorola Mobility, Inc. All rights reserved.

from kreatvipccaller import KreaTvIpcCaller
from binipcparameterstack import BinIpcParameterStack
from ToiAudioOverlayObserver import ToiAudioOverlayObserver


class ToiAudioOverlayService(KreaTvIpcCaller):

  def __init__(self, ipcproxy):
    KreaTvIpcCaller.__init__(self, ipcproxy.ip_address, ipcproxy.port)
    self._ipcproxy = ipcproxy
    self.lookup("AudioOverlayService")

  def Play(self, file, loop):
    """Plays an audio file if the format is supported.
    Any current active playback will be stopped.
    
    \param file     The audio file to play.
    \param loop     Indicates wheter the audio should be looped.
    """
    __request = BinIpcParameterStack()
    # Write file
    __request.push_string(file)
    # Write loop
    __request.push_bool(loop)
    self.call(__request, "IToiAudioOverlayService", "Play")

  def Stop(self):
    """Stops any current active playback. */
    """
    __request = BinIpcParameterStack()
    self.call(__request, "IToiAudioOverlayService", "Stop")

  def RegisterObserver(self, applicationId, address):
    """Register an observer of type IToiAudioOverlayObserver.
    The observer will receive events sent by this IToiAudioOverlayService instance.
    
    \param applicationId  The identifier of the observing application.
    \param address        The address where the IToiAudioOverlayService can find an
    IToiAudioOverlayObserver interface.
    \exception TToiInvalidArgumentException
    Raised if the specified address is invalid
    or if the application has already registered
    itself as an observer with the same address.
    \see IToiAudioOverlayObserver
    """
    __request = BinIpcParameterStack()
    # Write applicationId
    __request.push_int32(applicationId)
    # Write address
    __request.push_string(address)
    self.call(__request, "IToiAudioOverlayService", "RegisterObserver")

  def ReleaseObserver(self, address):
    """Release an observer previously registered with \ref RegisterObserver.
    
    \param address        The address where the IToiAudioOverlayService
    can find the registered IToiAudioOverlayObserver interface.
    \exception TToiInvalidArgumentException
    Raised if the specified address is invalid.
    \see IToiAudioOverlayObserver
    """
    __request = BinIpcParameterStack()
    # Write address
    __request.push_string(address)
    self.call(__request, "IToiAudioOverlayService", "ReleaseObserver")

  def CreateObserver(self):
    """Create and register an observer for the service.
       (This method is only available from Python.)
    """
    observer = ToiAudioOverlayObserver(self._ipcproxy, self)
    request = BinIpcParameterStack()
    request.push_int32(self._ipcproxy.application_id)
    request.push_string(observer.observer_address)
    self.call(request, "IToiAudioOverlayService", "RegisterObserver")
    return observer
