# Copyright (c) 2011 Motorola Mobility, Inc. All rights reserved.

from kreatvipcobserver import KreaTvIpcObserver
import collections
import sys

class ToiChannelObserver(KreaTvIpcObserver):
    OnChannelsAddedArguments = collections.namedtuple('OnChannelsAddedArguments', 'channels ')
    OnChannelsRemovedArguments = collections.namedtuple('OnChannelsRemovedArguments', 'channels ')
    OnChannelsChangedArguments = collections.namedtuple('OnChannelsChangedArguments', 'channels ')

    def __init__(self, ipcproxy, service=None, queue=None):
        KreaTvIpcObserver.__init__(self, ipcproxy, service, queue)

    def handle_callback(self, interface_name, method_name, stack):
        """
        Pushes the call details onto the log queue.
        Called on the worker thread.
        """
        call = KreaTvIpcObserver.Call(address=self.observer_address,
                                      interface=interface_name,
                                      method=method_name)
        if method_name == 'OnChannelsAdded':
            args = {}
            temp_channels = []
            for o in range(stack.pop_int32()):
              temp_channels.append(stack.pop_int32())
            args['channels'] = temp_channels
            call.arguments = ToiChannelObserver.OnChannelsAddedArguments(**args)
        if method_name == 'OnChannelsRemoved':
            args = {}
            temp_channels = []
            for o in range(stack.pop_int32()):
              temp_channels.append(stack.pop_int32())
            args['channels'] = temp_channels
            call.arguments = ToiChannelObserver.OnChannelsRemovedArguments(**args)
        if method_name == 'OnChannelsChanged':
            args = {}
            temp_channels = []
            for o in range(stack.pop_int32()):
              temp_channels.append(stack.pop_int32())
            args['channels'] = temp_channels
            call.arguments = ToiChannelObserver.OnChannelsChangedArguments(**args)
        # Thread-safe operation:
        self._queue.append(call)

    def wait_OnChannelsAdded(self, discard_non_matching_events=False,
                            timeout_sec=10, **argument_filters):
        """
        Convenience function to wait for callbacks to OnChannelsAdded.

        Keyword arguments can be given to specify parameter values to wait for.
        If a received event does not match the filter, a KreaTvIpcException
        will be raised, unless discard_non_matching_events is true in which
        case the event will be silently ignored (and the function will continue
        to wait for a matching event).
        """
        call = self.wait_callback(self.observer_address,
                                  'IToiChannelObserver',
                                  'OnChannelsAdded',
                                  discard_non_matching_events,
                                  timeout_sec,
                                  argument_filters)
        return call.arguments.channels

    def wait_OnChannelsRemoved(self, discard_non_matching_events=False,
                            timeout_sec=10, **argument_filters):
        """
        Convenience function to wait for callbacks to OnChannelsRemoved.

        Keyword arguments can be given to specify parameter values to wait for.
        If a received event does not match the filter, a KreaTvIpcException
        will be raised, unless discard_non_matching_events is true in which
        case the event will be silently ignored (and the function will continue
        to wait for a matching event).
        """
        call = self.wait_callback(self.observer_address,
                                  'IToiChannelObserver',
                                  'OnChannelsRemoved',
                                  discard_non_matching_events,
                                  timeout_sec,
                                  argument_filters)
        return call.arguments.channels

    def wait_OnChannelsChanged(self, discard_non_matching_events=False,
                            timeout_sec=10, **argument_filters):
        """
        Convenience function to wait for callbacks to OnChannelsChanged.

        Keyword arguments can be given to specify parameter values to wait for.
        If a received event does not match the filter, a KreaTvIpcException
        will be raised, unless discard_non_matching_events is true in which
        case the event will be silently ignored (and the function will continue
        to wait for a matching event).
        """
        call = self.wait_callback(self.observer_address,
                                  'IToiChannelObserver',
                                  'OnChannelsChanged',
                                  discard_non_matching_events,
                                  timeout_sec,
                                  argument_filters)
        return call.arguments.channels

