# Copyright (c) 2011 Motorola Mobility, Inc. All rights reserved.

from kreatvipccaller import KreaTvIpcCaller
from binipcparameterstack import BinIpcParameterStack
from ToiDiagnosticsDataModelObserver import ToiDiagnosticsDataModelObserver


class ToiDiagnosticsDataModel(KreaTvIpcCaller):

  def __init__(self, ipcproxy, object_address):
    KreaTvIpcCaller.__init__(self, ipcproxy.ip_address, ipcproxy.port)
    self._ipcproxy = ipcproxy
    self.set_object_address(object_address)

  def GetObject(self, objectId):
    """This function returns a \ref TToiObjectInfo corresponding
    to the given objectId.
    
    \param objectId
    The Id for the object
    
    \retval infoObject
    The \ref TToiObjectInfo representing that
    object in the data model.
    
    \exception TToiInvalidArgumentException
    Raised if the specified object identifier is invalid.
    
    """
    __request = BinIpcParameterStack()
    # Write objectId
    __request.push_string(objectId)
    __response = self.call(__request, "IToiDiagnosticsDataModel", "GetObject")
    __result = []
    # Read infoObject
    infoObject = {}
    infoObject["Children"] = []
    for k in range(__response.pop_int32()):
      infoObject["Children"].append(__response.pop_string())
    infoObject["Parameters"] = []
    for k in range(__response.pop_int32()):
      temp_l = {}
      temp_l["Value"] = __response.pop_string()
      temp_l["Name"] = __response.pop_string()
      infoObject["Parameters"].append(temp_l)
    infoObject["ObjectId"] = __response.pop_string()
    __result.append(infoObject)
    return __result[0]

  def SetObject(self, infoObject):
    """This function is used to set the parameters of an object
    in the data model.
    
    \param infoObject
    The object containing new parameter values.
    
    \exception TToiInvalidArgumentException
    Raised if the specified object indentifier is invalid.
    
    \exception TToiPermissionDeniedException
    Raised if the parameters of the data model object
    cannot be set.
    """
    __request = BinIpcParameterStack()
    # Write infoObject
    __request.push_string(infoObject["ObjectId"])
    for iter_k in infoObject["Parameters"][::-1]:
      __request.push_string(iter_k["Name"])
      __request.push_string(iter_k["Value"])
    __request.push_int32(len(infoObject["Parameters"]))
    for iter_k in infoObject["Children"][::-1]:
      __request.push_string(iter_k)
    __request.push_int32(len(infoObject["Children"]))
    self.call(__request, "IToiDiagnosticsDataModel", "SetObject")

  def SubscribeObject(self, address, objectId):
    """This function is used to register a
    IToiDiagnosticsDataModelObserver with the diagnostics framework
    to be notified of any changes to an object in the data model.
    
    \param address
    the address of the IToiDiagnosticsDataModelObserver
    
    \param objectId
    The object id that the application is interested in.
    
    \exception TToiInvalidArgumentException
    Raised if the specified object identifier is invalid.
    
    \see UnsubscribeObject()
    
    """
    __request = BinIpcParameterStack()
    # Write address
    __request.push_string(address)
    # Write objectId
    __request.push_string(objectId)
    self.call(__request, "IToiDiagnosticsDataModel", "SubscribeObject")

  def UnsubscribeObject(self, address, objectId):
    """This function is used to unregister a previously registered
    IToiDiagnosticsDataModelObserver.
    
    \param address
    the address of the  IToiDiagnosticsDataModelObserver
    
    \param objectId
    The object id that the application is interested in.
    
    \exception TToiInvalidArgumentException
    Raised if the specified object identifier is invalid.
    Raised if the address is invalid.
    
    \see SubscribeObject()
    
    """
    __request = BinIpcParameterStack()
    # Write address
    __request.push_string(address)
    # Write objectId
    __request.push_string(objectId)
    self.call(__request, "IToiDiagnosticsDataModel", "UnsubscribeObject")

  def ReleaseInstance(self):
    """This function releases the resources used by this instance.
    When this function returns all observers have also been released.
    """
    __request = BinIpcParameterStack()
    self.call(__request, "IToiDiagnosticsDataModel", "ReleaseInstance")

  def RegisterObserver(self, applicationId, address):
    """Register an observer of type IToiDiagnosticsDataModelObserver.
    The observer will receive events sent by this IToiDiagnosticsDataModel instance.
    
    \param applicationId  The identifier of the observing application.
    \param address        The address where the IToiDiagnosticsDataModel can find an
    IToiDiagnosticsDataModelObserver interface.
    \exception TToiInvalidArgumentException
    Raised if the specified address is invalid
    or if the application has already registered
    itself as an observer with the same address.
    \see IToiDiagnosticsDataModelObserver
    """
    __request = BinIpcParameterStack()
    # Write applicationId
    __request.push_int32(applicationId)
    # Write address
    __request.push_string(address)
    self.call(__request, "IToiDiagnosticsDataModel", "RegisterObserver")

  def ReleaseObserver(self, address):
    """Release an observer previously registered with \ref RegisterObserver.
    
    \param address        The address where the IToiDiagnosticsDataModel
    can find the registered IToiDiagnosticsDataModelObserver interface.
    \exception TToiInvalidArgumentException
    Raised if the specified address is invalid.
    \see IToiDiagnosticsDataModelObserver
    """
    __request = BinIpcParameterStack()
    # Write address
    __request.push_string(address)
    self.call(__request, "IToiDiagnosticsDataModel", "ReleaseObserver")

  def CreateObserver(self):
    """Create and register an observer for the service.
       (This method is only available from Python.)
    """
    observer = ToiDiagnosticsDataModelObserver(self._ipcproxy, self)
    request = BinIpcParameterStack()
    request.push_int32(self._ipcproxy.application_id)
    request.push_string(observer.observer_address)
    self.call(request, "IToiDiagnosticsDataModel", "RegisterObserver")
    return observer
