# Copyright (c) 2011 Motorola Mobility, Inc. All rights reserved.

from kreatvipcobserver import KreaTvIpcObserver
import collections
import sys

class ToiDiagnosticsDataModelObserver(KreaTvIpcObserver):
    OnObjectsChangedArguments = collections.namedtuple('OnObjectsChangedArguments', 'objects ')
    OnObjectsAddedArguments = collections.namedtuple('OnObjectsAddedArguments', 'objectIds ')
    OnObjectsRemovedArguments = collections.namedtuple('OnObjectsRemovedArguments', 'objectIds ')

    def __init__(self, ipcproxy, service=None, queue=None):
        KreaTvIpcObserver.__init__(self, ipcproxy, service, queue)

    def handle_callback(self, interface_name, method_name, stack):
        """
        Pushes the call details onto the log queue.
        Called on the worker thread.
        """
        call = KreaTvIpcObserver.Call(address=self.observer_address,
                                      interface=interface_name,
                                      method=method_name)
        if method_name == 'OnObjectsChanged':
            args = {}
            temp_objects = []
            for o in range(stack.pop_int32()):
              dict_temp_objects = {}
              temp_Children = []
              for p in range(stack.pop_int32()):
                temp_Children.append(stack.pop_string())
              dict_temp_objects['Children'] = temp_Children
              temp_Parameters = []
              for p in range(stack.pop_int32()):
                dict_temp_Parameters = {}
                dict_temp_Parameters['Value'] = stack.pop_string()
                dict_temp_Parameters['Name'] = stack.pop_string()
                temp_Parameters.append(dict_temp_Parameters)
              dict_temp_objects['Parameters'] = temp_Parameters
              dict_temp_objects['ObjectId'] = stack.pop_string()
              temp_objects.append(dict_temp_objects)
            args['objects'] = temp_objects
            call.arguments = ToiDiagnosticsDataModelObserver.OnObjectsChangedArguments(**args)
        if method_name == 'OnObjectsAdded':
            args = {}
            temp_objectIds = []
            for o in range(stack.pop_int32()):
              temp_objectIds.append(stack.pop_string())
            args['objectIds'] = temp_objectIds
            call.arguments = ToiDiagnosticsDataModelObserver.OnObjectsAddedArguments(**args)
        if method_name == 'OnObjectsRemoved':
            args = {}
            temp_objectIds = []
            for o in range(stack.pop_int32()):
              temp_objectIds.append(stack.pop_string())
            args['objectIds'] = temp_objectIds
            call.arguments = ToiDiagnosticsDataModelObserver.OnObjectsRemovedArguments(**args)
        # Thread-safe operation:
        self._queue.append(call)

    def wait_OnObjectsChanged(self, discard_non_matching_events=False,
                            timeout_sec=10, **argument_filters):
        """
        Convenience function to wait for callbacks to OnObjectsChanged.

        Keyword arguments can be given to specify parameter values to wait for.
        If a received event does not match the filter, a KreaTvIpcException
        will be raised, unless discard_non_matching_events is true in which
        case the event will be silently ignored (and the function will continue
        to wait for a matching event).
        """
        call = self.wait_callback(self.observer_address,
                                  'IToiDiagnosticsDataModelObserver',
                                  'OnObjectsChanged',
                                  discard_non_matching_events,
                                  timeout_sec,
                                  argument_filters)
        return call.arguments.objects

    def wait_OnObjectsAdded(self, discard_non_matching_events=False,
                            timeout_sec=10, **argument_filters):
        """
        Convenience function to wait for callbacks to OnObjectsAdded.

        Keyword arguments can be given to specify parameter values to wait for.
        If a received event does not match the filter, a KreaTvIpcException
        will be raised, unless discard_non_matching_events is true in which
        case the event will be silently ignored (and the function will continue
        to wait for a matching event).
        """
        call = self.wait_callback(self.observer_address,
                                  'IToiDiagnosticsDataModelObserver',
                                  'OnObjectsAdded',
                                  discard_non_matching_events,
                                  timeout_sec,
                                  argument_filters)
        return call.arguments.objectIds

    def wait_OnObjectsRemoved(self, discard_non_matching_events=False,
                            timeout_sec=10, **argument_filters):
        """
        Convenience function to wait for callbacks to OnObjectsRemoved.

        Keyword arguments can be given to specify parameter values to wait for.
        If a received event does not match the filter, a KreaTvIpcException
        will be raised, unless discard_non_matching_events is true in which
        case the event will be silently ignored (and the function will continue
        to wait for a matching event).
        """
        call = self.wait_callback(self.observer_address,
                                  'IToiDiagnosticsDataModelObserver',
                                  'OnObjectsRemoved',
                                  discard_non_matching_events,
                                  timeout_sec,
                                  argument_filters)
        return call.arguments.objectIds

