# Copyright (c) 2011 Motorola Mobility, Inc. All rights reserved.

from kreatvipccaller import KreaTvIpcCaller
from binipcparameterstack import BinIpcParameterStack


class ToiDiagnosticsService(KreaTvIpcCaller):

  # Const DATA_MODEL_MOTOROLA_STB
  DATA_MODEL_MOTOROLA_STB = "Motorola_STB"

  def __init__(self, ipcproxy):
    KreaTvIpcCaller.__init__(self, ipcproxy.ip_address, ipcproxy.port)
    self._ipcproxy = ipcproxy
    self.lookup("DiagnosticsService")

  def CreateDataModelInstance(self, applicationId, dataModelName):
    """This function creates a new Diagnostics Data Model instance.
    An instance of the data model is a client specific view of the data model.
    Although there might be different views, there is only one data model.
    An instance can be destroyed if the client is no longer using it.
    
    \param applicationId
    The identifier of the calling application.
    
    \param dataModelName
    A name for the data model. Typically this is used to
    indicate the type of client.
    
    \retval dataModelAddress
    The address to the \ref IToiDiagnosticsDataModel instance.
    
    \exception TToiInvalidArgumentException
    Raised if the dataNodelName is not a valid string or
    if the applicationId is not valid.
    
    """
    __request = BinIpcParameterStack()
    # Write applicationId
    __request.push_int32(applicationId)
    # Write dataModelName
    __request.push_string(dataModelName)
    __response = self.call(__request, "IToiDiagnosticsService", "CreateDataModelInstance")
    __result = []
    # Read dataModelAddress
    __result.append(__response.pop_string())
    return __result[0]

