# Copyright (c) 2011 Motorola Mobility, Inc. All rights reserved.

from kreatvipcobserver import KreaTvIpcObserver
import collections
import sys

class ToiDlnaRendererObserver(KreaTvIpcObserver):
    OnRendererSessionChangedArguments = collections.namedtuple('OnRendererSessionChangedArguments', 'info ')

    def __init__(self, ipcproxy, service=None, queue=None):
        KreaTvIpcObserver.__init__(self, ipcproxy, service, queue)

    def handle_callback(self, interface_name, method_name, stack):
        """
        Pushes the call details onto the log queue.
        Called on the worker thread.
        """
        call = KreaTvIpcObserver.Call(address=self.observer_address,
                                      interface=interface_name,
                                      method=method_name)
        if method_name == 'OnRendererSessionChanged':
            args = {}
            dict_info = {}
            temp_MediaProperties = []
            for o in range(stack.pop_int32()):
              dict_temp_MediaProperties = {}
              dict_temp_MediaProperties['Value'] = stack.pop_string()
              dict_temp_MediaProperties['SubIndex'] = stack.pop_int32()
              dict_temp_MediaProperties['Id'] = stack.pop_int32()
              temp_MediaProperties.append(dict_temp_MediaProperties)
            dict_info['MediaProperties'] = temp_MediaProperties
            dict_info['MimeType'] = stack.pop_string()
            dict_info['Uri'] = stack.pop_string()
            dict_info['State'] = stack.pop_int32()
            dict_info['Id'] = stack.pop_int32()
            args['info'] = dict_info
            call.arguments = ToiDlnaRendererObserver.OnRendererSessionChangedArguments(**args)
        # Thread-safe operation:
        self._queue.append(call)

    def wait_OnRendererSessionChanged(self, discard_non_matching_events=False,
                            timeout_sec=10, **argument_filters):
        """
        Convenience function to wait for callbacks to OnRendererSessionChanged.

        Keyword arguments can be given to specify parameter values to wait for.
        If a received event does not match the filter, a KreaTvIpcException
        will be raised, unless discard_non_matching_events is true in which
        case the event will be silently ignored (and the function will continue
        to wait for a matching event).
        """
        call = self.wait_callback(self.observer_address,
                                  'IToiDlnaRendererObserver',
                                  'OnRendererSessionChanged',
                                  discard_non_matching_events,
                                  timeout_sec,
                                  argument_filters)
        return call.arguments.info

