# Copyright (c) 2011 Motorola Mobility, Inc. All rights reserved.

from kreatvipccaller import KreaTvIpcCaller
from binipcparameterstack import BinIpcParameterStack
from ToiDrmObserver import ToiDrmObserver


class ToiDrmService(KreaTvIpcCaller):

  # Enum TToiCopyPermission
  COPY_FREE = 0
  COPY_NO_MORE = 1
  COPY_10 = 2
  COPY_NEVER = 3

  def __init__(self, ipcproxy):
    KreaTvIpcCaller.__init__(self, ipcproxy.ip_address, ipcproxy.port)
    self._ipcproxy = ipcproxy
    self.lookup("DrmService")

  def GetCurrentOutputControl(self, mediaSessionId):
    """Returns the current output control for a specific session.
    
    \param mediaSessionId The media session for which the CCI is requested.
    
    \retval outputControl The CCI.
    
    \exception TToiInvalidArgumentException
    The media session identifier is invalid.
    """
    __request = BinIpcParameterStack()
    # Write mediaSessionId
    __request.push_int32(mediaSessionId)
    __response = self.call(__request, "IToiDrmService", "GetCurrentOutputControl")
    __result = []
    # Read outputControl
    outputControl = {}
    outputControl["RecordingForMobileAllowed"] = __response.pop_bool()
    outputControl["RecordingAllowed"] = __response.pop_bool()
    __result.append(outputControl)
    return __result[0]

  def QueryCopyPermission(self, assetUrl):
    """Queries whether an asset can be copied.
    \param assetUrl          The URL to the asset
    \retval copyPermission   Indicates the permission.
    \retval remainingCopies  If copyPermission is COPY_10, this argument indicates the
    number of remaining copies.
    \exception TToiInvalidArgumentException
    The asset identifier is invalid.
    """
    __request = BinIpcParameterStack()
    # Write assetUrl
    __request.push_string(assetUrl)
    __response = self.call(__request, "IToiDrmService", "QueryCopyPermission")
    __result = []
    # Read remainingCopies
    __result.append(__response.pop_int32())
    # Read copyPermission
    __result.append(__response.pop_int32())
    return tuple(__result)

  def RegisterObserver(self, applicationId, address):
    """Register an observer of type IToiDrmObserver.
    The observer will receive events sent by this IToiDrmService instance.
    
    \param applicationId  The identifier of the observing application.
    \param address        The address where the IToiDrmService can find an
    IToiDrmObserver interface.
    \exception TToiInvalidArgumentException
    Raised if the specified address is invalid
    or if the application has already registered
    itself as an observer with the same address.
    \see IToiDrmObserver
    """
    __request = BinIpcParameterStack()
    # Write applicationId
    __request.push_int32(applicationId)
    # Write address
    __request.push_string(address)
    self.call(__request, "IToiDrmService", "RegisterObserver")

  def ReleaseObserver(self, address):
    """Release an observer previously registered with \ref RegisterObserver.
    
    \param address        The address where the IToiDrmService
    can find the registered IToiDrmObserver interface.
    \exception TToiInvalidArgumentException
    Raised if the specified address is invalid.
    \see IToiDrmObserver
    """
    __request = BinIpcParameterStack()
    # Write address
    __request.push_string(address)
    self.call(__request, "IToiDrmService", "ReleaseObserver")

  def CreateObserver(self):
    """Create and register an observer for the service.
       (This method is only available from Python.)
    """
    observer = ToiDrmObserver(self._ipcproxy, self)
    request = BinIpcParameterStack()
    request.push_int32(self._ipcproxy.application_id)
    request.push_string(observer.observer_address)
    self.call(request, "IToiDrmService", "RegisterObserver")
    return observer
