# Copyright (c) 2011 Motorola Mobility, Inc. All rights reserved.

from kreatvipccaller import KreaTvIpcCaller
from binipcparameterstack import BinIpcParameterStack


from ToiScanner import ToiScanner

class ToiDvbCScanner(ToiScanner):

  # Enum TToiDvbCModulation
  MODULATION_ANY = 0
  MODULATION_16_QAM = 1
  MODULATION_32_QAM = 2
  MODULATION_64_QAM = 3
  MODULATION_128_QAM = 4
  MODULATION_256_QAM = 5

  def __init__(self, ipcproxy, object_address):
    ToiScanner.__init__(self, ipcproxy, object_address)

  def AddFrequencyScanParameters(self, frontendParameters, startFrequency, endFrequency, step):
    """Add one set of parameters for a frequency band based scan.
    This method should be called before \ref IToiScanner::StartScan().
    
    Repeatly calling this method is allowed, parameters are stored and
    when the scan is started each set of parameters/band is scanned,
    one by one. If this method is called with the same values several
    times, only the first call has any effect.
    
    \param frontendParameters    DVB-C tuning parameters.
    
    \param startFrequency        Start frequency in Hz.
    
    \param endFrequency          End frequency in Hz.
    
    \param step                  Frequency interval used when scanning,
    in Hz.
    
    \exception TToiInvalidPreconditionException
    Raised if a scan is already active.
    
    \exception TToiInvalidArgumentException
    Raised if the arguments are not valid.
    
    \see IToiScanner::StartScan()
    \see TToiScanner::Clear()
    """
    __request = BinIpcParameterStack()
    # Write frontendParameters
    __request.push_int32(frontendParameters["Modulation"])
    __request.push_int32(frontendParameters["SymbolRate"])
    # Write startFrequency
    __request.push_int32(startFrequency)
    # Write endFrequency
    __request.push_int32(endFrequency)
    # Write step
    __request.push_int32(step)
    self.call(__request, "IToiDvbCScanner", "AddFrequencyScanParameters")

  def AddNitScanParametersByNetworkId(self, frontendParameters, frequency, networkId):
    """Add a set of parameters for a scan based on Network Id. The scan will
    tune to a specific frequency and start listening on SI on that
    frequency. This method should be called before
    \ref IToiScanner::StartScan().
    
    Repeatly calling this method is allowed, parameters are stored.
    When the scan is started the scan operation will perform the scan one
    parameter set by one, in the order they were added. If this method is
    called with the same values several times, only the first call will have
    any effect.
    
    When found, all Service Description Tables (SDTs) associated with that
    Network Information Table (NIT) are read. Service Information (SI) is
    acquired without actually scanning different frequencies. It is acquired
    by reading Service Information on the specified channel.
    
    \param frontendParameters Cable tuning parameters.
    
    \param frequency          The frequency in Hz.
    
    \param networkId          The network id to look for in NIT actual/other.
    
    \exception TToiInvalidPreconditionException
    Raised if a scan is already active.
    
    \exception TToiInvalidArgumentException
    Raised if the arguments are not valid.
    
    \see IToiScanner::StartScan()
    \see TToiScanner::Clear()
    """
    __request = BinIpcParameterStack()
    # Write frontendParameters
    __request.push_int32(frontendParameters["Modulation"])
    __request.push_int32(frontendParameters["SymbolRate"])
    # Write frequency
    __request.push_int32(frequency)
    # Write networkId
    __request.push_int32(networkId)
    self.call(__request, "IToiDvbCScanner", "AddNitScanParametersByNetworkId")

