# Copyright (c) 2011 Motorola Mobility, Inc. All rights reserved.

from kreatvipcobserver import KreaTvIpcObserver
import collections
import sys

class ToiDvbEitCacheObserver(KreaTvIpcObserver):
    OnCacheUpdatedArguments = collections.namedtuple('OnCacheUpdatedArguments', 'updates ')

    def __init__(self, ipcproxy, service=None, queue=None):
        KreaTvIpcObserver.__init__(self, ipcproxy, service, queue)

    def handle_callback(self, interface_name, method_name, stack):
        """
        Pushes the call details onto the log queue.
        Called on the worker thread.
        """
        call = KreaTvIpcObserver.Call(address=self.observer_address,
                                      interface=interface_name,
                                      method=method_name)
        if method_name == 'OnCacheUpdated':
            args = {}
            temp_updates = []
            for o in range(stack.pop_int32()):
              dict_temp_updates = {}
              dict_temp_updates['Duration'] = stack.pop_int32()
              dict_temp_updates['Time'] = stack.pop_uint32()
              dict_temp_updates['ServiceId'] = stack.pop_int32()
              dict_temp_updates['TransportStreamId'] = stack.pop_int32()
              dict_temp_updates['OriginalNetworkId'] = stack.pop_int32()
              temp_updates.append(dict_temp_updates)
            args['updates'] = temp_updates
            call.arguments = ToiDvbEitCacheObserver.OnCacheUpdatedArguments(**args)
        # Thread-safe operation:
        self._queue.append(call)

    def wait_OnCacheUpdated(self, discard_non_matching_events=False,
                            timeout_sec=10, **argument_filters):
        """
        Convenience function to wait for callbacks to OnCacheUpdated.

        Keyword arguments can be given to specify parameter values to wait for.
        If a received event does not match the filter, a KreaTvIpcException
        will be raised, unless discard_non_matching_events is true in which
        case the event will be silently ignored (and the function will continue
        to wait for a matching event).
        """
        call = self.wait_callback(self.observer_address,
                                  'IToiDvbEitCacheObserver',
                                  'OnCacheUpdated',
                                  discard_non_matching_events,
                                  timeout_sec,
                                  argument_filters)
        return call.arguments.updates

