# Copyright (c) 2011 Motorola Mobility, Inc. All rights reserved.

from kreatvipccaller import KreaTvIpcCaller
from binipcparameterstack import BinIpcParameterStack


class ToiDvbEitService(KreaTvIpcCaller):

  def __init__(self, ipcproxy):
    KreaTvIpcCaller.__init__(self, ipcproxy.ip_address, ipcproxy.port)
    self._ipcproxy = ipcproxy
    self.lookup("DvbEitService")

  def CreateInstance(self, applicationId):
    """Creates a new EIT cache and associates it with the calling
    application. It returns the address to the \ref IToiDvbEitCache interface
    of the new instance.
    
    When the application no longer needs the cache, it should release it by
    calling the \ref IToiDvbEitCache::ReleaseInstance() function.
    
    \param applicationId   The application identifier of the calling
    application.
    \retval address        The address to the \ref IToiDvbEitCache interface
    of the new EIT cache.
    
    \exception TToiNotEnoughResourcesException
    Raised if too many instances have already been
    created.
    
    \see IToiDvbEitCache
    \see IToiDvbEitCacheObserver
    
    """
    __request = BinIpcParameterStack()
    # Write applicationId
    __request.push_int32(applicationId)
    __response = self.call(__request, "IToiDvbEitService", "CreateInstance")
    __result = []
    # Read address
    __result.append(__response.pop_string())
    return __result[0]

