# Copyright (c) 2011 Motorola Mobility, Inc. All rights reserved.

from kreatvipccaller import KreaTvIpcCaller
from binipcparameterstack import BinIpcParameterStack


from ToiScanner import ToiScanner

class ToiDvbTScanner(ToiScanner):

  # Enum TToiDvbTBandwidth
  BANDWIDTH_ANY = 0
  BANDWIDTH_8MHZ = 1
  BANDWIDTH_7MHZ = 2
  BANDWIDTH_6MHZ = 3
  BANDWIDTH_5MHZ = 4

  def __init__(self, ipcproxy, object_address):
    ToiScanner.__init__(self, ipcproxy, object_address)

  def AddFrequencyScanParameters(self, tunerParams, startFrequency, endFrequency, step):
    """Add a set of parameters for a frequency band based scan.
    This method should be called before \ref IToiScanner::StartScan()
    
    Repeatly calling this method is allowed, parameters are stored and when
    the scan is started each set of parameters/band is scanned, one by one.
    
    \param tunerParams    Terrestrial tuning parameters.
    \param startFrequency Start frequency in Hz.
    \param endFrequency   End frequency in Hz.
    \param step           Step used when scanning in Hz.
    
    \exception TToiInvalidPreconditionException
    Raised if one scan is active.
    \exception TToiInvalidArgumentException
    Raised if the arguments are not valid.
    
    \see IToiScanner::StartScan()
    \see TToiScanner::Clear()
    """
    __request = BinIpcParameterStack()
    # Write tunerParams
    __request.push_int32(tunerParams["Bandwidth"])
    __request.push_bool(tunerParams["UseHighPriority"])
    # Write startFrequency
    __request.push_int32(startFrequency)
    # Write endFrequency
    __request.push_int32(endFrequency)
    # Write step
    __request.push_int32(step)
    self.call(__request, "IToiDvbTScanner", "AddFrequencyScanParameters")

