# Copyright (c) 2011 Motorola Mobility, Inc. All rights reserved.

from kreatvipccaller import KreaTvIpcCaller
from binipcparameterstack import BinIpcParameterStack


class ToiFrontPanelService(KreaTvIpcCaller):

  # Enum TToiColor
  COLOR_OFF = 0
  COLOR_GREEN = 1
  COLOR_RED = 2
  COLOR_ORANGE = 3
  COLOR_BLUE = 4
  COLOR_YELLOW = 5

  # Enum TToiSegmentDisplayModes
  SEGMENT_DISPLAY_MODE_TEXT = 0
  SEGMENT_DISPLAY_MODE_CLOCK = 1

  def __init__(self, ipcproxy):
    KreaTvIpcCaller.__init__(self, ipcproxy.ip_address, ipcproxy.port)
    self._ipcproxy = ipcproxy
    self.lookup("FrontPanelService")

  def GetLedCount(self):
    """Returns the number of LEDs.
    
    \retval numberOfLeds The number of LEDs deployed on the STB.
    LED indexing starts at zero and increments
    to the number of LEDs - 1.
    """
    __request = BinIpcParameterStack()
    __response = self.call(__request, "IToiFrontPanelService", "GetLedCount")
    __result = []
    # Read numberOfLeds
    __result.append(__response.pop_int32())
    return __result[0]

  def GetLedInfo(self, index):
    """Get information about a LED.
    
    \param index      The index for the LED.
    
    \retval info      Information about the LED.
    
    \exception TToiInvalidArgumentException
    Raised if any of the parameters to the call
    are invalid. Also raised if there are no LEDs
    present.
    """
    __request = BinIpcParameterStack()
    # Write index
    __request.push_int32(index)
    __response = self.call(__request, "IToiFrontPanelService", "GetLedInfo")
    __result = []
    # Read info
    info = {}
    info["SupportedColors"] = []
    for k in range(__response.pop_int32()):
      info["SupportedColors"].append(__response.pop_int32())
    info["Index"] = __response.pop_int32()
    __result.append(info)
    return __result[0]

  def SetLedState(self, index, state):
    """Turn the LED to the desired color and blinking state.
    
    \param index      The index for the LED.
    
    \param state      The state to use for the LED.
    
    \exception TToiInvalidArgumentException
    Raised if any of the parameters to the call
    are invalid. Also raised if there are no LEDs
    present.
    \exception TToiOperationNotSupportedException
    Raised if any of the parameters to the call
    are not supported.
    
    
    """
    __request = BinIpcParameterStack()
    # Write index
    __request.push_int32(index)
    # Write state
    __request.push_int32(state["Color"])
    __request.push_double(state["BlinkFrequency"])
    self.call(__request, "IToiFrontPanelService", "SetLedState")

  def GetLedState(self, index):
    """Get the current state of a given LED.
    
    \param  index      The index for the LED.
    
    \retval state      The current state of the LED.
    
    \exception TToiInvalidArgumentException
    Raised if any of the parameters to the call
    are invalid. Also raised if there are no LEDs
    present.
    """
    __request = BinIpcParameterStack()
    # Write index
    __request.push_int32(index)
    __response = self.call(__request, "IToiFrontPanelService", "GetLedState")
    __result = []
    # Read state
    state = {}
    state["BlinkFrequency"] = __response.pop_double()
    state["Color"] = __response.pop_int32()
    __result.append(state)
    return __result[0]

  def GetSegmentDisplayInfo(self):
    """Returns information about the segment display.
    
    \retval info Returns information about the segment display.
    
    \exception TToiOperationNotSupportedException
    Raised if the STB doesn't have a segment display.
    """
    __request = BinIpcParameterStack()
    __response = self.call(__request, "IToiFrontPanelService", "GetSegmentDisplayInfo")
    __result = []
    # Read info
    info = {}
    info["SupportsIntensity"] = __response.pop_bool()
    info["SupportsColon"] = __response.pop_bool()
    info["NumberOfBlocks"] = __response.pop_int32()
    __result.append(info)
    return __result[0]

  def SetSegmentDisplayState(self, state):
    """Starts a clock or sets text on the segment display.
    
    \param state          The state to use for the display.
    
    \exception TToiInvalidArgumentException
    Raised if any of the parameters to the
    call are invalid. An example would be if
    you try to display text and a clock at the
    same time.
    
    \exception TToiOperationNotSupportedException
    Raised if the STB doesn't have a segment display.
    """
    __request = BinIpcParameterStack()
    # Write state
    __request.push_string(state["Text"])
    __request.push_bool(state["ColonState"])
    __request.push_int32(state["Intensity"])
    __request.push_int32(state["CurrentMode"])
    self.call(__request, "IToiFrontPanelService", "SetSegmentDisplayState")

  def GetSegmentDisplayState(self):
    """Returns the current state of the segment display.
    
    \retval state          The current state of the display.
    
    \exception TToiOperationNotSupportedException
    Raised if the STB doesn't have a segment
    display.
    """
    __request = BinIpcParameterStack()
    __response = self.call(__request, "IToiFrontPanelService", "GetSegmentDisplayState")
    __result = []
    # Read state
    state = {}
    state["CurrentMode"] = __response.pop_int32()
    state["Intensity"] = __response.pop_int32()
    state["ColonState"] = __response.pop_bool()
    state["Text"] = __response.pop_string()
    __result.append(state)
    return __result[0]

  def IsSegmentDisplaySupported(self):
    """Indicates if the STB is equipped with a segment display.
    
    \retval isSupported Set to true if a segment display is present.
    """
    __request = BinIpcParameterStack()
    __response = self.call(__request, "IToiFrontPanelService", "IsSegmentDisplaySupported")
    __result = []
    # Read isSupported
    __result.append(__response.pop_bool())
    return __result[0]

