# Copyright (c) 2011 Motorola Mobility, Inc. All rights reserved.

from kreatvipccaller import KreaTvIpcCaller
from binipcparameterstack import BinIpcParameterStack


class ToiFrontendService(KreaTvIpcCaller):

  # Enum TToiModulationSystem
  CAPABILITY_SYSTEM_DVB_C = 0x1
  CAPABILITY_SYSTEM_DVB_S = 0x2
  CAPABILITY_SYSTEM_DVB_S2 = 0x4
  CAPABILITY_SYSTEM_DVB_T = 0x8
  CAPABILITY_SYSTEM_ISDB_C = 0x10
  CAPABILITY_SYSTEM_ISDB_S = 0x20
  CAPABILITY_SYSTEM_ISDB_T = 0x40
  CAPABILITY_SYSTEM_SBTVD = 0x80
  CAPABILITY_SYSTEM_SCTE = 0x100
  CAPABILITY_SYSTEM_ATSC = 0x200

  # Enum TToiChannelBandwidth
  CAPABILITY_BANDWIDTH_1_8MHZ = 0x1
  CAPABILITY_BANDWIDTH_5MHZ = 0x2
  CAPABILITY_BANDWIDTH_6MHZ = 0x4
  CAPABILITY_BANDWIDTH_7MHZ = 0x8
  CAPABILITY_BANDWIDTH_8MHZ = 0x10

  # Enum TToiModulationMode
  CAPABILITY_MODULATION_QAM_16 = 0x1
  CAPABILITY_MODULATION_QAM_32 = 0x2
  CAPABILITY_MODULATION_QAM_64 = 0x4
  CAPABILITY_MODULATION_QAM_128 = 0x8
  CAPABILITY_MODULATION_QAM_256 = 0x10
  CAPABILITY_MODULATION_QPSK = 0x20
  CAPABILITY_MODULATION_DVS178_QPSK = 0x40

  # Const FRONTEND_ID_ANY_FREE
  FRONTEND_ID_ANY_FREE = -1

  # Const PROPERTY_SIGNAL_STRENGTH_INDICATOR
  PROPERTY_SIGNAL_STRENGTH_INDICATOR = 1

  # Const PROPERTY_SIGNAL_QUALITY_INDICATOR
  PROPERTY_SIGNAL_QUALITY_INDICATOR = 2

  # Const PROPERTY_DERIVED_CHANNEL_ID
  PROPERTY_DERIVED_CHANNEL_ID = 3

  def __init__(self, ipcproxy):
    KreaTvIpcCaller.__init__(self, ipcproxy.ip_address, ipcproxy.port)
    self._ipcproxy = ipcproxy
    self.lookup("FrontendService")

  def GetOperationManager(self):
    """Returns the operation manager that handles asynchronous operations for
    all services derived from the Frontend service. The procedure of
    executing asynchronous operations is generic and looks like this:
    <ul>
    <li>Get a handle to the operation manager.</li>
    <li>Register an observer to get operation events from the operation
    manager.</li>
    <li>Create an operation in order to get an operation identifier.</li>
    <li>Call a asynchronous operation using the operation identifier.</li>
    <li>Receive progress events for the asynchronous operation.</li>
    <li>When the operation is completed, get result data (if any).</li>
    <li>Release the operation or reuse the identifier for another
    asynchronous call.</li>
    </ul>
    
    \retval address The address to the \ref IToiOperationManager.
    """
    __request = BinIpcParameterStack()
    __response = self.call(__request, "IToiFrontendService", "GetOperationManager")
    __result = []
    # Read address
    __result.append(__response.pop_string())
    return __result[0]

  def CreateDvbTScanner(self, applicationId, frontendId):
    """This function creates a new DVB-T scanner instance and associates it with
    the calling application. System resources are allocated when this
    function is called and released when \ref IToiScanner::ReleaseInstance()
    is called.
    
    When the application no longer needs the scanner instance, it must
    release it by calling \ref IToiScanner::ReleaseInstance().
    
    \param applicationId   The identifier of the calling application.
    
    \param frontendId      The frontend identifier which will be used to bind
    this specific frontend for the scan operation.
    
    \retval scannerAddress The address to the \ref IToiDvbTScanner instance.
    
    \exception TToiInvalidArgumentException
    Raised if the specified application identifier is
    invalid, or raised if the frontend id is invalid.
    
    \exception TToiNotEnoughResourcesException
    Raised if the specified frontend resource is in
    use.
    
    \exception TToiOperationNotSupportedException
    Raised if DVB-T is not supported.
    
    \see IToiScanner
    \see IToiDvbTScanner
    \see IToiScanner::ReleaseInstance()
    
    """
    __request = BinIpcParameterStack()
    # Write applicationId
    __request.push_int32(applicationId)
    # Write frontendId
    __request.push_int32(frontendId)
    __response = self.call(__request, "IToiFrontendService", "CreateDvbTScanner")
    __result = []
    # Read scannerAddress
    __result.append(__response.pop_string())
    return __result[0]

  def CreateDvbCScanner(self, applicationId, frontendId):
    """This function creates a new DVB-C scanner instance and associates it with
    the calling application. System resources are allocated when this
    function is called and released when \ref IToiScanner::ReleaseInstance()
    is called.
    
    When the application no longer needs the scanner instance, it must
    release it by calling \ref IToiScanner::ReleaseInstance().
    
    \param applicationId   The identifier of the calling application.
    
    \param frontendId      The frontend identifier which will be used to bind
    this specific frontend for the scan operation.
    
    \retval scannerAddress The address to the \ref IToiDvbCScanner instance.
    
    \exception TToiInvalidArgumentException
    Raised if the specified application identifier is
    invalid, or if frontend id is invalid.
    
    \exception TToiNotEnoughResourcesException
    Raised if the specified frontend resource is in
    use.
    
    \exception TToiOperationNotSupportedException
    Raised if DVB-C is not supported.
    
    \see IToiScanner
    \see IToiDvbCScanner
    \see IToiScanner::ReleaseInstance()
    
    """
    __request = BinIpcParameterStack()
    # Write applicationId
    __request.push_int32(applicationId)
    # Write frontendId
    __request.push_int32(frontendId)
    __response = self.call(__request, "IToiFrontendService", "CreateDvbCScanner")
    __result = []
    # Read scannerAddress
    __result.append(__response.pop_string())
    return __result[0]

  def CreateFrontendDownloader(self, applicationId, frontendId):
    """This function creates a new frontend downloader instance and
    associates it with the calling application. System resources are
    allocated when this function is called and released when
    \ref IToiFrontendDownloader::ReleaseInstance() is called.
    
    When the application no longer needs the frontend downloader
    instance, it must release it by calling
    \ref IToiFrontendDownloader::ReleaseInstance().
    
    \param applicationId      The identifier of the calling application.
    
    \param frontendId         The frontend identifier which will be used to
    bind this specific frontend to the downloader
    instance.
    
    \retval downloaderAddress The address to the \ref IToiFrontendDownloader
    instance.
    
    \exception TToiInvalidArgumentException
    Raised if the specified application identifier
    is invalid, or if frontend id is invalid.
    
    \exception TToiNotEnoughResourcesException
    Raised if the specified frontend resource is
    in use.
    
    \exception TToiOperationNotSupportedException
    Raised if the function is not supported.
    
    \see IToiFrontendDownloader
    \see IToiFrontendDownloader::ReleaseInstance()
    
    """
    __request = BinIpcParameterStack()
    # Write applicationId
    __request.push_int32(applicationId)
    # Write frontendId
    __request.push_int32(frontendId)
    __response = self.call(__request, "IToiFrontendService", "CreateFrontendDownloader")
    __result = []
    # Read downloaderAddress
    __result.append(__response.pop_string())
    return __result[0]

  def GetFrontends(self):
    """This function is used to get the frontend identifiers for the available
    frontends.
    
    \retval identifiers A list of frontend identifiers.
    
    \exception TToiOperationNotSupportedException
    Raised if there is no frontend available.
    """
    __request = BinIpcParameterStack()
    __response = self.call(__request, "IToiFrontendService", "GetFrontends")
    __result = []
    # Read identifiers
    identifiers = []
    for k in range(__response.pop_int32()):
      identifiers.append(__response.pop_int32())
    __result.append(identifiers)
    return __result[0]

  def GetFrontendInfo(self, frontendId):
    """Get status information about the frontend, i.e. tuner and demodulator,
    for the dedicated frontend.
    
    \param frontendId The frontend identifier.
    
    \retval info      The frontend status information.
    
    \exception TToiInvalidArgumentException
    Raised if the frontend id is not valid.
    
    \exception TToiNoDataException
    Raised if the frontend information is not available.
    
    \exception TToiOperationNotSupportedException
    Raised if there is no frontend available.
    """
    __request = BinIpcParameterStack()
    # Write frontendId
    __request.push_int32(frontendId)
    __response = self.call(__request, "IToiFrontendService", "GetFrontendInfo")
    __result = []
    # Read info
    info = {}
    info["ExtendedProperties"] = []
    for k in range(__response.pop_int32()):
      temp_l = {}
      temp_l["Value"] = __response.pop_string()
      temp_l["Id"] = __response.pop_int32()
      info["ExtendedProperties"].append(temp_l)
    info["RFLevel"] = __response.pop_int32()
    info["UncorrectedBlocks"] = __response.pop_int32()
    info["BitErrorRate"] = __response.pop_int32()
    info["SignalNoiseRatio"] = __response.pop_int32()
    info["CentreFrequency"] = __response.pop_int32()
    info["IsLocked"] = __response.pop_bool()
    info["FrontendCapabilities"] = {}
    info["FrontendCapabilities"]["ModulationModeMask"] = __response.pop_int32()
    info["FrontendCapabilities"]["ChannelBandwidthMask"] = __response.pop_int32()
    info["FrontendCapabilities"]["ModulationSystemMask"] = __response.pop_int32()
    info["SessionId"] = __response.pop_int32()
    __result.append(info)
    return __result[0]

  def GetFrontendInfoBySession(self, sessionId):
    """Get status information about the frontend, i.e. tuner and demodulator,
    for the dedicated session.
    
    \param  sessionId The media session identifier.
    
    \retval info      The frontend status information.
    
    \exception TToiInvalidArgumentException
    Raised if the session id is not valid
    or if the session does not exist.
    
    \exception TToiNoDataException
    Raised if the frontend information is not available.
    
    \exception TToiOperationNotSupportedException
    Raised if there is no frontend available
    or if the given media session is not using a frontend.
    """
    __request = BinIpcParameterStack()
    # Write sessionId
    __request.push_int32(sessionId)
    __response = self.call(__request, "IToiFrontendService", "GetFrontendInfoBySession")
    __result = []
    # Read info
    info = {}
    info["ExtendedProperties"] = []
    for k in range(__response.pop_int32()):
      temp_l = {}
      temp_l["Value"] = __response.pop_string()
      temp_l["Id"] = __response.pop_int32()
      info["ExtendedProperties"].append(temp_l)
    info["RFLevel"] = __response.pop_int32()
    info["UncorrectedBlocks"] = __response.pop_int32()
    info["BitErrorRate"] = __response.pop_int32()
    info["SignalNoiseRatio"] = __response.pop_int32()
    info["CentreFrequency"] = __response.pop_int32()
    info["IsLocked"] = __response.pop_bool()
    info["FrontendCapabilities"] = {}
    info["FrontendCapabilities"]["ModulationModeMask"] = __response.pop_int32()
    info["FrontendCapabilities"]["ChannelBandwidthMask"] = __response.pop_int32()
    info["FrontendCapabilities"]["ModulationSystemMask"] = __response.pop_int32()
    info["SessionId"] = __response.pop_int32()
    __result.append(info)
    return __result[0]

