# Copyright (c) 2011 Motorola Mobility, Inc. All rights reserved.

from kreatvipcobserver import KreaTvIpcObserver
import collections
import sys

class ToiInformationObserver(KreaTvIpcObserver):
    OnObjectsChangedArguments = collections.namedtuple('OnObjectsChangedArguments', 'objectNames ')
    OnObjectsRemovedArguments = collections.namedtuple('OnObjectsRemovedArguments', 'objectNames ')

    def __init__(self, ipcproxy, service=None, queue=None):
        KreaTvIpcObserver.__init__(self, ipcproxy, service, queue)

    def handle_callback(self, interface_name, method_name, stack):
        """
        Pushes the call details onto the log queue.
        Called on the worker thread.
        """
        call = KreaTvIpcObserver.Call(address=self.observer_address,
                                      interface=interface_name,
                                      method=method_name)
        if method_name == 'OnObjectsChanged':
            args = {}
            temp_objectNames = []
            for o in range(stack.pop_int32()):
              temp_objectNames.append(stack.pop_string())
            args['objectNames'] = temp_objectNames
            call.arguments = ToiInformationObserver.OnObjectsChangedArguments(**args)
        if method_name == 'OnObjectsRemoved':
            args = {}
            temp_objectNames = []
            for o in range(stack.pop_int32()):
              temp_objectNames.append(stack.pop_string())
            args['objectNames'] = temp_objectNames
            call.arguments = ToiInformationObserver.OnObjectsRemovedArguments(**args)
        # Thread-safe operation:
        self._queue.append(call)

    def wait_OnObjectsChanged(self, discard_non_matching_events=False,
                            timeout_sec=10, **argument_filters):
        """
        Convenience function to wait for callbacks to OnObjectsChanged.

        Keyword arguments can be given to specify parameter values to wait for.
        If a received event does not match the filter, a KreaTvIpcException
        will be raised, unless discard_non_matching_events is true in which
        case the event will be silently ignored (and the function will continue
        to wait for a matching event).
        """
        call = self.wait_callback(self.observer_address,
                                  'IToiInformationObserver',
                                  'OnObjectsChanged',
                                  discard_non_matching_events,
                                  timeout_sec,
                                  argument_filters)
        return call.arguments.objectNames

    def wait_OnObjectsRemoved(self, discard_non_matching_events=False,
                            timeout_sec=10, **argument_filters):
        """
        Convenience function to wait for callbacks to OnObjectsRemoved.

        Keyword arguments can be given to specify parameter values to wait for.
        If a received event does not match the filter, a KreaTvIpcException
        will be raised, unless discard_non_matching_events is true in which
        case the event will be silently ignored (and the function will continue
        to wait for a matching event).
        """
        call = self.wait_callback(self.observer_address,
                                  'IToiInformationObserver',
                                  'OnObjectsRemoved',
                                  discard_non_matching_events,
                                  timeout_sec,
                                  argument_filters)
        return call.arguments.objectNames

