# Copyright (c) 2011 Motorola Mobility, Inc. All rights reserved.

from kreatvipcobserver import KreaTvIpcObserver
import collections
import sys

class ToiMediaObserver(KreaTvIpcObserver):
    OnSessionStateChangedArguments = collections.namedtuple('OnSessionStateChangedArguments', 'sessionInfo ')
    OnSessionStreamInfoChangedArguments = collections.namedtuple('OnSessionStreamInfoChangedArguments', 'sessionInfo streamInfo ')

    def __init__(self, ipcproxy, service=None, queue=None):
        KreaTvIpcObserver.__init__(self, ipcproxy, service, queue)

    def handle_callback(self, interface_name, method_name, stack):
        """
        Pushes the call details onto the log queue.
        Called on the worker thread.
        """
        call = KreaTvIpcObserver.Call(address=self.observer_address,
                                      interface=interface_name,
                                      method=method_name)
        if method_name == 'OnSessionStateChanged':
            args = {}
            dict_sessionInfo = {}
            dict_sessionInfo['State'] = stack.pop_int32()
            dict_sessionInfo['Url'] = stack.pop_string()
            dict_sessionInfo['Type'] = stack.pop_int32()
            dict_sessionInfo['Id'] = stack.pop_int32()
            args['sessionInfo'] = dict_sessionInfo
            call.arguments = ToiMediaObserver.OnSessionStateChangedArguments(**args)
        if method_name == 'OnSessionStreamInfoChanged':
            args = {}
            dict_streamInfo = {}
            dict_streamInfo['ProgramNumber'] = stack.pop_int32()
            args['streamInfo'] = dict_streamInfo
            dict_sessionInfo = {}
            dict_sessionInfo['State'] = stack.pop_int32()
            dict_sessionInfo['Url'] = stack.pop_string()
            dict_sessionInfo['Type'] = stack.pop_int32()
            dict_sessionInfo['Id'] = stack.pop_int32()
            args['sessionInfo'] = dict_sessionInfo
            call.arguments = ToiMediaObserver.OnSessionStreamInfoChangedArguments(**args)
        # Thread-safe operation:
        self._queue.append(call)

    def wait_OnSessionStateChanged(self, discard_non_matching_events=False,
                            timeout_sec=10, **argument_filters):
        """
        Convenience function to wait for callbacks to OnSessionStateChanged.

        Keyword arguments can be given to specify parameter values to wait for.
        If a received event does not match the filter, a KreaTvIpcException
        will be raised, unless discard_non_matching_events is true in which
        case the event will be silently ignored (and the function will continue
        to wait for a matching event).
        """
        call = self.wait_callback(self.observer_address,
                                  'IToiMediaObserver',
                                  'OnSessionStateChanged',
                                  discard_non_matching_events,
                                  timeout_sec,
                                  argument_filters)
        return call.arguments.sessionInfo

    def wait_OnSessionStreamInfoChanged(self, discard_non_matching_events=False,
                            timeout_sec=10, **argument_filters):
        """
        Convenience function to wait for callbacks to OnSessionStreamInfoChanged.

        Keyword arguments can be given to specify parameter values to wait for.
        If a received event does not match the filter, a KreaTvIpcException
        will be raised, unless discard_non_matching_events is true in which
        case the event will be silently ignored (and the function will continue
        to wait for a matching event).
        """
        call = self.wait_callback(self.observer_address,
                                  'IToiMediaObserver',
                                  'OnSessionStreamInfoChanged',
                                  discard_non_matching_events,
                                  timeout_sec,
                                  argument_filters)
        return call.arguments

