# Copyright (c) 2011 Motorola Mobility, Inc. All rights reserved.

from kreatvipccaller import KreaTvIpcCaller
from binipcparameterstack import BinIpcParameterStack
from ToiMediaPlayerObserver import ToiMediaPlayerObserver


class ToiMediaPlayerBase(KreaTvIpcCaller):

  # Enum TToiComponentType
  COMPONENT_AUDIO = 0
  COMPONENT_VIDEO = 1
  COMPONENT_SUBTITLE = 2
  COMPONENT_SUBTITLE_CAPTION = 3
  COMPONENT_SUBTITLE_SUPERIMPOSE = 4
  COMPONENT_TELETEXT_VBI = 5

  # Enum TToiErrorSource
  SOURCE_UNKNOWN = 0
  SOURCE_INTERNAL = 1
  SOURCE_BITBAND = 2

  # Enum TToiState
  STATE_IDLE = 0
  STATE_CONNECTING = 1
  STATE_PAUSED = 2
  STATE_PLAYING = 3
  STATE_FASTFORWARDING = 4
  STATE_REWINDING = 5
  STATE_FAILED = 6

  # Enum TToiTimeshiftState
  TIMESHIFT_STATE_INACTIVE = 0
  TIMESHIFT_STATE_BUFFERING = 1
  TIMESHIFT_STATE_STOPPED = 2
  TIMESHIFT_STATE_BUFFERING_FAILED = 3

  # Enum TToiAudioAudienceType
  AUDIO_AUDIENCE_TYPE_UNKNOWN = 0
  AUDIO_AUDIENCE_TYPE_NORMAL = 1
  AUDIO_AUDIENCE_TYPE_VISUALLY_IMPAIRED = 2
  AUDIO_AUDIENCE_TYPE_HEARING_IMPAIRED = 3

  # Enum TToiAudioChannelFormat
  AUDIO_CHANNEL_FORMAT_UNKNOWN = 0
  AUDIO_CHANNEL_FORMAT_SINGLE_MONO = 1
  AUDIO_CHANNEL_FORMAT_DUAL_MONO = 2
  AUDIO_CHANNEL_FORMAT_STEREO = 3
  AUDIO_CHANNEL_FORMAT_3_FRONT_1_BACK = 4
  AUDIO_CHANNEL_FORMAT_3_FRONT_2_BACK = 5
  AUDIO_CHANNEL_FORMAT_3_FRONT_2_BACK_LFE = 6
  AUDIO_CHANNEL_FORMAT_3_FRONT_2_SIDE_2_BACK_LFE = 7

  # Enum TToiAudioChannelMode
  AUDIO_CHANNEL_MODE_NORMAL = 0
  AUDIO_CHANNEL_MODE_DUAL_MONO_LEFT_ONLY = 1
  AUDIO_CHANNEL_MODE_DUAL_MONO_RIGHT_ONLY = 2
  AUDIO_CHANNEL_MODE_DUAL_MONO_STEREO = 3
  AUDIO_CHANNEL_MODE_DUAL_MONO_MIXED = 4

  # Enum TToiAspectRatio
  ASPECT_RATIO_UNKNOWN = 0
  ASPECT_RATIO_4_3 = 1
  ASPECT_RATIO_16_9 = 2

  # Enum TToiActiveFormatDescriptor
  ACTIVE_FORMAT_UNKNOWN = 0
  ACTIVE_FORMAT_16_9_TOP = 1
  ACTIVE_FORMAT_14_9_TOP = 2
  ACTIVE_FORMAT_MORE_THAN_16_9_CENTRE = 3
  ACTIVE_FORMAT_SAME_AS_CODED = 4
  ACTIVE_FORMAT_4_3_CENTRE = 5
  ACTIVE_FORMAT_16_9_CENTRE = 6
  ACTIVE_FORMAT_14_9_CENTRE = 7
  ACTIVE_FORMAT_4_3_SHOOT_AND_PROTECT_14_9_CENTRE = 8
  ACTIVE_FORMAT_16_9_SHOOT_AND_PROTECT_14_9_CENTRE = 9
  ACTIVE_FORMAT_16_9_SHOOT_AND_PROTECT_4_3_CENTRE = 10

  # Enum TToiSubtitleType
  SUBTITLE_UNKNOWN = 0
  SUBTITLE_NORMAL = 1
  SUBTITLE_HEARING_IMPAIRED = 2

  # Enum TToiSubtitleFormat
  SUBTITLE_DVB = 0
  SUBTITLE_ARIB = 1
  SUBTITLE_TELETEXT = 2
  SUBTITLE_DVD = 3
  SUBTITLE_XSUB = 4

  # Enum TToiReceiveDmfMode
  RECEIVE_DMF_MODE_AUTO = 0
  RECEIVE_DMF_MODE_NON_AUTO = 1
  RECEIVE_DMF_MODE_SELECT = 2
  RECEIVE_DMF_MODE_SPECIAL = 3

  # Enum TToiRecordDmfMode
  RECORD_DMF_MODE_AUTO = 0
  RECORD_DMF_MODE_NON_AUTO = 1
  RECORD_DMF_MODE_SELECT = 2

  # Const PACE_PAUSE
  PACE_PAUSE = 0

  # Const PACE_PLAY
  PACE_PLAY = 1000

  # Const PACE_FASTFORWARD
  PACE_FASTFORWARD = 5000

  # Const PACE_REWIND
  PACE_REWIND = -5000

  # Const POSITION_LIVE
  POSITION_LIVE = -1616928864

  # Const CAPABILITY_PAUSE
  CAPABILITY_PAUSE = 1

  # Const CAPABILITY_REWIND
  CAPABILITY_REWIND = 2

  # Const CAPABILITY_FASTFORWARD
  CAPABILITY_FASTFORWARD = 4

  # Const CAPABILITY_GETPOSITION
  CAPABILITY_GETPOSITION = 8

  # Const CAPABILITY_SETPOSITION
  CAPABILITY_SETPOSITION = 16

  # Const CAPABILITY_FREEZE
  CAPABILITY_FREEZE = 32

  # Const CAPABILITY_TIMESHIFT
  CAPABILITY_TIMESHIFT = 64

  def __init__(self, ipcproxy, object_address):
    KreaTvIpcCaller.__init__(self, ipcproxy.ip_address, ipcproxy.port)
    self._ipcproxy = ipcproxy
    self.set_object_address(object_address)

  def GetState(self):
    """Returns the current state of the media player instance.
    
    \retval state     The current player state.
    """
    __request = BinIpcParameterStack()
    __response = self.call(__request, "IToiMediaPlayerBase", "GetState")
    __result = []
    # Read state
    __result.append(__response.pop_int32())
    return __result[0]

  def GetError(self):
    """Returns the last error. This is typically called when state is
    changed to \ref STATE_FAILED. The result is undefined if no error has
    occurred.
    \retval error The error
    """
    __request = BinIpcParameterStack()
    __response = self.call(__request, "IToiMediaPlayerBase", "GetError")
    __result = []
    # Read error
    error = {}
    error["Details"] = __response.pop_int32()
    error["Source"] = __response.pop_int32()
    __result.append(error)
    return __result[0]

  def GetSessionId(self):
    """Returns the session identifier. This identifier can be used to open a
    session object with \ref IToiMediaService::OpenSession().
    
    \retval id        The session identifier.
    
    \exception TToiSessionViolationException
    If the player is in \ref STATE_IDLE a session violation
    exception is raised.
    
    \see IToiMediaService::OpenSession()
    """
    __request = BinIpcParameterStack()
    __response = self.call(__request, "IToiMediaPlayerBase", "GetSessionId")
    __result = []
    # Read id
    __result.append(__response.pop_int32())
    return __result[0]

  def GetUrl(self):
    """Returns the URL for the media stream.
    
    \retval url       The URL for the media stream.
    
    \exception TToiSessionViolationException
    If the player is in \ref STATE_IDLE a session violation
    exception is raised.
    """
    __request = BinIpcParameterStack()
    __response = self.call(__request, "IToiMediaPlayerBase", "GetUrl")
    __result = []
    # Read url
    __result.append(__response.pop_string())
    return __result[0]

  def GetStreamInfo(self):
    """Returns information about the current stream in form of a stream info
    structure. The lists of audio and video streams provided as parts of this
    information can be used for further exploration using the
    \ref GetAudioStreamInfo() and \ref GetVideoStreamInfo() functions,
    respectively.
    
    \retval streamInfo  Information about the current stream.
    
    \exception TToiSessionViolationException
    If the player is in \ref STATE_IDLE a session violation
    exception is raised.
    
    \exception TToiUnavailableException
    TToiUnavailableException is raised if the player is
    temporarily unavailable due to platform resource
    management.
    """
    __request = BinIpcParameterStack()
    __response = self.call(__request, "IToiMediaPlayerBase", "GetStreamInfo")
    __result = []
    # Read streamInfo
    streamInfo = {}
    streamInfo["Paces"] = []
    for k in range(__response.pop_int32()):
      streamInfo["Paces"].append(__response.pop_int32())
    streamInfo["PlayTime"] = __response.pop_int32()
    streamInfo["ActiveComponents"] = []
    for k in range(__response.pop_int32()):
      temp_l = {}
      temp_l["Id"] = __response.pop_int32()
      temp_l["Type"] = __response.pop_int32()
      streamInfo["ActiveComponents"].append(temp_l)
    streamInfo["AvailableComponents"] = []
    for k in range(__response.pop_int32()):
      temp_l = {}
      temp_l["Id"] = __response.pop_int32()
      temp_l["Type"] = __response.pop_int32()
      streamInfo["AvailableComponents"].append(temp_l)
    streamInfo["Title"] = __response.pop_string()
    __result.append(streamInfo)
    return __result[0]

  def GetVideoStreamInfo(self, mediaComponent):
    """Returns information about a specified video stream in the
    current media stream multiplex in the form of a video stream
    info structure.
    
    \param mediaComponent     The media component entry for which the stream
    info has to be retrieved.
    
    \retval streamInfo  Information about the specified video stream.
    
    \exception TToiSessionViolationException
    If the player is in \ref STATE_IDLE a session violation
    exception is raised.
    
    \exception TToiInvalidArgumentException
    If the mediaComponent given as argument does not identify a
    video stream in the current stream multiplex, an
    invalid argument exception is raised.
    
    \exception TToiUnavailableException
    \ref TToiUnavailableException is raised if the player is
    temporarily unavailable due to platform resource
    management.
    """
    __request = BinIpcParameterStack()
    # Write mediaComponent
    __request.push_int32(mediaComponent["Type"])
    __request.push_int32(mediaComponent["Id"])
    __response = self.call(__request, "IToiMediaPlayerBase", "GetVideoStreamInfo")
    __result = []
    # Read streamInfo
    streamInfo = {}
    streamInfo["ComponentTag"] = __response.pop_int32()
    streamInfo["FrameRate"] = __response.pop_int32()
    streamInfo["ActiveFormatDescriptor"] = __response.pop_int32()
    streamInfo["AspectRatio"] = __response.pop_int32()
    streamInfo["ResolutionY"] = __response.pop_int32()
    streamInfo["ResolutionX"] = __response.pop_int32()
    streamInfo["Encoding"] = __response.pop_string()
    __result.append(streamInfo)
    return __result[0]

  def GetAudioStreamInfo(self, mediaComponent):
    """Returns information about a specified audio stream in the
    current media stream multiplex in the form of an audio stream
    info structure.
    
    \param mediaComponent     The media component entry for which the stream
    info has to be retrieved.
    
    \retval streamInfo  Information about the specified audio stream.
    
    \exception TToiSessionViolationException
    If the player is in \ref STATE_IDLE a session violation
    exception is raised.
    
    \exception TToiInvalidArgumentException
    If the mediaComponent given as argument does not identify an
    audio stream in the current stream multiplex, an
    invalid argument exception is raised.
    
    \exception TToiUnavailableException
    \ref TToiUnavailableException is raised if the player is
    temporarily unavailable due to platform resource
    management.
    """
    __request = BinIpcParameterStack()
    # Write mediaComponent
    __request.push_int32(mediaComponent["Type"])
    __request.push_int32(mediaComponent["Id"])
    __response = self.call(__request, "IToiMediaPlayerBase", "GetAudioStreamInfo")
    __result = []
    # Read streamInfo
    streamInfo = {}
    streamInfo["ComponentTag"] = __response.pop_int32()
    streamInfo["AudienceType"] = __response.pop_int32()
    streamInfo["SubLanguage"] = __response.pop_string()
    streamInfo["Language"] = __response.pop_string()
    streamInfo["ChannelMode"] = __response.pop_int32()
    streamInfo["ChannelFormat"] = __response.pop_int32()
    streamInfo["SampleRate"] = __response.pop_int32()
    streamInfo["Encoding"] = __response.pop_string()
    __result.append(streamInfo)
    return __result[0]

  def GetSubtitleStreamInfo(self, mediaComponent):
    """Returns information about a specified subtitle stream in the
    current media stream multiplex in the form of a subtitle stream
    info structure.
    
    \param mediaComponent     The media component entry for which the stream
    info has to be retrieved.
    
    \retval streamInfo  Information about the specified subtitle stream.
    
    \exception TToiSessionViolationException
    If the player is in \ref STATE_IDLE a session violation
    exception is raised.
    
    \exception TToiInvalidArgumentException
    If the streamId given as argument does not identify a
    subtitle stream in the current stream multiplex, an
    invalid argument exception is raised.
    
    \exception TToiUnavailableException
    \ref TToiUnavailableException is raised if the player is
    temporarily unavailable due to platform resource
    management.
    """
    __request = BinIpcParameterStack()
    # Write mediaComponent
    __request.push_int32(mediaComponent["Type"])
    __request.push_int32(mediaComponent["Id"])
    __response = self.call(__request, "IToiMediaPlayerBase", "GetSubtitleStreamInfo")
    __result = []
    # Read streamInfo
    streamInfo = {}
    streamInfo["RecordDmfMode"] = __response.pop_int32()
    streamInfo["ReceiveDmfMode"] = __response.pop_int32()
    streamInfo["AspectRatio"] = __response.pop_int32()
    streamInfo["Format"] = __response.pop_int32()
    streamInfo["Type"] = __response.pop_int32()
    streamInfo["Language"] = __response.pop_string()
    __result.append(streamInfo)
    return __result[0]

  def GetTeletextStreamInfo(self, mediaComponent):
    """Returns information about a specified teletext stream in the
    current media stream multiplex in the form of a teletext stream
    info structure.
    
    \param mediaComponent     The media component entry for which the stream
    info has to be retrieved.
    
    \retval streamInfo  Information about the specified teletext stream.
    
    \exception TToiSessionViolationException
    If the player is in \ref STATE_IDLE a session violation
    exception is raised.
    
    \exception TToiInvalidArgumentException
    If the mediaComponent given as argument does not identify a
    teletext stream in the current stream multiplex, an
    invalid argument exception is raised.
    
    \exception TToiUnavailableException
    \ref TToiUnavailableException is raised if the player is
    temporarily unavailable due to platform resource
    management.
    """
    __request = BinIpcParameterStack()
    # Write mediaComponent
    __request.push_int32(mediaComponent["Type"])
    __request.push_int32(mediaComponent["Id"])
    __response = self.call(__request, "IToiMediaPlayerBase", "GetTeletextStreamInfo")
    __result = []
    # Read streamInfo
    streamInfo = {}
    streamInfo["Language"] = __response.pop_string()
    __result.append(streamInfo)
    return __result[0]

  def GetCapabilities(self):
    """Returns the capabilities present for the current session.
    
    \retval capabilityMask
    A bit mask describing the capabilities of the current
    session. The mask can include the following capabilities:
    - Bit 0: \ref CAPABILITY_PAUSE
    - Bit 1: \ref CAPABILITY_REWIND
    - Bit 2: \ref CAPABILITY_FASTFORWARD
    - Bit 3: \ref CAPABILITY_GETPOSITION
    - Bit 4: \ref CAPABILITY_SETPOSITION
    - Bit 5: \ref CAPABILITY_FREEZE
    - Bit 6: \ref CAPABILITY_TIMESHIFT
    
    \exception TToiSessionViolationException
    If the player is in \ref STATE_IDLE a session violation
    exception is raised.
    
    \exception TToiUnavailableException
    \ref TToiUnavailableException is raised if the player is
    temporarily unavailable due to platform resource
    management.
    """
    __request = BinIpcParameterStack()
    __response = self.call(__request, "IToiMediaPlayerBase", "GetCapabilities")
    __result = []
    # Read capabilityMask
    __result.append(__response.pop_int32())
    return __result[0]

  def GetPosition(self):
    """Returns the current position in the media stream.
    
    \retval position  The current position in milliseconds. If asset
    start is 0, relative position from start is
    equal to current position. Otherwise, relative
    position from start is current position -
    asset start.
    
    \exception TToiSessionViolationException
    If the player is in \ref STATE_IDLE a session violation
    exception is raised.
    
    \exception TToiOperationNotSupportedException
    If the session lacks the capability to get position, the
    operation not supported exception is raised.
    
    \exception TToiUnavailableException
    \ref TToiUnavailableException is raised if the player is
    temporarily unavailable due to platform resource
    management.
    """
    __request = BinIpcParameterStack()
    __response = self.call(__request, "IToiMediaPlayerBase", "GetPosition")
    __result = []
    # Read position
    __result.append(__response.pop_int32())
    return __result[0]

  def GetPace(self):
    """Returns the current pace that the media stream is played with.
    
    \retval pace      The pace at which the position changes, given
    as a multiplier of normal playback times
    1000. If the stream is paused the pace is
    zero. When the stream is played at normal
    speed the pace is 1000. In
    \ref STATE_FASTFORWARDING the pace is generally
    above 1000, and in \ref STATE_REWINDING generally
    below -1000. For example, if the stream is
    rewinded at a pace three times normal
    playback, the pace is -3000.
    
    \exception TToiSessionViolationException
    If the player is in \ref STATE_IDLE a session violation
    exception is raised.
    
    \exception TToiOperationNotSupportedException
    If the session lacks the capability to get position, the
    operation not supported exception is raised.
    
    \exception TToiUnavailableException
    \ref TToiUnavailableException is raised if the player is
    temporarily unavailable due to platform resource
    management.
    """
    __request = BinIpcParameterStack()
    __response = self.call(__request, "IToiMediaPlayerBase", "GetPace")
    __result = []
    # Read pace
    __result.append(__response.pop_int32())
    return __result[0]

  def IsTimeshiftPlayback(self):
    """Returns true if the player is playing from the timeshift buffer.
    
    \retval timeshiftPlayback True if the player is playing from the
    timeshift buffer, otherwise false.
    
    \exception TToiSessionViolationException
    If the player is in \ref STATE_IDLE a session violation
    exception is raised.
    
    \exception TToiUnavailableException
    Raised if the player is temporarily unavailable due to
    platform resource management.
    """
    __request = BinIpcParameterStack()
    __response = self.call(__request, "IToiMediaPlayerBase", "IsTimeshiftPlayback")
    __result = []
    # Read timeshiftPlayback
    __result.append(__response.pop_bool())
    return __result[0]

  def GetTimeshiftInfo(self):
    """Retrieve information about the current timeshift operation.
    
    \retval timeshiftInfo Information about the current timeshift
    operation.
    
    \exception TToiSessionViolationException
    If the player is in \ref STATE_IDLE a session violation
    exception is raised.
    
    \exception TToiUnavailableException
    Raised if the player is temporarily unavailable due to
    platform resource management.
    """
    __request = BinIpcParameterStack()
    __response = self.call(__request, "IToiMediaPlayerBase", "GetTimeshiftInfo")
    __result = []
    # Read timeshiftInfo
    timeshiftInfo = {}
    timeshiftInfo["AssetId"] = __response.pop_string()
    timeshiftInfo["RecorderSessionId"] = __response.pop_int32()
    timeshiftInfo["BufferStartPosition"] = __response.pop_int32()
    timeshiftInfo["BufferUsed"] = __response.pop_int32()
    timeshiftInfo["BufferCapacity"] = __response.pop_int32()
    timeshiftInfo["State"] = __response.pop_int32()
    __result.append(timeshiftInfo)
    return __result[0]

  def GetTimeshiftError(self):
    """Returns the last timeshift error. This is typically called after a state
    change to \ref TIMESHIFT_STATE_BUFFERING_FAILED. The result is undefined
    if no error has occurred.
    
    \retval timeshiftError The error.
    """
    __request = BinIpcParameterStack()
    __response = self.call(__request, "IToiMediaPlayerBase", "GetTimeshiftError")
    __result = []
    # Read timeshiftError
    timeshiftError = {}
    timeshiftError["Details"] = __response.pop_int32()
    timeshiftError["Source"] = __response.pop_int32()
    __result.append(timeshiftError)
    return __result[0]

  def GetActiveComponents(self):
    """This function returns the selected media components currently
    selected.
    
    \deprecated Same information can be obtained by using GetStreamInfo().
    
    \retval components The media component entries currently selected.
    
    \exception TToiSessionViolationException
    If the player is in \ref STATE_IDLE a session violation
    exception is raised.
    
    \exception TToiUnavailableException
    Raised if the player is temporarily unavailable due to
    platform resource management.
    """
    __request = BinIpcParameterStack()
    __response = self.call(__request, "IToiMediaPlayerBase", "GetActiveComponents")
    __result = []
    # Read components
    components = []
    for k in range(__response.pop_int32()):
      temp_l = {}
      temp_l["Id"] = __response.pop_int32()
      temp_l["Type"] = __response.pop_int32()
      components.append(temp_l)
    __result.append(components)
    return __result[0]

  def GetParameter(self, name):
    """Get the current value of a parameter.
    
    \param name         Name of the requested parameter.
    
    \retval value       The current value of the parameter.
    
    \exception TToiInvalidArgumentException
    The parameter name is invalid or not
    recognized.
    
    \exception TToiUnavailableException
    Raised if the player is temporarily unavailable due to
    platform resource management.
    
    \see IToiMediaPlayer::SetParameter()
    \see IToiMediaPlayer::UnsetParameter()
    \see SubscribeParameter()
    """
    __request = BinIpcParameterStack()
    # Write name
    __request.push_string(name)
    __response = self.call(__request, "IToiMediaPlayerBase", "GetParameter")
    __result = []
    # Read value
    __result.append(__response.pop_string())
    return __result[0]

  def SubscribeParameter(self, address, name):
    """This function makes the application subscribe to updates of a
    specific named parameter. When the subscription is made there
    will be an immediate call to
    \ref IToiMediaPlayerObserver::OnParameterChanged() if there is a value
    available.
    
    \param address The observer address of the parameter that is
    subscribed.
    
    \param name The name of the parameter for which changes
    the application wants to subscribe to.
    
    \exception TToiInvalidArgumentException
    The parameter name is invalid or not
    recognized.
    
    \exception TToiUnavailableException
    Raised if the player is temporarily
    unavailable due to platform resource
    management.
    
    \see UnsubscribeParameter()
    """
    __request = BinIpcParameterStack()
    # Write address
    __request.push_string(address)
    # Write name
    __request.push_string(name)
    self.call(__request, "IToiMediaPlayerBase", "SubscribeParameter")

  def UnsubscribeParameter(self, address, name):
    """This function undoes the effect of a previous call to
    \ref SubscribeParameter().
    
    \param address The observer address of the parameter that is
    unsubscribed.
    
    \param name The name of the parameter that the
    application is no longer interested in.
    
    \exception TToiInvalidArgumentException
    The parameter name is invalid or not
    recognized.
    
    \exception TToiUnavailableException
    Raised if the player is temporarily
    unavailable due to platform resource
    management.
    
    \see SubscribeParameter()
    """
    __request = BinIpcParameterStack()
    # Write address
    __request.push_string(address)
    # Write name
    __request.push_string(name)
    self.call(__request, "IToiMediaPlayerBase", "UnsubscribeParameter")

  def ReleaseInstance(self):
    """This function releases the resources used by this instance.
    When this function returns all observers has also been released.
    """
    __request = BinIpcParameterStack()
    self.call(__request, "IToiMediaPlayerBase", "ReleaseInstance")

  def RegisterObserver(self, applicationId, address):
    """Register an observer of type IToiMediaPlayerObserver.
    The observer will receive events sent by this IToiMediaPlayerBase instance.
    
    \param applicationId  The identifier of the observing application.
    \param address        The address where the IToiMediaPlayerBase can find an
    IToiMediaPlayerObserver interface.
    \exception TToiInvalidArgumentException
    Raised if the specified address is invalid
    or if the application has already registered
    itself as an observer with the same address.
    \see IToiMediaPlayerObserver
    """
    __request = BinIpcParameterStack()
    # Write applicationId
    __request.push_int32(applicationId)
    # Write address
    __request.push_string(address)
    self.call(__request, "IToiMediaPlayerBase", "RegisterObserver")

  def ReleaseObserver(self, address):
    """Release an observer previously registered with \ref RegisterObserver.
    
    \param address        The address where the IToiMediaPlayerBase
    can find the registered IToiMediaPlayerObserver interface.
    \exception TToiInvalidArgumentException
    Raised if the specified address is invalid.
    \see IToiMediaPlayerObserver
    """
    __request = BinIpcParameterStack()
    # Write address
    __request.push_string(address)
    self.call(__request, "IToiMediaPlayerBase", "ReleaseObserver")

  def CreateObserver(self):
    """Create and register an observer for the service.
       (This method is only available from Python.)
    """
    observer = ToiMediaPlayerObserver(self._ipcproxy, self)
    request = BinIpcParameterStack()
    request.push_int32(self._ipcproxy.application_id)
    request.push_string(observer.observer_address)
    self.call(request, "IToiMediaPlayerBase", "RegisterObserver")
    return observer
