# Copyright (c) 2011 Motorola Mobility, Inc. All rights reserved.

from kreatvipcobserver import KreaTvIpcObserver
import collections
import sys

class ToiMediaPlayerObserver(KreaTvIpcObserver):
    OnStateChangedArguments = collections.namedtuple('OnStateChangedArguments', 'state reason ')
    OnDataAvailableArguments = collections.namedtuple('OnDataAvailableArguments', 'status ')
    OnPositionChangedArguments = collections.namedtuple('OnPositionChangedArguments', 'position pace ')
    OnStreamInfoChangedArguments = collections.namedtuple('OnStreamInfoChangedArguments', 'availableComponents activeComponents changedComponents ')
    OnActiveComponentChangedArguments = collections.namedtuple('OnActiveComponentChangedArguments', 'mediaComponent ')
    OnParameterChangedArguments = collections.namedtuple('OnParameterChangedArguments', 'name ')
    OnTimeshiftStateChangedArguments = collections.namedtuple('OnTimeshiftStateChangedArguments', 'state reason ')
    OnCapabilitiesChangedArguments = collections.namedtuple('OnCapabilitiesChangedArguments', 'capabilityMask ')

    def __init__(self, ipcproxy, service=None, queue=None):
        KreaTvIpcObserver.__init__(self, ipcproxy, service, queue)

    def handle_callback(self, interface_name, method_name, stack):
        """
        Pushes the call details onto the log queue.
        Called on the worker thread.
        """
        call = KreaTvIpcObserver.Call(address=self.observer_address,
                                      interface=interface_name,
                                      method=method_name)
        if method_name == 'OnStateChanged':
            args = {}
            args['reason'] = stack.pop_string()
            args['state'] = stack.pop_int32()
            call.arguments = ToiMediaPlayerObserver.OnStateChangedArguments(**args)
        if method_name == 'OnDataAvailable':
            args = {}
            args['status'] = stack.pop_bool()
            call.arguments = ToiMediaPlayerObserver.OnDataAvailableArguments(**args)
        if method_name == 'OnPositionChanged':
            args = {}
            args['pace'] = stack.pop_int32()
            args['position'] = stack.pop_int32()
            call.arguments = ToiMediaPlayerObserver.OnPositionChangedArguments(**args)
        if method_name == 'OnStreamInfoChanged':
            args = {}
            temp_changedComponents = []
            for o in range(stack.pop_int32()):
              dict_temp_changedComponents = {}
              dict_temp_changedComponents['Id'] = stack.pop_int32()
              dict_temp_changedComponents['Type'] = stack.pop_int32()
              temp_changedComponents.append(dict_temp_changedComponents)
            args['changedComponents'] = temp_changedComponents
            temp_activeComponents = []
            for o in range(stack.pop_int32()):
              dict_temp_activeComponents = {}
              dict_temp_activeComponents['Id'] = stack.pop_int32()
              dict_temp_activeComponents['Type'] = stack.pop_int32()
              temp_activeComponents.append(dict_temp_activeComponents)
            args['activeComponents'] = temp_activeComponents
            temp_availableComponents = []
            for o in range(stack.pop_int32()):
              dict_temp_availableComponents = {}
              dict_temp_availableComponents['Id'] = stack.pop_int32()
              dict_temp_availableComponents['Type'] = stack.pop_int32()
              temp_availableComponents.append(dict_temp_availableComponents)
            args['availableComponents'] = temp_availableComponents
            call.arguments = ToiMediaPlayerObserver.OnStreamInfoChangedArguments(**args)
        if method_name == 'OnActiveComponentChanged':
            args = {}
            dict_mediaComponent = {}
            dict_mediaComponent['Id'] = stack.pop_int32()
            dict_mediaComponent['Type'] = stack.pop_int32()
            args['mediaComponent'] = dict_mediaComponent
            call.arguments = ToiMediaPlayerObserver.OnActiveComponentChangedArguments(**args)
        if method_name == 'OnParameterChanged':
            args = {}
            args['name'] = stack.pop_string()
            call.arguments = ToiMediaPlayerObserver.OnParameterChangedArguments(**args)
        if method_name == 'OnTimeshiftStateChanged':
            args = {}
            args['reason'] = stack.pop_string()
            args['state'] = stack.pop_int32()
            call.arguments = ToiMediaPlayerObserver.OnTimeshiftStateChangedArguments(**args)
        if method_name == 'OnCapabilitiesChanged':
            args = {}
            args['capabilityMask'] = stack.pop_int32()
            call.arguments = ToiMediaPlayerObserver.OnCapabilitiesChangedArguments(**args)
        # Thread-safe operation:
        self._queue.append(call)

    def wait_OnStateChanged(self, discard_non_matching_events=False,
                            timeout_sec=10, **argument_filters):
        """
        Convenience function to wait for callbacks to OnStateChanged.

        Keyword arguments can be given to specify parameter values to wait for.
        If a received event does not match the filter, a KreaTvIpcException
        will be raised, unless discard_non_matching_events is true in which
        case the event will be silently ignored (and the function will continue
        to wait for a matching event).
        """
        call = self.wait_callback(self.observer_address,
                                  'IToiMediaPlayerObserver',
                                  'OnStateChanged',
                                  discard_non_matching_events,
                                  timeout_sec,
                                  argument_filters)
        return call.arguments

    def wait_OnDataAvailable(self, discard_non_matching_events=False,
                            timeout_sec=10, **argument_filters):
        """
        Convenience function to wait for callbacks to OnDataAvailable.

        Keyword arguments can be given to specify parameter values to wait for.
        If a received event does not match the filter, a KreaTvIpcException
        will be raised, unless discard_non_matching_events is true in which
        case the event will be silently ignored (and the function will continue
        to wait for a matching event).
        """
        call = self.wait_callback(self.observer_address,
                                  'IToiMediaPlayerObserver',
                                  'OnDataAvailable',
                                  discard_non_matching_events,
                                  timeout_sec,
                                  argument_filters)
        return call.arguments.status

    def wait_OnPositionChanged(self, discard_non_matching_events=False,
                            timeout_sec=10, **argument_filters):
        """
        Convenience function to wait for callbacks to OnPositionChanged.

        Keyword arguments can be given to specify parameter values to wait for.
        If a received event does not match the filter, a KreaTvIpcException
        will be raised, unless discard_non_matching_events is true in which
        case the event will be silently ignored (and the function will continue
        to wait for a matching event).
        """
        call = self.wait_callback(self.observer_address,
                                  'IToiMediaPlayerObserver',
                                  'OnPositionChanged',
                                  discard_non_matching_events,
                                  timeout_sec,
                                  argument_filters)
        return call.arguments

    def wait_OnStreamInfoChanged(self, discard_non_matching_events=False,
                            timeout_sec=10, **argument_filters):
        """
        Convenience function to wait for callbacks to OnStreamInfoChanged.

        Keyword arguments can be given to specify parameter values to wait for.
        If a received event does not match the filter, a KreaTvIpcException
        will be raised, unless discard_non_matching_events is true in which
        case the event will be silently ignored (and the function will continue
        to wait for a matching event).
        """
        call = self.wait_callback(self.observer_address,
                                  'IToiMediaPlayerObserver',
                                  'OnStreamInfoChanged',
                                  discard_non_matching_events,
                                  timeout_sec,
                                  argument_filters)
        return call.arguments

    def wait_OnActiveComponentChanged(self, discard_non_matching_events=False,
                            timeout_sec=10, **argument_filters):
        """
        Convenience function to wait for callbacks to OnActiveComponentChanged.

        Keyword arguments can be given to specify parameter values to wait for.
        If a received event does not match the filter, a KreaTvIpcException
        will be raised, unless discard_non_matching_events is true in which
        case the event will be silently ignored (and the function will continue
        to wait for a matching event).
        """
        call = self.wait_callback(self.observer_address,
                                  'IToiMediaPlayerObserver',
                                  'OnActiveComponentChanged',
                                  discard_non_matching_events,
                                  timeout_sec,
                                  argument_filters)
        return call.arguments.mediaComponent

    def wait_OnParameterChanged(self, discard_non_matching_events=False,
                            timeout_sec=10, **argument_filters):
        """
        Convenience function to wait for callbacks to OnParameterChanged.

        Keyword arguments can be given to specify parameter values to wait for.
        If a received event does not match the filter, a KreaTvIpcException
        will be raised, unless discard_non_matching_events is true in which
        case the event will be silently ignored (and the function will continue
        to wait for a matching event).
        """
        call = self.wait_callback(self.observer_address,
                                  'IToiMediaPlayerObserver',
                                  'OnParameterChanged',
                                  discard_non_matching_events,
                                  timeout_sec,
                                  argument_filters)
        return call.arguments.name

    def wait_OnTimeshiftStateChanged(self, discard_non_matching_events=False,
                            timeout_sec=10, **argument_filters):
        """
        Convenience function to wait for callbacks to OnTimeshiftStateChanged.

        Keyword arguments can be given to specify parameter values to wait for.
        If a received event does not match the filter, a KreaTvIpcException
        will be raised, unless discard_non_matching_events is true in which
        case the event will be silently ignored (and the function will continue
        to wait for a matching event).
        """
        call = self.wait_callback(self.observer_address,
                                  'IToiMediaPlayerObserver',
                                  'OnTimeshiftStateChanged',
                                  discard_non_matching_events,
                                  timeout_sec,
                                  argument_filters)
        return call.arguments

    def wait_OnCapabilitiesChanged(self, discard_non_matching_events=False,
                            timeout_sec=10, **argument_filters):
        """
        Convenience function to wait for callbacks to OnCapabilitiesChanged.

        Keyword arguments can be given to specify parameter values to wait for.
        If a received event does not match the filter, a KreaTvIpcException
        will be raised, unless discard_non_matching_events is true in which
        case the event will be silently ignored (and the function will continue
        to wait for a matching event).
        """
        call = self.wait_callback(self.observer_address,
                                  'IToiMediaPlayerObserver',
                                  'OnCapabilitiesChanged',
                                  discard_non_matching_events,
                                  timeout_sec,
                                  argument_filters)
        return call.arguments.capabilityMask

