# Copyright (c) 2011 Motorola Mobility, Inc. All rights reserved.

from kreatvipccaller import KreaTvIpcCaller
from binipcparameterstack import BinIpcParameterStack
from ToiMediaObserver import ToiMediaObserver


class ToiMediaService(KreaTvIpcCaller):

  # Const REASON_UNKNOWN
  REASON_UNKNOWN = "Unknown"

  # Const REASON_COMMAND_OPEN
  REASON_COMMAND_OPEN = "CommandOpen"

  # Const REASON_COMMAND_CLOSE
  REASON_COMMAND_CLOSE = "CommandClose"

  # Const REASON_COMMAND_PAUSE
  REASON_COMMAND_PAUSE = "CommandPause"

  # Const REASON_COMMAND_PLAY
  REASON_COMMAND_PLAY = "CommandPlay"

  # Const REASON_COMMAND_FASTFORWARD
  REASON_COMMAND_FASTFORWARD = "CommandFastForward"

  # Const REASON_COMMAND_REWIND
  REASON_COMMAND_REWIND = "CommandRewind"

  # Const REASON_COMMAND_PLAYFROMPOSITION
  REASON_COMMAND_PLAYFROMPOSITION = "CommandPlayFromPosition"

  # Const REASON_COMMAND_RECORD
  REASON_COMMAND_RECORD = "CommandRecord"

  # Const REASON_COMMAND_CONSUME
  REASON_COMMAND_CONSUME = "CommandConsume"

  # Const REASON_POSITION_END
  REASON_POSITION_END = "PositionEnd"

  # Const REASON_POSITION_START
  REASON_POSITION_START = "PositionStart"

  # Const REASON_TIMESHIFT_BUFFER_OPEN
  REASON_TIMESHIFT_BUFFER_OPEN = "TimeshiftBufferOpen"

  # Const REASON_TIMESHIFT_BUFFER_FULL
  REASON_TIMESHIFT_BUFFER_FULL = "TimeshiftBufferFull"

  # Const REASON_TIMESHIFT_BUFFER_DISCARDED
  REASON_TIMESHIFT_BUFFER_DISCARDED = "TimeshiftBufferDiscarded"

  # Const REASON_ERROR_COMMAND_NOT_SUPPORTED
  REASON_ERROR_COMMAND_NOT_SUPPORTED = "ErrorCommandNotSupported"

  # Const REASON_ERROR_STREAM_CLIENT
  REASON_ERROR_STREAM_CLIENT = "ErrorStreamClient"

  # Const REASON_ERROR_CONNECTION_FAILED
  REASON_ERROR_CONNECTION_FAILED = "ErrorConnectionFailed"

  # Const REASON_ERROR_INVALID_PRECONDITION
  REASON_ERROR_INVALID_PRECONDITION = "ErrorInvalidPrecondition"

  # Const REASON_ERROR_PERMISSION_DENIED
  REASON_ERROR_PERMISSION_DENIED = "ErrorPermissionDenied"

  # Const REASON_ERROR_INVALID_DISKASSET
  REASON_ERROR_INVALID_DISKASSET = "ErrorInvalidDiskasset"

  # Const REASON_ERROR_UNSUPPORTED_DISKASSET_VERSION
  REASON_ERROR_UNSUPPORTED_DISKASSET_VERSION = "ErrorUnsupportedDiskAssetVersion"

  # Const REASON_ERROR_DISK_FULL
  REASON_ERROR_DISK_FULL = "ErrorStorageFull"

  # Const REASON_ERROR_LOWLEVEL_IO_FAILURE
  REASON_ERROR_LOWLEVEL_IO_FAILURE = "ErrorLowlevelIOFailure"

  # Const REASON_ERROR_POSITION_INVALID
  REASON_ERROR_POSITION_INVALID = "ErrorPositionInvalid"

  # Const REASON_ERROR_DISK_OVERHEATED
  REASON_ERROR_DISK_OVERHEATED = "ErrorStorageOverheated"

  # Const REASON_ERROR_HOST_UNREACHABLE
  REASON_ERROR_HOST_UNREACHABLE = "ErrorHostUnreachable"

  # Const REASON_ERROR_UNEXISTING_RESOURCE
  REASON_ERROR_UNEXISTING_RESOURCE = "ErrorUnexistingResource"

  # Const REASON_ERROR_NO_EXTERNAL_INPUT_SIGNAL
  REASON_ERROR_NO_EXTERNAL_INPUT_SIGNAL = "ErrorNoExternalInputSignal"

  def __init__(self, ipcproxy):
    KreaTvIpcCaller.__init__(self, ipcproxy.ip_address, ipcproxy.port)
    self._ipcproxy = ipcproxy
    self.lookup("MediaService")

  def CreatePlayerInstance(self, applicationId):
    """This function creates a new media player and associates it with
    the calling application.
    
    When the application no longer needs the media player, it must
    release it by calling \ref IToiMediaPlayer::ReleaseInstance(). This
    will return the underlying resources to the system.
    
    \param applicationId
    The identifier of the calling application.
    
    \retval playerAddress
    The address to the \ref IToiMediaPlayer instance.
    
    \exception TToiInvalidArgumentException
    Raised if the specified application identifier is
    invalid.
    
    \exception TToiNotEnoughResourcesException
    Raised if a new player instance cannot be
    created due to lack of system resources.
    
    \exception TToiPermissionDeniedException
    Raised if the calling application is not allowed
    to create instances of the media player. This
    restriction applies to popup applications.
    
    \see IToiMediaPlayer
    \see IToiMediaPlayer::ReleaseInstance()
    
    """
    __request = BinIpcParameterStack()
    # Write applicationId
    __request.push_int32(applicationId)
    __response = self.call(__request, "IToiMediaService", "CreatePlayerInstance")
    __result = []
    # Read playerAddress
    __result.append(__response.pop_string())
    return __result[0]

  def CreatePipPlayerInstance(self, applicationId):
    """This function creates a new Picture in Picture media player and
    associates it with the calling application.
    
    When the application no longer needs the media player, it must release
    it by calling \ref IToiMediaPlayer::ReleaseInstance(). This will
    return the underlying resources to the system.
    
    \param applicationId
    The identifier of the calling application.
    
    \retval pipAddress
    The address to the \ref IToiMediaPlayer instance.
    
    \exception TToiInvalidArgumentException
    Raised if the specified application identifier is
    invalid.
    
    \exception TToiOperationNotSupportedException
    Raised if a new player instance cannot be
    created.
    
    \exception TToiPermissionDeniedException
    Raised if the calling application is not allowed
    to create instances of the media player. This
    restriction applies to popup applications.
    
    \see IToiMediaPlayer
    \see IToiMediaPlayer::ReleaseInstance()
    
    """
    __request = BinIpcParameterStack()
    # Write applicationId
    __request.push_int32(applicationId)
    __response = self.call(__request, "IToiMediaService", "CreatePipPlayerInstance")
    __result = []
    # Read pipAddress
    __result.append(__response.pop_string())
    return __result[0]

  def CreateRecorderInstance(self, applicationId):
    """This function creates a new media recorder and associates it
    with the calling application.
    
    When the application no longer needs the media recorder, it must
    release it by calling \ref IToiMediaRecorder::ReleaseInstance().
    This will return the underlying resources to the system.
    
    \param applicationId
    The identifier of the calling application.
    
    \retval recorderAddress
    The address to the \ref IToiMediaRecorder instance
    
    \exception TToiInvalidArgumentException
    Raised if the specified application identifier
    is invalid.
    
    \exception TToiNotEnoughResourcesException
    Raised if a new recorder instance cannot be
    created due to lack of system resources.
    
    \exception TToiOperationNotSupportedException
    Raised if recording is not supported on this
    system.
    
    \see IToiMediaRecorder
    \see IToiMediaRecorder::ReleaseInstance()
    
    """
    __request = BinIpcParameterStack()
    # Write applicationId
    __request.push_int32(applicationId)
    __response = self.call(__request, "IToiMediaService", "CreateRecorderInstance")
    __result = []
    # Read recorderAddress
    __result.append(__response.pop_string())
    return __result[0]

  def CreateConsumerInstance(self, applicationId):
    """
    \techpreview This method is a TECHNICAL PREVIEW, it may change.
    
    This function creates a new media consumer. A media consumer can
    be used to fetch data from a media stream without allocating
    media decoder and recorder resources. This may be useful for
    applications that wants to filter out stream data without
    actually playing the stream.
    
    When the application no longer needs the media consumer, it
    must release it by calling \ref IToiMediaConsumer::ReleaseInstance().
    This will return the underlying resources to the system.
    
    \param applicationId
    The identifier of the calling application.
    
    \retval consumerAddress
    The address to the \ref IToiMediaConsumer instance
    
    \exception TToiInvalidArgumentException
    Raised if the specified application identifier
    is invalid.
    
    \exception TToiNotEnoughResourcesException
    Raised if a new consumer instance cannot be
    created due to lack of system resources.
    
    \see IToiMediaConsumer
    \see IToiMediaConsumer::ReleaseInstance()
    """
    __request = BinIpcParameterStack()
    # Write applicationId
    __request.push_int32(applicationId)
    __response = self.call(__request, "IToiMediaService", "CreateConsumerInstance")
    __result = []
    # Read consumerAddress
    __result.append(__response.pop_string())
    return __result[0]

  def EnumerateSessions(self, type):
    """This function is used to get a list of all sessions of a certain
    type. Note that a session may be of more than one type.
    
    \param type       Only return sessions with this type.
    
    \retval sessions  A list of matching media sessions.
    *
    """
    __request = BinIpcParameterStack()
    # Write type
    __request.push_int32(type)
    __response = self.call(__request, "IToiMediaService", "EnumerateSessions")
    __result = []
    # Read sessions
    sessions = []
    for k in range(__response.pop_int32()):
      temp_l = {}
      temp_l["State"] = __response.pop_int32()
      temp_l["Url"] = __response.pop_string()
      temp_l["Type"] = __response.pop_int32()
      temp_l["Id"] = __response.pop_int32()
      sessions.append(temp_l)
    __result.append(sessions)
    return __result[0]

  def OpenSession(self, applicationId, sessionId):
    """Open an existing session. Each call to this method returns the
    address of a unique object that must be released when no longer
    needed, by calling \ref IToiMediaSession::ReleaseInstance().
    
    Note that the actual session does not necessarily have to be
    owned by the application calling this function. It may still be
    used by the owner to access the functionality of the
    \ref IToiMediaSession interface.
    
    \param applicationId
    The identifier of the calling application.
    
    \param sessionId  The identifier of the session.
    
    \retval sessionAddress
    The address of the new session object which is
    mapped to the requested session. The session
    object implements the \ref IToiMediaSession
    interface.
    
    \exception TToiInvalidArgumentException
    Raised if the specified application identifier
    is invalid.
    
    \exception TToiInvalidArgumentException
    Raised if the session identifier does not
    identify an existing session. Note that all
    applications must be designed to handle this
    exception since sessions can be closed at any
    time, rendering the session identifier invalid.
    
    \see IToiMediaSession
    """
    __request = BinIpcParameterStack()
    # Write applicationId
    __request.push_int32(applicationId)
    # Write sessionId
    __request.push_int32(sessionId)
    __response = self.call(__request, "IToiMediaService", "OpenSession")
    __result = []
    # Read sessionAddress
    __result.append(__response.pop_string())
    return __result[0]

  def RegisterObserver(self, applicationId, address):
    """Register an observer of type IToiMediaObserver.
    The observer will receive events sent by this IToiMediaService instance.
    
    \param applicationId  The identifier of the observing application.
    \param address        The address where the IToiMediaService can find an
    IToiMediaObserver interface.
    \exception TToiInvalidArgumentException
    Raised if the specified address is invalid
    or if the application has already registered
    itself as an observer with the same address.
    \see IToiMediaObserver
    """
    __request = BinIpcParameterStack()
    # Write applicationId
    __request.push_int32(applicationId)
    # Write address
    __request.push_string(address)
    self.call(__request, "IToiMediaService", "RegisterObserver")

  def ReleaseObserver(self, address):
    """Release an observer previously registered with \ref RegisterObserver.
    
    \param address        The address where the IToiMediaService
    can find the registered IToiMediaObserver interface.
    \exception TToiInvalidArgumentException
    Raised if the specified address is invalid.
    \see IToiMediaObserver
    """
    __request = BinIpcParameterStack()
    # Write address
    __request.push_string(address)
    self.call(__request, "IToiMediaService", "ReleaseObserver")

  def CreateObserver(self):
    """Create and register an observer for the service.
       (This method is only available from Python.)
    """
    observer = ToiMediaObserver(self._ipcproxy, self)
    request = BinIpcParameterStack()
    request.push_int32(self._ipcproxy.application_id)
    request.push_string(observer.observer_address)
    self.call(request, "IToiMediaService", "RegisterObserver")
    return observer
