# Copyright (c) 2011 Motorola Mobility, Inc. All rights reserved.

from kreatvipccaller import KreaTvIpcCaller
from binipcparameterstack import BinIpcParameterStack
from ToiMediaSessionObserver import ToiMediaSessionObserver


class ToiMediaSession(KreaTvIpcCaller):

  # Enum TToiState
  STATE_CONNECTING = 0
  STATE_RUNNING = 1
  STATE_FAILED = 2
  STATE_CLOSED = 3

  # Enum TToiCipher
  CIPHER_NONE = 0
  CIPHER_DVB_CSA = 1
  CIPHER_ARIB_MULTI2 = 2
  CIPHER_AES_128 = 3
  CIPHER_TRIPLE_DES = 4

  # Enum TToiCipherBlockMode
  CIPHER_BLOCK_MODE_NONE = 0
  CIPHER_BLOCK_MODE_ECB = 1
  CIPHER_BLOCK_MODE_CBC = 2
  CIPHER_BLOCK_MODE_CFB = 3
  CIPHER_BLOCK_MODE_OFB = 4
  CIPHER_BLOCK_MODE_CTR = 5

  # Enum TToiCipherResidueMode
  CIPHER_RESIDUE_MODE_NONE = 0
  CIPHER_RESIDUE_MODE_FRONT = 1
  CIPHER_RESIDUE_MODE_BACK = 2
  CIPHER_RESIDUE_MODE_CTS = 3
  CIPHER_RESIDUE_MODE_OFB = 4

  # Const TYPE_PLAYER
  TYPE_PLAYER = 1

  # Const TYPE_RECORDER
  TYPE_RECORDER = 2

  # Const TYPE_CONSUMER
  TYPE_CONSUMER = 4

  # Const TYPE_DISTRIBUTOR
  TYPE_DISTRIBUTOR = 8

  def __init__(self, ipcproxy, object_address):
    KreaTvIpcCaller.__init__(self, ipcproxy.ip_address, ipcproxy.port)
    self._ipcproxy = ipcproxy
    self.set_object_address(object_address)

  def GetId(self):
    """Get the session identifier.
    
    \retval id        The session identifier.
    """
    __request = BinIpcParameterStack()
    __response = self.call(__request, "IToiMediaSession", "GetId")
    __result = []
    # Read id
    __result.append(__response.pop_int32())
    return __result[0]

  def GetType(self):
    """Get the session type bitmask. Note that a session can combine
    the different type constants.
    
    \retval type      The type of the session, e.g. \ref TYPE_PLAYER.
    This field is a bitmask.
    
    \exception TToiSessionViolationException
    If the session no longer exist an session violation
    exception is raised.
    """
    __request = BinIpcParameterStack()
    __response = self.call(__request, "IToiMediaSession", "GetType")
    __result = []
    # Read type
    __result.append(__response.pop_int32())
    return __result[0]

  def GetUrl(self):
    """Get the source URL of the session.
    
    \retval url       The source URL of the session.
    
    \exception TToiSessionViolationException
    If the session no longer exist an session violation
    exception is raised.
    """
    __request = BinIpcParameterStack()
    __response = self.call(__request, "IToiMediaSession", "GetUrl")
    __result = []
    # Read url
    __result.append(__response.pop_string())
    return __result[0]

  def GetState(self):
    """Returns the current state of the media session.
    
    \retval state     The current session state.
    
    \exception TToiSessionViolationException
    If the session no longer exist an session violation
    exception is raised.
    """
    __request = BinIpcParameterStack()
    __response = self.call(__request, "IToiMediaSession", "GetState")
    __result = []
    # Read state
    __result.append(__response.pop_int32())
    return __result[0]

  def GetStreamInfo(self):
    """Returns information about the stream handled by the session.
    
    \retval info      Information about the stream.
    
    \exception TToiNoDataException
    If no information is yet available an no data
    exception is raised.
    
    \exception TToiSessionViolationException
    If the session no longer exist an session violation
    exception is raised.
    """
    __request = BinIpcParameterStack()
    __response = self.call(__request, "IToiMediaSession", "GetStreamInfo")
    __result = []
    # Read info
    info = {}
    info["ProgramNumber"] = __response.pop_int32()
    __result.append(info)
    return __result[0]

  def GetAssetId(self):
    """Returns the asset identifier used for recording.
    
    \deprecated This method is not reliable since several assets may be
    used simultaneously in the same session.
    
    \retval assetId   The asset identifier.
    
    \exception TToiOperationNotSupportedException
    If the stream is not being recorded an operation
    is not supported exception is raised.
    
    \exception TToiSessionViolationException
    If the session no longer exist an session violation
    exception is raised.
    """
    __request = BinIpcParameterStack()
    __response = self.call(__request, "IToiMediaSession", "GetAssetId")
    __result = []
    # Read assetId
    __result.append(__response.pop_string())
    return __result[0]

  def AddSectionDataRelay(self, pid, data, include, exclude, length, socketName):
    """Extract section data from the stream and send to a Unix domain datagram
    socket. The data to extract is identified by the pid and the filter.
    
    The first byte in the filter correspond to the first byte in the section.
    The second filter byte correspond to the fourth byte in the section,
    as the second and third section bytes contain the section length
    and the filtering mechanism skips these.
    
    Inclusion mask:
    For all inclusion mask bits that are set to 1 all of the corresponding
    bits in the section must match the bits specified in the data coefficient
    in order to be included.
    
    Exclusion mask:
    For all exclusion mask bits that are set to 1 at least one of the
    corresponding bits in the section must not match the bits specified
    by the data coefficient in order to be included.
    
    Results from both comparisons are AND'ed together to form the final
    pass decision.
    
    \param pid        The transport stream packet identifier.
    
    \param data       This is the data for which the section data will
    be compared against. Note that the include
    and exclude masks are part of the filter.
    
    \param include    This is the include mask. Can be left empty to
    include all filter data. If not empty the length
    must match the data length.
    
    \param exclude    This is the exclude mask. Can be left empty for no
    exclusion. If not empty the length must match
    the data length.
    
    \param length     The length of the data, include and exclude vectors.
    
    \param socketName The name of a Unix domain datagram socket to
    which filtered data should be sent.
    
    \retval dataRelayId
    The identifier of the created data relay.
    
    \exception TToiInvalidArgumentException
    If the pid is out of range,
    if the filter does not apply to the current
    stream format, or if the length is too large
    an invalid argument exception is raised.
    It is also raised if an identical relay
    already has been setup for the same socket.
    
    \exception TToiNotEnoughResourcesException
    If too many relays have been setup an not enough
    resources exception is raised.
    
    \exception TToiSessionViolationException
    If the session no longer exist an session violation
    exception is raised.
    
    \see RemoveDataRelay()
    
    """
    __request = BinIpcParameterStack()
    # Write pid
    __request.push_int32(pid)
    # Write data
    assert len(data) == length
    __request.push_binary(data)
    __request.push_int32(len(data))
    # Write include
    assert len(include) == length
    __request.push_binary(include)
    __request.push_int32(len(include))
    # Write exclude
    assert len(exclude) == length
    __request.push_binary(exclude)
    __request.push_int32(len(exclude))
    # Write length
    __request.push_int32(length)
    # Write socketName
    __request.push_string(socketName)
    __response = self.call(__request, "IToiMediaSession", "AddSectionDataRelay")
    __result = []
    # Read dataRelayId
    __result.append(__response.pop_int32())
    return __result[0]

  def AddPacketDataRelay(self, pid, socketName):
    """Extract packet data from the stream and send to a Unix domain datagram
    socket. The data to extract is identified by the pid.
    
    \param pid        The transport stream packet identifier.
    
    \param socketName The name of a Unix domain datagram socket to
    which filtered data should be sent.
    
    \retval dataRelayId
    The identifier of the created data relay.
    
    \exception TInvalidArgumentException
    If the pid is out of range or if it the filter
    does not apply to the current stream format an
    invalid argument exception is raised.
    It is also raised if an identical relay
    already has been setup for the same socket.
    
    \exception TNotEnoughResourcesException
    If too many relays have been setup an not enough
    resources exception is raised.
    
    \exception TToiSessionViolationException
    If the session no longer exist an session violation
    exception is raised.
    
    \see RemoveDataRelay()
    """
    __request = BinIpcParameterStack()
    # Write pid
    __request.push_int32(pid)
    # Write socketName
    __request.push_string(socketName)
    __response = self.call(__request, "IToiMediaSession", "AddPacketDataRelay")
    __result = []
    # Read dataRelayId
    __result.append(__response.pop_int32())
    return __result[0]

  def RemoveDataRelay(self, dataRelayId):
    """Remove a previously added data relay.
    
    \param dataRelayId  The identifier of the data relay to be removed.
    
    \exception TToiInvalidArgumentException
    If the data relay identifier does not exist an invalid
    argument exception is raised.
    
    \exception TToiSessionViolationException
    If the session no longer exist an session violation
    exception is raised.
    
    \see AddPacketDataRelay()
    \see AddSectionDataRelay()
    """
    __request = BinIpcParameterStack()
    # Write dataRelayId
    __request.push_int32(dataRelayId)
    self.call(__request, "IToiMediaSession", "RemoveDataRelay")

  def GetParameter(self, name):
    """Get the current value of a parameter.
    
    \param name       Name of the requested parameter.
    
    \retval value     The value of the parameter.
    
    \exception TToiInvalidArgumentException
    If no parameter with this name can be found an
    invalid argument exception is raised.
    
    \exception TToiSessionViolationException
    If the session no longer exist an session violation
    exception is raised.
    """
    __request = BinIpcParameterStack()
    # Write name
    __request.push_string(name)
    __response = self.call(__request, "IToiMediaSession", "GetParameter")
    __result = []
    # Read value
    __result.append(__response.pop_string())
    return __result[0]

  def SetParameter(self, name, value):
    """Set the current value of a parameter.
    
    \param name       Name of the  parameter.
    
    \retval value     The value to be set to parameter.
    
    \exception TToiInvalidArgumentException
    If no parameter with this name can be found an
    invalid argument exception is raised.
    
    \exception TToiSessionViolationException
    If the session no longer exist an session violation
    exception is raised.
    """
    __request = BinIpcParameterStack()
    # Write name
    __request.push_string(name)
    # Write value
    __request.push_string(value)
    self.call(__request, "IToiMediaSession", "SetParameter")

  def SubscribeParameter(self, address, name):
    """This function makes the application subscribe to updates of a
    specific named parameter. When the subscription is made there
    will be an immediate call to
    \ref IToiMediaSessionObserver::OnParameterChanged() if there is a
    value available.
    
    \param address    The observer address of the parameter that is
    subscribed.
    
    \param name       The name of the subscribed parameter.
    
    \exception TToiInvalidArgumentException
    If the parameter name is invalid or not recognized
    an invalid argument exception is raised.
    
    \exception TToiSessionViolationException
    If the session no longer exist an session violation
    exception is raised.
    
    \see UnsubscribeParameter()
    
    """
    __request = BinIpcParameterStack()
    # Write address
    __request.push_string(address)
    # Write name
    __request.push_string(name)
    self.call(__request, "IToiMediaSession", "SubscribeParameter")

  def UnsubscribeParameter(self, address, name):
    """This function undoes the effect of a previous call to
    \ref SubscribeParameter().
    
    \param address    The observer address of the parameter that is
    unsubscribed.
    
    \param name       The name of the unsubscribed parameter.
    
    \exception TToiInvalidArgumentException
    If the parameter name is invalid or not recognized
    an invalid argument exception is raised.
    
    \exception TToiSessionViolationException
    If the session no longer exist an session violation
    exception is raised.
    
    \see SubscribeParameter()
    
    """
    __request = BinIpcParameterStack()
    # Write address
    __request.push_string(address)
    # Write name
    __request.push_string(name)
    self.call(__request, "IToiMediaSession", "UnsubscribeParameter")

  def SetDecryptionCipher(self, streamIds, cipher, blockMode, residueMode):
    """Set cipher mode on stream identifiers to prepare them for use
    of SetDecryptionKeys().
    
    If a license is no longer valid it is possible to immediately
    disable decryption by setting the cipher to CIPHER_NONE.
    
    \param streamIds  The identifiers of the stream to be decrypted
    with the specified cipher.
    
    \param cipher     The cipher to use.
    
    \param blockMode  The cipher block mode to use.
    
    \param residueMode
    The cipher residue mode to use.
    
    \exception TToiOperationNotSupportedException
    If the requested cipher is not supported
    by the implementation an operation not supported
    exception is raised.
    
    \exception TToiSessionViolationException
    If the session no longer exist an session violation
    exception is raised.
    
    \see SetDecryptionKeys()
    """
    __request = BinIpcParameterStack()
    # Write streamIds
    for iter_k in streamIds[::-1]:
      __request.push_int32(iter_k)
    __request.push_int32(len(streamIds))
    # Write cipher
    __request.push_int32(cipher)
    # Write blockMode
    __request.push_int32(blockMode)
    # Write residueMode
    __request.push_int32(residueMode)
    self.call(__request, "IToiMediaSession", "SetDecryptionCipher")

  def SetDecryptionKeys(self, streamIds, isEncrypted, evenKey, evenIv, oddKey, oddIv, length):
    """Set keys to be used for stream decryption.
    Before calling this method the cipher mode must be set with
    SetDecryptionCipher().
    
    \param streamIds  The identifiers of the stream to be decrypted.
    
    \param isEncrypted
    Flag indicating whether the keys passed to this
    function are encrypted.
    
    \param evenKey    The even key. Leave empty if only odd key is set.
    
    \param evenIv     The even initialization vector. Leave empty to use
    an IV with just zeroes, also leave empty if IV is not
    applicable.
    
    \param oddKey     The odd key. Leave empty if only even key is set.
    
    \param oddIv      The odd initialization vector. Leave empty to use
    an IV with just zeroes, also leave empty if IV is not
    applicable.
    
    \param length     The length of the key and IV.
    
    \exception TToiOperationNotSupportedException
    If the length of the keys are not consistent
    with the selected cipher mode an operation
    is not supported exception is raised.
    
    \exception TToiSessionViolationException
    If the session no longer exist an session violation
    exception is raised.
    
    \see SetDecryptionCipher()
    
    
    """
    __request = BinIpcParameterStack()
    # Write streamIds
    for iter_k in streamIds[::-1]:
      __request.push_int32(iter_k)
    __request.push_int32(len(streamIds))
    # Write isEncrypted
    __request.push_bool(isEncrypted)
    # Write evenKey
    assert len(evenKey) == length
    __request.push_binary(evenKey)
    __request.push_int32(len(evenKey))
    # Write evenIv
    assert len(evenIv) == length
    __request.push_binary(evenIv)
    __request.push_int32(len(evenIv))
    # Write oddKey
    assert len(oddKey) == length
    __request.push_binary(oddKey)
    __request.push_int32(len(oddKey))
    # Write oddIv
    assert len(oddIv) == length
    __request.push_binary(oddIv)
    __request.push_int32(len(oddIv))
    # Write length
    __request.push_int32(length)
    self.call(__request, "IToiMediaSession", "SetDecryptionKeys")

  def ReleaseInstance(self):
    """This method releases the resources used by this object and
    removes all registered observers. Upon return this object will
    no longer be available and must not be called.
    
    Note that this method does not close the associated
    session. This can only be done by calling
    \ref IToiMediaPlayer::Close() or \ref IToiMediaRecorder::Close().
    """
    __request = BinIpcParameterStack()
    self.call(__request, "IToiMediaSession", "ReleaseInstance")

  def RegisterObserver(self, applicationId, address):
    """Register an observer of type IToiMediaSessionObserver.
    The observer will receive events sent by this IToiMediaSession instance.
    
    \param applicationId  The identifier of the observing application.
    \param address        The address where the IToiMediaSession can find an
    IToiMediaSessionObserver interface.
    \exception TToiInvalidArgumentException
    Raised if the specified address is invalid
    or if the application has already registered
    itself as an observer with the same address.
    \see IToiMediaSessionObserver
    """
    __request = BinIpcParameterStack()
    # Write applicationId
    __request.push_int32(applicationId)
    # Write address
    __request.push_string(address)
    self.call(__request, "IToiMediaSession", "RegisterObserver")

  def ReleaseObserver(self, address):
    """Release an observer previously registered with \ref RegisterObserver.
    
    \param address        The address where the IToiMediaSession
    can find the registered IToiMediaSessionObserver interface.
    \exception TToiInvalidArgumentException
    Raised if the specified address is invalid.
    \see IToiMediaSessionObserver
    """
    __request = BinIpcParameterStack()
    # Write address
    __request.push_string(address)
    self.call(__request, "IToiMediaSession", "ReleaseObserver")

  def CreateObserver(self):
    """Create and register an observer for the service.
       (This method is only available from Python.)
    """
    observer = ToiMediaSessionObserver(self._ipcproxy, self)
    request = BinIpcParameterStack()
    request.push_int32(self._ipcproxy.application_id)
    request.push_string(observer.observer_address)
    self.call(request, "IToiMediaSession", "RegisterObserver")
    return observer
