# Copyright (c) 2011 Motorola Mobility, Inc. All rights reserved.

from kreatvipccaller import KreaTvIpcCaller
from binipcparameterstack import BinIpcParameterStack
from ToiOperationObserver import ToiOperationObserver


class ToiOperationManager(KreaTvIpcCaller):

  # Enum TToiOperationState
  OPERATION_PENDING = 0
  OPERATION_COMPLETED = 1
  OPERATION_FAILED = 2

  # Const OP_ERROR_FAILED
  OP_ERROR_FAILED = -1

  # Const OP_ERROR_CANCELLED
  OP_ERROR_CANCELLED = -2

  # Const OP_RESULT_NONE
  OP_RESULT_NONE = 0

  def __init__(self, ipcproxy, object_address):
    KreaTvIpcCaller.__init__(self, ipcproxy.ip_address, ipcproxy.port)
    self._ipcproxy = ipcproxy
    self.set_object_address(object_address)

  def CreateOperation(self, applicationId, userData):
    __request = BinIpcParameterStack()
    # Write applicationId
    __request.push_int32(applicationId)
    # Write userData
    __request.push_string(userData)
    __response = self.call(__request, "IToiOperationManager", "CreateOperation")
    __result = []
    # Read operationId
    __result.append(__response.pop_int32())
    return __result[0]

  def CancelOperation(self, operationId):
    """
    Requests an operation to stop execution.
    
    \param operationId      Operation identifier.
    
    \exception TToiInvalidArgumentException
    Raised if any of the parameters to the call are
    invalid.
    """
    __request = BinIpcParameterStack()
    # Write operationId
    __request.push_int32(operationId)
    self.call(__request, "IToiOperationManager", "CancelOperation")

  def ReleaseOperation(self, operationId):
    """
    Releases any system resources associated with the operation and if
    needed cancels the operation as well. This method shall be called when
    the application is done with the operation and will use it no more.
    
    \param operationId      Operation identifier.
    
    \exception TToiInvalidArgumentException
    Raised if any of the parameters to the call are
    invalid.
    """
    __request = BinIpcParameterStack()
    # Write operationId
    __request.push_int32(operationId)
    self.call(__request, "IToiOperationManager", "ReleaseOperation")

  def GetOperationInfo(self, operationId):
    """
    Returns information about given operation. This method is
    normally not useful for an application that is listening to
    operation events since the same information is provided in the
    callback.
    
    In some circumstances it may still be relevant to use this method
    and that is when the application is using a polled approach to
    handle operation results instead of callbacks. But also if the
    application do not handle an incoming callback directly and want
    to get the latest updated operation information.
    
    \param operationId     Operation identifier.
    
    \retval operationInfo  The current operation information.
    
    \exception TToiInvalidArgumentException
    Raised if any of the parameters to the call are
    """
    __request = BinIpcParameterStack()
    # Write operationId
    __request.push_int32(operationId)
    __response = self.call(__request, "IToiOperationManager", "GetOperationInfo")
    __result = []
    # Read operationInfo
    operationInfo = {}
    operationInfo["Properties"] = []
    for k in range(__response.pop_int32()):
      temp_l = {}
      temp_l["Value"] = __response.pop_string()
      temp_l["Id"] = __response.pop_int32()
      operationInfo["Properties"].append(temp_l)
    operationInfo["ErrorCode"] = __response.pop_int32()
    operationInfo["Progress"] = __response.pop_int32()
    operationInfo["State"] = __response.pop_int32()
    operationInfo["UserData"] = __response.pop_string()
    operationInfo["Id"] = __response.pop_int32()
    __result.append(operationInfo)
    return __result[0]

  def RegisterObserver(self, applicationId, address):
    """Register an observer of type IToiOperationObserver.
    The observer will receive events sent by this IToiOperationManager instance.
    
    \param applicationId  The identifier of the observing application.
    \param address        The address where the IToiOperationManager can find an
    IToiOperationObserver interface.
    \exception TToiInvalidArgumentException
    Raised if the specified address is invalid
    or if the application has already registered
    itself as an observer with the same address.
    \see IToiOperationObserver
    """
    __request = BinIpcParameterStack()
    # Write applicationId
    __request.push_int32(applicationId)
    # Write address
    __request.push_string(address)
    self.call(__request, "IToiOperationManager", "RegisterObserver")

  def ReleaseObserver(self, address):
    """Release an observer previously registered with \ref RegisterObserver.
    
    \param address        The address where the IToiOperationManager
    can find the registered IToiOperationObserver interface.
    \exception TToiInvalidArgumentException
    Raised if the specified address is invalid.
    \see IToiOperationObserver
    """
    __request = BinIpcParameterStack()
    # Write address
    __request.push_string(address)
    self.call(__request, "IToiOperationManager", "ReleaseObserver")

  def CreateObserver(self):
    """Create and register an observer for the service.
       (This method is only available from Python.)
    """
    observer = ToiOperationObserver(self._ipcproxy, self)
    request = BinIpcParameterStack()
    request.push_int32(self._ipcproxy.application_id)
    request.push_string(observer.observer_address)
    self.call(request, "IToiOperationManager", "RegisterObserver")
    return observer
