# Copyright (c) 2011 Motorola Mobility, Inc. All rights reserved.

from kreatvipcobserver import KreaTvIpcObserver
import collections
import sys

class ToiOperationObserver(KreaTvIpcObserver):
    OnOperationResultArguments = collections.namedtuple('OnOperationResultArguments', 'operation result ')

    def __init__(self, ipcproxy, service=None, queue=None):
        KreaTvIpcObserver.__init__(self, ipcproxy, service, queue)

    def handle_callback(self, interface_name, method_name, stack):
        """
        Pushes the call details onto the log queue.
        Called on the worker thread.
        """
        call = KreaTvIpcObserver.Call(address=self.observer_address,
                                      interface=interface_name,
                                      method=method_name)
        if method_name == 'OnOperationResult':
            args = {}
            args['result'] = stack.pop_int32()
            dict_operation = {}
            temp_Properties = []
            for o in range(stack.pop_int32()):
              dict_temp_Properties = {}
              dict_temp_Properties['Value'] = stack.pop_string()
              dict_temp_Properties['Id'] = stack.pop_int32()
              temp_Properties.append(dict_temp_Properties)
            dict_operation['Properties'] = temp_Properties
            dict_operation['ErrorCode'] = stack.pop_int32()
            dict_operation['Progress'] = stack.pop_int32()
            dict_operation['State'] = stack.pop_int32()
            dict_operation['UserData'] = stack.pop_string()
            dict_operation['Id'] = stack.pop_int32()
            args['operation'] = dict_operation
            call.arguments = ToiOperationObserver.OnOperationResultArguments(**args)
        # Thread-safe operation:
        self._queue.append(call)

    def wait_OnOperationResult(self, discard_non_matching_events=False,
                            timeout_sec=10, **argument_filters):
        """
        Convenience function to wait for callbacks to OnOperationResult.

        Keyword arguments can be given to specify parameter values to wait for.
        If a received event does not match the filter, a KreaTvIpcException
        will be raised, unless discard_non_matching_events is true in which
        case the event will be silently ignored (and the function will continue
        to wait for a matching event).
        """
        call = self.wait_callback(self.observer_address,
                                  'IToiOperationObserver',
                                  'OnOperationResult',
                                  discard_non_matching_events,
                                  timeout_sec,
                                  argument_filters)
        return call.arguments

