# Copyright (c) 2011 Motorola Mobility, Inc. All rights reserved.

from kreatvipcobserver import KreaTvIpcObserver
import collections
import sys

class ToiPlatformObserver(KreaTvIpcObserver):
    OnComponentInfoChangedArguments = collections.namedtuple('OnComponentInfoChangedArguments', 'info ')

    def __init__(self, ipcproxy, service=None, queue=None):
        KreaTvIpcObserver.__init__(self, ipcproxy, service, queue)

    def handle_callback(self, interface_name, method_name, stack):
        """
        Pushes the call details onto the log queue.
        Called on the worker thread.
        """
        call = KreaTvIpcObserver.Call(address=self.observer_address,
                                      interface=interface_name,
                                      method=method_name)
        if method_name == 'OnComponentInfoChanged':
            args = {}
            dict_info = {}
            dict_info['State'] = stack.pop_int32()
            dict_info['IsDynamic'] = stack.pop_bool()
            dict_info['Name'] = stack.pop_string()
            args['info'] = dict_info
            call.arguments = ToiPlatformObserver.OnComponentInfoChangedArguments(**args)
        # Thread-safe operation:
        self._queue.append(call)

    def wait_OnComponentInfoChanged(self, discard_non_matching_events=False,
                            timeout_sec=10, **argument_filters):
        """
        Convenience function to wait for callbacks to OnComponentInfoChanged.

        Keyword arguments can be given to specify parameter values to wait for.
        If a received event does not match the filter, a KreaTvIpcException
        will be raised, unless discard_non_matching_events is true in which
        case the event will be silently ignored (and the function will continue
        to wait for a matching event).
        """
        call = self.wait_callback(self.observer_address,
                                  'IToiPlatformObserver',
                                  'OnComponentInfoChanged',
                                  discard_non_matching_events,
                                  timeout_sec,
                                  argument_filters)
        return call.arguments.info

