# Copyright (c) 2011 Motorola Mobility, Inc. All rights reserved.

from kreatvipccaller import KreaTvIpcCaller
from binipcparameterstack import BinIpcParameterStack


class ToiScanner(KreaTvIpcCaller):

  # Enum TToiStoreMode
  STORE_MODE_OVERWRITE = 0
  STORE_MODE_UPDATE = 1

  # Const OP_PROPERTY_SERVICES_FOUND
  OP_PROPERTY_SERVICES_FOUND = 1

  def __init__(self, ipcproxy, object_address):
    KreaTvIpcCaller.__init__(self, ipcproxy.ip_address, ipcproxy.port)
    self._ipcproxy = ipcproxy
    self.set_object_address(object_address)

  def Clear(self):
    """Clear all the scan parameters set by the derived scan interface.
    
    \exception TToiInvalidPreconditionException Raised if a scan is active.
    """
    __request = BinIpcParameterStack()
    self.call(__request, "IToiScanner", "Clear")

  def StartScan(self, operationId, storeMode):
    """Starts a scan based on the parameters set by the functions of the
    derived scan interface. The found channel information is stored into
    permanent storage based on the store mode once the scan process is
    completed.
    
    When the application receives a
    \ref IToiOperationObserver::OnOperationResult() call with the
    operation result \ref IToiOperationManager::OP_RESULT_NONE,
    then the application can get the number of found services from \ref
    IToiOperationManager::TToiPropertySequence. The property for this is
    identified by id \ref OP_PROPERTY_SERVICES_FOUND.
    Only when the state in \ref IToiOperationManager::TToiOperationInfo
    reach OPERATION_COMPLETED, the application can get channel info via
    \ref IToiChannelService.
    
    Calling \ref IToiOperationManager::CancelOperation() will
    immediately stop an ongoing channel scan. The found services since
    StartScan() are not saved to permanent storage.
    Calling \ref IToiOperationManager::ReleaseOperation() will remove
    the operation identified by operationId. If a scan is in progress,
    it will immediatly stop and the found services since StartScan() are not
    saved to permanent storage.
    
    \param operationId The operation identifier created by
    \ref IToiOperationManager retrieved from
    \ref IToiFrontendService.
    
    \param storeMode   The store mode used for saving the found service
    information.
    
    \exception TToiInvalidArgumentException
    Raised if the function was called with invalid
    arguments.
    
    \exception TToiInvalidPreconditionException
    Raised if no scan parameters are set or one scan is
    active.
    """
    __request = BinIpcParameterStack()
    # Write operationId
    __request.push_int32(operationId)
    # Write storeMode
    __request.push_int32(storeMode)
    self.call(__request, "IToiScanner", "StartScan")

  def ReleaseInstance(self):
    """This function releases the resources used by this instance. Upon return
    this object will no longer be available and must not be called.
    """
    __request = BinIpcParameterStack()
    self.call(__request, "IToiScanner", "ReleaseInstance")

