# Copyright (c) 2011 Motorola Mobility, Inc. All rights reserved.

from kreatvipcobserver import KreaTvIpcObserver
import collections
import sys

class ToiSchedulerObserver(KreaTvIpcObserver):
    OnScheduledStartArguments = collections.namedtuple('OnScheduledStartArguments', 'booking ')
    OnScheduledStopArguments = collections.namedtuple('OnScheduledStopArguments', 'booking ')
    OnScheduledTriggerArguments = collections.namedtuple('OnScheduledTriggerArguments', 'booking delta ')
    OnBookingsAddedArguments = collections.namedtuple('OnBookingsAddedArguments', 'bookings ')
    OnBookingsRemovedArguments = collections.namedtuple('OnBookingsRemovedArguments', 'bookings ')
    OnBookingsChangedArguments = collections.namedtuple('OnBookingsChangedArguments', 'bookings ')
    OnSeriesChangedArguments = collections.namedtuple('OnSeriesChangedArguments', 'series ')

    def __init__(self, ipcproxy, service=None, queue=None):
        KreaTvIpcObserver.__init__(self, ipcproxy, service, queue)

    def handle_callback(self, interface_name, method_name, stack):
        """
        Pushes the call details onto the log queue.
        Called on the worker thread.
        """
        call = KreaTvIpcObserver.Call(address=self.observer_address,
                                      interface=interface_name,
                                      method=method_name)
        if method_name == 'OnScheduledStart':
            args = {}
            dict_booking = {}
            dict_booking['SeriesId'] = stack.pop_int32()
            dict_booking['Duration'] = stack.pop_int32()
            dict_booking['Start'] = stack.pop_int32()
            dict_booking['ActivityType'] = stack.pop_string()
            dict_booking['Category'] = stack.pop_string()
            dict_booking['Id'] = stack.pop_int32()
            args['booking'] = dict_booking
            call.arguments = ToiSchedulerObserver.OnScheduledStartArguments(**args)
        if method_name == 'OnScheduledStop':
            args = {}
            dict_booking = {}
            dict_booking['SeriesId'] = stack.pop_int32()
            dict_booking['Duration'] = stack.pop_int32()
            dict_booking['Start'] = stack.pop_int32()
            dict_booking['ActivityType'] = stack.pop_string()
            dict_booking['Category'] = stack.pop_string()
            dict_booking['Id'] = stack.pop_int32()
            args['booking'] = dict_booking
            call.arguments = ToiSchedulerObserver.OnScheduledStopArguments(**args)
        if method_name == 'OnScheduledTrigger':
            args = {}
            args['delta'] = stack.pop_int32()
            dict_booking = {}
            dict_booking['SeriesId'] = stack.pop_int32()
            dict_booking['Duration'] = stack.pop_int32()
            dict_booking['Start'] = stack.pop_int32()
            dict_booking['ActivityType'] = stack.pop_string()
            dict_booking['Category'] = stack.pop_string()
            dict_booking['Id'] = stack.pop_int32()
            args['booking'] = dict_booking
            call.arguments = ToiSchedulerObserver.OnScheduledTriggerArguments(**args)
        if method_name == 'OnBookingsAdded':
            args = {}
            temp_bookings = []
            for o in range(stack.pop_int32()):
              dict_temp_bookings = {}
              dict_temp_bookings['SeriesId'] = stack.pop_int32()
              dict_temp_bookings['Duration'] = stack.pop_int32()
              dict_temp_bookings['Start'] = stack.pop_int32()
              dict_temp_bookings['ActivityType'] = stack.pop_string()
              dict_temp_bookings['Category'] = stack.pop_string()
              dict_temp_bookings['Id'] = stack.pop_int32()
              temp_bookings.append(dict_temp_bookings)
            args['bookings'] = temp_bookings
            call.arguments = ToiSchedulerObserver.OnBookingsAddedArguments(**args)
        if method_name == 'OnBookingsRemoved':
            args = {}
            temp_bookings = []
            for o in range(stack.pop_int32()):
              dict_temp_bookings = {}
              dict_temp_bookings['SeriesId'] = stack.pop_int32()
              dict_temp_bookings['Duration'] = stack.pop_int32()
              dict_temp_bookings['Start'] = stack.pop_int32()
              dict_temp_bookings['ActivityType'] = stack.pop_string()
              dict_temp_bookings['Category'] = stack.pop_string()
              dict_temp_bookings['Id'] = stack.pop_int32()
              temp_bookings.append(dict_temp_bookings)
            args['bookings'] = temp_bookings
            call.arguments = ToiSchedulerObserver.OnBookingsRemovedArguments(**args)
        if method_name == 'OnBookingsChanged':
            args = {}
            temp_bookings = []
            for o in range(stack.pop_int32()):
              dict_temp_bookings = {}
              dict_temp_bookings['SeriesId'] = stack.pop_int32()
              dict_temp_bookings['Duration'] = stack.pop_int32()
              dict_temp_bookings['Start'] = stack.pop_int32()
              dict_temp_bookings['ActivityType'] = stack.pop_string()
              dict_temp_bookings['Category'] = stack.pop_string()
              dict_temp_bookings['Id'] = stack.pop_int32()
              temp_bookings.append(dict_temp_bookings)
            args['bookings'] = temp_bookings
            call.arguments = ToiSchedulerObserver.OnBookingsChangedArguments(**args)
        if method_name == 'OnSeriesChanged':
            args = {}
            dict_series = {}
            dict_series['OriginalNumberOfOccurences'] = stack.pop_int32()
            dict_series['Weekdays'] = stack.pop_int32()
            dict_series['Duration'] = stack.pop_int32()
            dict_series['Start'] = stack.pop_int32()
            dict_series['ActivityType'] = stack.pop_string()
            dict_series['Category'] = stack.pop_string()
            dict_series['Id'] = stack.pop_int32()
            args['series'] = dict_series
            call.arguments = ToiSchedulerObserver.OnSeriesChangedArguments(**args)
        # Thread-safe operation:
        self._queue.append(call)

    def wait_OnScheduledStart(self, discard_non_matching_events=False,
                            timeout_sec=10, **argument_filters):
        """
        Convenience function to wait for callbacks to OnScheduledStart.

        Keyword arguments can be given to specify parameter values to wait for.
        If a received event does not match the filter, a KreaTvIpcException
        will be raised, unless discard_non_matching_events is true in which
        case the event will be silently ignored (and the function will continue
        to wait for a matching event).
        """
        call = self.wait_callback(self.observer_address,
                                  'IToiSchedulerObserver',
                                  'OnScheduledStart',
                                  discard_non_matching_events,
                                  timeout_sec,
                                  argument_filters)
        return call.arguments.booking

    def wait_OnScheduledStop(self, discard_non_matching_events=False,
                            timeout_sec=10, **argument_filters):
        """
        Convenience function to wait for callbacks to OnScheduledStop.

        Keyword arguments can be given to specify parameter values to wait for.
        If a received event does not match the filter, a KreaTvIpcException
        will be raised, unless discard_non_matching_events is true in which
        case the event will be silently ignored (and the function will continue
        to wait for a matching event).
        """
        call = self.wait_callback(self.observer_address,
                                  'IToiSchedulerObserver',
                                  'OnScheduledStop',
                                  discard_non_matching_events,
                                  timeout_sec,
                                  argument_filters)
        return call.arguments.booking

    def wait_OnScheduledTrigger(self, discard_non_matching_events=False,
                            timeout_sec=10, **argument_filters):
        """
        Convenience function to wait for callbacks to OnScheduledTrigger.

        Keyword arguments can be given to specify parameter values to wait for.
        If a received event does not match the filter, a KreaTvIpcException
        will be raised, unless discard_non_matching_events is true in which
        case the event will be silently ignored (and the function will continue
        to wait for a matching event).
        """
        call = self.wait_callback(self.observer_address,
                                  'IToiSchedulerObserver',
                                  'OnScheduledTrigger',
                                  discard_non_matching_events,
                                  timeout_sec,
                                  argument_filters)
        return call.arguments

    def wait_OnBookingsAdded(self, discard_non_matching_events=False,
                            timeout_sec=10, **argument_filters):
        """
        Convenience function to wait for callbacks to OnBookingsAdded.

        Keyword arguments can be given to specify parameter values to wait for.
        If a received event does not match the filter, a KreaTvIpcException
        will be raised, unless discard_non_matching_events is true in which
        case the event will be silently ignored (and the function will continue
        to wait for a matching event).
        """
        call = self.wait_callback(self.observer_address,
                                  'IToiSchedulerObserver',
                                  'OnBookingsAdded',
                                  discard_non_matching_events,
                                  timeout_sec,
                                  argument_filters)
        return call.arguments.bookings

    def wait_OnBookingsRemoved(self, discard_non_matching_events=False,
                            timeout_sec=10, **argument_filters):
        """
        Convenience function to wait for callbacks to OnBookingsRemoved.

        Keyword arguments can be given to specify parameter values to wait for.
        If a received event does not match the filter, a KreaTvIpcException
        will be raised, unless discard_non_matching_events is true in which
        case the event will be silently ignored (and the function will continue
        to wait for a matching event).
        """
        call = self.wait_callback(self.observer_address,
                                  'IToiSchedulerObserver',
                                  'OnBookingsRemoved',
                                  discard_non_matching_events,
                                  timeout_sec,
                                  argument_filters)
        return call.arguments.bookings

    def wait_OnBookingsChanged(self, discard_non_matching_events=False,
                            timeout_sec=10, **argument_filters):
        """
        Convenience function to wait for callbacks to OnBookingsChanged.

        Keyword arguments can be given to specify parameter values to wait for.
        If a received event does not match the filter, a KreaTvIpcException
        will be raised, unless discard_non_matching_events is true in which
        case the event will be silently ignored (and the function will continue
        to wait for a matching event).
        """
        call = self.wait_callback(self.observer_address,
                                  'IToiSchedulerObserver',
                                  'OnBookingsChanged',
                                  discard_non_matching_events,
                                  timeout_sec,
                                  argument_filters)
        return call.arguments.bookings

    def wait_OnSeriesChanged(self, discard_non_matching_events=False,
                            timeout_sec=10, **argument_filters):
        """
        Convenience function to wait for callbacks to OnSeriesChanged.

        Keyword arguments can be given to specify parameter values to wait for.
        If a received event does not match the filter, a KreaTvIpcException
        will be raised, unless discard_non_matching_events is true in which
        case the event will be silently ignored (and the function will continue
        to wait for a matching event).
        """
        call = self.wait_callback(self.observer_address,
                                  'IToiSchedulerObserver',
                                  'OnSeriesChanged',
                                  discard_non_matching_events,
                                  timeout_sec,
                                  argument_filters)
        return call.arguments.series

