# Copyright (c) 2011 Motorola Mobility, Inc. All rights reserved.

from kreatvipcobserver import KreaTvIpcObserver
import collections
import sys

class ToiSoftwareObserver(KreaTvIpcObserver):
    OnIipAvailableArguments = collections.namedtuple('OnIipAvailableArguments', 'name dependencyType ')
    OnAddRefResponseArguments = collections.namedtuple('OnAddRefResponseArguments', 'name dependencyType addRefResult ')
    OnReleaseResponseArguments = collections.namedtuple('OnReleaseResponseArguments', 'name dependencyType releaseResult ')
    OnAddIipsResponseArguments = collections.namedtuple('OnAddIipsResponseArguments', 'url success ')

    def __init__(self, ipcproxy, service=None, queue=None):
        KreaTvIpcObserver.__init__(self, ipcproxy, service, queue)

    def handle_callback(self, interface_name, method_name, stack):
        """
        Pushes the call details onto the log queue.
        Called on the worker thread.
        """
        call = KreaTvIpcObserver.Call(address=self.observer_address,
                                      interface=interface_name,
                                      method=method_name)
        if method_name == 'OnIipAvailable':
            args = {}
            args['dependencyType'] = stack.pop_int32()
            args['name'] = stack.pop_string()
            call.arguments = ToiSoftwareObserver.OnIipAvailableArguments(**args)
        if method_name == 'OnAddRefResponse':
            args = {}
            args['addRefResult'] = stack.pop_int32()
            args['dependencyType'] = stack.pop_int32()
            args['name'] = stack.pop_string()
            call.arguments = ToiSoftwareObserver.OnAddRefResponseArguments(**args)
        if method_name == 'OnReleaseResponse':
            args = {}
            args['releaseResult'] = stack.pop_int32()
            args['dependencyType'] = stack.pop_int32()
            args['name'] = stack.pop_string()
            call.arguments = ToiSoftwareObserver.OnReleaseResponseArguments(**args)
        if method_name == 'OnAddIipsResponse':
            args = {}
            args['success'] = stack.pop_bool()
            args['url'] = stack.pop_string()
            call.arguments = ToiSoftwareObserver.OnAddIipsResponseArguments(**args)
        # Thread-safe operation:
        self._queue.append(call)

    def wait_OnIipAvailable(self, discard_non_matching_events=False,
                            timeout_sec=10, **argument_filters):
        """
        Convenience function to wait for callbacks to OnIipAvailable.

        Keyword arguments can be given to specify parameter values to wait for.
        If a received event does not match the filter, a KreaTvIpcException
        will be raised, unless discard_non_matching_events is true in which
        case the event will be silently ignored (and the function will continue
        to wait for a matching event).
        """
        call = self.wait_callback(self.observer_address,
                                  'IToiSoftwareObserver',
                                  'OnIipAvailable',
                                  discard_non_matching_events,
                                  timeout_sec,
                                  argument_filters)
        return call.arguments

    def wait_OnAddRefResponse(self, discard_non_matching_events=False,
                            timeout_sec=10, **argument_filters):
        """
        Convenience function to wait for callbacks to OnAddRefResponse.

        Keyword arguments can be given to specify parameter values to wait for.
        If a received event does not match the filter, a KreaTvIpcException
        will be raised, unless discard_non_matching_events is true in which
        case the event will be silently ignored (and the function will continue
        to wait for a matching event).
        """
        call = self.wait_callback(self.observer_address,
                                  'IToiSoftwareObserver',
                                  'OnAddRefResponse',
                                  discard_non_matching_events,
                                  timeout_sec,
                                  argument_filters)
        return call.arguments

    def wait_OnReleaseResponse(self, discard_non_matching_events=False,
                            timeout_sec=10, **argument_filters):
        """
        Convenience function to wait for callbacks to OnReleaseResponse.

        Keyword arguments can be given to specify parameter values to wait for.
        If a received event does not match the filter, a KreaTvIpcException
        will be raised, unless discard_non_matching_events is true in which
        case the event will be silently ignored (and the function will continue
        to wait for a matching event).
        """
        call = self.wait_callback(self.observer_address,
                                  'IToiSoftwareObserver',
                                  'OnReleaseResponse',
                                  discard_non_matching_events,
                                  timeout_sec,
                                  argument_filters)
        return call.arguments

    def wait_OnAddIipsResponse(self, discard_non_matching_events=False,
                            timeout_sec=10, **argument_filters):
        """
        Convenience function to wait for callbacks to OnAddIipsResponse.

        Keyword arguments can be given to specify parameter values to wait for.
        If a received event does not match the filter, a KreaTvIpcException
        will be raised, unless discard_non_matching_events is true in which
        case the event will be silently ignored (and the function will continue
        to wait for a matching event).
        """
        call = self.wait_callback(self.observer_address,
                                  'IToiSoftwareObserver',
                                  'OnAddIipsResponse',
                                  discard_non_matching_events,
                                  timeout_sec,
                                  argument_filters)
        return call.arguments

