# Copyright (c) 2011 Motorola Mobility, Inc. All rights reserved.

from kreatvipcobserver import KreaTvIpcObserver
import collections
import sys

class ToiStorageObserver(KreaTvIpcObserver):
    OnDeviceConnectedArguments = collections.namedtuple('OnDeviceConnectedArguments', 'info ')
    OnDeviceDisconnectedArguments = collections.namedtuple('OnDeviceDisconnectedArguments', 'info ')
    OnDeviceSpinningChangedArguments = collections.namedtuple('OnDeviceSpinningChangedArguments', 'info ')
    OnDevicePartitionsChangedArguments = collections.namedtuple('OnDevicePartitionsChangedArguments', 'info ')
    OnDevicePartitionMountChangedArguments = collections.namedtuple('OnDevicePartitionMountChangedArguments', 'info partitionInfo ')
    OnDevicePartitionFullArguments = collections.namedtuple('OnDevicePartitionFullArguments', 'info partitionInfo spaceRequired ')
    OnDeviceTemperatureThresholdReachedArguments = collections.namedtuple('OnDeviceTemperatureThresholdReachedArguments', 'info thresholdName ')

    def __init__(self, ipcproxy, service=None, queue=None):
        KreaTvIpcObserver.__init__(self, ipcproxy, service, queue)

    def handle_callback(self, interface_name, method_name, stack):
        """
        Pushes the call details onto the log queue.
        Called on the worker thread.
        """
        call = KreaTvIpcObserver.Call(address=self.observer_address,
                                      interface=interface_name,
                                      method=method_name)
        if method_name == 'OnDeviceConnected':
            args = {}
            dict_info = {}
            dict_info['IsApproved'] = stack.pop_bool()
            dict_info['Temperature'] = stack.pop_int32()
            dict_info['FreeCapacity'] = stack.pop_int64()
            dict_info['Capacity'] = stack.pop_int64()
            temp_Partitions = []
            for o in range(stack.pop_int32()):
              temp_Partitions.append(stack.pop_uint32())
            dict_info['Partitions'] = temp_Partitions
            dict_info['IsOverheated'] = stack.pop_bool()
            dict_info['IsSpinning'] = stack.pop_bool()
            dict_info['IsConnected'] = stack.pop_bool()
            dict_info['IsInternal'] = stack.pop_bool()
            dict_info['SerialNumber'] = stack.pop_string()
            dict_info['Id'] = stack.pop_uint32()
            args['info'] = dict_info
            call.arguments = ToiStorageObserver.OnDeviceConnectedArguments(**args)
        if method_name == 'OnDeviceDisconnected':
            args = {}
            dict_info = {}
            dict_info['IsApproved'] = stack.pop_bool()
            dict_info['Temperature'] = stack.pop_int32()
            dict_info['FreeCapacity'] = stack.pop_int64()
            dict_info['Capacity'] = stack.pop_int64()
            temp_Partitions = []
            for o in range(stack.pop_int32()):
              temp_Partitions.append(stack.pop_uint32())
            dict_info['Partitions'] = temp_Partitions
            dict_info['IsOverheated'] = stack.pop_bool()
            dict_info['IsSpinning'] = stack.pop_bool()
            dict_info['IsConnected'] = stack.pop_bool()
            dict_info['IsInternal'] = stack.pop_bool()
            dict_info['SerialNumber'] = stack.pop_string()
            dict_info['Id'] = stack.pop_uint32()
            args['info'] = dict_info
            call.arguments = ToiStorageObserver.OnDeviceDisconnectedArguments(**args)
        if method_name == 'OnDeviceSpinningChanged':
            args = {}
            dict_info = {}
            dict_info['IsApproved'] = stack.pop_bool()
            dict_info['Temperature'] = stack.pop_int32()
            dict_info['FreeCapacity'] = stack.pop_int64()
            dict_info['Capacity'] = stack.pop_int64()
            temp_Partitions = []
            for o in range(stack.pop_int32()):
              temp_Partitions.append(stack.pop_uint32())
            dict_info['Partitions'] = temp_Partitions
            dict_info['IsOverheated'] = stack.pop_bool()
            dict_info['IsSpinning'] = stack.pop_bool()
            dict_info['IsConnected'] = stack.pop_bool()
            dict_info['IsInternal'] = stack.pop_bool()
            dict_info['SerialNumber'] = stack.pop_string()
            dict_info['Id'] = stack.pop_uint32()
            args['info'] = dict_info
            call.arguments = ToiStorageObserver.OnDeviceSpinningChangedArguments(**args)
        if method_name == 'OnDevicePartitionsChanged':
            args = {}
            dict_info = {}
            dict_info['IsApproved'] = stack.pop_bool()
            dict_info['Temperature'] = stack.pop_int32()
            dict_info['FreeCapacity'] = stack.pop_int64()
            dict_info['Capacity'] = stack.pop_int64()
            temp_Partitions = []
            for o in range(stack.pop_int32()):
              temp_Partitions.append(stack.pop_uint32())
            dict_info['Partitions'] = temp_Partitions
            dict_info['IsOverheated'] = stack.pop_bool()
            dict_info['IsSpinning'] = stack.pop_bool()
            dict_info['IsConnected'] = stack.pop_bool()
            dict_info['IsInternal'] = stack.pop_bool()
            dict_info['SerialNumber'] = stack.pop_string()
            dict_info['Id'] = stack.pop_uint32()
            args['info'] = dict_info
            call.arguments = ToiStorageObserver.OnDevicePartitionsChangedArguments(**args)
        if method_name == 'OnDevicePartitionMountChanged':
            args = {}
            dict_partitionInfo = {}
            dict_partitionInfo['IsReadWrite'] = stack.pop_bool()
            dict_partitionInfo['FsStatus'] = stack.pop_int32()
            dict_partitionInfo['FreeCapacity'] = stack.pop_int64()
            dict_partitionInfo['Capacity'] = stack.pop_int64()
            dict_partitionInfo['MountPoint'] = stack.pop_string()
            dict_partitionInfo['IsMounted'] = stack.pop_bool()
            dict_partitionInfo['UniqueId'] = stack.pop_string()
            dict_partitionInfo['Id'] = stack.pop_uint32()
            args['partitionInfo'] = dict_partitionInfo
            dict_info = {}
            dict_info['IsApproved'] = stack.pop_bool()
            dict_info['Temperature'] = stack.pop_int32()
            dict_info['FreeCapacity'] = stack.pop_int64()
            dict_info['Capacity'] = stack.pop_int64()
            temp_Partitions = []
            for o in range(stack.pop_int32()):
              temp_Partitions.append(stack.pop_uint32())
            dict_info['Partitions'] = temp_Partitions
            dict_info['IsOverheated'] = stack.pop_bool()
            dict_info['IsSpinning'] = stack.pop_bool()
            dict_info['IsConnected'] = stack.pop_bool()
            dict_info['IsInternal'] = stack.pop_bool()
            dict_info['SerialNumber'] = stack.pop_string()
            dict_info['Id'] = stack.pop_uint32()
            args['info'] = dict_info
            call.arguments = ToiStorageObserver.OnDevicePartitionMountChangedArguments(**args)
        if method_name == 'OnDevicePartitionFull':
            args = {}
            args['spaceRequired'] = stack.pop_int64()
            dict_partitionInfo = {}
            dict_partitionInfo['IsReadWrite'] = stack.pop_bool()
            dict_partitionInfo['FsStatus'] = stack.pop_int32()
            dict_partitionInfo['FreeCapacity'] = stack.pop_int64()
            dict_partitionInfo['Capacity'] = stack.pop_int64()
            dict_partitionInfo['MountPoint'] = stack.pop_string()
            dict_partitionInfo['IsMounted'] = stack.pop_bool()
            dict_partitionInfo['UniqueId'] = stack.pop_string()
            dict_partitionInfo['Id'] = stack.pop_uint32()
            args['partitionInfo'] = dict_partitionInfo
            dict_info = {}
            dict_info['IsApproved'] = stack.pop_bool()
            dict_info['Temperature'] = stack.pop_int32()
            dict_info['FreeCapacity'] = stack.pop_int64()
            dict_info['Capacity'] = stack.pop_int64()
            temp_Partitions = []
            for o in range(stack.pop_int32()):
              temp_Partitions.append(stack.pop_uint32())
            dict_info['Partitions'] = temp_Partitions
            dict_info['IsOverheated'] = stack.pop_bool()
            dict_info['IsSpinning'] = stack.pop_bool()
            dict_info['IsConnected'] = stack.pop_bool()
            dict_info['IsInternal'] = stack.pop_bool()
            dict_info['SerialNumber'] = stack.pop_string()
            dict_info['Id'] = stack.pop_uint32()
            args['info'] = dict_info
            call.arguments = ToiStorageObserver.OnDevicePartitionFullArguments(**args)
        if method_name == 'OnDeviceTemperatureThresholdReached':
            args = {}
            args['thresholdName'] = stack.pop_string()
            dict_info = {}
            dict_info['IsApproved'] = stack.pop_bool()
            dict_info['Temperature'] = stack.pop_int32()
            dict_info['FreeCapacity'] = stack.pop_int64()
            dict_info['Capacity'] = stack.pop_int64()
            temp_Partitions = []
            for o in range(stack.pop_int32()):
              temp_Partitions.append(stack.pop_uint32())
            dict_info['Partitions'] = temp_Partitions
            dict_info['IsOverheated'] = stack.pop_bool()
            dict_info['IsSpinning'] = stack.pop_bool()
            dict_info['IsConnected'] = stack.pop_bool()
            dict_info['IsInternal'] = stack.pop_bool()
            dict_info['SerialNumber'] = stack.pop_string()
            dict_info['Id'] = stack.pop_uint32()
            args['info'] = dict_info
            call.arguments = ToiStorageObserver.OnDeviceTemperatureThresholdReachedArguments(**args)
        # Thread-safe operation:
        self._queue.append(call)

    def wait_OnDeviceConnected(self, discard_non_matching_events=False,
                            timeout_sec=10, **argument_filters):
        """
        Convenience function to wait for callbacks to OnDeviceConnected.

        Keyword arguments can be given to specify parameter values to wait for.
        If a received event does not match the filter, a KreaTvIpcException
        will be raised, unless discard_non_matching_events is true in which
        case the event will be silently ignored (and the function will continue
        to wait for a matching event).
        """
        call = self.wait_callback(self.observer_address,
                                  'IToiStorageObserver',
                                  'OnDeviceConnected',
                                  discard_non_matching_events,
                                  timeout_sec,
                                  argument_filters)
        return call.arguments.info

    def wait_OnDeviceDisconnected(self, discard_non_matching_events=False,
                            timeout_sec=10, **argument_filters):
        """
        Convenience function to wait for callbacks to OnDeviceDisconnected.

        Keyword arguments can be given to specify parameter values to wait for.
        If a received event does not match the filter, a KreaTvIpcException
        will be raised, unless discard_non_matching_events is true in which
        case the event will be silently ignored (and the function will continue
        to wait for a matching event).
        """
        call = self.wait_callback(self.observer_address,
                                  'IToiStorageObserver',
                                  'OnDeviceDisconnected',
                                  discard_non_matching_events,
                                  timeout_sec,
                                  argument_filters)
        return call.arguments.info

    def wait_OnDeviceSpinningChanged(self, discard_non_matching_events=False,
                            timeout_sec=10, **argument_filters):
        """
        Convenience function to wait for callbacks to OnDeviceSpinningChanged.

        Keyword arguments can be given to specify parameter values to wait for.
        If a received event does not match the filter, a KreaTvIpcException
        will be raised, unless discard_non_matching_events is true in which
        case the event will be silently ignored (and the function will continue
        to wait for a matching event).
        """
        call = self.wait_callback(self.observer_address,
                                  'IToiStorageObserver',
                                  'OnDeviceSpinningChanged',
                                  discard_non_matching_events,
                                  timeout_sec,
                                  argument_filters)
        return call.arguments.info

    def wait_OnDevicePartitionsChanged(self, discard_non_matching_events=False,
                            timeout_sec=10, **argument_filters):
        """
        Convenience function to wait for callbacks to OnDevicePartitionsChanged.

        Keyword arguments can be given to specify parameter values to wait for.
        If a received event does not match the filter, a KreaTvIpcException
        will be raised, unless discard_non_matching_events is true in which
        case the event will be silently ignored (and the function will continue
        to wait for a matching event).
        """
        call = self.wait_callback(self.observer_address,
                                  'IToiStorageObserver',
                                  'OnDevicePartitionsChanged',
                                  discard_non_matching_events,
                                  timeout_sec,
                                  argument_filters)
        return call.arguments.info

    def wait_OnDevicePartitionMountChanged(self, discard_non_matching_events=False,
                            timeout_sec=10, **argument_filters):
        """
        Convenience function to wait for callbacks to OnDevicePartitionMountChanged.

        Keyword arguments can be given to specify parameter values to wait for.
        If a received event does not match the filter, a KreaTvIpcException
        will be raised, unless discard_non_matching_events is true in which
        case the event will be silently ignored (and the function will continue
        to wait for a matching event).
        """
        call = self.wait_callback(self.observer_address,
                                  'IToiStorageObserver',
                                  'OnDevicePartitionMountChanged',
                                  discard_non_matching_events,
                                  timeout_sec,
                                  argument_filters)
        return call.arguments

    def wait_OnDevicePartitionFull(self, discard_non_matching_events=False,
                            timeout_sec=10, **argument_filters):
        """
        Convenience function to wait for callbacks to OnDevicePartitionFull.

        Keyword arguments can be given to specify parameter values to wait for.
        If a received event does not match the filter, a KreaTvIpcException
        will be raised, unless discard_non_matching_events is true in which
        case the event will be silently ignored (and the function will continue
        to wait for a matching event).
        """
        call = self.wait_callback(self.observer_address,
                                  'IToiStorageObserver',
                                  'OnDevicePartitionFull',
                                  discard_non_matching_events,
                                  timeout_sec,
                                  argument_filters)
        return call.arguments

    def wait_OnDeviceTemperatureThresholdReached(self, discard_non_matching_events=False,
                            timeout_sec=10, **argument_filters):
        """
        Convenience function to wait for callbacks to OnDeviceTemperatureThresholdReached.

        Keyword arguments can be given to specify parameter values to wait for.
        If a received event does not match the filter, a KreaTvIpcException
        will be raised, unless discard_non_matching_events is true in which
        case the event will be silently ignored (and the function will continue
        to wait for a matching event).
        """
        call = self.wait_callback(self.observer_address,
                                  'IToiStorageObserver',
                                  'OnDeviceTemperatureThresholdReached',
                                  discard_non_matching_events,
                                  timeout_sec,
                                  argument_filters)
        return call.arguments

