# Copyright (c) 2011 Motorola Mobility, Inc. All rights reserved.

from kreatvipccaller import KreaTvIpcCaller
from binipcparameterstack import BinIpcParameterStack
from ToiVideoOutputConfigurationObserver import ToiVideoOutputConfigurationObserver


class ToiVideoOutputConfiguration(KreaTvIpcCaller):

  # Enum TToiColorSystem
  COLOR_SYSTEM_NTSC_M = 0
  COLOR_SYSTEM_NTSC_J = 1
  COLOR_SYSTEM_PAL_B = 2
  COLOR_SYSTEM_PAL_M = 3
  COLOR_SYSTEM_PAL_N = 4
  COLOR_SYSTEM_PAL_NC = 5

  # Enum TToiVideoMode
  NO_VIDEO_MODE = 0
  VIDEO_MODE_480I60 = 1
  VIDEO_MODE_576I50 = 2
  VIDEO_MODE_480P60 = 3
  VIDEO_MODE_576P50 = 4
  VIDEO_MODE_720P50 = 5
  VIDEO_MODE_720P60 = 6
  VIDEO_MODE_1080I50 = 7
  VIDEO_MODE_1080I60 = 8
  VIDEO_MODE_1080P23976 = 9
  VIDEO_MODE_1080P24 = 10
  VIDEO_MODE_1080P25 = 11
  VIDEO_MODE_1080P29970 = 12
  VIDEO_MODE_1080P30 = 13
  VIDEO_MODE_1080P50 = 14
  VIDEO_MODE_1080P59940 = 15
  VIDEO_MODE_1080P60 = 16

  # Enum TToiScartMode
  NO_SCART_MODE = 0
  SCART_MODE_CVBS = 1
  SCART_MODE_YC = 2
  SCART_MODE_RGB_CVBS = 3

  # Enum TToiColorSpace
  NO_COLOR_SPACE = 0
  DIGITAL_COLOR_SPACE_SRGB = 1
  DIGITAL_COLOR_SPACE_YCBCR_444 = 2
  DIGITAL_COLOR_SPACE_YCBCR_422 = 3
  DIGITAL_COLOR_SPACE_XVYCC = 4
  ANALOG_COLOR_SPACE_YPBPR = 5

  # Enum TToiValueMode
  NO_VALUE_MODE = 0
  VALUE_MODE_ENABLED = 1
  VALUE_MODE_DISABLED = 2

  # Enum TToiVideoConnectionType
  VIDEO_CONNECTION_TYPE_HDMI = 0
  VIDEO_CONNECTION_TYPE_COMPONENT = 1
  VIDEO_CONNECTION_TYPE_SCART = 2
  VIDEO_CONNECTION_TYPE_COMPOSITE = 3
  VIDEO_CONNECTION_TYPE_SVIDEO = 4
  VIDEO_CONNECTION_TYPE_RF = 5

  # Enum TToiVideoOutputType
  VIDEO_OUTPUT_TYPE_HD = 0
  VIDEO_OUTPUT_TYPE_SD = 1

  # Enum TToiAspectRatio
  ASPECT_RATIO_UNKNOWN = 0
  ASPECT_RATIO_4_3 = 1
  ASPECT_RATIO_16_9 = 2
  ASPECT_RATIO_16_10 = 3

  # Enum TToiDisplayStatus
  DISPLAY_STATUS_UNKNOWN = 0
  DISPLAY_STATUS_CONNECTED = 1
  DISPLAY_STATUS_CONNECTED_CAPABILITIES_UNKNOWN = 2
  DISPLAY_STATUS_DISCONNECTED = 3

  # Enum TToiActiveFormatDescriptor
  ACTIVE_FORMAT_UNKNOWN = 0
  ACTIVE_FORMAT_16_9_TOP = 1
  ACTIVE_FORMAT_14_9_TOP = 2
  ACTIVE_FORMAT_MORE_THAN_16_9_CENTRE = 3
  ACTIVE_FORMAT_SAME_AS_CODED = 4
  ACTIVE_FORMAT_4_3_CENTRE = 5
  ACTIVE_FORMAT_16_9_CENTRE = 6
  ACTIVE_FORMAT_14_9_CENTRE = 7
  ACTIVE_FORMAT_4_3_SHOOT_AND_PROTECT_14_9_CENTRE = 8
  ACTIVE_FORMAT_16_9_SHOOT_AND_PROTECT_14_9_CENTRE = 9
  ACTIVE_FORMAT_16_9_SHOOT_AND_PROTECT_4_3_CENTRE = 10

  def __init__(self, ipcproxy, object_address):
    KreaTvIpcCaller.__init__(self, ipcproxy.ip_address, ipcproxy.port)
    self._ipcproxy = ipcproxy
    self.set_object_address(object_address)

  def GetDisplayInfo(self, outputId):
    """Returns information about the display connected to a video
    output.  This is primarily used for HDMI outputs that may
    receive EDID information. It is possible to override
    capabilities of a display using
    \ref IToiVideoOutputConfigurationSession::SetDisplayInfo(). If
    there are no overrides, \ref GetDisplayInfo() returns information from
    the EDID in the case of HDMI and default values for other
    outputs. If overrides have been set, these are returned instead.
    
    \param outputId          Video output identifier.
    \retval info             Information for available displays.
    """
    __request = BinIpcParameterStack()
    # Write outputId
    __request.push_uint32(outputId)
    __response = self.call(__request, "IToiVideoOutputConfiguration", "GetDisplayInfo")
    __result = []
    # Read info
    info = {}
    info["SupportedColorSpaces"] = []
    for k in range(__response.pop_int32()):
      info["SupportedColorSpaces"].append(__response.pop_int32())
    info["HasOverriddenValues"] = __response.pop_bool()
    info["Status"] = __response.pop_int32()
    info["AspectRatio"] = __response.pop_int32()
    info["PreferredVideoMode"] = __response.pop_int32()
    info["SupportedVideoModes"] = []
    for k in range(__response.pop_int32()):
      info["SupportedVideoModes"].append(__response.pop_int32())
    info["VideoOutputId"] = __response.pop_uint32()
    info["VendorId"] = __response.pop_string()
    __result.append(info)
    return __result[0]

  def GetColorSystems(self):
    """Get supported color systems.
    
    \retval colorSystems       Supported color systems.
    """
    __request = BinIpcParameterStack()
    __response = self.call(__request, "IToiVideoOutputConfiguration", "GetColorSystems")
    __result = []
    # Read colorSystems
    colorSystems = []
    for k in range(__response.pop_int32()):
      colorSystems.append(__response.pop_int32())
    __result.append(colorSystems)
    return __result[0]

  def GetColorSystem(self):
    """Get current color system.
    
    \retval colorSystem        Current color system.
    """
    __request = BinIpcParameterStack()
    __response = self.call(__request, "IToiVideoOutputConfiguration", "GetColorSystem")
    __result = []
    # Read colorSystem
    __result.append(__response.pop_int32())
    return __result[0]

  def GetVideoOutputs(self):
    """Returns available video outputs.
    
    \retval identifiers       List of video output identifiers.
    """
    __request = BinIpcParameterStack()
    __response = self.call(__request, "IToiVideoOutputConfiguration", "GetVideoOutputs")
    __result = []
    # Read identifiers
    identifiers = []
    for k in range(__response.pop_int32()):
      identifiers.append(__response.pop_uint32())
    __result.append(identifiers)
    return __result[0]

  def GetVideoOutputInfo(self, outputId):
    """Get information about current settings for a video output.
    
    \param outputId        Video output identifier.
    
    \retval info           The video output information.
    
    \exception             TToiOperationNotSupportedException
    Raised if outputs can not be listed.
    
    \exception             TToiInvalidArgumentException
    Raised if outputId is an invalid output.
    """
    __request = BinIpcParameterStack()
    # Write outputId
    __request.push_uint32(outputId)
    __response = self.call(__request, "IToiVideoOutputConfiguration", "GetVideoOutputInfo")
    __result = []
    # Read info
    info = {}
    info["DviMode"] = __response.pop_int32()
    info["AllowedDviModes"] = []
    for k in range(__response.pop_int32()):
      info["AllowedDviModes"].append(__response.pop_int32())
    info["ScartLoopThroughMode"] = __response.pop_int32()
    info["AllowedScartLoopThroughModes"] = []
    for k in range(__response.pop_int32()):
      info["AllowedScartLoopThroughModes"].append(__response.pop_int32())
    info["ScartMode"] = __response.pop_int32()
    info["AllowedScartModes"] = []
    for k in range(__response.pop_int32()):
      info["AllowedScartModes"].append(__response.pop_int32())
    info["ColorSpace"] = __response.pop_int32()
    info["AllowedColorSpaceModes"] = []
    for k in range(__response.pop_int32()):
      info["AllowedColorSpaceModes"].append(__response.pop_int32())
    info["VideoMode"] = __response.pop_int32()
    info["AllowedVideoModes"] = []
    for k in range(__response.pop_int32()):
      info["AllowedVideoModes"].append(__response.pop_int32())
    info["Index"] = __response.pop_uint32()
    info["ConnectionType"] = __response.pop_int32()
    info["OutputType"] = __response.pop_int32()
    info["Id"] = __response.pop_uint32()
    __result.append(info)
    return __result[0]

  def GetVideoOutputCapabilityInfo(self, outputId):
    """Get capability information about a video output.
    
    \param outputId        Video output identifier.
    
    \retval info           The video output capability information.
    
    \exception             TToiOperationNotSupportedException
    Raised if outputs can not be listed.
    
    \exception             TToiInvalidArgumentException
    Raised if outputId is an invalid output.
    """
    __request = BinIpcParameterStack()
    # Write outputId
    __request.push_uint32(outputId)
    __response = self.call(__request, "IToiVideoOutputConfiguration", "GetVideoOutputCapabilityInfo")
    __result = []
    # Read info
    info = {}
    info["DviModes"] = []
    for k in range(__response.pop_int32()):
      info["DviModes"].append(__response.pop_int32())
    info["ScartLoopThroughModes"] = []
    for k in range(__response.pop_int32()):
      info["ScartLoopThroughModes"].append(__response.pop_int32())
    info["ScartModes"] = []
    for k in range(__response.pop_int32()):
      info["ScartModes"].append(__response.pop_int32())
    info["ColorSpaceModes"] = []
    for k in range(__response.pop_int32()):
      info["ColorSpaceModes"].append(__response.pop_int32())
    info["VideoModes"] = []
    for k in range(__response.pop_int32()):
      info["VideoModes"].append(__response.pop_int32())
    info["Id"] = __response.pop_uint32()
    __result.append(info)
    return __result[0]

  def RegisterObserver(self, applicationId, address):
    """Register an observer of type IToiVideoOutputConfigurationObserver.
    The observer will receive events sent by this IToiVideoOutputConfiguration instance.
    
    \param applicationId  The identifier of the observing application.
    \param address        The address where the IToiVideoOutputConfiguration can find an
    IToiVideoOutputConfigurationObserver interface.
    \exception TToiInvalidArgumentException
    Raised if the specified address is invalid
    or if the application has already registered
    itself as an observer with the same address.
    \see IToiVideoOutputConfigurationObserver
    """
    __request = BinIpcParameterStack()
    # Write applicationId
    __request.push_int32(applicationId)
    # Write address
    __request.push_string(address)
    self.call(__request, "IToiVideoOutputConfiguration", "RegisterObserver")

  def ReleaseObserver(self, address):
    """Release an observer previously registered with \ref RegisterObserver.
    
    \param address        The address where the IToiVideoOutputConfiguration
    can find the registered IToiVideoOutputConfigurationObserver interface.
    \exception TToiInvalidArgumentException
    Raised if the specified address is invalid.
    \see IToiVideoOutputConfigurationObserver
    """
    __request = BinIpcParameterStack()
    # Write address
    __request.push_string(address)
    self.call(__request, "IToiVideoOutputConfiguration", "ReleaseObserver")

  def CreateObserver(self):
    """Create and register an observer for the service.
       (This method is only available from Python.)
    """
    observer = ToiVideoOutputConfigurationObserver(self._ipcproxy, self)
    request = BinIpcParameterStack()
    request.push_int32(self._ipcproxy.application_id)
    request.push_string(observer.observer_address)
    self.call(request, "IToiVideoOutputConfiguration", "RegisterObserver")
    return observer
