# Copyright (c) 2011 Motorola Mobility, Inc. All rights reserved.

from kreatvipcobserver import KreaTvIpcObserver
import collections
import sys

class ToiVideoOutputConfigurationObserver(KreaTvIpcObserver):
    OnVideoOutputSettingsChangedArguments = collections.namedtuple('OnVideoOutputSettingsChangedArguments', 'info ')
    OnDisplayChangedArguments = collections.namedtuple('OnDisplayChangedArguments', 'info ')
    OnColorSystemChangedArguments = collections.namedtuple('OnColorSystemChangedArguments', 'colorSystem ')

    def __init__(self, ipcproxy, service=None, queue=None):
        KreaTvIpcObserver.__init__(self, ipcproxy, service, queue)

    def handle_callback(self, interface_name, method_name, stack):
        """
        Pushes the call details onto the log queue.
        Called on the worker thread.
        """
        call = KreaTvIpcObserver.Call(address=self.observer_address,
                                      interface=interface_name,
                                      method=method_name)
        if method_name == 'OnVideoOutputSettingsChanged':
            args = {}
            dict_info = {}
            dict_info['DviMode'] = stack.pop_int32()
            temp_AllowedDviModes = []
            for o in range(stack.pop_int32()):
              temp_AllowedDviModes.append(stack.pop_int32())
            dict_info['AllowedDviModes'] = temp_AllowedDviModes
            dict_info['ScartLoopThroughMode'] = stack.pop_int32()
            temp_AllowedScartLoopThroughModes = []
            for o in range(stack.pop_int32()):
              temp_AllowedScartLoopThroughModes.append(stack.pop_int32())
            dict_info['AllowedScartLoopThroughModes'] = temp_AllowedScartLoopThroughModes
            dict_info['ScartMode'] = stack.pop_int32()
            temp_AllowedScartModes = []
            for o in range(stack.pop_int32()):
              temp_AllowedScartModes.append(stack.pop_int32())
            dict_info['AllowedScartModes'] = temp_AllowedScartModes
            dict_info['ColorSpace'] = stack.pop_int32()
            temp_AllowedColorSpaceModes = []
            for o in range(stack.pop_int32()):
              temp_AllowedColorSpaceModes.append(stack.pop_int32())
            dict_info['AllowedColorSpaceModes'] = temp_AllowedColorSpaceModes
            dict_info['VideoMode'] = stack.pop_int32()
            temp_AllowedVideoModes = []
            for o in range(stack.pop_int32()):
              temp_AllowedVideoModes.append(stack.pop_int32())
            dict_info['AllowedVideoModes'] = temp_AllowedVideoModes
            dict_info['Index'] = stack.pop_uint32()
            dict_info['ConnectionType'] = stack.pop_int32()
            dict_info['OutputType'] = stack.pop_int32()
            dict_info['Id'] = stack.pop_uint32()
            args['info'] = dict_info
            call.arguments = ToiVideoOutputConfigurationObserver.OnVideoOutputSettingsChangedArguments(**args)
        if method_name == 'OnDisplayChanged':
            args = {}
            dict_info = {}
            temp_SupportedColorSpaces = []
            for o in range(stack.pop_int32()):
              temp_SupportedColorSpaces.append(stack.pop_int32())
            dict_info['SupportedColorSpaces'] = temp_SupportedColorSpaces
            dict_info['HasOverriddenValues'] = stack.pop_bool()
            dict_info['Status'] = stack.pop_int32()
            dict_info['AspectRatio'] = stack.pop_int32()
            dict_info['PreferredVideoMode'] = stack.pop_int32()
            temp_SupportedVideoModes = []
            for o in range(stack.pop_int32()):
              temp_SupportedVideoModes.append(stack.pop_int32())
            dict_info['SupportedVideoModes'] = temp_SupportedVideoModes
            dict_info['VideoOutputId'] = stack.pop_uint32()
            dict_info['VendorId'] = stack.pop_string()
            args['info'] = dict_info
            call.arguments = ToiVideoOutputConfigurationObserver.OnDisplayChangedArguments(**args)
        if method_name == 'OnColorSystemChanged':
            args = {}
            args['colorSystem'] = stack.pop_int32()
            call.arguments = ToiVideoOutputConfigurationObserver.OnColorSystemChangedArguments(**args)
        # Thread-safe operation:
        self._queue.append(call)

    def wait_OnVideoOutputSettingsChanged(self, discard_non_matching_events=False,
                            timeout_sec=10, **argument_filters):
        """
        Convenience function to wait for callbacks to OnVideoOutputSettingsChanged.

        Keyword arguments can be given to specify parameter values to wait for.
        If a received event does not match the filter, a KreaTvIpcException
        will be raised, unless discard_non_matching_events is true in which
        case the event will be silently ignored (and the function will continue
        to wait for a matching event).
        """
        call = self.wait_callback(self.observer_address,
                                  'IToiVideoOutputConfigurationObserver',
                                  'OnVideoOutputSettingsChanged',
                                  discard_non_matching_events,
                                  timeout_sec,
                                  argument_filters)
        return call.arguments.info

    def wait_OnDisplayChanged(self, discard_non_matching_events=False,
                            timeout_sec=10, **argument_filters):
        """
        Convenience function to wait for callbacks to OnDisplayChanged.

        Keyword arguments can be given to specify parameter values to wait for.
        If a received event does not match the filter, a KreaTvIpcException
        will be raised, unless discard_non_matching_events is true in which
        case the event will be silently ignored (and the function will continue
        to wait for a matching event).
        """
        call = self.wait_callback(self.observer_address,
                                  'IToiVideoOutputConfigurationObserver',
                                  'OnDisplayChanged',
                                  discard_non_matching_events,
                                  timeout_sec,
                                  argument_filters)
        return call.arguments.info

    def wait_OnColorSystemChanged(self, discard_non_matching_events=False,
                            timeout_sec=10, **argument_filters):
        """
        Convenience function to wait for callbacks to OnColorSystemChanged.

        Keyword arguments can be given to specify parameter values to wait for.
        If a received event does not match the filter, a KreaTvIpcException
        will be raised, unless discard_non_matching_events is true in which
        case the event will be silently ignored (and the function will continue
        to wait for a matching event).
        """
        call = self.wait_callback(self.observer_address,
                                  'IToiVideoOutputConfigurationObserver',
                                  'OnColorSystemChanged',
                                  discard_non_matching_events,
                                  timeout_sec,
                                  argument_filters)
        return call.arguments.colorSystem

