# -----------------------------------------------------------------------------
#
# Copyright (c) 2011 Motorola Mobility, Inc. All rights reserved.
#
# -----------------------------------------------------------------------------

from ktestcase import KattTestCase
from subtitletestenv import SubtitleTestEnv
import rteutils
import toi

class SubtitleTestCase(KattTestCase, SubtitleTestEnv):

    # Get expected (actual) subtitle info of a stream in compact format.
    def get_expected_subtitles(self, stream_name):
        subtitles = []
        if stream_name == 'kms1178':
            base_subtitle = { 'CompType': self._player.COMPONENT_SUBTITLE,
                              'Language': 'swe',
                              'Type': self._player.SUBTITLE_NORMAL,
                              'Format': self._player.SUBTITLE_DVB,
                              'AspectRatio': self._player.ASPECT_RATIO_16_9
                            }
            for lang in ('swe', 'fre'):
                new_subtitle = base_subtitle.copy()
                new_subtitle['Language'] = lang
                subtitles.append(new_subtitle)
        elif stream_name == 'kms0018':
            base_subtitle = { 'CompType': self._player.COMPONENT_SUBTITLE,
                              'Language': 'swe',
                              'Type': self._player.SUBTITLE_NORMAL,
                              'Format': self._player.SUBTITLE_TELETEXT,
                              'AspectRatio': self._player.ASPECT_RATIO_UNKNOWN
                            }
            for lang in ('swe', 'dan', 'nor', 'fra', 'fin'):
                new_subtitle = base_subtitle.copy()
                new_subtitle['Language'] = lang
                subtitles.append(new_subtitle)
   
        elif stream_name == 'kms0625':    
            base_subtitle = { 'CompType': self._player.COMPONENT_SUBTITLE,
                              'Language': 'swe',
                              'Type': self._player.SUBTITLE_NORMAL,
                              'Format': self._player.SUBTITLE_TELETEXT,
                              'AspectRatio': self._player.ASPECT_RATIO_UNKNOWN
                            }
            for lang in ('swe', 'dan', 'nor', 'fin'):
                new_subtitle = base_subtitle.copy()
                new_subtitle['Language'] = lang
                subtitles.append(new_subtitle)

            base_subtitle['Format'] = self._player.SUBTITLE_DVB
            for lang in ('swe', 'eng', 'chi'):
                new_subtitle = base_subtitle.copy()
                new_subtitle['Language'] = lang
                subtitles.append(new_subtitle)

        elif stream_name == 'kms0050':
            new_subtitle = { 'CompType': self._player.COMPONENT_SUBTITLE,
                             'Language': 'nor',
                             'Type': self._player.SUBTITLE_NORMAL,
                             'Format': self._player.SUBTITLE_TELETEXT,
                             'AspectRatio': self._player.ASPECT_RATIO_UNKNOWN
                           }
            subtitles.append(new_subtitle)

            new_subtitle = new_subtitle.copy()
            new_subtitle['Format'] = self._player.SUBTITLE_DVB
            subtitles.append(new_subtitle)

            new_subtitle = new_subtitle.copy()
            new_subtitle['Type'] = self._player.SUBTITLE_HEARING_IMPAIRED
            subtitles.append(new_subtitle)

        elif stream_name == 'kms1699':
            new_subtitle = { 'CompType': self._player.COMPONENT_SUBTITLE,
                             'Language': 'fra',
                             'Type': self._player.SUBTITLE_HEARING_IMPAIRED,
                             'Format': self._player.SUBTITLE_TELETEXT,
                             'AspectRatio': self._player.ASPECT_RATIO_UNKNOWN
                           }
            subtitles.append(new_subtitle)

            new_subtitle = new_subtitle.copy()
            new_subtitle['Format'] = self._player.SUBTITLE_DVB
            subtitles.append(new_subtitle)

            new_subtitle = new_subtitle.copy()
            new_subtitle['Language'] = 'eng'
            new_subtitle['Type'] = self._player.SUBTITLE_NORMAL
            subtitles.append(new_subtitle)
     
        # Sort subtitles before return.
        subtitles.sort()
        return subtitles

    def setUp(self):
        self._rte = rteutils.createRteConnection(self.stbIp)
        self._proxy = rteutils.IpcProxy(self._rte, self.stbIp)
        self._proxy.enable_callbacks()

        self.kill_applications(self._rte)

        self._media_service = toi.ToiMediaService(self._proxy)
        address = self._media_service.CreatePlayerInstance(
            self._proxy.application_id)
        self._player = toi.ToiMediaPlayer(self._proxy, address)
        self._observer = self._player.CreateObserver()

        self._is = toi.ToiInformationService(self._proxy)

    def tearDown(self):
        self.safe_close()
        self._player.ReleaseInstance()
        self._proxy.release()

    # Plays a media stream, stopping the current stream if one is playing.
    def safe_play(self, uri):
        self.safe_close()
        self._player.Open(uri)
        self._player.Play(1000)
    
    # Closes a stream even if it is not running
    def safe_close(self):
        result = self._player.GetState()
        if result != self._player.STATE_IDLE:
            try:
                self._player.Close()
            except:
                pass

    # Get all subtitle info from current stream
    def get_all_subtitles(self):
        components = self._player.GetStreamInfo()["AvailableComponents"]

        self._all_subtitles = []
        for comp in components:
            if comp["Type"] == self._player.COMPONENT_SUBTITLE:
                info = self._player.GetSubtitleStreamInfo(comp)
                self._all_subtitles.append({"Comp" : comp,
                                            "Info" : info})
            elif comp["Type"] == self._player.COMPONENT_SUBTITLE_CAPTION \
                 or comp["Type"] == self._player.COMPONENT_SUBTITLE_SUPERIMPOSE:
                self._all_subtitles.append({"Comp" : comp})


    # Get current subtitle info in compact format, for comparing.
    def get_compact_subtitles(self):
        compact_subtitles = []
        for subtitle in self._all_subtitles:
            comp = subtitle["Comp"]
            info = subtitle["Info"]
            if comp["Type"] == self._player.COMPONENT_SUBTITLE:
                compact_subtitles.append({"CompType"    : comp["Type"],
                                          "Language"    : info["Language"],
                                          "Type"        : info["Type"],
                                          "Format"      : info["Format"],
                                          "AspectRatio" : info["AspectRatio"]
                                         });
            else:
                compact_subtiles.append({"CompType"    : comp["Type"]})

        compact_subtitles.sort()
        return compact_subtitles
    
    def filter_subtitles(self, components):
        subtitles = []
        for comp in components:
            if comp["Type"] == self._player.COMPONENT_SUBTITLE \
                or comp["Type"] == self._player.COMPONENT_SUBTITLE_CAPTION \
                or comp["Type"] == self._player.COMPONENT_SUBTITLE_SUPERIMPOSE:
                subtitles.append(comp)
        return subtitles

    # Get current active subtitle
    def get_active_subtitle(self):
        components = self._player.GetActiveComponents()
        return self.filter_subtitles(components)

    # Get a subtitle component by subtitle info (language, type, format)
    def get_component_by_info(self, lang, t=None, f=None):
        comp = None
        for subtitle in self._all_subtitles:
            if subtitle["Comp"]["Type"] == self._player.COMPONENT_SUBTITLE \
                and subtitle["Info"]["Language"] == lang \
                and (t == None or subtitle["Info"]["Type"] == t) \
                and (f == None or subtitle["Info"]["Format"] == f):
                comp = subtitle["Comp"]
                break
        return comp

    # Wait until all expected subtitles appear in current stream.
    def wait_all_subtitles(self, stream_name):
        expected_subtitles = self.get_expected_subtitles(stream_name)
       
        # Try several times after starting playback.
        self._observer.wait_OnStateChanged(True, reason='CommandPlay',
                                           state=self._player.STATE_PLAYING)
        self._rte.system.PrintToLog('wait all subtitles: STATE_PLAYING')

        matched = False
        try_cnt = 0
        while try_cnt < 10:
            args = self._observer.wait_OnStreamInfoChanged(True)
            self.get_all_subtitles()
            compact_subtitles = self.get_compact_subtitles()

            if len(args.availableComponents) == 0 \
                or len(args.changedComponents) == 0:
                continue

            if len(compact_subtitles) > len(expected_subtitles):
                break

            matched = True
            i = 0
            for j in range(len(compact_subtitles)):
                while i < len(expected_subtitles):
                    if compact_subtitles[j] == expected_subtitles[i]:
                        break
                    else:
                        i = i + 1
                if i >= len(expected_subtitles):
                    matched = False
                    break
                i = i + 1

            if not matched or len(compact_subtitles) == len(expected_subtitles):
                break
            else:
                matched = False
                try_cnt = try_cnt + 1
        
        if not matched:
            print 'Expected subtitles: ', expected_subtitles
            print 'Actual subtitles: ', compact_subtitles
        return matched

    # Wait until expected active subtitle appear in current stream.
    def wait_active_subtitle(self, expected_subtitle):
        if expected_subtitle == self.get_active_subtitle():
            return True
        
        # Try several times.
        for try_cnt in range(5):
            args = self._observer.wait_OnActiveComponentChanged(True)
            if expected_subtitle == [args]:
                return True
        return False

    # Safely play a stream and wait for expected subtitles.
    def safe_play_wait(self, stream_name):
        uri = self.get_uri(stream_name)
        print 'Playing stream %s: %s' % (stream_name, uri)

        self.safe_play(uri)
        matched = self.wait_all_subtitles(stream_name)
        self.assertTrue(matched, 'Failed to detect expected subtitles.')
 

    # Set subtitling information object.
    def set_is(self, obj, value):
        self._is.SetObject('cfg.media.subtitling.'+ obj, value, self._is.STORAGE_VOLATILE)

