# -----------------------------------------------------------------------------
#
# Copyright (c) 2011 Motorola Mobility, Inc. All rights reserved.
#
# -----------------------------------------------------------------------------
import os
import time
import kresource

class SubtitleTestEnv():
    """ Enviroment settings for KATT subtitle testing.
    """

    # Environment variable name prefix
    _env_prefix = 'KATT_STREAM_URI_'

    # Default URIs for subtitle testing streams. To overwrite them, set
    # environment variables like:
    #   KATT_STREAM_URI_kms0018=224.1.1.131:11111
    # Required streams include:
    #   kms1178 kms0018 kms0050 kms1699 kms0625

    # Get URI from stream name.
    def get_uri(self, stream_name):
        env_name = self._env_prefix + stream_name
        if env_name in os.environ:
            uri = os.environ[env_name]
        else:
            res_name = "SUBTITLE_" + stream_name.upper() + "_STREAM_URL"
            uri = kresource.get_localized_resource(res_name)
        return uri

    def kill_applications(self, rte):
        # The function makes sure no applications are running except for KATT
        # and RTE. This is to ensure RTE can use KreaTvGfx for subtitle
 	# renderer wrapper without problems. Also we expect that subtitle
 	# testcases are not affected by failure of other testcases.
        appIds = rte.app.GetApplicationIds()
            for appId in appIds:
 		appInfo = rte.app.GetInfo(appId)
 		# State = STARTING, INVISIBLE, VISIBLE or ACTIVE
 		if ((appInfo["State"] == 3 or
 		    appInfo["State"] == 4 or
 		    appInfo["State"] == 5 or
 		    appInfo["State"] == 6) \
 		    and appInfo["Name"].find("RTE") == -1 \
 		    and appInfo["Name"].find("KATT") == -1):
 		    # Kill all applications that are not RTE or KATT
 		 rte.app.Kill(appId)
 		                # Wait a bit to make sure application is really
 		# dead. Would have been nice to wait for state change
 		# events...
 		 time.sleep(1)
