# -----------------------------------------------------------------------------
#
# Copyright (c) 2011 Motorola Mobility, Inc. All rights reserved.
#
# This program is confidential and proprietary to Motorola Mobility, Inc and
# may not be copied, reproduced, disclosed to others, published or used, in
# whole or in part, without the expressed prior written permission of Motorola
# Mobility, Inc.
#
# -----------------------------------------------------------------------------

from subtitletestcase import SubtitleTestCase
import ktestrunner

#KATTrequirement: COMP-6(v2), Activation of automatic subtitle selection.
#KATTrequirement: COMP-7(v2), Automatic subtitle selection - language codes.
#KATTrequirement: COMP-8(v2), Automatic subtitle selection - language list.

class TestSubtitleLanguage(SubtitleTestCase):

    def test_subtitle_language(self):
        self.safe_play_wait('kms1178')

        self.set_is('autoselection', 'TRUE')
        self.set_is('typepriority', 'normal,hearing_impaired')
        self.set_is('modepriority', 'DVB,Teletext')

        # test config format: [expected component language, language priority]
        # 1. Test different language codes in the prioritized list.
        # 2. Test to add and remove language codes from the prioritized list.
        #    (Behaviour shall follow the updated list of language codes)
        # 3. Test the same language code in both lower and upper case.
        #    (Behaviour shall be agnostic to case)
        # 4. Test to vary the B/T code, i.e. "fre" or "fra" for french.
        #    (Behaviour shall be agnostic to B/T code)
        test_configs = [['fre', 'fra,swe'],
                        ['swe', 'swe,fra'],
                        ['swe', 'eng,swe,fra'],
                        ['fre', 'fra,eng'],
                        ['swe', 'SWE,FRA'],
                        ['fre', 'fre,swe']
                       ]

        for config in test_configs:
            self.set_is('languagepriority', config[1])
            comp = self.get_component_by_info(config[0])
            self.assertTrue(self.wait_active_subtitle([comp]))
       
        # Test that if no language code matches the stream content,
        # NO subtitles shall be displayed.
        self.set_is('languagepriority', 'eng,dan')
        self.assertTrue(self.wait_active_subtitle([]))

        # Test to add language codes that are 2 characters and 4 characters.
        # (Negative test)
        self.set_is('languagepriority', 'fr,fran')
        self.assertTrue(self.wait_active_subtitle([]))

        self.set_is('languagepriority', 'swe,fra')

        # Close media player.
        self.safe_close()


if __name__ == '__main__':
    ktestrunner.RunTests()

