# -----------------------------------------------------------------------------
#
# Copyright (c) 2011 Motorola Mobility, Inc. All rights reserved.
#
# This program is confidential and proprietary to Motorola Mobility, Inc and
# may not be copied, reproduced, disclosed to others, published or used, in
# whole or in part, without the expressed prior written permission of Motorola
# Mobility, Inc.
#
# -----------------------------------------------------------------------------

from subtitletestcase import SubtitleTestCase
import ktestrunner

#KATTrequirement: COMP-6(v2), Activation of automatic subtitle selection.
#KATTrequirement: COMP-11, Automatic subtitle selection - priority order.

class TestSubtitlePriorityOrder(SubtitleTestCase):

    def check_lang_type(self):
        self.safe_play_wait('kms1699')
 
        self.set_is('autoselection', 'TRUE')
        self.set_is('modepriority', 'DVB,Teletext')

        # Language has higher priority than type
        self.set_is('languagepriority', 'eng,fra')
        self.set_is('typepriority', 'hearing_impaired,normal')
        comp = self.get_component_by_info('eng')
        self.assertTrue(self.wait_active_subtitle([comp]))

        self.set_is('languagepriority', 'fra,eng')
        self.set_is('typepriority', 'normal,hearing_impaired')
        comp = self.get_component_by_info('fra', f=self._player.SUBTITLE_DVB)
        self.assertTrue(self.wait_active_subtitle([comp]))

        self.set_is('languagepriority', 'eng,fra')
        self.set_is('typepriority', 'normal,hearing_impaired')
        comp = self.get_component_by_info('eng')
        self.assertTrue(self.wait_active_subtitle([comp]))

        # Close media player.
        self.safe_close()

    def check_type_mode(self):
        self.safe_play_wait('kms0050')
 
        self.set_is('autoselection', 'TRUE')
        self.set_is('languagepriority', 'nor,swe')

        # Type has higher priority than mode
        self.set_is('typepriority', 'hearing_impaired,normal')
        self.set_is('modepriority', 'DVB,Teletext')
        comp = self.get_component_by_info('nor', t=self._player.SUBTITLE_HEARING_IMPAIRED,
                                          f=self._player.SUBTITLE_DVB)
        self.assertTrue(self.wait_active_subtitle([comp]))

        self.set_is('typepriority', 'normal,hearing_impaired')
        comp = self.get_component_by_info('nor', t=self._player.SUBTITLE_NORMAL,
                                          f=self._player.SUBTITLE_DVB)
        self.assertTrue(self.wait_active_subtitle([comp]))

        self.set_is('typepriority', 'hearing_impaired,normal')
        self.set_is('modepriority', 'Teletext,DVB')
        comp = self.get_component_by_info('nor', t=self._player.SUBTITLE_HEARING_IMPAIRED,
                                          f=self._player.SUBTITLE_DVB)
        self.assertTrue(self.wait_active_subtitle([comp]))

        # Close media player.
        self.safe_close()

    def check_lang_mode(self):
        self.safe_play_wait('kms0625')

        self.set_is('autoselection', 'TRUE')
        self.set_is('typepriority', 'normal,hearing_impaired')

        # Language has higher priority than mode
        self.set_is('languagepriority', 'dan,eng')
        self.set_is('modepriority', 'DVB,Teletext')
        comp = self.get_component_by_info('dan', f=self._player.SUBTITLE_TELETEXT)
        self.assertTrue(self.wait_active_subtitle([comp]))

        self.set_is('languagepriority', 'eng,nor')
        self.set_is('modepriority', 'Teletext,DVB')
        comp = self.get_component_by_info('eng', f=self._player.SUBTITLE_DVB)
        self.assertTrue(self.wait_active_subtitle([comp]))

        # Close media player.
        self.safe_close()

    def test_subtitle_priority_order(self):
        self.check_lang_type()
        self.check_type_mode()
        self.check_lang_mode()

if __name__ == '__main__':
    ktestrunner.RunTests()

