# -----------------------------------------------------------------------------
#
# Copyright (c) 2011 Motorola Mobility, Inc. All rights reserved.
#
# This program is confidential and proprietary to Motorola Mobility, Inc and
# may not be copied, reproduced, disclosed to others, published or used, in
# whole or in part, without the expressed prior written permission of Motorola
# Mobility, Inc.
#
# -----------------------------------------------------------------------------

from subtitletestcase import SubtitleTestCase
import ktestrunner

#KATTrequirement: COMP-6(v2), Activation of automatic subtitle selection.
#KATTrequirement: COMP-12(v2), Automatic subtitle selection - handling changes

class TestSubtitleZap(SubtitleTestCase):

    def test_subtitle_zap(self):
        self.safe_play_wait('kms1178')

        self.set_is('autoselection', 'TRUE')
        self.set_is('languagepriority', 'swe,fra')
        self.set_is('typepriority', 'normal,hearing_impaired')
        self.set_is('modepriority', 'DVB,Teletext')

        # 'swe' should be auto selected.
        comp = self.get_component_by_info('swe')
        self.assertTrue(self.wait_active_subtitle([comp]))

        # Select 'fre' manually
        comp = self.get_component_by_info('fre')
        self._player.ActivateComponent(comp)
        self.assertTrue(self.wait_active_subtitle([comp]))

        # Zap to another stream
        self.safe_play_wait('kms0625')

        # 'swe' should be auto selected again.
        comp = self.get_component_by_info('swe', f=self._player.SUBTITLE_DVB)
        self.assertTrue(self.wait_active_subtitle([comp]))

        # Trun-off automatic subtitle selection
        self.set_is('autoselection', 'FALSE')
        self.assertTrue(self.wait_active_subtitle([]))

        # Trun-on automatic subtitle selection again
        self.set_is('autoselection', 'TRUE')
        self.assertTrue(self.wait_active_subtitle([comp]))

        # Close media player.
        self.safe_close()

if __name__ == '__main__':
    ktestrunner.RunTests()

