# -----------------------------------------------------------------------------
#
# Copyright (c) 2011 Motorola, Inc. All Rights Reserved.
#
# -----------------------------------------------------------------------------

# KATTlevel: 2
# KATTtimeout: 20
# KATTrequirement: GFX-3, GFX-4, GFX-7, GFX-8
# KATTdashboard: 3
# KATTdisabled: all, BUGZ:16143, Problem with bliting when alpha blending enabled.

import xmlrpclib

import os
import sys

cwd = os.path.dirname(os.path.realpath(__file__))
sys.path.append(cwd + "/../../common/py")

import ktestrunner
from ktestcase import KattTestCase
import krtehelpers
from ctypes import c_uint32


class TestBlitterInterface(KattTestCase):

    def setUp(self):

        try:
            self.rte = xmlrpclib.ServerProxy("http://" + self.stbIp + ":8888/")
        except:
            raise Exception("Unable to establish RTE connection")

        krtehelpers.wait_for_rte_startup(self.rte, 10)

    def test_interfaces(self):

        passed = 0
        failed = 0
        self.rte.kgfxmark.Setup()
        for i in range(self.rte.kgfxmark.GetTestCount()):
            result = self.rte.kgfxmark.RunInterfaceTest(i)
            if result['result'] == 1:
                passed = passed + 1
            else:
                failed = failed + 1
            name = self.rte.kgfxmark.GetName(i)
            try:
                group = self.rte.kgfxmark.GetGroup(i) + "_"
            except:
                group = ""

            print "#%2.2d: %50.50s: success %d: %50.50s: " \
                  "BlendingEnabled: %d: %s[0x%X->0x%X]: " \
                  "expected 0x%X, actual 0x%X" % \
                  (i, \
                  name,
                  result['result'], \
                  result['msg'], \
                  result['isBlendingEnabled'], \
                  result['type'], \
                  c_uint32(result['fgColor']).value, \
                  c_uint32(result['bgColor']).value, \
                  c_uint32(result['expected']).value, \
                  c_uint32(result['actual']).value)

        self.rte.kgfxmark.Teardown()

        print 'PASSED: ' + str(passed) + ' FAILED:' + str(failed) + \
            ' TOTAL: ' + str(i + 1)

        if failed > 0:
            raise Exception(str(failed) + ' tests failed')

if __name__ == '__main__':
    ktestrunner.RunTests()
