/*
 *---------------------------------------------------------------------------
 *
 * kbox.h
 *
 * Copyright (c) 2003 Kreatel Communications AB
 * Copyright (c) 2008-2009 Motorola, Inc.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the following disclaimer in the
 * documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 * derived from this software without specific prior written permission.
 *
 * Alternatively, this software may be distributed under the terms of the
 * GNU General Public License ("GPL") version 2 as published by the Free
 * Software Foundation.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
 * IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 *---------------------------------------------------------------------------
 */

#ifndef _LINUX_KBOX_H
#define _LINUX_KBOX_H

#ifdef __KERNEL__

#include <linux/ioctl.h>
#include <linux/types.h>

extern int read_xml_config(char *tag, char *buf, size_t size);
extern int read_xml_user_config(char *tag, char *buf, size_t size);

#endif /* __KERNEL__ */

#define KBOX_MAJOR        		120
#define KBOX_DEVICE_NAME  		"kbox"
#define KPROGRESS_MAJOR        		121
#define KPROGRESS_DEVICE_NAME  		"kprogress"

/* Scart modes */
#define KBOX_SCART_MODE_OFF	0x01
#define KBOX_SCART_MODE_AV	0x02
#define KBOX_SCART_MODE_TV	0x04
#define KBOX_SCART_MODE_WS	0x08
#define KBOX_SCART_MODE_UNKNOWN	0x10
#define KBOX_IOCTL_GET_SCART_MODE	_IOR('K',1,int)
#define KBOX_IOCTL_SCART_MODE	_IO('K',1)

/* Watchdog */
#define KBOX_IOCTL_SET_WD	_IO('K',2)
#define KBOX_IOCTL_FEED_WD	_IO('K',3)

/* Misc */
#define KBOX_IOCTL_PWR_SWITCH	_IOR('K',4,int)
#define KBOX_IOCTL_TVMUX	_IO('K',5)

/* TV signal modes */
#define KBOX_TV_MODE_OFF	0x00
#define KBOX_TV_MODE_CVBS	0x01
#define KBOX_TV_MODE_SVIDEO	0x02
#define KBOX_TV_MODE_RGB	0x04
#define KBOX_IOCTL_TV_MODE	_IO('K',6)
#define KBOX_IOCTL_GET_TV_MODE	_IOR('K',7, int)

/* Hardware type */
#define KBOX_HW_UNKNOWN		0
#define KBOX_HW_KBOX1		1
#define KBOX_HW_SVB		2
#define KBOX_HW_KBOX2		3
#define KBOX_IOCTL_GET_HW_TYPE	_IOR('K',8,int)

/* Boot image */
#define KBOX_BOOT_UNKNOWN	-1
#define KBOX_BOOT_OLD_RBL	0
#define KBOX_BOOT_DBL1		1
#define KBOX_BOOT_DBL2		2
#define KBOX_IOCTL_GET_BOOT_IMAGE _IOR('K',9,int)

#define KBOX_IOCTL_GET_AUX_SCART _IOR('K',12,int)

/* Framebuffer details */
#define KBOX_IOCTL_GET_HEIGHT _IOR('K',13, int)
#define KBOX_IOCTL_GET_WIDTH  _IOR('K',14, int)
#define KBOX_IOCTL_GET_PITCH  _IOR('K',15, int)
#define KBOX_IOCTL_GET_OFFSET _IOR('K',16, int)

#define KBOX_PIX_FORMAT_565       1
#define KBOX_PIX_FORMAT_8888      2
#define KBOX_IOCTL_GET_PIXEL_FORMAT _IOR('K',17, int)

struct kbox_gfx_param {
	unsigned long p1;
	unsigned long p2;
	unsigned long p3;
	unsigned long p4;
	unsigned long p5;
};

/* Graphics */
#define KBOX_GFX_FILL_RECT_AREA _IOW('K',18, struct kbox_gfx_param)

#define KBOX_TV_SYSTEM_UNKNOWN  0
#define KBOX_TV_SYSTEM_PAL      1
#define KBOX_TV_SYSTEM_NTSC     2
#define KBOX_GFX_GET_TV_SYSTEM  _IOR('K', 18, int)

#if 0
/* STC */
#define KBOX_IOCTL_GET_STC _IOR('K',19,sizeof(unsigned long long))

/* PCR */
#define KBOX_IOCTL_GET_PCR _IOR('K',20,sizeof(unsigned long long))
#endif

/* Add proc files */
#define KBOX_PROC_DATA_PRODPARAM_INDEX         0
#define KBOX_PROC_DATA_USERPARAM_INDEX         1
#define KBOX_PROC_DATA_USPPARAM_INDEX          2
#define KBOX_PROC_DATA_USPPRODPARAM_INDEX      3
#define KBOX_PROC_DATA_CAKEY_INDEX             4
#define KBOX_PROC_DATA_HDCPKEY_INDEX           5
#define KBOX_PROC_DATA_USERPARAMOVERRIDE_INDEX 6
#define KBOX_PROC_DATA_ENTRIES                 7
struct kbox_add_proc_param
{
  unsigned int Index;
  char Path[64];
  char RootTag[64];
  unsigned int DataSize;
  unsigned char* Data;

};

#define KBOX_IOCTL_ADD_PROC_DATA _IOW('K',24, struct kbox_add_proc_param)

#endif /* _LINUX_KBOX_H */
