/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
*---------------------------------------------------------------------------
*
* TErrorCodeException.h
*
* Copyright (c) 2007-2008 Motorola Inc. All Rights Reserved.
*
*---------------------------------------------------------------------------
*/

#ifndef TERRORCODEEXCEPTION_H
#define TERRORCODEEXCEPTION_H

#include "TException.h"

#include <cstdio>

class TErrorCodeException : public TException
{
private:
  const int ErrorCode;

  
  
  
  
  mutable std::string ErrorMessage;

  void CreateErrorMessage() const throw ();

public:
  TErrorCodeException(const std::string& message, int errorCode) throw ();
  virtual ~TErrorCodeException() throw ();

  int GetErrorCode() const throw ();

  
  
  virtual std::string GetErrorCodeString() const throw ();

  
  virtual const char* GetText() const throw ();
  virtual const std::string& GetString() const throw ();
};

inline void TErrorCodeException::CreateErrorMessage() const throw ()
{
  if (ErrorMessage.empty()) {
    ErrorMessage = TException::GetString() + ": " + GetErrorCodeString();
  }
}

inline TErrorCodeException::
TErrorCodeException(const std::string& message, int errorCode) throw ()
  : TException(message),
    ErrorCode(errorCode)
{
  
}

inline TErrorCodeException::~TErrorCodeException() throw ()
{
  
}

inline int TErrorCodeException::GetErrorCode() const throw ()
{
  return ErrorCode;
}

inline std::string TErrorCodeException::GetErrorCodeString() const throw ()
{
  char str[32];
  ::snprintf(str, sizeof(str), "error %d", GetErrorCode());
  return str;
}

inline const char* TErrorCodeException::GetText() const throw ()
{
  CreateErrorMessage();
  return ErrorMessage.c_str();
}

inline const std::string& TErrorCodeException::GetString() const throw ()
{
  CreateErrorMessage();
  return ErrorMessage;
}

#endif
