/*
 *--------------------------------------------------------------------
 *
 * TCIpcCallHandler.h
 *
 * This is the C port of the old cpp TIpcEventHandler.
 * 
 * Copyright (c) 2004 Kreatel Communications AB. All Rights Reserved.
 * Copyright (c) 2008 Motorola, Inc. All Rights Reserved.
 *
 *--------------------------------------------------------------------
 */

#ifndef TCIPCCALLHANDLER_H
#define TCIPCCALLHANDLER_H

#ifdef __cplusplus
extern "C" {
#endif

#include "IpcDefs.h"

#include "bin-ipc/TBinIpcParameterStack.h"

#include <sys/socket.h>
  
#define IPC_HANDLER_MAX_OBJECTS     192
#define IPC_HANDLER_MAX_CONNECTIONS  96



typedef bool (*TDispatchFunction) (const char* interfaceName,
                                   const char* methodName,
                                   TBinIpcParameterStack* request,
                                   bool* errorResponse, 
                                   TBinIpcParameterStack* response, 
                                   void* userData);




typedef void (*TAddDescriptorFunction)(int fd, void* userData);
typedef void (*TRemoveDescriptorFunction)(int fd, void* userData);

typedef void (*TOnConnectionClosedFunction)(int pid, void* userData);


typedef struct TCIpcHandlerObjectStruct
{
  uint32_t Id;
  TDispatchFunction Dispatch;
  void* UserData;
} TCIpcHandlerObject;

typedef struct TCIpcHandlerConnectionStruct
{
  int Fd;
  struct ucred PeerInfo;
} TCIpcHandlerConnection;

typedef struct TCIpcCallHandlerStruct
{
  TAddDescriptorFunction AddDescriptor;
  TRemoveDescriptorFunction RemoveDescriptor;
  
  void* DescriptorUserData;

  TOnConnectionClosedFunction OnConnectionClosed;
  
  void* ObserverUserData;

  char* ServerSocketName;
  int ServerSocketFd;

  TCIpcHandlerObject Objects[IPC_HANDLER_MAX_OBJECTS];
  uint32_t ObjectCount;
  uint32_t NextObjectId;
  bool IgnoreUnknownObjects;

  TCIpcHandlerConnection Connections[IPC_HANDLER_MAX_CONNECTIONS];
  uint32_t ConnectionCount;
  
  TCIpcHandlerConnection* CurrentConnection;

  char* ErrorMessage;
  bool Suspended;
} TCIpcCallHandler;


void TCIpcCallHandler_FlushCalls(TCIpcCallHandler* _this, int fd);

void TCIpcCallHandler_Init(TCIpcCallHandler* _this,
                           bool ignoreUnknownObjects);
void TCIpcCallHandler_CleanUp(TCIpcCallHandler* _this);

void TCIpcCallHandler_CloseServerSocket(TCIpcCallHandler* _this);


void TCIpcCallHandler_SetAddress(TCIpcCallHandler* _this,
                                 const char* address);
void TCIpcCallHandler_SetEventLoop(TCIpcCallHandler* _this,
                                   TAddDescriptorFunction addDescriptor,
                                   TRemoveDescriptorFunction removeDescriptor,
                                   void* userData);
void TCIpcCallHandler_SetConnectionObserver(TCIpcCallHandler* _this,
                                            TOnConnectionClosedFunction
                                              onConnectionClosed,
                                            void* userData);

void TCIpcCallHandler_ReleaseEventLoop(TCIpcCallHandler* _this);


void TCIpcCallHandler_SuspendEvents(TCIpcCallHandler* _this, bool suspend);





void TCIpcCallHandler_CreateObject(TCIpcCallHandler* _this,
                                   char* name, uint32_t size);

void TCIpcCallHandler_AddDispatcher(TCIpcCallHandler* _this,
                                    const char* address,
                                    TDispatchFunction dispatch,
                                    void* userData);
void TCIpcCallHandler_RemoveDispatcher(TCIpcCallHandler* _this,
                                       const char* address);



int TCIpcCallHandler_GetPeerPid(TCIpcCallHandler* _this);
int TCIpcCallHandler_GetCallDescriptor(TCIpcCallHandler* _this);
int TCIpcCallHandler_GetPeerUid(TCIpcCallHandler* _this);
int TCIpcCallHandler_GetPeerGid(TCIpcCallHandler* _this);


bool TCIpcCallHandler_IsDescriptorConnected(const TCIpcCallHandler* _this, int fd);



int TCIpcCallHandler_GetCurrentConnectionFd(TCIpcCallHandler* _this);

bool TCIpcCallHandler_HandleEvent(TCIpcCallHandler* _this,
                                  int fileDescriptor);

#ifdef __cplusplus
}
#endif

#endif
