







#ifndef C_IPC_TCIPCCLIENT_H
#define C_IPC_TCIPCCLIENT_H

#ifdef __cplusplus
extern "C" {
#endif

#include <stdint.h>
#include <stdbool.h>
#include "bin-ipc/TBinIpcParameterStack.h"

/* The IPC client implements a connection pool where existing
   connections are reused as much as possible. Connections are kept
   open until one of the following events occur:

   1. A connection to a new socket is requested but the connection
      limit has already been reached. In this case a LRU (Least
      Recently Used) algorithm determines which existing connection
      should be closed.

   2. An I/O error (e g timeout or protocol error) is detected.

   3. The users calls CloseConnections().

   4. The IPC client is destructed.
*/

#define IPC_MAX_CONNECTIONS 64

#define IPC_CALL_TIMEOUT 5000
#define IPC_CALL_TIMEOUT_WARNING 750  

typedef struct TCIpcClientConnectionStruct
{
  char* SocketName;
  int FileDescriptor;
  int PeerPid;
  uint32_t TimeStamp;
} TCIpcClientConnection;


typedef struct TCIpcClientStruct
{
  TCIpcClientConnection ConnectionList[IPC_MAX_CONNECTIONS];
  uint32_t ConnectionCount;
  uint32_t NextTimeStamp;
  
  
  uint32_t CallerCount;
  
  
  
  TCIpcClientConnection* CurrentConnection;

  char* ErrorMessage;
  bool DisableTimeout;
} TCIpcClient;


bool TCIpcClient_ValidateAddress(const char* address);


void TCIpcClient_Init(TCIpcClient* _this);
void TCIpcClient_CleanUp(TCIpcClient* _this);



void TCIpcClient_CloseConnections(TCIpcClient* _this);

void TCIpcClient_AddCaller(TCIpcClient* _this, const char* address);
void TCIpcClient_RemoveCaller(TCIpcClient* _this, const char* address);

bool TCIpcClient_InvokeOneway(TCIpcClient* _this,
                              const char* objectName,
                              const char* interfaceName,
                              const char* methodName, 
                              TBinIpcParameterStack* reqestStack);

bool TCIpcClient_Invoke(TCIpcClient* _this,
                        const char* objectName,
                        const char* interfaceName,
                        const char* methodName, 
                        TBinIpcParameterStack* reqestStack,
                        bool* errorResponse,
                        TBinIpcParameterStack* responseStack);

#ifdef __cplusplus
}
#endif

#endif
