/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *---------------------------------------------------------------------------
 *
 * comm/emio/TDatagramSocket.h
 *
 * Copyright (c) 2003 Kreatel Communications AB. All Rights Reserved.
 *
 *---------------------------------------------------------------------------
 */

#ifndef COMM_EMIO_TDATAGRAMSOCKET_H
#define COMM_EMIO_TDATAGRAMSOCKET_H

#include "TSocket.h"

#include <cerrno>
#include <sys/socket.h>
#include <sys/un.h>

class TDatagramSocket : public TSocket
{
protected:
  sockaddr ToAddr;

  void Open(int protocolFamily);

public:
  TDatagramSocket(bool doOpen, int protocolFamily);
  
  virtual int Write(const void* buffer, unsigned long size);
  virtual int Write(const void* buffer, unsigned long size, sockaddr_un* addr);
  int Read(void* buffer, unsigned long size);
  int Read(void* buffer, unsigned long size,
           sockaddr* from, socklen_t& fromlen);
  void Flush();
};

inline TDatagramSocket::TDatagramSocket(bool doOpen, int protocolFamily)
{
  if (doOpen) {
    TSocket::Open(protocolFamily, SOCK_DGRAM);
  }
}

inline void TDatagramSocket::Open(int protocolFamily)
{
  TSocket::Open(protocolFamily, SOCK_DGRAM);
}


inline int TDatagramSocket::Write(const void* buffer, unsigned long size)
{
  while (true) {
    int result = ::sendto(Descriptor, buffer, size, MSG_NOSIGNAL,
                          &ToAddr, sizeof(sockaddr));
    if (!(result == -1 && errno == EINTR)) {
      return result;
    }
  }
}


inline int TDatagramSocket::Write(const void* buffer, unsigned long size, 
  sockaddr_un* addr )
{
  while (true) {
    int result = ::sendto(Descriptor, buffer, size, MSG_NOSIGNAL,
                          (struct sockaddr*)addr, sizeof(sockaddr_un));
    if (!(result == -1 && errno == EINTR)) {
      return result;
    }
  }
}

inline int TDatagramSocket::Read(void* buffer, unsigned long size)
{
  while (true) {
    int result = ::recvfrom(Descriptor, buffer, size, MSG_NOSIGNAL,
                            NULL, NULL);
    if (!(result == -1 && errno == EINTR)) {
      return result;
    }
  }
}

inline int TDatagramSocket::Read(void* buffer, unsigned long size,
                                 sockaddr* from, socklen_t& fromlen)
{
  while (true) {
    int result = ::recvfrom(Descriptor, buffer, size, MSG_NOSIGNAL,
                            from, &fromlen);
    if (!(result == -1 && errno == EINTR)) {
      return result;
    }
  }
}

inline void TDatagramSocket::Flush()
{
  fd_set set;
  int ret;
  char buffer[1316*4];
  struct timeval wait;

  wait.tv_sec = 0;
  wait.tv_usec = 0;
  while (true) {
    FD_ZERO(&set);
    FD_SET(Descriptor, &set);
    ret = ::select(FD_SETSIZE, &set, NULL, NULL, &wait);
    if (ret == 0) {
      break;
    }
    ::recvfrom(Descriptor, buffer, 1316*4, MSG_NOSIGNAL, NULL, NULL);
  }
}


#endif 
