/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *--------------------------------------------------------------------
 *
 * comm/emio/TSocket.h
 *
 * This class is the base class of all emio socket classes.
 *
 * Copyright (c) 2003 Kreatel Communications AB. All Rights Reserved.
 *
 *--------------------------------------------------------------------
 */

#ifndef COMM_EMIO_TSOCKET_H
#define COMM_EMIO_TSOCKET_H

#include "IEmioSocket.h"

#include <unistd.h>
#include <sys/socket.h>
#include <cassert>


class TSocket : virtual public IEmioSocket
{
private:
  void Copy(const TSocket& f);

protected:
  int Descriptor;

  void Open(int protocolFamily, int domain);

public:
  enum { INVALID_FILE_DESCRIPTOR = -1 };

  TSocket();
  TSocket(const TSocket& f);
  virtual ~TSocket();

  TSocket& operator= (const TSocket& f);
  operator int() const;

  bool Close();

  void SetFD(int fd);
  int GetFD() const;

  bool IsOpen() const;

  
  
  
  
  
  virtual int Read(void* buffer, unsigned long count) = 0;

  
  
  
  
  
  virtual int Write(const void* buffer, unsigned long count) = 0;

  
  
  bool SetBlocking();
  bool SetNonBlocking();

  
  
  
  bool SetCloseOnExec();
  
  int GetPendingError();

  
  
  bool ShutdownRD();
  bool ShutdownWR();
  bool ShutdownRDWR();

  void SetReceiveBufferSize(unsigned long size);
};


inline TSocket::TSocket()
  : Descriptor(INVALID_FILE_DESCRIPTOR)
{
  
}

inline TSocket::TSocket(const TSocket& f)
  : IEmioSocket(f)
{
  Copy(f);
} 

inline TSocket::~TSocket()
{
  Close();
}

inline TSocket& TSocket::operator=(const TSocket& f)
{
  Close();
  Copy(f);
  return *this;
}

inline TSocket::operator int() const
{
  return Descriptor;
}

inline void TSocket::Open(int protocolFamily, int domain)
{
  assert(Descriptor == INVALID_FILE_DESCRIPTOR);
  Descriptor = ::socket(protocolFamily, domain, 0);
}

inline void TSocket::SetFD(int fd)
{
  Descriptor = fd;
}

inline int TSocket::GetFD() const
{
  return Descriptor;
}

inline bool TSocket::IsOpen() const
{
  return Descriptor != INVALID_FILE_DESCRIPTOR;
}

inline int TSocket::GetPendingError()
{
  int error;
  socklen_t length = sizeof(int);
  ::getsockopt(Descriptor, SOL_SOCKET, SO_ERROR, &error, &length);
  return error;
}

inline bool TSocket::ShutdownRD()
{
  return ::shutdown(Descriptor, SHUT_RD) == 0;
}

inline bool TSocket::ShutdownWR()
{
  return ::shutdown(Descriptor, SHUT_WR) == 0;
}

inline bool TSocket::ShutdownRDWR()
{
  return ::shutdown(Descriptor, SHUT_RDWR) == 0;
}

inline void TSocket::SetReceiveBufferSize(unsigned long size)
{
  ::setsockopt(Descriptor, SOL_SOCKET, SO_RCVBUF,
               &size, sizeof(unsigned long));
}

#endif 
