/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *---------------------------------------------------------------------------
 *
 * comm/emio/TStreamSocket.h
 *
 * Base class for all stream sockets.
 *
 * Copyright (c) 2003 Kreatel Communications AB. All Rights Reserved.
 *
 *---------------------------------------------------------------------------
 */

#ifndef COMM_EMIO_TSTREAMSOCKET_H
#define COMM_EMIO_TSTREAMSOCKET_H

#include "TSocket.h"
#include "IStreamSocket.h"

#include <cerrno>
#include <sys/socket.h>
#include <unistd.h>

class TStreamSocket : public TSocket,
                      virtual public IStreamSocket
{
protected:
  void Open(int protocolFamily);

public:
  TStreamSocket(bool doOpen, int protocolFamily);
  virtual ~TStreamSocket() {}

  
  
  
  int Write(const void* buffer, unsigned long size);

  
  
  
  
  int Read(void* buffer, unsigned long size);

  
  
  bool Listen(int backlog = 1);
};

inline void TStreamSocket::Open(int protocolFamily)
{
  TSocket::Open(protocolFamily, SOCK_STREAM);
}

inline int TStreamSocket::Write(const void* buffer, unsigned long size)
{
  while (true) {
    int result = ::send(Descriptor, buffer, size, MSG_NOSIGNAL);
    if (!(result == -1 && errno == EINTR)) {
      return result;
    }
  }
}

inline int TStreamSocket::Read(void* buffer, unsigned long size)
{
  while (true) {
    int result = ::recv(Descriptor, buffer, size, MSG_NOSIGNAL);
    if (!(result == -1 && errno == EINTR)) {
      return result;
    }
  }
}

inline bool TStreamSocket::Listen(int backlog)
{
  return ::listen(Descriptor, backlog) == 0;
}

#endif 
