/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *---------------------------------------------------------------------------
 *
 * comm/emio/TTcpSocket.h
 *
 *
 * Copyright (c) 2003 Kreatel Communications AB. All Rights Reserved.
 *
 *---------------------------------------------------------------------------
 */

#ifndef COMM_EMIO_TTCPSOCKET_H
#define COMM_EMIO_TTCPSOCKET_H

#include "TStreamSocket.h"
#include "ITcpSocket.h"
#include "TInetProto.h"

#include <netinet/in.h>
#include <sys/types.h>
#include <sys/socket.h>


class TTcpSocket : public TStreamSocket,
                   virtual public ITcpSocket
{
public:
  TTcpSocket(bool doOpen = true);

  void Open();

  
  
  bool GetSocketName(sockaddr_in& address);

  
  
  bool GetPeerName(sockaddr_in& address);

  bool GetPeerName(char* address, unsigned int size);

  

  
  
  bool Bind(const char* address, unsigned short port);
  bool Bind(unsigned long address, unsigned short port);
  bool Bind(unsigned short port);
  bool Bind(sockaddr_in& address);

  
  
  bool Accept(ITcpSocket* s);

  
  
  bool Accept(TTcpSocket& s);

  

  
  
  
  
  
  int Connect(const sockaddr_in& address);

  
  bool SetTcpFlag(int optname, bool flag);

  /* Obsolete:
  bool Connect(const char* address, unsigned short port);
  bool Connect(unsigned long address, unsigned short port);
  bool Connect(unsigned short port);
  */
};


inline TTcpSocket::TTcpSocket(bool doOpen)
  : TStreamSocket(doOpen, PF_INET)
{
  
}

inline void TTcpSocket::Open()
{
  TStreamSocket::Open(PF_INET);
}

inline bool TTcpSocket::Bind(const char* address, unsigned short port)
{
  return TInetProto::Bind(Descriptor, address, port);
}

inline bool TTcpSocket::Bind(unsigned long address, unsigned short port)
{
  return TInetProto::Bind(Descriptor, address, port);
}

inline bool TTcpSocket::Bind(unsigned short port)
{
  return TInetProto::Bind(Descriptor, port);
}

inline bool TTcpSocket::Bind(sockaddr_in& address)
{
  return TInetProto::Bind(Descriptor, address);
}

inline bool TTcpSocket::Accept(TTcpSocket& s)
{
  return Accept(&s);
}

inline bool TTcpSocket::SetTcpFlag(int optname, bool flag)
{
  int value = flag ? 1 : 0;
  return 0 == ::setsockopt(GetFD(), optname, IPPROTO_TCP, &value, sizeof(int));
}

#endif 
