/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *---------------------------------------------------------------------------
 *
 * TLogMessage.h
 *
 * Structures used for encoding log messages.
 *
 * Copyright (c) 2004 Kreatel Communications AB. All Rights Reserved.
 *
 *---------------------------------------------------------------------------
 */

#ifndef DEBUG_TLOGMESSAGE_H
#define DEBUG_TLOGMESSAGE_H

#include "debug/TLogLevel.h"

#include <stdint.h>
#include <sys/time.h>
#include <sys/types.h>

/*************************************************
 * Log message format:
 *
 * [header][data]
 *
 * The header always contains the total length of the message
 * and the type of message. Depending on the type, the contents of
 * the header and the encoding of the data area differs.
 *
 * LOG_TYPE_MESSAGE:
 *   Log messages of type LOG_TYPE_MESSAGE use the LogHeader
 *   field of the TMessageHeader header. The data is a NULL terminated
 *   string.
 *
 * LOG_TYPE_TEXT:
 *   Log messages of type LOG_TYPE_TEXT use the TextHeader
 *   field of the TMessageHeader header. The data is a NULL terminated
 *   string.
 * 
 *************************************************/

typedef enum {
  LOG_TYPE_MESSAGE,
  LOG_TYPE_TEXT
} TLogType;

typedef struct {
  int32_t Sec;                /* Time when log was generated */
  int32_t uSec;               /* microseconds */
  pid_t Pid;                  /* Process which generated log */
  TLogLevel Level;            /* Log level */
  int Seq;                    /* Log sequence number */
} TLogHeader;

typedef struct {
  int32_t Sec;                /* Time when log was received */
  int32_t uSec;               /* microseconds */
  pid_t Pid;                  /* Process which generated log */
} TTextHeader;

typedef struct {
  uint32_t MsgSize;           /* Total length of entire log message. */
  TLogType MsgType;           /* The type of log message. */
  union {
    TLogHeader LogHeader;     /* MsgType == LOG_TYPE_MESSAGE */
    TTextHeader TextHeader;   /* MsgType == LOG_TYPE_TEXT */
  } MsgHeader;
} TLogMessageHeader;

#endif
