/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *---------------------------------------------------------------------------
 *
 * TNetTracker.h
 *
 * A TTracker specialization that forwards all traces to a logclient.
 * Aimed to be used when procman is not around.
 *
 * Copyright (c) 2009 Motorola, Inc. All Rights Reserved.
 *
 *---------------------------------------------------------------------------
 */

#ifndef TNETTRACKER_H
#define TNETTRACKER_H

#include "TTracker.h"

#include "debug/TLogMessage.h"
#include "event/IIOEventHandler.h"
#include "comm/emio/TTcpSocket.h"
#include "unix-wrappers/TTimeVal.h"
#include "event/IIOTimeoutHandler.h"


#include <queue>


class TNetTracker : public TTracker,
  public IIOEventHandler,
  public IIOTimeoutHandler
{
private:
  typedef struct __attribute__ ((packed))
  {
    uint32_t Sec;
    uint32_t uSec;
    uint32_t Pid;
    uint32_t Level;
    uint32_t Seq;
  } TPILogHeader; 

  typedef struct __attribute__ ((packed))
  {
    uint32_t Sec;
    uint32_t uSec;
    uint32_t Pid;
  } TPITextHeader; 

  typedef struct {
    uint32_t MsgSize;
    uint32_t MsgType;
    union {
      TPILogHeader  LogHeader;
      TPITextHeader TextHeader;
    } MsgHeader;
  } TPILogMessageHeader;

  struct TNetMessageHeader
  {
    int MsgSize;
    int MsgSeq;
    TLogType MsgType;
    union {
      TPILogHeader LogHeader;
      TPITextHeader TextHeader;
    };
  } __attribute__ ((packed));

  typedef struct {
    uint32_t Size;
    char*    Buffer;
  } TMessage;

  enum {
    BUFFER_SIZE          = 2048,
    TIMEOUT              = 100,
    MESSAGES_PER_TIMEOUT = 5,
    MESSAGES_QUEUE_SIZE  = 1000
  };
  TSystemTime StartTime;
  IIOEventLoop* EventLoop;
  int ServerPort;
  TTcpSocket TcpServer;
  TTcpSocket TcpClient;
  uint32_t Sequence;
  int StderrFds[2];
  int32_t TimerId;
  char* Buffer;
  std::queue<TMessage> Messages;

  void AttachStderr();
  void DetachStderr();
  void AddMessage(TLevel level,
                  const char* module,
                  const char* text);

  void SendMessages(uint32_t messages) throw();

protected:
  virtual void Output(TLevel level, const char* format, va_list ap) throw ();

public:
  TNetTracker(TLevel level,
              IIOEventLoop* eventLoop,
              uint32_t serverPort = 19999) throw ();

  ~TNetTracker() throw();

  
  virtual void HandleEvent(int fd, uint32_t event) throw ();

  
  virtual void HandleTimeout(int timerId) throw ();

};

#endif 
