/* -*- Mode: C; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *---------------------------------------------------------------------------
 *
 * logoutput.h
 *
 * C functions for log output.
 *
 * Copyright (c) 2004 Kreatel Communications AB. All Rights Reserved.
 *
 *---------------------------------------------------------------------------
 */

#ifndef DEBUG_LOGOUTPUT_H
#define DEBUG_LOGOUTPUT_H

#include "debug/TLogLevel.h"

#include <stdarg.h>
#include <stddef.h>
#include <sys/cdefs.h>

__BEGIN_DECLS

#define WEAK_LINKAGE __attribute__((weak))


/*
  Implement this function in your process to receive log messages. If
  no such implementation is provided, the messages will be silently
  discarded.

  AVOID CALLING THIS FUNCTION DIRECTLY!
 
  Because of its weak linkage, the address of this function will be
  NULL if it is not defined (the linker will not complain). When you
  call this function, you have to check that it is not NULL
  first. Failing to do so may result in a segmentation fault.

  The following is an example implementation that outputs all log
  messages to standard out:

  #include <stdio.h>
  void LogOutputVa(TLogLevel level, const char* format, va_list ap)
  {
    vprintf(format, ap);
  }
*/
extern void LogOutputVa(TLogLevel level, const char* format,
                        va_list ap) WEAK_LINKAGE;


/*
  This function is a wrapper that calls LogOutputVa safely and should
  be used for appending messages to the log.
*/
static inline void LogOutput(TLogLevel level, const char* format, ...)
  __attribute__ ((format (printf, 2, 3)));

__END_DECLS


static inline void LogOutput(TLogLevel level, const char* format, ...)
{
  if (&LogOutputVa != NULL) {
    va_list ap;
    va_start(ap, format);
    LogOutputVa(level, format, ap);
    va_end(ap);
  }
}

#endif 
