/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *---------------------------------------------------------------------------
 *
 * TPollableMessageQueue.h
 *
 * This class implements a very simple pollable message queue using a
 * Unix pipe.
 *
 * Copyright (c) 2002 Kreatel Communications AB. All Rights Reserved.
 *
 *---------------------------------------------------------------------------
 */

#ifndef COMMON_CPP_THREAD_TPOLLABLEMESSAGEQUEUE_H
#define COMMON_CPP_THREAD_TPOLLABLEMESSAGEQUEUE_H

#include "base/TException.h"

#include <unistd.h>
#include <poll.h>
#include <cerrno>
#include <cstring>


class TPollableMessageQueue
{
protected:
  int Descriptors[2];

public:
  TPollableMessageQueue() throw (TException);
  ~TPollableMessageQueue();

  
  
  int GetFileDescriptor() const;

  void Push(char message);
  char Pop();

  bool IsEmpty();

  void Write(const void* message, size_t size);
  void Read(void* message, size_t size);
};

inline TPollableMessageQueue::TPollableMessageQueue() throw (TException)
{
  if (::pipe(Descriptors) == -1) {
    throw TException("Could not create pipe");
  }
}

inline TPollableMessageQueue::~TPollableMessageQueue()
{
  ::close(Descriptors[0]);
  ::close(Descriptors[1]);
}

inline int TPollableMessageQueue::GetFileDescriptor() const
{
  return Descriptors[0];
}

inline void TPollableMessageQueue::Push(char message)
{
  Write(&message, 1);
}

inline char TPollableMessageQueue::Pop()
{
  char message;
  Read(&message, 1);
  return message;
}

inline bool TPollableMessageQueue::IsEmpty()
{
  struct pollfd pfd;
  memset(&pfd, 0, sizeof(pfd));
  pfd.fd = Descriptors[0];
  pfd.events = POLLIN;
  if (TEMP_FAILURE_RETRY(poll(&pfd, 1, 0)) != 0) {
    return false;
  }
  else {
    return true;
  }
}

#endif
