/* -*- Mode: C++; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *---------------------------------------------------------------------------
 *
 * Copyright (c) 2009 Motorola, Inc. All Rights Reserved.
 *
 *---------------------------------------------------------------------------
 */

#ifndef EVENT_TPRIMITIVEDEFERREDFUNCTIONCALLER_H
#define EVENT_TPRIMITIVEDEFERREDFUNCTIONCALLER_H

#include "event/IIOEventHandler.h"
#include "event/IIOTimeoutHandler.h"
#include "TPollableMessageQueue.h"
#include <map>
#include <queue>

class IIOEventLoop;

/**
 * This class implements deferred function calls, just like
 * TDeferredMethodCaller but with a more primitive interface and without a
 * dependency to Boost. Use TDeferredMethodCaller instead of this class unless
 * you can't use Boost.
 */
class TPrimitiveDeferredFunctionCaller : public IIOEventHandler,
                                         public IIOTimeoutHandler
{
public:
  typedef void (*TFunction)(void* context);

private:
  IIOEventLoop& EventLoop;

  typedef std::pair<TFunction, void*> TFunctionPair;

  typedef std::queue<TFunctionPair> TFunctionQueue;
  TFunctionQueue FunctionQueue;
  TPollableMessageQueue MessageQueue;

  typedef std::map<int, TFunctionPair> TTimerFunctionMap;
  TTimerFunctionMap TimerFunctionMap;

  void HandleEvent(int fd, uint32_t event) throw ();
  void HandleTimeout(int timerId) throw ();

public:
  /**
   * Constructor. The object will keep a reference to the event loop object and
   * the caller is responsible for assuring deletion order.
   *
   * @param eventLoop The eventloop object to use.
   */
  TPrimitiveDeferredFunctionCaller(IIOEventLoop& eventLoop);

  ~TPrimitiveDeferredFunctionCaller();

  /**
   * Queues a function call that should be called "later". The function will
   * get the context as a parameter.
   *
   * @param function The function to run.
   *
   * @param context The context to pass to the function.
   */
  void Push(TFunction function, void* context = NULL);

  /**
   * Hands over a function call that should be executed upon timeout. The
   * function will be executed once.
   *
   * @param milliseconds Time in milliseconds when the runner is run. If
   * milliseconds is 0, Push will be called.
   *
   * @param function The function to run.
   *
   * @param context The context to pass to the function.
   */
  void PushAt(const int milliseconds, TFunction function, void* context = NULL);
};

#endif
