/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *---------------------------------------------------------------------------
 *
 * event/TTimerSet.h
 *
 * Copyright (c) 2002 Kreatel Communications AB. All Rights Reserved.
 * Copyright (c) 2008-2011 Motorola, Inc. All Rights Reserved.
 *
 *---------------------------------------------------------------------------
 */

#ifndef EVENT_TTIMERSET_H
#define EVENT_TTIMERSET_H

#include <stdint.h>
#include <ctime>

class IClockSource;
class IIOTimeoutHandler;
class TTracker;

class TTimerSet
{
  friend class TTimerSetTests;

private:
  class TTimer
  {
  public:
    uint32_t Interval;
    IIOTimeoutHandler* Handler;
    clock_t NextTimeout;

    TTimer() : Interval(0), Handler(NULL), NextTimeout(0) { /* Empty. */ }
  };

  enum { MAX_TIMERS = 64 };

private:
  TTracker* Tracker;

  TTimer Timers[MAX_TIMERS];

  
  int HighestTimerId;

  
  clock_t NextTimeout;
  clock_t LatestCheckTime;

  IClockSource& ClockSource;

  
  TTimerSet(const TTimerSet& other);
  TTimerSet& operator=(const TTimerSet& other);

public:
  TTimerSet(IClockSource& clocksource) throw ();
  ~TTimerSet() throw ();

  int GetNextTimeout() throw ();

  
  
  
  
  
  
  
  enum TMode {
    DEFAULT,
    EXECUTE_EXPIRED_ON_CHECK };
  bool HandleTimeout(TMode mode = DEFAULT) throw ();

  
  int AddTimer(uint32_t interval, IIOTimeoutHandler* handler) throw ();
  void RemoveTimer(int timerId) throw ();
};

#endif 
