/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *---------------------------------------------------------------------------
 *
 * event/TUnixEventLoop.h
 *
 * Copyright (c) 2002 Kreatel Communications AB. All Rights Reserved.
 * Copyright (c) 2008-2011 Motorola, Inc. All Rights Reserved.
 *
 *---------------------------------------------------------------------------
 */

#ifndef EVENT_TUNIXEVENTLOOP_H
#define EVENT_TUNIXEVENTLOOP_H

#include "event/IIOEventLoop.h"
#include "THandlerSet.h"
#include "TRealClockSource.h"
#include "TTimerSet.h"

#include <stdint.h>

class IIOEventHandler;
class IIOTimeoutHandler;
class TTracker;

class TUnixEventLoop : public IIOEventLoop
{
private:
  class TEventHandler
  {
  public:
    int32_t Priority;
    IIOEventHandler* Callback;

    TEventHandler(int32_t prio, IIOEventHandler* handler)
      : Priority(prio), Callback(handler) { /* Empty. */ }
  };

private:
  TTracker* Tracker;

  THandlerSet HandlerSet;

  TTimerSet TimerSet;

  int StopReadFd;
  int StopWriteFd;

  static uint32_t InstanceCount;
  uint32_t Instance;
  bool ShouldRun;
  TRealClockSource ClockSource;
  bool PollArrayInvalidated;
public:
  enum TMode {
    DEFAULT,  
    EXECUTE_TIMEOUTS_WITH_EVENTS  
                                  
  };
private:
  const TMode Mode;

  
  TUnixEventLoop(const TUnixEventLoop& other);
  TUnixEventLoop& operator=(const TUnixEventLoop& other);

  
 void init() throw ();

public:
  TUnixEventLoop() throw ();
  TUnixEventLoop(TMode mode) throw ();
  virtual ~TUnixEventLoop() throw ();

  /**
   * Runs one iteration of the eventloop.
   *
   * @param millis Maximum number of milliseconds to wait for new
   * events. -1 means wait forever.
   *
   * @return true if io events or timers were dispatched, otherwise false
   */
  bool RunOnce(int millis = -1) throw ();
  
  void Run() throw ();
  
  
  void Stop() throw ();

  
  virtual void AddDescriptor(int fd, uint32_t eventMask, TPriority priority,
                             IIOEventHandler* handler) throw ();
  virtual void RemoveDescriptor(int fd) throw ();
  virtual int AddTimer(uint32_t interval, IIOTimeoutHandler* handler) throw ();
  virtual void RemoveTimer(int timerId) throw ();
};

#endif 
