







/** \file */

#ifndef ISUBTITLERENDEREXTENSION_H
#define ISUBTITLERENDEREXTENSION_H

#include "interface/kreatvgfx/IKreaTvGfx.h"
#include "interface/kreatvgfx/IKreaTvGfxExtension.h"
#include "interface/kreatvgfx/IKreaTvGfxExtensionObserver.h"
#include "ipc/TIpcServer.h"

#include <boost/shared_ptr.hpp>


/** This function will create a subtitle renderer that can draw
    subtitles as decided by the active media player and Information
    Service parameters. The application level graphics rendering
    engine can use this extension to draw subtitles in the graphics
    buffer to show on screen.

    \param[in] kreaTvGfx
                      KreaTvGfx instance that library will use to
                      create internally needed pixmaps.

    \param[in] applicationId
                      The id of the application.

    \param[in] eventLoop
                      Event loop needed to allow library to talk to
                      the media session.

    \param[in] observer
                      Observer interface where update events will be
                      signalled when new subtitle graphics is
                      available.

    \param[out] subtitleExtension
                      The entry point for the KreaTvGfx extension.
 */
void CreateSubtitleRenderExtension(
  TKreaTvGfxPointer kreaTvGfx,
  int applicationId,
  IIOEventLoop& eventLoop,
  IKreaTvGfxExtensionObserver* observer,
  TKreaTvGfxExtensionPointer& subtitleExtension)
  throw ();

#endif
