/*
 *---------------------------------------------------------------------------
 *
 * Copyright (c) 2009 Motorola, Inc. All Rights Reserved.
 *
 *---------------------------------------------------------------------------
 */

#ifndef TREGIONBUFFER_H
#define TREGIONBUFFER_H

#include "IRegionBuffer.h"
#include <sys/types.h>

#include <cstring>
#include <boost/shared_array.hpp>

class TTracker;

class TRegionBuffer : public IRegionBuffer
{
private:
  TTracker* Tracker;

  TRect MaxRegion;
  std::vector<TRect> Regions;

  int BytesPerPixel;
  int BufferSize;
  bool ClutBased;
  uint ClearClutIndex;
  TClut Clut;
  class RegionComp
  {
  public:
    bool operator ()(const TRect& a, const TRect& b) const
    {
      return (a.Y == b.Y) ?  a.X < b.X : a.Y < b.Y;
    }
  };

  std::vector<TRect>::const_iterator FindRegion(uint16_t x, uint16_t y) const;

public:
  TRegionBuffer();

  void UseClut(bool clutBased);
  uint16_t GetWidth() const;
  uint16_t GetHeight() const;

  bool SetPixel(uint16_t x, uint16_t y, uint8_t color);
  bool SetPixelRow(uint16_t x, uint16_t y, uint8_t color, uint16_t length);
  bool SetPixelWord(uint16_t x, uint16_t y, uint8_t fgcolor, uint8_t bgcolor,
                    uint16_t data, uint8_t length);
  uint8_t GetPixel(uint16_t x, uint16_t y);

  bool SetRegions(const std::vector<TRect>& regions);
  void FillRegion(uint16_t x, uint16_t y, uint8_t color);

  int16_t GetOffsetY() const;
  int16_t GetOffsetX() const;
  void Clear();
  void Initiate();

  boost::shared_array<uint8_t> Buffer;

  
  virtual void SetClut(const TClut& clut, TClut::size_type clearIndex);
  virtual const TRect& GetBoundingRect();
};

inline void TRegionBuffer::UseClut(bool clutBased)
{
  ClutBased = clutBased;
  BytesPerPixel = ClutBased ? 1 : 4;
}

inline void TRegionBuffer::SetClut(const TClut& clut,
                                   TClut::size_type clearIndex)
{
  Clut = clut;
  if (Clut.size() <= ClearClutIndex ||
      Clut[clearIndex].Value != Clut[ClearClutIndex].Value) {
    ClearClutIndex = clearIndex;
    Clear();
  }
}

inline uint16_t TRegionBuffer::GetWidth() const
{
  return MaxRegion.Width;
}

inline uint16_t TRegionBuffer::GetHeight() const
{
  return MaxRegion.Height;
}

inline int16_t TRegionBuffer::GetOffsetX() const
{
  return MaxRegion.X;
}

inline int16_t TRegionBuffer::GetOffsetY() const
{
  return MaxRegion.Y;
}

inline void TRegionBuffer::Clear()
{
  ::memset(Buffer.get(), ClearClutIndex, BufferSize);
}

inline const TRect& TRegionBuffer::GetBoundingRect()
{
  return MaxRegion;
}

#endif
