/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *---------------------------------------------------------------------------
 *
 * TSubtitlePlugin.h
 *
 * Copyright (c) 2011 Motorola Mobility Inc. All Rights Reserved.
 *
 *---------------------------------------------------------------------------
 */

#ifndef TSUBTITLEPLUGIN_H
#define TSUBTITLEPLUGIN_H

#include "ISubtitlePlugin.h"
#include "media/streamer/IStreamerTypes.h"

#define PLUGIN_CTOR __plugin_ctor
#define PLUGIN_DTOR __plugin_dtor

#define CREATE_PLUGIN(Class, Name, Format)                              \
  extern "C" ISubtitlePlugin* PLUGIN_CTOR() {                           \
    return new Class(Name, Format);                                     \
  }                                                                     \
  extern "C" void PLUGIN_DTOR(Class* ptr) {                             \
    delete ptr;                                                         \
  }

#define STR(x) #x
#define XSTR(x) STR(x)
#define PLUGIN_CTOR_NAME XSTR(PLUGIN_CTOR)
#define PLUGIN_DTOR_NAME XSTR(PLUGIN_DTOR)

template<typename T>
class TSubtitlePlugin
  : public ISubtitlePlugin
{
public:
  TSubtitlePlugin(const char* name,
                  IStreamerTypes::TSubtitleFormat format)
    : Name(name),
      Format(format)
  {
    
  }

  const std::string& GetName() const throw ()
  {
    return Name;
  }

  const IStreamerTypes::TSubtitleFormat& GetSupportFormat() const
    throw ()
  {
    return Format;
  }

  ISubtitlePageDescriptor* GetSubtitlePageDescriptor() const
    throw ()
  {
    return (ISubtitlePageDescriptor*) &Object;
  }

private:
  T Object;
  std::string Name;
  IStreamerTypes::TSubtitleFormat Format;
};

#endif 
