/*
 *-----------------------------------------------------------------------------
 *
 * IToiApplicationService.h
 *
 * This file was created from .idlcpp/IToiApplicationService.idl
 * by the Motorola IDL backend, which is based on the OmniORB IDL parser.
 *
 * This file was generated from the AbstractHeader.tpl template.
 *
 * Copyright (c) 2009 Motorola, Inc. All Rights Reserved.
 *
 *-----------------------------------------------------------------------------
 */

#ifndef ITOIAPPLICATIONSERVICE_H
#define ITOIAPPLICATIONSERVICE_H

#include "interface/TToiInvalidArgumentException.h"
#include "interface/TToiUnhandledValueException.h"

#include "ipc/TDispatchableExceptionTemplate.h"
#include "ipc/TIpcException.h"

#include <stdint.h>
#include <cstring>
#include <string>
#include <vector>






class IToiApplicationService
{
public:
  struct TToiApplicationInfo;
  
  enum TToiApplicationState
  {
    STATE_REGISTERED = 0,
    STATE_INSTALLING = 1,
    STATE_INSTALLED = 2,
    STATE_STARTING = 3,
    STATE_INVISIBLE = 4,
    STATE_VISIBLE = 5,
    STATE_ACTIVE = 6,
    STATE_STOPPED = 7
  };
  typedef int TToiApplicationId;
  typedef std::vector< TToiApplicationId > TToiApplicationIdSequence;
  typedef std::vector< TToiApplicationInfo > TToiApplicationInfoSequence;
  typedef std::vector< std::string > TToiPropertyStringSequence;
  
  struct TToiApplicationInfo {
    TToiApplicationId ApplicationId;
    std::string ApplicationName;
    TToiApplicationState State;
    
    TToiApplicationInfo() :
      ApplicationId(0), 
      ApplicationName(""), 
      State(STATE_REGISTERED)  {   }
    
    TToiApplicationInfo(
      const TToiApplicationId& _ApplicationId, 
      const std::string& _ApplicationName, 
      const TToiApplicationState& _State) :
        ApplicationId(_ApplicationId), 
        ApplicationName(_ApplicationName), 
        State(_State)
    {
    }
    
  };
  
  virtual ~IToiApplicationService() { /* Empty */ }
  virtual void RegisterApplication(const std::string& propertyFilename) throw (TToiInvalidArgumentException, TIpcException) = 0;
  virtual void ReportStarted(const TToiApplicationId& applicationId, const std::string& address) throw (TToiInvalidArgumentException, TIpcException) = 0;
  virtual void ReportPing(const TToiApplicationId& applicationId) throw (TToiInvalidArgumentException, TIpcException) = 0;
  virtual void Activate(const TToiApplicationId& applicationId) throw (TToiInvalidArgumentException, TIpcException) = 0;
  virtual void ActivateWithUri(const TToiApplicationId& applicationId, const std::string& uri, const std::string& mimeType) throw (TToiInvalidArgumentException, TIpcException) = 0;
  virtual void ActivateWithCommand(const TToiApplicationId& applicationId, const std::string& command) throw (TToiInvalidArgumentException, TIpcException) = 0;
  virtual void Deactivate(const TToiApplicationId& applicationId) throw (TToiInvalidArgumentException, TIpcException) = 0;
  virtual void Kill(const TToiApplicationId& applicationId) throw (TToiInvalidArgumentException, TIpcException) = 0;
  virtual void GetOwnApplicationId(TToiApplicationId& applicationId) throw (TIpcException) = 0;
  virtual void GetApplicationIds(TToiApplicationIdSequence& applicationIds) throw (TIpcException) = 0;
  virtual void GetBoolProperty(const TToiApplicationId& applicationId, const std::string& property, bool& value) throw (TToiInvalidArgumentException, TIpcException) = 0;
  virtual void GetIntProperty(const TToiApplicationId& applicationId, const std::string& property, int& value) throw (TToiInvalidArgumentException, TIpcException) = 0;
  virtual void GetStringProperty(const TToiApplicationId& applicationId, const std::string& property, std::string& value) throw (TToiInvalidArgumentException, TIpcException) = 0;
  virtual void GetStringSequenceProperty(const TToiApplicationId& applicationId, const std::string& property, TToiPropertyStringSequence& values) throw (TToiInvalidArgumentException, TIpcException) = 0;
  virtual void GetInfo(const TToiApplicationId& applicationId, TToiApplicationInfo& info) throw (TToiInvalidArgumentException, TIpcException) = 0;
  virtual void LoadUri(const std::string& uri, const std::string& mimeType) throw (TToiUnhandledValueException, TIpcException) = 0;
  virtual void RegisterObserver(const TToiApplicationId& applicationId, const std::string& address) throw (TToiInvalidArgumentException, TIpcException) = 0;
  virtual void ReleaseObserver(const std::string& address) throw (TToiInvalidArgumentException, TIpcException) = 0;
};



#define IMPLEMENTS_ITOIAPPLICATIONSERVICE \
virtual void RegisterApplication(const std::string& propertyFilename) throw (TToiInvalidArgumentException); \
virtual void ReportStarted(const TToiApplicationId& applicationId, const std::string& address) throw (TToiInvalidArgumentException); \
virtual void ReportPing(const TToiApplicationId& applicationId) throw (TToiInvalidArgumentException); \
virtual void Activate(const TToiApplicationId& applicationId) throw (TToiInvalidArgumentException); \
virtual void ActivateWithUri(const TToiApplicationId& applicationId, const std::string& uri, const std::string& mimeType) throw (TToiInvalidArgumentException); \
virtual void ActivateWithCommand(const TToiApplicationId& applicationId, const std::string& command) throw (TToiInvalidArgumentException); \
virtual void Deactivate(const TToiApplicationId& applicationId) throw (TToiInvalidArgumentException); \
virtual void Kill(const TToiApplicationId& applicationId) throw (TToiInvalidArgumentException); \
virtual void GetOwnApplicationId(TToiApplicationId& applicationId) throw (); \
virtual void GetApplicationIds(TToiApplicationIdSequence& applicationIds) throw (); \
virtual void GetBoolProperty(const TToiApplicationId& applicationId, const std::string& property, bool& value) throw (TToiInvalidArgumentException); \
virtual void GetIntProperty(const TToiApplicationId& applicationId, const std::string& property, int& value) throw (TToiInvalidArgumentException); \
virtual void GetStringProperty(const TToiApplicationId& applicationId, const std::string& property, std::string& value) throw (TToiInvalidArgumentException); \
virtual void GetStringSequenceProperty(const TToiApplicationId& applicationId, const std::string& property, TToiPropertyStringSequence& values) throw (TToiInvalidArgumentException); \
virtual void GetInfo(const TToiApplicationId& applicationId, TToiApplicationInfo& info) throw (TToiInvalidArgumentException); \
virtual void LoadUri(const std::string& uri, const std::string& mimeType) throw (TToiUnhandledValueException); \
virtual void RegisterObserver(const TToiApplicationId& applicationId, const std::string& address) throw (TToiInvalidArgumentException); \
virtual void ReleaseObserver(const std::string& address) throw (TToiInvalidArgumentException);

#endif 

