/*
 *-----------------------------------------------------------------------------
 *
 * IToiAudioOutputConfiguration.h
 *
 * This file was created from .idlcpp/IToiAudioOutputConfiguration.idl
 * by the Motorola IDL backend, which is based on the OmniORB IDL parser.
 *
 * This file was generated from the AbstractHeader.tpl template.
 *
 * Copyright (c) 2009 Motorola, Inc. All Rights Reserved.
 *
 *-----------------------------------------------------------------------------
 */

#ifndef ITOIAUDIOOUTPUTCONFIGURATION_H
#define ITOIAUDIOOUTPUTCONFIGURATION_H

#include "interface/TToiInvalidArgumentException.h"
#include "interface/TToiInvalidPreconditionException.h"
#include "interface/TToiOperationNotSupportedException.h"
#include "interface/TToiUnhandledValueException.h"
#include "interface/IToiApplicationService.h"
#include "interface/IToiAudioOutputService.h"

#include "ipc/TDispatchableExceptionTemplate.h"
#include "ipc/TIpcException.h"

#include <stdint.h>
#include <cstring>
#include <string>
#include <vector>






class IToiAudioOutputConfiguration
{
public:
  struct TToiManufacturerInfo;
  struct TToiHdmiSinkInfo;
  struct TToiAudioFormatOption;
  struct TToiAudioFormatMapping;
  
  enum TToiAudioFormat
  {
    AUDIO_FORMAT_NULL = 0,
    AUDIO_FORMAT_MPEG_1 = 1,
    AUDIO_FORMAT_MPEG_2 = 2,
    AUDIO_FORMAT_AC3 = 3,
    AUDIO_FORMAT_AC3_PLUS = 4,
    AUDIO_FORMAT_AAC_HE = 5,
    AUDIO_FORMAT_AAC_LC = 6,
    AUDIO_FORMAT_DTS = 7,
    AUDIO_FORMAT_DTS_HD = 8,
    AUDIO_FORMAT_PCM_2CH = 9,
    AUDIO_FORMAT_PCM_MULTI = 10
  };
  enum TToiHdmiConnectionStatus
  {
    HDMI_CONNECTION_STATUS_UNKNOWN = 0,
    HDMI_CONNECTION_STATUS_CONNECTED = 1,
    HDMI_CONNECTION_STATUS_CONNECTED_CAPABILITIES_UNKNOWN = 2,
    HDMI_CONNECTION_STATUS_DISCONNECTED = 3
  };
  typedef std::vector< TToiAudioFormat > TToiAudioFormatSequence;
  typedef std::vector< TToiAudioFormatOption > TToiAudioFormatOptionSequence;
  typedef std::vector< TToiAudioFormatMapping > TToiAudioFormatMappingSequence;
  
  struct TToiManufacturerInfo {
    std::string Name;
    int ProductId;
    int SerialNumber;
    int WeekOfManufacture;
    int YearOfManufacture;
    
    TToiManufacturerInfo() :
      Name(""), 
      ProductId(0), 
      SerialNumber(0), 
      WeekOfManufacture(0), 
      YearOfManufacture(0)  {   }
    
    TToiManufacturerInfo(
      const std::string& _Name, 
      const int& _ProductId, 
      const int& _SerialNumber, 
      const int& _WeekOfManufacture, 
      const int& _YearOfManufacture) :
        Name(_Name), 
        ProductId(_ProductId), 
        SerialNumber(_SerialNumber), 
        WeekOfManufacture(_WeekOfManufacture), 
        YearOfManufacture(_YearOfManufacture)
    {
    }
    
  };
  
  struct TToiHdmiSinkInfo {
    TToiHdmiConnectionStatus Status;
    TToiAudioFormatSequence AudioFormatList;
    TToiManufacturerInfo ManufacturerInfo;
    
    TToiHdmiSinkInfo() :
      Status(HDMI_CONNECTION_STATUS_UNKNOWN)  {   }
    
    TToiHdmiSinkInfo(
      const TToiHdmiConnectionStatus& _Status, 
      const TToiAudioFormatSequence& _AudioFormatList, 
      const TToiManufacturerInfo& _ManufacturerInfo) :
        Status(_Status), 
        AudioFormatList(_AudioFormatList), 
        ManufacturerInfo(_ManufacturerInfo)
    {
    }
    
  };
  
  struct TToiAudioFormatOption {
    TToiAudioFormat Format;
    bool MixFlag;
    
    TToiAudioFormatOption() :
      Format(AUDIO_FORMAT_NULL), 
      MixFlag(false)  {   }
    
    TToiAudioFormatOption(
      const TToiAudioFormat& _Format, 
      const bool& _MixFlag) :
        Format(_Format), 
        MixFlag(_MixFlag)
    {
    }
    
  };
  
  struct TToiAudioFormatMapping {
    TToiAudioFormat InputFormat;
    TToiAudioFormatOptionSequence OutputFormatOptionList;
    TToiAudioFormat ActiveOutputFormat;
    bool ActiveMixFlag;
    
    TToiAudioFormatMapping() :
      InputFormat(AUDIO_FORMAT_NULL), 
      ActiveOutputFormat(AUDIO_FORMAT_NULL), 
      ActiveMixFlag(false)  {   }
    
    TToiAudioFormatMapping(
      const TToiAudioFormat& _InputFormat, 
      const TToiAudioFormatOptionSequence& _OutputFormatOptionList, 
      const TToiAudioFormat& _ActiveOutputFormat, 
      const bool& _ActiveMixFlag) :
        InputFormat(_InputFormat), 
        OutputFormatOptionList(_OutputFormatOptionList), 
        ActiveOutputFormat(_ActiveOutputFormat), 
        ActiveMixFlag(_ActiveMixFlag)
    {
    }
    
  };
  
  virtual ~IToiAudioOutputConfiguration() { /* Empty */ }
  virtual void GetFormatMapping(const IToiAudioOutputService::TToiAudioConnectionId& connectionId, const TToiAudioFormat& inputFormat, TToiAudioFormatMapping& formatMapping) throw (TToiInvalidArgumentException, TIpcException) = 0;
  virtual void GetFormatMappingList(const IToiAudioOutputService::TToiAudioConnectionId& connectionId, TToiAudioFormatMappingSequence& formatMappingList) throw (TToiInvalidArgumentException, TIpcException) = 0;
  virtual void GetHdmiSinkInfo(const IToiAudioOutputService::TToiAudioConnectionId& connectionId, TToiHdmiSinkInfo& sinkInfo) throw (TToiInvalidArgumentException, TIpcException) = 0;
};



#define IMPLEMENTS_ITOIAUDIOOUTPUTCONFIGURATION \
virtual void GetFormatMapping(const IToiAudioOutputService::TToiAudioConnectionId& connectionId, const TToiAudioFormat& inputFormat, TToiAudioFormatMapping& formatMapping) throw (TToiInvalidArgumentException); \
virtual void GetFormatMappingList(const IToiAudioOutputService::TToiAudioConnectionId& connectionId, TToiAudioFormatMappingSequence& formatMappingList) throw (TToiInvalidArgumentException); \
virtual void GetHdmiSinkInfo(const IToiAudioOutputService::TToiAudioConnectionId& connectionId, TToiHdmiSinkInfo& sinkInfo) throw (TToiInvalidArgumentException);

#endif 

