/*
 *-----------------------------------------------------------------------------
 *
 * IToiChannelService.h
 *
 * This file was created from .idlcpp/IToiChannelService.idl
 * by the Motorola IDL backend, which is based on the OmniORB IDL parser.
 *
 * This file was generated from the AbstractHeader.tpl template.
 *
 * Copyright (c) 2009 Motorola, Inc. All Rights Reserved.
 *
 *-----------------------------------------------------------------------------
 */

#ifndef ITOICHANNELSERVICE_H
#define ITOICHANNELSERVICE_H

#include "interface/TToiInvalidArgumentException.h"
#include "interface/TToiNotEnoughResourcesException.h"
#include "interface/TToiUnhandledValueException.h"
#include "interface/IToiApplicationService.h"
#include "interface/IToiOperationManager.h"
#include "interface/TToiOperationNotSupportedException.h"
#include "interface/TToiNoDataException.h"
#include "interface/TToiSessionViolationException.h"
#include "interface/IToiMediaSession.h"
#include "interface/IToiFrontendService.h"

#include "ipc/TDispatchableExceptionTemplate.h"
#include "ipc/TIpcException.h"

#include <stdint.h>
#include <cstring>
#include <string>
#include <vector>





#define PROPERTY_MODULATION_SYSTEM_TYPE "system.modulationsystemtype"
#define PROPERTY_NETWORK_NAME "system.networkname"
#define PROPERTY_NETWORK_ID "system.networkid"
#define PROPERTY_SERVICE_TYPE "system.servicetype"
#define PROPERTY_SERVICE_PROVIDER "system.serviceprovider"
#define PROPERTY_SERVICE_ID "system.serviceid"
#define PROPERTY_SERVICE_NAME "system.servicename"
#define PROPERTY_LOGICAL_CHANNEL_NUMBER "system.logicalchannelnumber"
#define PROPERTY_VISIBLE "system.visible"
#define PROPERTY_FREE "system.free"
#define PROPERTY_ORIGINAL_NETWORK_ID "system.originalnetworkid"
#define PROPERTY_TRANSPORT_STREAM_ID "system.transportstreamid"
#define PROPERTY_DELIVERY_SYSTEM_DESCRIPTOR "system.deliverysystemdescriptor"
#define PROPERTY_URI "system.uri"
#define PROPERTY_SOURCE_ID "system.sourceid"
#define PROPERTY_FREQUENCY "system.frequency"
#define PROPERTY_MODULATION_FORMAT "system.modulationformat"

class IToiChannelService
{
public:
  enum { OP_RESULT_CHANNEL_INFO = 1 };
  struct TToiPropertyDefinition;
  struct TToiProperty;
  struct TToiQueryCondition;
  struct TToiChannelInfo;
  
  enum TToiPropertyType
  {
    TYPE_BOOLEAN = 0,
    TYPE_STRING = 1,
    TYPE_INTEGER = 2
  };
  enum TToiConditionOperation
  {
    EQUALS = 0,
    NOT_EQUAL = 1
  };
  typedef std::string TToiPropertyId;
  typedef std::vector< TToiPropertyId > TToiPropertyIdSequence;
  typedef std::string TToiPropertyValue;
  typedef int TToiChannelId;
  typedef std::vector< TToiChannelId > TToiChannelIdSequence;
  typedef std::vector< TToiProperty > TToiPropertySequence;
  typedef std::vector< TToiQueryCondition > TToiQueryConditionSequence;
  typedef std::vector< TToiChannelInfo > TToiChannelInfoSequence;
  
  struct TToiPropertyDefinition {
    TToiPropertyId PropertyId;
    TToiPropertyType Type;
    bool IsSystemDefined;
    
    TToiPropertyDefinition() :
      PropertyId(""), 
      Type(TYPE_BOOLEAN), 
      IsSystemDefined(false)  {   }
    
    TToiPropertyDefinition(
      const TToiPropertyId& _PropertyId, 
      const TToiPropertyType& _Type, 
      const bool& _IsSystemDefined) :
        PropertyId(_PropertyId), 
        Type(_Type), 
        IsSystemDefined(_IsSystemDefined)
    {
    }
    
  };
  
  struct TToiProperty {
    TToiPropertyId Id;
    TToiPropertyValue Value;
    
    TToiProperty() :
      Id(""), 
      Value("")  {   }
    
    TToiProperty(
      const TToiPropertyId& _Id, 
      const TToiPropertyValue& _Value) :
        Id(_Id), 
        Value(_Value)
    {
    }
    
  };
  
  struct TToiQueryCondition {
    TToiProperty Operand;
    TToiConditionOperation Operation;
    
    TToiQueryCondition() :
      Operation(EQUALS)  {   }
    
    TToiQueryCondition(
      const TToiProperty& _Operand, 
      const TToiConditionOperation& _Operation) :
        Operand(_Operand), 
        Operation(_Operation)
    {
    }
    
  };
  
  struct TToiChannelInfo {
    TToiChannelId Id;
    std::string Name;
    TToiPropertySequence Properties;
    
    TToiChannelInfo() :
      Id(0), 
      Name("")  {   }
    
    TToiChannelInfo(
      const TToiChannelId& _Id, 
      const std::string& _Name, 
      const TToiPropertySequence& _Properties) :
        Id(_Id), 
        Name(_Name), 
        Properties(_Properties)
    {
    }
    
  };
  
  virtual ~IToiChannelService() { /* Empty */ }
  virtual void GetOperationManager(std::string& address) throw (TIpcException) = 0;
  virtual void GetPropertyDefinition(const TToiPropertyId& propertyId, TToiPropertyDefinition& definition) throw (TToiInvalidArgumentException, TToiNotEnoughResourcesException, TIpcException) = 0;
  virtual void RunQuery(const IToiOperationManager::TToiOperationId& operationId, const TToiQueryConditionSequence& queryConditions, const TToiPropertyIdSequence& requestedProperties) throw (TToiInvalidArgumentException, TToiNotEnoughResourcesException, TIpcException) = 0;
  virtual void GetChannelsInfo(const TToiChannelIdSequence& channels, const TToiPropertyIdSequence& requestedProperties, TToiChannelInfoSequence& channelsInfo) throw (TToiInvalidArgumentException, TToiNotEnoughResourcesException, TIpcException) = 0;
  virtual void GetChannelInfoResult(const IToiOperationManager::TToiOperationId& operationId, TToiChannelInfoSequence& results) throw (TToiInvalidArgumentException, TToiNoDataException, TIpcException) = 0;
  virtual void RegisterObserver(const IToiApplicationService::TToiApplicationId& applicationId, const std::string& address) throw (TToiInvalidArgumentException, TIpcException) = 0;
  virtual void ReleaseObserver(const std::string& address) throw (TToiInvalidArgumentException, TIpcException) = 0;
};



#define IMPLEMENTS_ITOICHANNELSERVICE \
virtual void GetOperationManager(std::string& address) throw (); \
virtual void GetPropertyDefinition(const TToiPropertyId& propertyId, TToiPropertyDefinition& definition) throw (TToiInvalidArgumentException, TToiNotEnoughResourcesException); \
virtual void RunQuery(const IToiOperationManager::TToiOperationId& operationId, const TToiQueryConditionSequence& queryConditions, const TToiPropertyIdSequence& requestedProperties) throw (TToiInvalidArgumentException, TToiNotEnoughResourcesException); \
virtual void GetChannelsInfo(const TToiChannelIdSequence& channels, const TToiPropertyIdSequence& requestedProperties, TToiChannelInfoSequence& channelsInfo) throw (TToiInvalidArgumentException, TToiNotEnoughResourcesException); \
virtual void GetChannelInfoResult(const IToiOperationManager::TToiOperationId& operationId, TToiChannelInfoSequence& results) throw (TToiInvalidArgumentException, TToiNoDataException); \
virtual void RegisterObserver(const IToiApplicationService::TToiApplicationId& applicationId, const std::string& address) throw (TToiInvalidArgumentException); \
virtual void ReleaseObserver(const std::string& address) throw (TToiInvalidArgumentException);

#endif 

