/*
 *-----------------------------------------------------------------------------
 *
 * IToiFrontendService.h
 *
 * This file was created from .idlcpp/IToiFrontendService.idl
 * by the Motorola IDL backend, which is based on the OmniORB IDL parser.
 *
 * This file was generated from the AbstractHeader.tpl template.
 *
 * Copyright (c) 2009 Motorola, Inc. All Rights Reserved.
 *
 *-----------------------------------------------------------------------------
 */

#ifndef ITOIFRONTENDSERVICE_H
#define ITOIFRONTENDSERVICE_H

#include "interface/TToiInvalidArgumentException.h"
#include "interface/TToiOperationNotSupportedException.h"
#include "interface/TToiNotEnoughResourcesException.h"
#include "interface/TToiUnhandledValueException.h"
#include "interface/IToiApplicationService.h"
#include "interface/TToiNoDataException.h"
#include "interface/TToiSessionViolationException.h"
#include "interface/IToiMediaSession.h"

#include "ipc/TDispatchableExceptionTemplate.h"
#include "ipc/TIpcException.h"

#include <stdint.h>
#include <cstring>
#include <string>
#include <vector>






class IToiFrontendService
{
public:
  enum { FRONTEND_ID_ANY_FREE = -1 };
  enum { PROPERTY_SIGNAL_STRENGTH_INDICATOR = 1 };
  enum { PROPERTY_SIGNAL_QUALITY_INDICATOR = 2 };
  enum { PROPERTY_DERIVED_CHANNEL_ID = 3 };
  struct TToiProperty;
  struct TToiFrontendCapabilities;
  struct TToiFrontendInfo;
  
  enum TToiModulationSystem
  {
    CAPABILITY_SYSTEM_DVB_C = 0x1,
    CAPABILITY_SYSTEM_DVB_S = 0x2,
    CAPABILITY_SYSTEM_DVB_S2 = 0x4,
    CAPABILITY_SYSTEM_DVB_T = 0x8,
    CAPABILITY_SYSTEM_ISDB_C = 0x10,
    CAPABILITY_SYSTEM_ISDB_S = 0x20,
    CAPABILITY_SYSTEM_ISDB_T = 0x40,
    CAPABILITY_SYSTEM_SBTVD = 0x80,
    CAPABILITY_SYSTEM_SCTE = 0x100,
    CAPABILITY_SYSTEM_ATSC = 0x200
  };
  enum TToiChannelBandwidth
  {
    CAPABILITY_BANDWIDTH_1_8MHZ = 0x1,
    CAPABILITY_BANDWIDTH_5MHZ = 0x2,
    CAPABILITY_BANDWIDTH_6MHZ = 0x4,
    CAPABILITY_BANDWIDTH_7MHZ = 0x8,
    CAPABILITY_BANDWIDTH_8MHZ = 0x10
  };
  enum TToiModulationMode
  {
    CAPABILITY_MODULATION_QAM_16 = 0x1,
    CAPABILITY_MODULATION_QAM_32 = 0x2,
    CAPABILITY_MODULATION_QAM_64 = 0x4,
    CAPABILITY_MODULATION_QAM_128 = 0x8,
    CAPABILITY_MODULATION_QAM_256 = 0x10,
    CAPABILITY_MODULATION_QPSK = 0x20,
    CAPABILITY_MODULATION_DVS178_QPSK = 0x40
  };
  typedef int TToiFrontendId;
  typedef std::vector< TToiFrontendId > TToiFrontendIdSequence;
  typedef int TToiPropertyId;
  typedef std::vector< TToiProperty > TToiPropertySequence;
  
  struct TToiProperty {
    TToiPropertyId Id;
    std::string Value;
    
    TToiProperty() :
      Id(0), 
      Value("")  {   }
    
    TToiProperty(
      const TToiPropertyId& _Id, 
      const std::string& _Value) :
        Id(_Id), 
        Value(_Value)
    {
    }
    
  };
  
  struct TToiFrontendCapabilities {
    int ModulationSystemMask;
    int ChannelBandwidthMask;
    int ModulationModeMask;
    
    TToiFrontendCapabilities() :
      ModulationSystemMask(0), 
      ChannelBandwidthMask(0), 
      ModulationModeMask(0)  {   }
    
    TToiFrontendCapabilities(
      const int& _ModulationSystemMask, 
      const int& _ChannelBandwidthMask, 
      const int& _ModulationModeMask) :
        ModulationSystemMask(_ModulationSystemMask), 
        ChannelBandwidthMask(_ChannelBandwidthMask), 
        ModulationModeMask(_ModulationModeMask)
    {
    }
    
  };
  
  struct TToiFrontendInfo {
    IToiMediaSession::TToiSessionId SessionId;
    TToiFrontendCapabilities FrontendCapabilities;
    bool IsLocked;
    int CentreFrequency;
    int SignalNoiseRatio;
    int BitErrorRate;
    int UncorrectedBlocks;
    int RFLevel;
    TToiPropertySequence ExtendedProperties;
    
    TToiFrontendInfo() :
      SessionId(0), 
      IsLocked(false), 
      CentreFrequency(0), 
      SignalNoiseRatio(0), 
      BitErrorRate(0), 
      UncorrectedBlocks(0), 
      RFLevel(0)  {   }
    
    TToiFrontendInfo(
      const IToiMediaSession::TToiSessionId& _SessionId, 
      const TToiFrontendCapabilities& _FrontendCapabilities, 
      const bool& _IsLocked, 
      const int& _CentreFrequency, 
      const int& _SignalNoiseRatio, 
      const int& _BitErrorRate, 
      const int& _UncorrectedBlocks, 
      const int& _RFLevel, 
      const TToiPropertySequence& _ExtendedProperties) :
        SessionId(_SessionId), 
        FrontendCapabilities(_FrontendCapabilities), 
        IsLocked(_IsLocked), 
        CentreFrequency(_CentreFrequency), 
        SignalNoiseRatio(_SignalNoiseRatio), 
        BitErrorRate(_BitErrorRate), 
        UncorrectedBlocks(_UncorrectedBlocks), 
        RFLevel(_RFLevel), 
        ExtendedProperties(_ExtendedProperties)
    {
    }
    
  };
  
  virtual ~IToiFrontendService() { /* Empty */ }
  virtual void GetOperationManager(std::string& address) throw (TIpcException) = 0;
  virtual void CreateDvbTScanner(const IToiApplicationService::TToiApplicationId& applicationId, const TToiFrontendId& frontendId, std::string& scannerAddress) throw (TToiInvalidArgumentException, TToiNotEnoughResourcesException, TToiOperationNotSupportedException, TIpcException) = 0;
  virtual void CreateDvbCScanner(const IToiApplicationService::TToiApplicationId& applicationId, const TToiFrontendId& frontendId, std::string& scannerAddress) throw (TToiInvalidArgumentException, TToiNotEnoughResourcesException, TToiOperationNotSupportedException, TIpcException) = 0;
  virtual void CreateFrontendDownloader(const IToiApplicationService::TToiApplicationId& applicationId, const TToiFrontendId& frontendId, std::string& downloaderAddress) throw (TToiInvalidArgumentException, TToiNotEnoughResourcesException, TToiOperationNotSupportedException, TIpcException) = 0;
  virtual void GetFrontends(TToiFrontendIdSequence& identifiers) throw (TToiOperationNotSupportedException, TIpcException) = 0;
  virtual void GetFrontendInfo(const TToiFrontendId& frontendId, TToiFrontendInfo& info) throw (TToiInvalidArgumentException, TToiNoDataException, TToiOperationNotSupportedException, TIpcException) = 0;
  virtual void GetFrontendInfoBySession(const IToiMediaSession::TToiSessionId& sessionId, TToiFrontendInfo& info) throw (TToiInvalidArgumentException, TToiNoDataException, TToiOperationNotSupportedException, TIpcException) = 0;
};



#define IMPLEMENTS_ITOIFRONTENDSERVICE \
virtual void GetOperationManager(std::string& address) throw (); \
virtual void CreateDvbTScanner(const IToiApplicationService::TToiApplicationId& applicationId, const TToiFrontendId& frontendId, std::string& scannerAddress) throw (TToiInvalidArgumentException, TToiNotEnoughResourcesException, TToiOperationNotSupportedException); \
virtual void CreateDvbCScanner(const IToiApplicationService::TToiApplicationId& applicationId, const TToiFrontendId& frontendId, std::string& scannerAddress) throw (TToiInvalidArgumentException, TToiNotEnoughResourcesException, TToiOperationNotSupportedException); \
virtual void CreateFrontendDownloader(const IToiApplicationService::TToiApplicationId& applicationId, const TToiFrontendId& frontendId, std::string& downloaderAddress) throw (TToiInvalidArgumentException, TToiNotEnoughResourcesException, TToiOperationNotSupportedException); \
virtual void GetFrontends(TToiFrontendIdSequence& identifiers) throw (TToiOperationNotSupportedException); \
virtual void GetFrontendInfo(const TToiFrontendId& frontendId, TToiFrontendInfo& info) throw (TToiInvalidArgumentException, TToiNoDataException, TToiOperationNotSupportedException); \
virtual void GetFrontendInfoBySession(const IToiMediaSession::TToiSessionId& sessionId, TToiFrontendInfo& info) throw (TToiInvalidArgumentException, TToiNoDataException, TToiOperationNotSupportedException);

#endif 

