/*
 *-----------------------------------------------------------------------------
 *
 * IToiMediaSession.h
 *
 * This file was created from .idlcpp/IToiMediaSession.idl
 * by the Motorola IDL backend, which is based on the OmniORB IDL parser.
 *
 * This file was generated from the AbstractHeader.tpl template.
 *
 * Copyright (c) 2009 Motorola, Inc. All Rights Reserved.
 *
 *-----------------------------------------------------------------------------
 */

#ifndef ITOIMEDIASESSION_H
#define ITOIMEDIASESSION_H

#include "interface/TToiOperationNotSupportedException.h"
#include "interface/TToiNoDataException.h"
#include "interface/TToiNotEnoughResourcesException.h"
#include "interface/TToiInvalidArgumentException.h"
#include "interface/TToiSessionViolationException.h"
#include "interface/TToiUnhandledValueException.h"
#include "interface/IToiApplicationService.h"

#include "ipc/TDispatchableExceptionTemplate.h"
#include "ipc/TIpcException.h"

#include <stdint.h>
#include <cstring>
#include <string>
#include <vector>






class IToiMediaSession
{
public:
  enum { TYPE_PLAYER = 1 };
  enum { TYPE_RECORDER = 2 };
  enum { TYPE_CONSUMER = 4 };
  enum { TYPE_DISTRIBUTOR = 8 };
  struct TToiStreamInfo;
  
  enum TToiState
  {
    STATE_CONNECTING = 0,
    STATE_RUNNING = 1,
    STATE_FAILED = 2,
    STATE_CLOSED = 3
  };
  enum TToiCipher
  {
    CIPHER_NONE = 0,
    CIPHER_DVB_CSA = 1,
    CIPHER_ARIB_MULTI2 = 2,
    CIPHER_AES_128 = 3,
    CIPHER_TRIPLE_DES = 4
  };
  enum TToiCipherBlockMode
  {
    CIPHER_BLOCK_MODE_NONE = 0,
    CIPHER_BLOCK_MODE_ECB = 1,
    CIPHER_BLOCK_MODE_CBC = 2,
    CIPHER_BLOCK_MODE_CFB = 3,
    CIPHER_BLOCK_MODE_OFB = 4,
    CIPHER_BLOCK_MODE_CTR = 5
  };
  enum TToiCipherResidueMode
  {
    CIPHER_RESIDUE_MODE_NONE = 0,
    CIPHER_RESIDUE_MODE_FRONT = 1,
    CIPHER_RESIDUE_MODE_BACK = 2,
    CIPHER_RESIDUE_MODE_CTS = 3,
    CIPHER_RESIDUE_MODE_OFB = 4
  };
  typedef uint8_t* TToiDataRelayData;
  typedef int TToiStreamId;
  typedef std::vector< TToiStreamId > TToiStreamIdSequence;
  typedef uint8_t* TToiCipherData;
  typedef int TToiSessionId;
  typedef int TToiSessionType;
  
  struct TToiStreamInfo {
    int ProgramNumber;
    
    TToiStreamInfo() :
      ProgramNumber(0)  {   }
    
    TToiStreamInfo(
      const int& _ProgramNumber) :
        ProgramNumber(_ProgramNumber)
    {
    }
    
  };
  
  virtual ~IToiMediaSession() { /* Empty */ }
  virtual void GetId(TToiSessionId& id) throw (TIpcException) = 0;
  virtual void GetType(TToiSessionType& type) throw (TToiSessionViolationException, TIpcException) = 0;
  virtual void GetUrl(std::string& url) throw (TToiSessionViolationException, TIpcException) = 0;
  virtual void GetState(TToiState& state) throw (TIpcException) = 0;
  virtual void GetStreamInfo(TToiStreamInfo& info) throw (TToiNoDataException, TToiSessionViolationException, TIpcException) = 0;
  virtual void GetAssetId(std::string& assetId) throw (TToiOperationNotSupportedException, TToiSessionViolationException, TIpcException) = 0;
  virtual void AddSectionDataRelay(const int& pid, const uint8_t* data, const uint8_t* include, const uint8_t* exclude, const int& length, const std::string& socketName, int& dataRelayId) throw (TToiInvalidArgumentException, TToiNotEnoughResourcesException, TToiSessionViolationException, TIpcException) = 0;
  virtual void AddPacketDataRelay(const int& pid, const std::string& socketName, int& dataRelayId) throw (TToiInvalidArgumentException, TToiNotEnoughResourcesException, TToiSessionViolationException, TIpcException) = 0;
  virtual void RemoveDataRelay(const int& dataRelayId) throw (TToiInvalidArgumentException, TToiSessionViolationException, TIpcException) = 0;
  virtual void GetParameter(const std::string& name, std::string& value) throw (TToiInvalidArgumentException, TToiSessionViolationException, TIpcException) = 0;
  virtual void SetParameter(const std::string& name, const std::string& value) throw (TToiInvalidArgumentException, TToiSessionViolationException, TIpcException) = 0;
  virtual void SubscribeParameter(const std::string& address, const std::string& name) throw (TToiInvalidArgumentException, TToiSessionViolationException, TIpcException) = 0;
  virtual void UnsubscribeParameter(const std::string& address, const std::string& name) throw (TToiInvalidArgumentException, TToiSessionViolationException, TIpcException) = 0;
  virtual void SetDecryptionCipher(const TToiStreamIdSequence& streamIds, const TToiCipher& cipher, const TToiCipherBlockMode& blockMode, const TToiCipherResidueMode& residueMode) throw (TToiOperationNotSupportedException, TToiSessionViolationException, TIpcException) = 0;
  virtual void SetDecryptionKeys(const TToiStreamIdSequence& streamIds, const bool& isEncrypted, const uint8_t* evenKey, const uint8_t* evenIv, const uint8_t* oddKey, const uint8_t* oddIv, const int& length) throw (TToiOperationNotSupportedException, TToiSessionViolationException, TIpcException) = 0;
  virtual void ReleaseInstance() throw (TIpcException) = 0;
  virtual void RegisterObserver(const IToiApplicationService::TToiApplicationId& applicationId, const std::string& address) throw (TToiInvalidArgumentException, TIpcException) = 0;
  virtual void ReleaseObserver(const std::string& address) throw (TToiInvalidArgumentException, TIpcException) = 0;
};



#define IMPLEMENTS_ITOIMEDIASESSION \
virtual void GetId(TToiSessionId& id) throw (); \
virtual void GetType(TToiSessionType& type) throw (TToiSessionViolationException); \
virtual void GetUrl(std::string& url) throw (TToiSessionViolationException); \
virtual void GetState(TToiState& state) throw (); \
virtual void GetStreamInfo(TToiStreamInfo& info) throw (TToiNoDataException, TToiSessionViolationException); \
virtual void GetAssetId(std::string& assetId) throw (TToiOperationNotSupportedException, TToiSessionViolationException); \
virtual void AddSectionDataRelay(const int& pid, const uint8_t* data, const uint8_t* include, const uint8_t* exclude, const int& length, const std::string& socketName, int& dataRelayId) throw (TToiInvalidArgumentException, TToiNotEnoughResourcesException, TToiSessionViolationException); \
virtual void AddPacketDataRelay(const int& pid, const std::string& socketName, int& dataRelayId) throw (TToiInvalidArgumentException, TToiNotEnoughResourcesException, TToiSessionViolationException); \
virtual void RemoveDataRelay(const int& dataRelayId) throw (TToiInvalidArgumentException, TToiSessionViolationException); \
virtual void GetParameter(const std::string& name, std::string& value) throw (TToiInvalidArgumentException, TToiSessionViolationException); \
virtual void SetParameter(const std::string& name, const std::string& value) throw (TToiInvalidArgumentException, TToiSessionViolationException); \
virtual void SubscribeParameter(const std::string& address, const std::string& name) throw (TToiInvalidArgumentException, TToiSessionViolationException); \
virtual void UnsubscribeParameter(const std::string& address, const std::string& name) throw (TToiInvalidArgumentException, TToiSessionViolationException); \
virtual void SetDecryptionCipher(const TToiStreamIdSequence& streamIds, const TToiCipher& cipher, const TToiCipherBlockMode& blockMode, const TToiCipherResidueMode& residueMode) throw (TToiOperationNotSupportedException, TToiSessionViolationException); \
virtual void SetDecryptionKeys(const TToiStreamIdSequence& streamIds, const bool& isEncrypted, const uint8_t* evenKey, const uint8_t* evenIv, const uint8_t* oddKey, const uint8_t* oddIv, const int& length) throw (TToiOperationNotSupportedException, TToiSessionViolationException); \
virtual void ReleaseInstance() throw (); \
virtual void RegisterObserver(const IToiApplicationService::TToiApplicationId& applicationId, const std::string& address) throw (TToiInvalidArgumentException); \
virtual void ReleaseObserver(const std::string& address) throw (TToiInvalidArgumentException);

#endif 

