/*
 *-----------------------------------------------------------------------------
 *
 * IToiSchedulerService.h
 *
 * This file was created from .idlcpp/IToiSchedulerService.idl
 * by the Motorola IDL backend, which is based on the OmniORB IDL parser.
 *
 * This file was generated from the AbstractHeader.tpl template.
 *
 * Copyright (c) 2009 Motorola, Inc. All Rights Reserved.
 *
 *-----------------------------------------------------------------------------
 */

#ifndef ITOISCHEDULERSERVICE_H
#define ITOISCHEDULERSERVICE_H

#include "interface/TToiInvalidArgumentException.h"
#include "interface/TToiNotEnoughResourcesException.h"
#include "interface/TToiOperationNotSupportedException.h"
#include "interface/TToiPermissionDeniedException.h"
#include "interface/TToiUnhandledValueException.h"
#include "interface/IToiApplicationService.h"

#include "ipc/TDispatchableExceptionTemplate.h"
#include "ipc/TIpcException.h"

#include <stdint.h>
#include <cstring>
#include <string>
#include <vector>






class IToiSchedulerService
{
public:
  enum { BOOKING_ID_NONE = 0 };
  enum { SERIES_ID_NONE = 0 };
  enum { TIME_NOW = -1 };
  enum { WEEKDAY_MONDAY = 1 };
  enum { WEEKDAY_TUESDAY = 2 };
  enum { WEEKDAY_WEDNESDAY = 4 };
  enum { WEEKDAY_THURSDAY = 8 };
  enum { WEEKDAY_FRIDAY = 16 };
  enum { WEEKDAY_SATURDAY = 32 };
  enum { WEEKDAY_SUNDAY = 64 };
  enum { WEEKDAY_ALL_WEEKDAYS = 127 };
  struct TToiSeries;
  struct TToiBooking;
  struct TToiSolution;
  
  enum TToiConflictStrategy
  {
    CONFLICT_STRATEGY_FAIL = 0,
    CONFLICT_STRATEGY_SKIP_BOOKINGS = 1
  };
  enum TToiSolutionType
  {
    SOLUTION_TYPE_NONE = 0,
    SOLUTION_TYPE_REMOVE = 1
  };
  typedef int TToiBookingId;
  typedef int TToiSeriesId;
  typedef int TToiTime;
  typedef int TToiWeekdayMask;
  typedef std::vector< TToiBookingId > TToiBookingIdSequence;
  typedef std::vector< TToiSeriesId > TToiSeriesIdSequence;
  typedef std::vector< TToiBooking > TToiBookingSequence;
  typedef std::vector< TToiSolution > TToiConflictSequence;
  typedef int TToiDelta;
  typedef std::vector< TToiDelta > TToiDeltaSequence;
  typedef std::vector< std::string > TToiParameterNameSequence;
  typedef std::vector< std::string > TToiParameterValueSequence;
  
  struct TToiSeries {
    TToiSeriesId Id;
    std::string Category;
    std::string ActivityType;
    TToiTime Start;
    int Duration;
    int Weekdays;
    int OriginalNumberOfOccurences;
    
    TToiSeries() :
      Id(0), 
      Category(""), 
      ActivityType(""), 
      Start(0), 
      Duration(0), 
      Weekdays(0), 
      OriginalNumberOfOccurences(0)  {   }
    
    TToiSeries(
      const TToiSeriesId& _Id, 
      const std::string& _Category, 
      const std::string& _ActivityType, 
      const TToiTime& _Start, 
      const int& _Duration, 
      const int& _Weekdays, 
      const int& _OriginalNumberOfOccurences) :
        Id(_Id), 
        Category(_Category), 
        ActivityType(_ActivityType), 
        Start(_Start), 
        Duration(_Duration), 
        Weekdays(_Weekdays), 
        OriginalNumberOfOccurences(_OriginalNumberOfOccurences)
    {
    }
    
  };
  
  struct TToiBooking {
    TToiBookingId Id;
    std::string Category;
    std::string ActivityType;
    TToiTime Start;
    int Duration;
    TToiSeriesId SeriesId;
    
    TToiBooking() :
      Id(0), 
      Category(""), 
      ActivityType(""), 
      Start(0), 
      Duration(0), 
      SeriesId(0)  {   }
    
    TToiBooking(
      const TToiBookingId& _Id, 
      const std::string& _Category, 
      const std::string& _ActivityType, 
      const TToiTime& _Start, 
      const int& _Duration, 
      const TToiSeriesId& _SeriesId) :
        Id(_Id), 
        Category(_Category), 
        ActivityType(_ActivityType), 
        Start(_Start), 
        Duration(_Duration), 
        SeriesId(_SeriesId)
    {
    }
    
  };
  
  struct TToiSolution {
    TToiSolutionType Type;
    TToiBookingIdSequence AffectedBookings;
    
    TToiSolution() :
      Type(SOLUTION_TYPE_NONE)  {   }
    
    TToiSolution(
      const TToiSolutionType& _Type, 
      const TToiBookingIdSequence& _AffectedBookings) :
        Type(_Type), 
        AffectedBookings(_AffectedBookings)
    {
    }
    
  };
  
  virtual ~IToiSchedulerService() { /* Empty */ }
  virtual void Schedule(const std::string& category, const std::string& activity, const TToiTime& start, const int& duration, TToiBookingId& id) throw (TToiInvalidArgumentException, TToiNotEnoughResourcesException, TIpcException) = 0;
  virtual void ScheduleSeries(const std::string& category, const std::string& activity, const TToiTime& start, const int& duration, const int& weekdays, const int& numberOfOccurrences, const TToiConflictStrategy& conflictStrategy, TToiSeriesId& id) throw (TToiInvalidArgumentException, TToiNotEnoughResourcesException, TIpcException) = 0;
  virtual void Reschedule(const TToiBookingId& id, const std::string& category, const std::string& activity, const TToiTime& start, const int& duration) throw (TToiInvalidArgumentException, TToiNotEnoughResourcesException, TIpcException) = 0;
  virtual void Remove(const TToiBookingId& id) throw (TToiInvalidArgumentException, TIpcException) = 0;
  virtual void RemoveSeries(const TToiSeriesId& id) throw (TToiInvalidArgumentException, TIpcException) = 0;
  virtual void GetConflicts(const TToiBookingId& id, const std::string& category, const std::string& activity, const TToiTime& start, const int& duration, TToiConflictSequence& conflicts) throw (TToiInvalidArgumentException, TIpcException) = 0;
  virtual void GetSeriesConflicts(const TToiSeriesId& id, const std::string& category, const std::string& activity, const TToiTime& start, const int& duration, const int& weekdays, const int& occurrence, TToiConflictSequence& conflicts) throw (TToiInvalidArgumentException, TIpcException) = 0;
  virtual void GetNextStartTime(const std::string& categoryExpression, TToiTime& nextStart) throw (TIpcException) = 0;
  virtual void GetBooking(const TToiBookingId& id, TToiBooking& booking) throw (TToiInvalidArgumentException, TIpcException) = 0;
  virtual void GetSeries(const TToiSeriesId& id, TToiSeries& series) throw (TToiInvalidArgumentException, TIpcException) = 0;
  virtual void GetBookingIds(const std::string& categoryExpression, const TToiTime& start, const TToiTime& stop, TToiBookingIdSequence& bookings) throw (TToiInvalidArgumentException, TIpcException) = 0;
  virtual void GetSeriesIds(const std::string& categoryExpression, TToiSeriesIdSequence& seriesIds) throw (TIpcException) = 0;
  virtual void GetBookingIdsForSeries(const TToiSeriesId& seriesId, TToiBookingIdSequence& bookings) throw (TToiInvalidArgumentException, TIpcException) = 0;
  virtual void AddTrigger(const TToiBookingId& id, const TToiDelta& delta) throw (TToiInvalidArgumentException, TIpcException) = 0;
  virtual void RemoveTrigger(const TToiBookingId& id, const TToiDelta& delta) throw (TToiInvalidArgumentException, TIpcException) = 0;
  virtual void AddTriggerOnBookings(const TToiBookingIdSequence& ids, const TToiDelta& delta) throw (TToiInvalidArgumentException, TIpcException) = 0;
  virtual void RemoveTriggerOnBookings(const TToiBookingIdSequence& ids, const TToiDelta& delta) throw (TToiInvalidArgumentException, TIpcException) = 0;
  virtual void GetTriggers(const TToiBookingId& id, TToiDeltaSequence& deltas) throw (TToiInvalidArgumentException, TIpcException) = 0;
  virtual void SetParameter(const TToiBookingId& id, const std::string& name, const std::string& value) throw (TToiInvalidArgumentException, TIpcException) = 0;
  virtual void UnsetParameter(const TToiBookingId& id, const std::string& name) throw (TToiInvalidArgumentException, TIpcException) = 0;
  virtual void GetParameter(const TToiBookingId& id, const std::string& name, std::string& value) throw (TToiInvalidArgumentException, TIpcException) = 0;
  virtual void GetParameterNames(const TToiBookingId& id, TToiParameterNameSequence& names) throw (TToiInvalidArgumentException, TIpcException) = 0;
  virtual void SetParametersOnBookings(const TToiBookingIdSequence& ids, const TToiParameterNameSequence& names, const TToiParameterValueSequence& values) throw (TToiInvalidArgumentException, TIpcException) = 0;
  virtual void SetSeriesParameter(const TToiSeriesId& id, const std::string& name, const std::string& value) throw (TToiInvalidArgumentException, TIpcException) = 0;
  virtual void UnsetSeriesParameter(const TToiSeriesId& id, const std::string& name) throw (TToiInvalidArgumentException, TIpcException) = 0;
  virtual void GetSeriesParameter(const TToiSeriesId& id, const std::string& name, std::string& value) throw (TToiInvalidArgumentException, TIpcException) = 0;
  virtual void GetSeriesParameterNames(const TToiSeriesId& id, TToiParameterNameSequence& names) throw (TToiInvalidArgumentException, TIpcException) = 0;
  virtual void SetCategorySubscription(const std::string& address, const std::string& categoryExpression) throw (TToiInvalidArgumentException, TIpcException) = 0;
  virtual void RegisterObserver(const IToiApplicationService::TToiApplicationId& applicationId, const std::string& address) throw (TToiInvalidArgumentException, TIpcException) = 0;
  virtual void ReleaseObserver(const std::string& address) throw (TToiInvalidArgumentException, TIpcException) = 0;
};



#define IMPLEMENTS_ITOISCHEDULERSERVICE \
virtual void Schedule(const std::string& category, const std::string& activity, const TToiTime& start, const int& duration, TToiBookingId& id) throw (TToiInvalidArgumentException, TToiNotEnoughResourcesException); \
virtual void ScheduleSeries(const std::string& category, const std::string& activity, const TToiTime& start, const int& duration, const int& weekdays, const int& numberOfOccurrences, const TToiConflictStrategy& conflictStrategy, TToiSeriesId& id) throw (TToiInvalidArgumentException, TToiNotEnoughResourcesException); \
virtual void Reschedule(const TToiBookingId& id, const std::string& category, const std::string& activity, const TToiTime& start, const int& duration) throw (TToiInvalidArgumentException, TToiNotEnoughResourcesException); \
virtual void Remove(const TToiBookingId& id) throw (TToiInvalidArgumentException); \
virtual void RemoveSeries(const TToiSeriesId& id) throw (TToiInvalidArgumentException); \
virtual void GetConflicts(const TToiBookingId& id, const std::string& category, const std::string& activity, const TToiTime& start, const int& duration, TToiConflictSequence& conflicts) throw (TToiInvalidArgumentException); \
virtual void GetSeriesConflicts(const TToiSeriesId& id, const std::string& category, const std::string& activity, const TToiTime& start, const int& duration, const int& weekdays, const int& occurrence, TToiConflictSequence& conflicts) throw (TToiInvalidArgumentException); \
virtual void GetNextStartTime(const std::string& categoryExpression, TToiTime& nextStart) throw (); \
virtual void GetBooking(const TToiBookingId& id, TToiBooking& booking) throw (TToiInvalidArgumentException); \
virtual void GetSeries(const TToiSeriesId& id, TToiSeries& series) throw (TToiInvalidArgumentException); \
virtual void GetBookingIds(const std::string& categoryExpression, const TToiTime& start, const TToiTime& stop, TToiBookingIdSequence& bookings) throw (TToiInvalidArgumentException); \
virtual void GetSeriesIds(const std::string& categoryExpression, TToiSeriesIdSequence& seriesIds) throw (); \
virtual void GetBookingIdsForSeries(const TToiSeriesId& seriesId, TToiBookingIdSequence& bookings) throw (TToiInvalidArgumentException); \
virtual void AddTrigger(const TToiBookingId& id, const TToiDelta& delta) throw (TToiInvalidArgumentException); \
virtual void RemoveTrigger(const TToiBookingId& id, const TToiDelta& delta) throw (TToiInvalidArgumentException); \
virtual void AddTriggerOnBookings(const TToiBookingIdSequence& ids, const TToiDelta& delta) throw (TToiInvalidArgumentException); \
virtual void RemoveTriggerOnBookings(const TToiBookingIdSequence& ids, const TToiDelta& delta) throw (TToiInvalidArgumentException); \
virtual void GetTriggers(const TToiBookingId& id, TToiDeltaSequence& deltas) throw (TToiInvalidArgumentException); \
virtual void SetParameter(const TToiBookingId& id, const std::string& name, const std::string& value) throw (TToiInvalidArgumentException); \
virtual void UnsetParameter(const TToiBookingId& id, const std::string& name) throw (TToiInvalidArgumentException); \
virtual void GetParameter(const TToiBookingId& id, const std::string& name, std::string& value) throw (TToiInvalidArgumentException); \
virtual void GetParameterNames(const TToiBookingId& id, TToiParameterNameSequence& names) throw (TToiInvalidArgumentException); \
virtual void SetParametersOnBookings(const TToiBookingIdSequence& ids, const TToiParameterNameSequence& names, const TToiParameterValueSequence& values) throw (TToiInvalidArgumentException); \
virtual void SetSeriesParameter(const TToiSeriesId& id, const std::string& name, const std::string& value) throw (TToiInvalidArgumentException); \
virtual void UnsetSeriesParameter(const TToiSeriesId& id, const std::string& name) throw (TToiInvalidArgumentException); \
virtual void GetSeriesParameter(const TToiSeriesId& id, const std::string& name, std::string& value) throw (TToiInvalidArgumentException); \
virtual void GetSeriesParameterNames(const TToiSeriesId& id, TToiParameterNameSequence& names) throw (TToiInvalidArgumentException); \
virtual void SetCategorySubscription(const std::string& address, const std::string& categoryExpression) throw (TToiInvalidArgumentException); \
virtual void RegisterObserver(const IToiApplicationService::TToiApplicationId& applicationId, const std::string& address) throw (TToiInvalidArgumentException); \
virtual void ReleaseObserver(const std::string& address) throw (TToiInvalidArgumentException);

#endif 

