/*
 *-----------------------------------------------------------------------------
 *
 * IToiStorageDevice.h
 *
 * This file was created from .idlcpp/IToiStorageDevice.idl
 * by the Motorola IDL backend, which is based on the OmniORB IDL parser.
 *
 * This file was generated from the AbstractHeader.tpl template.
 *
 * Copyright (c) 2009 Motorola, Inc. All Rights Reserved.
 *
 *-----------------------------------------------------------------------------
 */

#ifndef ITOISTORAGEDEVICE_H
#define ITOISTORAGEDEVICE_H

#include "interface/TToiInvalidArgumentException.h"
#include "interface/TToiNotEnoughResourcesException.h"
#include "interface/TToiUnhandledValueException.h"
#include "interface/IToiApplicationService.h"
#include "interface/IToiOperationManager.h"
#include "interface/TToiInvalidPreconditionException.h"
#include "interface/TToiOperationNotSupportedException.h"

#include "ipc/TDispatchableExceptionTemplate.h"
#include "ipc/TIpcException.h"

#include <stdint.h>
#include <cstring>
#include <string>
#include <vector>






class IToiStorageDevice
{
public:
  struct TToiStorageInfo;
  struct TToiPartitionInfo;
  
  enum TToiFileSystemStatus
  {
    FS_STATUS_OK = 0,
    FS_STATUS_ERRORS_CORRECTED = 1,
    FS_STATUS_ERRORS_LEFT_UNCORRECTED = 2,
    FS_STATUS_MAJOR_FAILURE = 3
  };
  typedef unsigned int TToiStorageDeviceId;
  typedef unsigned int TToiPartitionId;
  typedef std::vector< TToiPartitionId > TToiPartitionIdSequence;
  typedef TToiFileSystemStatus TFileSystemStatus;
  
  struct TToiStorageInfo {
    TToiStorageDeviceId Id;
    std::string SerialNumber;
    bool IsInternal;
    bool IsConnected;
    bool IsSpinning;
    bool IsOverheated;
    TToiPartitionIdSequence Partitions;
    int64_t Capacity;
    int64_t FreeCapacity;
    int Temperature;
    bool IsApproved;
    
    TToiStorageInfo() :
      Id(0), 
      SerialNumber(""), 
      IsInternal(false), 
      IsConnected(false), 
      IsSpinning(false), 
      IsOverheated(false), 
      Capacity(0), 
      FreeCapacity(0), 
      Temperature(0), 
      IsApproved(false)  {   }
    
    TToiStorageInfo(
      const TToiStorageDeviceId& _Id, 
      const std::string& _SerialNumber, 
      const bool& _IsInternal, 
      const bool& _IsConnected, 
      const bool& _IsSpinning, 
      const bool& _IsOverheated, 
      const TToiPartitionIdSequence& _Partitions, 
      const int64_t& _Capacity, 
      const int64_t& _FreeCapacity, 
      const int& _Temperature, 
      const bool& _IsApproved) :
        Id(_Id), 
        SerialNumber(_SerialNumber), 
        IsInternal(_IsInternal), 
        IsConnected(_IsConnected), 
        IsSpinning(_IsSpinning), 
        IsOverheated(_IsOverheated), 
        Partitions(_Partitions), 
        Capacity(_Capacity), 
        FreeCapacity(_FreeCapacity), 
        Temperature(_Temperature), 
        IsApproved(_IsApproved)
    {
    }
    
  };
  
  struct TToiPartitionInfo {
    TToiPartitionId Id;
    std::string UniqueId;
    bool IsMounted;
    std::string MountPoint;
    int64_t Capacity;
    int64_t FreeCapacity;
    TToiFileSystemStatus FsStatus;
    bool IsReadWrite;
    
    TToiPartitionInfo() :
      Id(0), 
      UniqueId(""), 
      IsMounted(false), 
      MountPoint(""), 
      Capacity(0), 
      FreeCapacity(0), 
      FsStatus(FS_STATUS_OK), 
      IsReadWrite(false)  {   }
    
    TToiPartitionInfo(
      const TToiPartitionId& _Id, 
      const std::string& _UniqueId, 
      const bool& _IsMounted, 
      const std::string& _MountPoint, 
      const int64_t& _Capacity, 
      const int64_t& _FreeCapacity, 
      const TToiFileSystemStatus& _FsStatus, 
      const bool& _IsReadWrite) :
        Id(_Id), 
        UniqueId(_UniqueId), 
        IsMounted(_IsMounted), 
        MountPoint(_MountPoint), 
        Capacity(_Capacity), 
        FreeCapacity(_FreeCapacity), 
        FsStatus(_FsStatus), 
        IsReadWrite(_IsReadWrite)
    {
    }
    
  };
  
  virtual ~IToiStorageDevice() { /* Empty */ }
  virtual void GetStorageInfo(TToiStorageInfo& info) throw (TIpcException) = 0;
  virtual void GetPartitionInfo(const TToiPartitionId& partitionId, TToiPartitionInfo& info) throw (TToiInvalidArgumentException, TToiInvalidPreconditionException, TIpcException) = 0;
  virtual void MountPartition(const IToiOperationManager::TToiOperationId& operationId, const TToiPartitionId& partitionId, const std::string& mountPoint) throw (TToiInvalidArgumentException, TToiInvalidPreconditionException, TIpcException) = 0;
  virtual void UnmountPartition(const IToiOperationManager::TToiOperationId& operationId, const TToiPartitionId& partitionId) throw (TToiInvalidArgumentException, TToiInvalidPreconditionException, TIpcException) = 0;
  virtual void Reformat(const IToiOperationManager::TToiOperationId& operationId) throw (TToiInvalidArgumentException, TToiInvalidPreconditionException, TToiOperationNotSupportedException, TIpcException) = 0;
  virtual void ScheduleReformatOnNextReboot(const TToiPartitionId& partitionId) throw (TToiOperationNotSupportedException, TToiInvalidArgumentException, TIpcException) = 0;
  virtual void ScheduleFileSystemRepairOnNextReboot(const TToiPartitionId& partitionId) throw (TToiOperationNotSupportedException, TToiInvalidArgumentException, TIpcException) = 0;
  virtual void SpinDown() throw (TToiInvalidPreconditionException, TIpcException) = 0;
  virtual void SpinUp() throw (TToiInvalidPreconditionException, TIpcException) = 0;
};



#define IMPLEMENTS_ITOISTORAGEDEVICE \
virtual void GetStorageInfo(TToiStorageInfo& info) throw (); \
virtual void GetPartitionInfo(const TToiPartitionId& partitionId, TToiPartitionInfo& info) throw (TToiInvalidArgumentException, TToiInvalidPreconditionException); \
virtual void MountPartition(const IToiOperationManager::TToiOperationId& operationId, const TToiPartitionId& partitionId, const std::string& mountPoint) throw (TToiInvalidArgumentException, TToiInvalidPreconditionException); \
virtual void UnmountPartition(const IToiOperationManager::TToiOperationId& operationId, const TToiPartitionId& partitionId) throw (TToiInvalidArgumentException, TToiInvalidPreconditionException); \
virtual void Reformat(const IToiOperationManager::TToiOperationId& operationId) throw (TToiInvalidArgumentException, TToiInvalidPreconditionException, TToiOperationNotSupportedException); \
virtual void ScheduleReformatOnNextReboot(const TToiPartitionId& partitionId) throw (TToiOperationNotSupportedException, TToiInvalidArgumentException); \
virtual void ScheduleFileSystemRepairOnNextReboot(const TToiPartitionId& partitionId) throw (TToiOperationNotSupportedException, TToiInvalidArgumentException); \
virtual void SpinDown() throw (TToiInvalidPreconditionException); \
virtual void SpinUp() throw (TToiInvalidPreconditionException);

#endif 

