/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *---------------------------------------------------------------------------
 *
 * IKreaTvGfxDisplayObserver.h
 *
 * Copyright (c) 2009 Motorola, Inc. All Rights Reserved.
 *
 *---------------------------------------------------------------------------
 */

#ifndef IKREATVGFXDISPLAYOBSERVER_H
#define IKREATVGFXDISPLAYOBSERVER_H

#include "KreaTvGfxTypes.h"

/** \class IKreaTvGfxDisplayObserver IKreaTvGfxDisplayObserver.h "kreatvgfx/IKreaTvGfxDisplayObserver.h"

    This interface needs to be implemented by applications that want
    to be informed about display related events.

    \see IKreaTvGfx::GetDisplay()
    \see IKreaTvGfxDisplay
*/
class IKreaTvGfxDisplayObserver
{
protected:
  virtual ~IKreaTvGfxDisplayObserver() { /* Empty */ }

public:
  /** Callback when an update is done, i.e. video layer settings have
      been applied and the pixmap has been flipped. When you get this
      event it is safe to draw in the previously used pixmap without
      causing tearing, e.g. when double buffering.

      \param[in] display
                        The display that has been updated.

      \see IKreaTvGfxDisplay::Update()
  */
  virtual void OnUpdate(TKreaTvGfxDisplayPointer display) throw ()= 0;


  /** Callback when the display video mode has changed.

      \param[in] display
                        The display that has a new video mode.

      \param[in] videoMode
                        The new video mode.

      \todo How to handle scaling problems in hw when this event is a
      bit delayed, i.e. HAL has changed resolution and application
      still renders in old res. Now applications must be able to
      handle the KGFX_RESULT_SCALING_ERROR.

      \todo Get this event from video output service instead?
  */
  virtual void
  OnVideoModeChanged(TKreaTvGfxDisplayPointer display,
                     const TKreaTvGfxVideoMode& videoMode) throw ()= 0;
};

#endif
