/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *---------------------------------------------------------------------------
 *
 * IKreaTvInputObserver.h
 *
 * Copyright (c) 2009 Motorola, Inc. All Rights Reserved.
 *
 *---------------------------------------------------------------------------
 */

#ifndef IKREATVINPUTOBSERVER_H
#define IKREATVINPUTOBSERVER_H

#include "KreaTvInputTypes.h"

/** class IKreaTvInputObserver IKreaTvInputObserver.h "kreatvinput/IKreaTvInputObserver.h"

    The input observer interface should be implemented by
    applications that want to be informed about keyboard events.
*/
class IKreaTvInputObserver
{
protected:
  virtual ~IKreaTvInputObserver() { /* Empty */ }

public:
  /** Callback when a key has been pressed.

      \param[in] key    The key that was pressed.

      \param[in] repeated
                        Flag indicating whether this is a repeated
                        event.

      \param[in] modifiers
                        A bitmask with modifiers.
                        See TKreaTvInputModifier.
  */
  virtual void OnKeyDown(TKreaTvInputKey key, bool repeated,
                         uint32_t modifiers) throw () = 0;


  /** Callback when a key has been released.

      \param[in] key    The key that was released.

      \param[in] modifiers
                        A bitmask with modifiers.
                        See TKreaTvInputModifier.
 */
  virtual void OnKeyUp(TKreaTvInputKey key,
                       uint32_t modifiers) throw () = 0;
};

#endif
