/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *---------------------------------------------------------------------------
 *
 * KreaTvInputTypes.h
 *
 * Copyright (c) 2009 Motorola, Inc. All Rights Reserved.
 *
 *---------------------------------------------------------------------------
 */

/** \file */

#ifndef KREATVINPUTTYPES_H
#define KREATVINPUTTYPES_H

#include <stdint.h>


/** \enum TKreaTvInputResult

    Possible return values.
*/
enum TKreaTvInputResult
{
  /** The operation completed successfully. */
  KINPUT_RESULT_OK,

  /** One or several arguments are invalid. */
  KINPUT_RESULT_INVALID_ARGUMENT,

  /** Reserve key failed (Key has been reserved by another app) . */
  KINPUT_RESULT_RESERVE_KEY_FAILED,

  /** Release key failed (Key has not been reserved by this app). */
  KINPUT_RESULT_RELEASE_KEY_FAILED,

  /** Some internal error */
  KINPUT_RESULT_INTERNAL_ERROR
};


/** \enum TKreaTvInputModifier

    Enumeration of possible input modifiers.
*/
enum TKreaTvInputModifier
{
  /** None modifier **/
  KINPUT_NO_MODIFIER = 0x00000000,

  /** Shift */
  KINPUT_MODIFIER_SHIFT = 0x00000001,

  /** Caps lock */
  KINPUT_MODIFIER_CAPS_LOCK = 0x00000002,

  /** Control */
  KINPUT_MODIFIER_CTRL = 0x00000004,

  /** Alt */
  KINPUT_MODIFIER_ALT = 0x00000008,

  /** Num lock */
  KINPUT_MODIFIER_NUM_LOCK = 0x00000010,

  /** Group shift (AltGr) */
  KINPUT_MODIFIER_GROUP_SHIFT = 0x00000080
};


/** \enum TKreaTvInputKey

    Enumeration of keys.
*/
enum TKreaTvInputKey
{
  KINPUT_KEY_E0 = 49,
  KINPUT_KEY_E1 = 10,
  KINPUT_KEY_E2 = 11,
  KINPUT_KEY_E3 = 12,
  KINPUT_KEY_E4 = 13,
  KINPUT_KEY_E5 = 14,
  KINPUT_KEY_E6 = 15,
  KINPUT_KEY_E7 = 16,
  KINPUT_KEY_E8 = 17,
  KINPUT_KEY_E9 = 18,
  KINPUT_KEY_E10 = 19,
  KINPUT_KEY_E11 = 20,
  KINPUT_KEY_E12 = 21,
  KINPUT_KEY_BACKSPACE = 22,

  KINPUT_KEY_TAB = 23,
  KINPUT_KEY_D1 = 24,
  KINPUT_KEY_D2 = 25,
  KINPUT_KEY_D3 = 26,
  KINPUT_KEY_D4 = 27,
  KINPUT_KEY_D5 = 28,
  KINPUT_KEY_D6 = 29,
  KINPUT_KEY_D7 = 30,
  KINPUT_KEY_D8 = 31,
  KINPUT_KEY_D9 = 32,
  KINPUT_KEY_D10 = 33,
  KINPUT_KEY_D11 = 34,
  KINPUT_KEY_D12 = 35,
  /** Alias for US keyboards, same as C12 */
  KINPUT_KEY_D13 = 51,
  KINPUT_KEY_ENTER = 36,

  KINPUT_KEY_CAPS_LOCK = 66,
  KINPUT_KEY_C1 = 38,
  KINPUT_KEY_C2 = 39,
  KINPUT_KEY_C3 = 40,
  KINPUT_KEY_C4 = 41,
  KINPUT_KEY_C5 = 42,
  KINPUT_KEY_C6 = 43,
  KINPUT_KEY_C7 = 44,
  KINPUT_KEY_C8 = 45,
  KINPUT_KEY_C9 = 46,
  KINPUT_KEY_C10 = 47,
  KINPUT_KEY_C11 = 48,
  /** International keyboards, same as D13 */
  KINPUT_KEY_C12 = 51,

  KINPUT_KEY_LEFT_SHIFT = 50,
  /** Not on US or japanese keyboards */
  KINPUT_KEY_B0 = 94,
  KINPUT_KEY_B1 = 52,
  KINPUT_KEY_B2 = 53,
  KINPUT_KEY_B3 = 54,
  KINPUT_KEY_B4 = 55,
  KINPUT_KEY_B5 = 56,
  KINPUT_KEY_B6 = 57,
  KINPUT_KEY_B7	= 58,
  KINPUT_KEY_B8 = 59,
  KINPUT_KEY_B9 = 60,
  KINPUT_KEY_B10 = 61,
  KINPUT_KEY_RIGHT_SHIFT = 62,

  KINPUT_KEY_LEFT_CTRL = 37,
  KINPUT_KEY_LEFT_WIN = 115,
  KINPUT_KEY_LEFT_ALT = 64,
  KINPUT_KEY_SPACE = 65,
  KINPUT_KEY_RIGHT_ALT = 113,
  KINPUT_KEY_RIGHT_WIN = 116,
  KINPUT_KEY_MENU = 117,
  KINPUT_KEY_RIGHT_CTRL = 109,

  KINPUT_KEY_ESC = 9,
  KINPUT_KEY_F1 = 67,
  KINPUT_KEY_F2 = 68,
  KINPUT_KEY_F3 = 69,
  KINPUT_KEY_F4 = 70,
  KINPUT_KEY_F5 = 71,
  KINPUT_KEY_F6 = 72,
  KINPUT_KEY_F7 = 73,
  KINPUT_KEY_F8 = 74,
  KINPUT_KEY_F9 = 75,
  KINPUT_KEY_F10 = 76,
  KINPUT_KEY_F11 = 95,
  KINPUT_KEY_F12 = 96,

  KINPUT_KEY_PRINT_SCREEN = 111,
  KINPUT_KEY_SCROLL_LOCK = 78,
  KINPUT_KEY_PAUSE = 110,

  KINPUT_KEY_INSERT = 106,
  KINPUT_KEY_HOME = 97,
  KINPUT_KEY_PAGE_UP = 99,
  KINPUT_KEY_DELETE = 107,
  KINPUT_KEY_END = 103,
  KINPUT_KEY_PAGE_DOWN = 105,

  KINPUT_KEY_UP = 98,
  KINPUT_KEY_LEFT = 100,
  KINPUT_KEY_DOWN = 104,
  KINPUT_KEY_RIGHT = 102,

  KINPUT_KEY_NUM_LOCK = 77,
  KINPUT_KEY_KEYPAD_DIVIDE = 112,
  KINPUT_KEY_KEYPAD_MULTIPLY = 63,
  KINPUT_KEY_KEYPAD_SUBTRACT = 82,

  KINPUT_KEY_KEYPAD_7 = 79,
  KINPUT_KEY_KEYPAD_8 = 80,
  KINPUT_KEY_KEYPAD_9 = 81,

  KINPUT_KEY_KEYPAD_4 = 83,
  KINPUT_KEY_KEYPAD_5 = 84,
  KINPUT_KEY_KEYPAD_6 = 85,
  KINPUT_KEY_KEYPAD_ADD = 86,

  KINPUT_KEY_KEYPAD_1 = 87,
  KINPUT_KEY_KEYPAD_2 = 88,
  KINPUT_KEY_KEYPAD_3 = 89,

  KINPUT_KEY_KEYPAD_0 = 90,
  KINPUT_KEY_KEYPAD_SEPARATOR = 91,
  KINPUT_KEY_KEYPAD_ENTER = 108
};

#endif
