/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *---------------------------------------------------------------------------
 *
 * ipc/TIpcEventHandler.h
 *
 * Copyright (c) 2002 Kreatel Communications AB. All Rights Reserved.
 * Copyright (c) 2009 Motorola, Inc. All Rights Reserved.
 *
 *---------------------------------------------------------------------------
 */

#ifndef IPC_TIPCEVENTHANDLER_H
#define IPC_TIPCEVENTHANDLER_H

#include "IConnectionObserver.h"
#include "IIpcDispatcher.h"

#include "cipc/TCIpcCallHandler.h"
#include "event/IIOEventHandler.h"
#include "event/IIOEventLoop.h"

#include <stdint.h>
#include "IIpcEventHandler.h"
#include <string>

class TIpcEventHandler : public IIOEventHandler, public IIpcEventHandler
{
protected:
  mutable TCIpcCallHandler CallHandler;
  IIOEventLoop* EventLoop;
  IConnectionObserver* Observer;

  
  TIpcEventHandler(const TIpcEventHandler& other);
  TIpcEventHandler& operator=(const TIpcEventHandler& other);

  static bool InvokeDispatcher(const char* interfaceName,
                               const char* methodName,
                               TBinIpcParameterStack* requestStack,
                               bool* errorResponse,
                               TBinIpcParameterStack* responseStack,
                               void* userData);
  static void AddDescriptor(int fd, void* userData);
  static void RemoveDescriptor(int fd, void* userData);
  static void OnConnectionClosed(int fd, void* userData);

  void SetAddress(const std::string& address) throw ();
  void SetEventLoop(IIOEventLoop* eventLoop) throw ();
  void ReleaseEventLoop() throw ();

public:
  TIpcEventHandler() throw ();
  TIpcEventHandler(IIOEventLoop* eventLoop) throw ();
  virtual ~TIpcEventHandler() throw ();

  
  virtual IIOEventLoop* GetEventLoop() throw ();

  
  
  void CloseServerSocket() throw ();

  
  
  
  
  virtual std::string CreateObject() throw ();

  
  virtual void AddDispatcher(const std::string& address,
                             IIpcDispatcher* dispatcher) throw ();
  virtual void RemoveDispatcher(const std::string& address,
                                IIpcDispatcher* /*dispatcher*/) throw ();
  void SetConnectionObserver(IConnectionObserver* observer) throw ();

  
  virtual void SuspendEvents(bool suspend);

  
  
  int GetPeerPid() const throw ();
  int GetCallDescriptor() const throw ();
  int GetPeerUid() const throw ();
  int GetPeerGid() const throw ();

  bool IsDescriptorConnected(int fd) const throw ();

  
  
  int GetCurrentConnectionFd() const throw ();

  
  virtual void HandleEvent(int fd, uint32_t /*event*/) throw ();
};




inline void TIpcEventHandler::SetAddress(const std::string& address) throw ()
{
  TCIpcCallHandler_SetAddress(&CallHandler, address.c_str());
}

inline void TIpcEventHandler::SetEventLoop(IIOEventLoop* eventLoop) throw ()
{
  EventLoop = eventLoop;
  TCIpcCallHandler_SetEventLoop(&CallHandler,
                                &AddDescriptor, &RemoveDescriptor, this);
}

inline void TIpcEventHandler::ReleaseEventLoop() throw ()
{
  TCIpcCallHandler_ReleaseEventLoop(&CallHandler);
}

inline IIOEventLoop* TIpcEventHandler::GetEventLoop() throw ()
{
  return EventLoop;
}

inline void TIpcEventHandler::CloseServerSocket() throw ()
{
  TCIpcCallHandler_CloseServerSocket(&CallHandler);
}

inline void
TIpcEventHandler::AddDispatcher(const std::string& address,
                                IIpcDispatcher* dispatcher) throw ()
{
  TCIpcCallHandler_AddDispatcher(&CallHandler, address.c_str(),
                                 &TIpcEventHandler::InvokeDispatcher,
                                 dispatcher);
}

inline void
TIpcEventHandler::RemoveDispatcher(const std::string& address,
                                   IIpcDispatcher* /*dispatcher*/) throw ()
{
  TCIpcCallHandler_RemoveDispatcher(&CallHandler, address.c_str());
}

#endif 
