/* -*- mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset:2 -*-
 *--------------------------------------------------------------------
 *
 * TKreaTvGfxVideoLayerInternal.h
 *
 *
 * Copyright (c) 2009-2010 Motorola, Inc. All rights reserved.
 *
 *--------------------------------------------------------------------
 */

#ifndef TKREATVGFXVIDEOLAYERINTERNAL_H_
#define TKREATVGFXVIDEOLAYERINTERNAL_H_

#include "hal/THaiGfxTypes.h"
#include "interface/kreatvgfx/KreaTvGfxTypes.h"
#include "interface/kreatvgfx/IKreaTvGfxVideoLayer.h"

/** Capture format
 */
typedef enum {
  /** Capture the visible video layer with video positioned, scaled
      and cropped according to the current viewport. The graphics
      surface given to CaptureVideo() must be of the same
      dimension (i.e. the same horizontal and vertical size) as the
      current video mode.
  */
  KGFX_CAPTURE_FORMAT_SCREEN,

  /** Capture the video component of the decoded content stream. The
      video component is scaled to fit the graphics surface given to
      CaptureVideo().
  */
  KGFX_CAPTURE_FORMAT_VIDEO,

  /** Capture the video component of the decoded content stream and
      mirror it vertically. The video component is scaled to fit the
      graphics surface given to CaptureVideo().
  */
  KGFX_CAPTURE_FORMAT_VIDEO_VERTICAL_MIRRORING
} TKreaTvGfxCaptureFormat;

class IKreaTvGfxVideoLayerInternal : public IKreaTvGfxVideoLayer
{
public:
  /**
     Captures video from the source selected by SetSource().

     \param[in] pixmap
                        A pixmap to put the capture in.
   */
  virtual TKreaTvGfxResult
  CaptureVideo(TKreaTvGfxCaptureFormat format,
               const TKreaTvGfxPixmapPointer pixmap) throw () = 0;

  /**
     Gets the video size.
   */
  virtual void
  GetVideoSize(uint32_t& width, uint32_t& height) throw () = 0;
};

#endif /* TKREATVGFXVIDEOLAYERINTERNAL_H_ */
