/*
 *--------------------------------------------------------------------
 *
 * libH222/TAvcVideoDescriptorHandle
 *
 * AVC Video Descriptor
 *
 * Copyright (c) 2011 Motorola Mobility, Inc. All rights reserved.
 *
 *--------------------------------------------------------------------
*/

#ifndef LIBH222_TAVCVIDEODESCRIPTORHANDLE_H
#define LIBH222_TAVCVIDEODESCRIPTORHANDLE_H

#include "media/libH222/TMpeg2DescriptorHandle.h"
#include "media/libH222/TMpeg2Util.h"
#include "media/libH222/Mpeg2Si.h"

#include <endian.h>

#ifndef NDEBUG
#include <iostream>
#endif

#ifndef __LITTLE_ENDIAN
#error Big endian target environment not supported!
#endif


class TAvcVideoDescriptorHandle : public TMpeg2DescriptorHandle
{
public:
  TAvcVideoDescriptorHandle(const TMpeg2DescriptorHandle& desc) throw ();

  
  bool GetStillPictureFlag() const throw();
};





inline TAvcVideoDescriptorHandle::
TAvcVideoDescriptorHandle(const TMpeg2DescriptorHandle& desc) throw ()
  : TMpeg2DescriptorHandle(desc)
{
  if (GetTag() != Mpeg2Si::TAG_AVC_VIDEO
      || GetSize() != 6 ) {
    
    Header = NULL;
  }
}

inline bool TAvcVideoDescriptorHandle::
GetStillPictureFlag() const throw()
{
  if (!IsValid()) {
    return false;
  }
  const uint8_t *p = GetDataPtr();
  const int stillPresentFlagBit = 0x80;
  return (p[5] & stillPresentFlagBit);
}

#endif
