/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *-----------------------------------------------------------------------------
 *
 * TDsmccDownloadInfoIndicationHandle.h
 * 
 * 13818-6 Table 7-6
 *
 * Copyright (c) 2007 Motorola, Inc. All Rights Reserved.
 *
 *-----------------------------------------------------------------------------
 */

#ifndef TDSMCCDOWNLOADINFOINDICATIONHANDLE_H
#define TDSMCCDOWNLOADINFOINDICATIONHANDLE_H

#include "TDsmccMessageHeaderHandle.h"

#include <vector>


struct TDsmccDII
{
  uint8_t downloadId[4];
  uint8_t blockSize[2];
  uint8_t windowSize;
  uint8_t ackPeriod;
  uint8_t tCDownloadWindow[4];
  uint8_t tCDownloadScenario[4];
  uint8_t compatibilityDescriptorLength[2];
};

struct TScmccDIIModule
{
  uint16_t ModuleId;
  uint32_t ModuleSize;
  uint8_t  ModuleVersion;
  uint8_t  ModuleInfoLength;
  const uint8_t *ModuleInfo;
};

class TDsmccDownloadInfoIndicationHandle : public TDsmccMessageHeaderHandle
{
private:
  const TDsmccDII* Dii;
  std::vector<TScmccDIIModule> Modules;
  uint16_t PrivateDataLength;
  const uint8_t *PrivateData;
  void Parse() throw ();

public:
  TDsmccDownloadInfoIndicationHandle(const uint8_t* buffer) throw ();
  uint32_t GetDownloadId() const throw ();
  uint16_t GetBlockSize() const throw ();
  uint8_t  GetWindowSize() const throw ();
  uint8_t  GetAckPeriod() const throw ();
  uint32_t GetTcDownloadWindow() const throw ();
  uint32_t GetTcDownloadScenario() const throw ();
  uint16_t GetCompatibilityDescriptorLength() const throw ();
  const std::vector<TScmccDIIModule> &GetModules() const throw ();
  const uint8_t *GetPrivateData() const throw ();
  uint16_t GetPrivateDataLength() const throw ();
};

inline 
TDsmccDownloadInfoIndicationHandle::
TDsmccDownloadInfoIndicationHandle(const uint8_t* buffer) throw ()
  : TDsmccMessageHeaderHandle(buffer)
{
  Dii = reinterpret_cast<const TDsmccDII*>(GetSectionDataBegin() + 
                                           GetMessageHeaderSize());
  Parse();
}

inline 
uint32_t TDsmccDownloadInfoIndicationHandle::GetDownloadId() const throw ()
{
  return TMpeg2Util::Make32(Dii->downloadId[0],
                            Dii->downloadId[1],
                            Dii->downloadId[2],
                            Dii->downloadId[3]);
}

inline 
uint16_t TDsmccDownloadInfoIndicationHandle::GetBlockSize() const throw ()
{
  return TMpeg2Util::Make16(Dii->blockSize[0], Dii->blockSize[1]);
}

inline 
uint8_t TDsmccDownloadInfoIndicationHandle::GetWindowSize() const throw ()
{
  return Dii->windowSize;
}

inline 
uint8_t TDsmccDownloadInfoIndicationHandle::GetAckPeriod() const throw ()
{
  return Dii->ackPeriod;
}

inline 
uint32_t TDsmccDownloadInfoIndicationHandle::GetTcDownloadWindow()
  const throw ()
{
  return TMpeg2Util::Make32(Dii->tCDownloadWindow[0],
                            Dii->tCDownloadWindow[1],
                            Dii->tCDownloadWindow[2],
                            Dii->tCDownloadWindow[3]);
}

inline uint32_t 
TDsmccDownloadInfoIndicationHandle::GetTcDownloadScenario() const throw ()
{
  return TMpeg2Util::Make32(Dii->tCDownloadScenario[0],
                            Dii->tCDownloadScenario[1],
                            Dii->tCDownloadScenario[2],
                            Dii->tCDownloadScenario[3]);
}

inline uint16_t 
TDsmccDownloadInfoIndicationHandle::GetCompatibilityDescriptorLength() 
  const throw ()
{
  return TMpeg2Util::Make16(Dii->compatibilityDescriptorLength[0],
                            Dii->compatibilityDescriptorLength[1]);
}

inline const std::vector<TScmccDIIModule> &
TDsmccDownloadInfoIndicationHandle::GetModules() const throw ()
{
  return Modules;
}

inline const uint8_t *
TDsmccDownloadInfoIndicationHandle::GetPrivateData() const throw ()
{
  return PrivateData;
}

inline uint16_t
TDsmccDownloadInfoIndicationHandle::GetPrivateDataLength() const throw ()
{
  return PrivateDataLength;
}
#endif 
