/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *-----------------------------------------------------------------------------
 *
 * TDsmccMessageHeader.h
 *
 * 13818-6 Table 2-1
 *
 * Note reused as dsmccDownloadDataHeader also (13818-6 Table 7-3)
 * (hence GetDownloadId alias for GetTransactionId)
 *
 * Copyright (c) 2007 Motorola, Inc. All Rights Reserved.
 *
 *-----------------------------------------------------------------------------
 */

#ifndef TDSMCCMESSAGEHEADERHANDLE_H
#define TDSMCCMESSAGEHEADERHANDLE_H
#include "media/libH222/TMpeg2Util.h"
#include "media/libH222/TTsSectionHandle.h"

struct TDsmccMessageHeader 
{
  uint8_t protocolDiscriminator;
  uint8_t dsmccType;
  uint8_t messageId[2];
  uint8_t transactionId[4];
  uint8_t reserved;
  uint8_t adaptationLength;
  uint8_t messageLength[2];
};

class TDsmccMessageHeaderHandle:  public TTsSectionHandle
{
 private:
  const TDsmccMessageHeader* Header;

 public:
  TDsmccMessageHeaderHandle(const uint8_t* buffer) throw ();
  uint8_t  GetProtocolDiscriminator() const throw ();
  uint8_t  GetDsmccType() const throw ();
  uint16_t GetMessageId() const throw ();
  uint32_t GetTransactionId() const throw ();
  uint32_t GetDownloadId() const throw ();
  uint8_t  GetAdaptationLength() const throw ();
  uint16_t GetMessageLength() const throw ();
  uint16_t GetMessageHeaderSize() const throw ();
};

inline 
TDsmccMessageHeaderHandle::TDsmccMessageHeaderHandle(
  const uint8_t* buffer) throw ()
  : TTsSectionHandle(buffer)
{
  Header = 
    reinterpret_cast<const TDsmccMessageHeader*>(GetSectionDataBegin());
}

inline 
uint8_t TDsmccMessageHeaderHandle::GetProtocolDiscriminator() const throw ()
{
  return Header->protocolDiscriminator;
}

inline 
uint8_t TDsmccMessageHeaderHandle::GetDsmccType() const throw ()
{
  return Header->dsmccType;
}

inline 
uint16_t TDsmccMessageHeaderHandle::GetMessageId() const throw ()
{
  return TMpeg2Util::Make16(Header->messageId[0], Header->messageId[1]);
}

inline 
uint32_t TDsmccMessageHeaderHandle::GetTransactionId() const throw ()
{
  return TMpeg2Util::Make32(Header->transactionId[0],
			    Header->transactionId[1],
			    Header->transactionId[2],
			    Header->transactionId[3]);
}

inline 
uint32_t TDsmccMessageHeaderHandle::GetDownloadId() const throw ()
{
  
  return GetTransactionId();
}

inline 
uint8_t TDsmccMessageHeaderHandle::GetAdaptationLength() const throw ()
{
  return Header->adaptationLength;
}

inline 
uint16_t TDsmccMessageHeaderHandle::GetMessageLength() const throw ()
{
  return TMpeg2Util::Make16(Header->messageLength[0],
                            Header->messageLength[1]);
}

inline 
uint16_t TDsmccMessageHeaderHandle::GetMessageHeaderSize() const throw ()
{
  return sizeof(TDsmccMessageHeader) + GetAdaptationLength();
}
#endif 
