/*
 *--------------------------------------------------------------------
 *
 * libH222/TDsmccStreamDescriptorHandle.h
 *
 * Copyright (c) 2005 Kreatel Communications AB. All rights reserved.
 *
 *--------------------------------------------------------------------
 */

#ifndef LIBH222_TDSMCCSTREAMDESCRIPTORHANDLE_H
#define LIBH222_TDSMCCSTREAMDESCRIPTORHANDLE_H

#include "media/libH222/TMpeg2DescriptorHandle.h"
#include "media/libH222/TMpeg2Util.h"

#include <stdint.h>
#include <string>
#include <endian.h>

#ifndef __LITTLE_ENDIAN
#error Big endian target environment not supported!
#endif

struct TDsmccStreamEventDescriptorInfo
{
  unsigned eventId_high      :8;
  unsigned eventId_low       :8;
  unsigned eventNPT_lsb      :1;
  unsigned                   :7; 
  unsigned                   :8; 
  unsigned                   :8; 
  unsigned                   :8; 
  unsigned eventNPT_high     :8;
  unsigned eventNPT_midh     :8;
  unsigned eventNPT_midl     :8;
  unsigned eventNPT_low      :8;
};

#define SIZEOF_DSMCCSTREAMDESCRIPTOR 10
#define STREAM_EVENT_DESCRIPTOR_TAG 26

class TDsmccStreamDescriptorHandle : public TMpeg2DescriptorHandle
{
 private:
  const TDsmccStreamEventDescriptorInfo* DataPtr;

 public:
  TDsmccStreamDescriptorHandle() throw();
  explicit TDsmccStreamDescriptorHandle(const uint8_t* data) throw();
  TDsmccStreamDescriptorHandle(const TMpeg2DescriptorHandle& desc) throw();

  bool IsValid() const throw();
  uint16_t GetEventId() const throw();
  uint64_t GetEventNPT() const throw();
  uint32_t GetEventDataSize() const throw();
  uint32_t GetEventData(uint8_t* buffer, uint32_t size) const throw();
};

inline TDsmccStreamDescriptorHandle::TDsmccStreamDescriptorHandle() throw()
  : TMpeg2DescriptorHandle(), 
    DataPtr(0)
{
  
}

inline TDsmccStreamDescriptorHandle::TDsmccStreamDescriptorHandle(const uint8_t* data) throw()
  : TMpeg2DescriptorHandle(data),
    DataPtr(reinterpret_cast<const TDsmccStreamEventDescriptorInfo*>(data+SIZEOF_TMPEG2DESCRIPTORHEADER))
{
  
}

inline TDsmccStreamDescriptorHandle::
TDsmccStreamDescriptorHandle(const TMpeg2DescriptorHandle& desc) throw()
  : TMpeg2DescriptorHandle(desc),
    DataPtr(reinterpret_cast<const TDsmccStreamEventDescriptorInfo*>(GetBodyData()))
{
  
}

inline bool TDsmccStreamDescriptorHandle::IsValid() const throw()
{
  return (DataPtr != 0);
}

inline uint16_t TDsmccStreamDescriptorHandle::GetEventId() const throw()
{
  return TMpeg2Util::Make16(DataPtr->eventId_high,
			    DataPtr->eventId_low);
}

inline uint64_t TDsmccStreamDescriptorHandle::GetEventNPT() const throw()
{
  uint64_t npt = TMpeg2Util::Make64(0, 0, 0,
				    DataPtr->eventNPT_high,
				    DataPtr->eventNPT_midh,
				    DataPtr->eventNPT_midl,
				    DataPtr->eventNPT_low,
				    (uint8_t)DataPtr->eventNPT_lsb);

  return npt;
}

inline uint32_t TDsmccStreamDescriptorHandle::GetEventDataSize() const throw()
{
  return GetLength() - SIZEOF_DSMCCSTREAMDESCRIPTOR;
}

#endif

