/*
 *--------------------------------------------------------------------
 *
 * libH222/TDvbSubtitlingDescriptorHandle.h
 *
 * Copyright (c) 2003 Kreatel Communications AB. All rights reserved.
 *
 *--------------------------------------------------------------------
 */

#ifndef LIBH222_TDVBSUBTITLINGDESCRIPTORHANDLE_H
#define LIBH222_TDVBSUBTITLINGDESCRIPTORHANDLE_H

#include "media/libH222/TMpeg2DescriptorHandle.h"
#include "media/libH222/TMpeg2Util.h"

#include <stdint.h>
#include <string>




struct TDvbSubtitlingDescriptorInfo
{
  unsigned ISO_639_langugage_code       :24;
  unsigned subtitling_type              :8;
  unsigned composition_page_id_high     :8;
  unsigned composition_page_id_low      :8;
  unsigned ancillary_page_id_high       :8;
  unsigned ancillary_page_id_low        :8;
};

#define SIZEOF_TDVBSUBTITLINGDESCRIPTORINFO 8


class TDvbSubtitlingDescriptorHandle : public TMpeg2DescriptorHandle
{
public:
  class TDvbSubtitlingEntryDescriptorHandle
  {
    friend class TDvbSubtitlingDescriptorHandle;

  private:
    const TDvbSubtitlingDescriptorInfo* DataPtr;

    explicit TDvbSubtitlingEntryDescriptorHandle(const uint8_t* dataPtr)
      throw ();

  public:
    TDvbSubtitlingEntryDescriptorHandle() throw ();

    bool IsValid() const throw ();
    const uint8_t* GetDataPtr() const throw ();
    uint32_t GetSize() const throw ();
    std::string GetLanguageCodeString() const throw ();
    uint8_t GetSubtitlingType() const throw ();
    uint16_t GetCompositionPageId() const throw ();
    uint16_t GetAncillaryPageId() const throw ();
    TDvbSubtitlingEntryDescriptorHandle GetNext(const uint8_t* start,
                                                uint32_t size) const throw ();
  };
  
  TDvbSubtitlingDescriptorHandle() throw ();
  explicit TDvbSubtitlingDescriptorHandle(const uint8_t* buffer) throw ();
  TDvbSubtitlingDescriptorHandle(const TMpeg2DescriptorHandle &desc) throw ();
  
  TDvbSubtitlingEntryDescriptorHandle GetFirstEntry() const throw ();
  TDvbSubtitlingEntryDescriptorHandle GetNextEntry
  (const TDvbSubtitlingEntryDescriptorHandle &previous) const throw ();
};


inline TDvbSubtitlingDescriptorHandle::
TDvbSubtitlingDescriptorHandle() throw ()
{
  
}

inline TDvbSubtitlingDescriptorHandle::
TDvbSubtitlingDescriptorHandle(const uint8_t* buffer) throw ()
  : TMpeg2DescriptorHandle(buffer)
{
  
}

inline TDvbSubtitlingDescriptorHandle::
TDvbSubtitlingDescriptorHandle(const TMpeg2DescriptorHandle &desc) throw ()
  : TMpeg2DescriptorHandle(desc)
{
  
}

inline TDvbSubtitlingDescriptorHandle::TDvbSubtitlingEntryDescriptorHandle::
TDvbSubtitlingEntryDescriptorHandle() throw ()
  : DataPtr(NULL)
{
  
}

inline TDvbSubtitlingDescriptorHandle::TDvbSubtitlingEntryDescriptorHandle::
TDvbSubtitlingEntryDescriptorHandle(const uint8_t* buffer) throw ()
  : DataPtr(reinterpret_cast<const TDvbSubtitlingDescriptorInfo*>(buffer))
{
  
}

inline bool TDvbSubtitlingDescriptorHandle::
TDvbSubtitlingEntryDescriptorHandle::IsValid() const throw ()
{
  return DataPtr != NULL;
}

inline const uint8_t* TDvbSubtitlingDescriptorHandle::
TDvbSubtitlingEntryDescriptorHandle::GetDataPtr() const throw ()
{
  return reinterpret_cast<const uint8_t*>(DataPtr);
}

inline uint32_t TDvbSubtitlingDescriptorHandle::
TDvbSubtitlingEntryDescriptorHandle::GetSize() const throw ()
{
  return SIZEOF_TDVBSUBTITLINGDESCRIPTORINFO;
}

inline uint8_t TDvbSubtitlingDescriptorHandle::
TDvbSubtitlingEntryDescriptorHandle::GetSubtitlingType() const throw ()
{
  return DataPtr->subtitling_type;
}

inline uint16_t TDvbSubtitlingDescriptorHandle::
TDvbSubtitlingEntryDescriptorHandle::GetCompositionPageId() const throw ()
{
  return TMpeg2Util::Make16(DataPtr->composition_page_id_high,
                            DataPtr->composition_page_id_low);
}

inline uint16_t TDvbSubtitlingDescriptorHandle::
TDvbSubtitlingEntryDescriptorHandle::GetAncillaryPageId() const throw ()
{
  return TMpeg2Util::Make16(DataPtr->ancillary_page_id_high,
                            DataPtr->ancillary_page_id_low);
}

#endif
