/*
 *--------------------------------------------------------------------
 *
 * libH222/TDvbTeletextDescriptorHandle
 *
 * Copyright (c) 2003 Kreatel Communications AB. All rights reserved.
 *
 *--------------------------------------------------------------------
 */

#ifndef LIBH222_TDVBTELETEXTDESCRIPTORHANDLE_H
#define LIBH222_TDVBTELETEXTDESCRIPTORHANDLE_H

#include "media/libH222/TMpeg2DescriptorHandle.h"
#include "media/libH222/TMpeg2Util.h"

#include <string>
#include <stdint.h>




struct TDvbTeletextDescriptorInfo
{
  unsigned ISO_639_langugage_code       :24;
  unsigned teletext_magazine_number     :3;
  unsigned teletext_type                :5;
  unsigned teletext_page_number1        :4;
  unsigned teletext_page_number10       :4;
};

#define SIZEOF_TDVBTELETEXTDESCRIPTORINFO 5

class TDvbTeletextDescriptorHandle : public TMpeg2DescriptorHandle
{
public:
  class TDvbTeletextEntryDescriptorHandle
  {
    friend class TDvbTeletextDescriptorHandle;

  private:
    const TDvbTeletextDescriptorInfo* DataPtr;

    explicit TDvbTeletextEntryDescriptorHandle(const uint8_t* dataPtr) throw ();

  public:
    TDvbTeletextEntryDescriptorHandle() throw ();

    bool IsValid() const throw ();
    const uint8_t* GetDataPtr() const throw ();
    uint8_t GetSize() const throw ();
    std::string GetLanguageCodeString() const throw ();
    uint8_t GetTeletextType() const throw ();
    uint16_t GetMagazineNumber() const throw ();
    uint16_t GetPageNumber10() const throw ();
    uint16_t GetPageNumber1() const throw ();
    TDvbTeletextEntryDescriptorHandle GetNext(const uint8_t* start,
                                              uint32_t size) const throw ();
  };
  
  TDvbTeletextDescriptorHandle() throw ();
  explicit TDvbTeletextDescriptorHandle(const uint8_t* buffer) throw ();
  TDvbTeletextDescriptorHandle(const TMpeg2DescriptorHandle &desc) throw ();
  
  TDvbTeletextEntryDescriptorHandle GetFirstEntry() const throw ();
  TDvbTeletextEntryDescriptorHandle GetNextEntry
  (const TDvbTeletextEntryDescriptorHandle &previous) const throw ();
};

inline TDvbTeletextDescriptorHandle::TDvbTeletextDescriptorHandle() throw ()
{
  
}

inline TDvbTeletextDescriptorHandle
::TDvbTeletextDescriptorHandle(const uint8_t* buffer) throw ()
  : TMpeg2DescriptorHandle(buffer)
{
  
}

inline TDvbTeletextDescriptorHandle::
TDvbTeletextDescriptorHandle(const TMpeg2DescriptorHandle &desc) throw ()
  : TMpeg2DescriptorHandle(desc)
{
  
}

inline TDvbTeletextDescriptorHandle::
TDvbTeletextEntryDescriptorHandle::TDvbTeletextEntryDescriptorHandle() throw ()
  : DataPtr(NULL)
{
  
}

inline TDvbTeletextDescriptorHandle::TDvbTeletextEntryDescriptorHandle::
TDvbTeletextEntryDescriptorHandle(const uint8_t* buffer) throw ()
  : DataPtr(reinterpret_cast<const TDvbTeletextDescriptorInfo*>(buffer))
{
  
}

inline bool TDvbTeletextDescriptorHandle::
TDvbTeletextEntryDescriptorHandle::IsValid() const throw ()
{
  return DataPtr != NULL;
}

inline const uint8_t* TDvbTeletextDescriptorHandle::
TDvbTeletextEntryDescriptorHandle::GetDataPtr() const throw ()
{
  return reinterpret_cast<const uint8_t*>(DataPtr);
}

inline uint8_t TDvbTeletextDescriptorHandle::
TDvbTeletextEntryDescriptorHandle::GetSize() const throw ()
{
  return SIZEOF_TDVBTELETEXTDESCRIPTORINFO;
}

inline uint8_t TDvbTeletextDescriptorHandle::
TDvbTeletextEntryDescriptorHandle::GetTeletextType() const throw ()
{
  return DataPtr->teletext_type;
}

inline uint16_t TDvbTeletextDescriptorHandle::
TDvbTeletextEntryDescriptorHandle::GetMagazineNumber() const throw ()
{
  return DataPtr->teletext_magazine_number;
}

inline uint16_t TDvbTeletextDescriptorHandle::
TDvbTeletextEntryDescriptorHandle::GetPageNumber10() const throw ()
{
  return DataPtr->teletext_page_number10;
}

inline uint16_t TDvbTeletextDescriptorHandle::
TDvbTeletextEntryDescriptorHandle::GetPageNumber1() const throw ()
{
  return DataPtr->teletext_page_number1;
}

#endif
