/*
 *--------------------------------------------------------------------
 *
 * libH222/TMpeg2CaDescriptorHandle.h
 *
 * ISO/IEC 138181-1 conditional access descriptor.
 *
 * Copyright (c) 2002 Kreatel Communications AB. All rights reserved.
 *
 *--------------------------------------------------------------------
 */

#ifndef MEDIA_COMMON_TMPEG2CADESCHANDLE_H
#define MEDIA_COMMON_TMPEG2CADESCHANDLE_H


#include "media/libH222/TMpeg2DescriptorHandle.h"
#include "media/libH222/TMpeg2Util.h"
#include <string>



struct TMpeg2CaDescriptorInfo
{
  unsigned CA_system_ID_high            :8;
  
  unsigned CA_system_ID_low             :8;

  unsigned CA_PID_high                  :5;
  unsigned                              :3;  
  
  unsigned CA_PID_low                   :8;

  
};

#define SIZEOF_TMPEG2CADESCRIPTORBODY 4


class TMpeg2CaDescriptorHandle : public TMpeg2DescriptorHandle
{
public:
  TMpeg2CaDescriptorHandle() throw ();
  explicit TMpeg2CaDescriptorHandle(const uint8_t* buffer) throw ();
  TMpeg2CaDescriptorHandle(const TMpeg2DescriptorHandle &desc) throw ();

  uint16_t GetCaSystemId() const throw ();
  uint16_t GetCaPid() const throw ();
  const uint8_t* GetPrivateDataBegin() const throw ();
  const uint8_t* GetPrivateDataEnd() const throw ();
};


inline TMpeg2CaDescriptorHandle::TMpeg2CaDescriptorHandle() throw ()
{
  
}

inline TMpeg2CaDescriptorHandle::
TMpeg2CaDescriptorHandle(const uint8_t* buffer) throw ()
  : TMpeg2DescriptorHandle(buffer)
{
  
}

inline TMpeg2CaDescriptorHandle::
TMpeg2CaDescriptorHandle(const TMpeg2DescriptorHandle &desc) throw ()
  : TMpeg2DescriptorHandle(desc)
{
  
}

inline uint16_t
TMpeg2CaDescriptorHandle::GetCaSystemId() const throw ()
{
  const TMpeg2CaDescriptorInfo* body
    = reinterpret_cast<const TMpeg2CaDescriptorInfo*>(GetBodyData());
  return TMpeg2Util::Make16(body->CA_system_ID_high, body->CA_system_ID_low);
}

inline uint16_t
TMpeg2CaDescriptorHandle::GetCaPid() const throw ()
{
  const TMpeg2CaDescriptorInfo* body
    = reinterpret_cast<const TMpeg2CaDescriptorInfo*>(GetBodyData());
  return TMpeg2Util::Make16(body->CA_PID_high, body->CA_PID_low);
}

inline
const uint8_t* TMpeg2CaDescriptorHandle::GetPrivateDataBegin() const throw ()
{
  return GetBodyData() + SIZEOF_TMPEG2CADESCRIPTORBODY;
}

inline
const uint8_t* TMpeg2CaDescriptorHandle::GetPrivateDataEnd() const throw ()
{
  return GetDataPtr() + GetSize();
}

#endif
