/*
 *--------------------------------------------------------------------
 *
 * libH222/TMpeg2DescriptorHandle.h
 *
 * A base class for descriptor structures used in MPEG2 transport and
 * program streams.
 *
 * Copyright (c) 2002 Kreatel Communications AB. All rights reserved.
 *
 *--------------------------------------------------------------------
 */

#ifndef MEDIA_COMMON_TMPEG2DESCRIPTORHANDLE_H
#define MEDIA_COMMON_TMPEG2DESCRIPTORHANDLE_H

#include <stdint.h>
#include <cstdio>

#ifndef NDEBUG
#include <iostream>
#endif



struct TMpeg2DescriptorHeader
{
  unsigned descriptor_tag               :8;

  unsigned descriptor_length            :8;
};

#define SIZEOF_TMPEG2DESCRIPTORHEADER 2

class TMpeg2DescriptorHandle
{
public:
  enum { HEADER_SIZE = 2 };

protected:
  const TMpeg2DescriptorHeader* Header;

  const uint8_t* GetBodyData() const throw ();

public:
  TMpeg2DescriptorHandle() throw ();
  TMpeg2DescriptorHandle(const uint8_t* buffer) throw ();

  bool IsValid() const throw ();
  const uint8_t* GetDataPtr() const throw ();
  uint32_t GetSize() const throw ();

  uint8_t GetTag() const throw ();
  uint8_t GetLength() const throw ();

  TMpeg2DescriptorHandle GetNext(const uint8_t* start, uint32_t size) const
    throw ();

  uint32_t Write(uint8_t* buffer, uint32_t size) const throw ();

#ifndef NDEBUG
  void PrintInfo(std::ostream& s) const throw ();
#endif
};


inline const uint8_t* TMpeg2DescriptorHandle::GetBodyData() const throw ()
{
  return reinterpret_cast<const uint8_t*>(Header)
    + SIZEOF_TMPEG2DESCRIPTORHEADER;
}

inline TMpeg2DescriptorHandle::TMpeg2DescriptorHandle() throw ()
  : Header(NULL)
{
  
}

inline
TMpeg2DescriptorHandle::TMpeg2DescriptorHandle(const uint8_t* buffer) throw ()
  : Header(reinterpret_cast<const TMpeg2DescriptorHeader*>(buffer))
{
  
}

inline bool TMpeg2DescriptorHandle::IsValid() const throw ()
{
  return Header != NULL;
}

inline const uint8_t* TMpeg2DescriptorHandle::GetDataPtr() const throw ()
{
  return reinterpret_cast<const uint8_t*>(Header);
}

inline uint32_t TMpeg2DescriptorHandle::GetSize() const throw ()
{
  return SIZEOF_TMPEG2DESCRIPTORHEADER + Header->descriptor_length;
}

inline uint8_t TMpeg2DescriptorHandle::GetTag() const throw ()
{
  return Header->descriptor_tag;
}

inline uint8_t TMpeg2DescriptorHandle::GetLength() const throw ()
{
  return Header->descriptor_length;
}

#endif
