/*
 *--------------------------------------------------------------------
 *
 * libH222/TMpeg2LanguageDescriptorHandle.h
 *
 * ISO 639 language descriptor.
 *
 * Copyright (c) 2002 Kreatel Communications AB. All rights reserved.
 *
 *--------------------------------------------------------------------
 */

#ifndef MEDIA_COMMON_TMPEG2LANGDESCHANDLE_H
#define MEDIA_COMMON_TMPEG2LANGDESCHANDLE_H


#include "media/libH222/TMpeg2DescriptorHandle.h"
#include <string>



struct TMpeg2LanguageDescriptorInfo
{
  unsigned ISO_639_langugage_code       :24;
  
  unsigned audio_type                   :8;
};

#define SIZEOF_TMPEG2LANGUAGEDESCRIPTORINFO 4

class TMpeg2LanguageDescriptorHandle : public TMpeg2DescriptorHandle
{
public:
  class TLanguageHandle
  {
    friend class TMpeg2LanguageDescriptorHandle;

  private:
    const TMpeg2LanguageDescriptorInfo* DataPtr;
    explicit TLanguageHandle(const uint8_t* dataPtr) throw ();

  public:
    TLanguageHandle() throw ();

    bool IsValid() const throw ();
    const uint8_t* GetDataPtr() const throw ();
    uint32_t GetSize() const throw ();
    uint32_t GetLanguageCode() const throw ();
    std::string GetLanguageCodeString() const throw ();
    uint8_t GetAudioType() const throw ();
    TLanguageHandle GetNext(const uint8_t* start, uint32_t size) const throw ();
  };
  
  TMpeg2LanguageDescriptorHandle() throw ();
  explicit TMpeg2LanguageDescriptorHandle(const uint8_t* buffer) throw ();
  TMpeg2LanguageDescriptorHandle(const TMpeg2DescriptorHandle &desc) throw ();

  TLanguageHandle GetFirstLanguage() const throw ();
  TLanguageHandle GetNextLanguage(const TLanguageHandle &previous) const
    throw ();
};

inline
TMpeg2LanguageDescriptorHandle::TMpeg2LanguageDescriptorHandle() throw ()
{
  
}

inline TMpeg2LanguageDescriptorHandle::
TMpeg2LanguageDescriptorHandle(const uint8_t* buffer) throw ()
  : TMpeg2DescriptorHandle(buffer)
{
  
}

inline TMpeg2LanguageDescriptorHandle::
TMpeg2LanguageDescriptorHandle(const TMpeg2DescriptorHandle &desc) throw ()
  : TMpeg2DescriptorHandle(desc)
{
  
}

inline
TMpeg2LanguageDescriptorHandle::TLanguageHandle::TLanguageHandle() throw ()
  : DataPtr(NULL)
{
  
}

inline TMpeg2LanguageDescriptorHandle::TLanguageHandle::
TLanguageHandle(const uint8_t* buffer) throw ()
  : DataPtr(reinterpret_cast<const TMpeg2LanguageDescriptorInfo*>(buffer))
{
  
}

inline
bool TMpeg2LanguageDescriptorHandle::TLanguageHandle::IsValid() const throw ()
{
  return DataPtr != NULL;
}

inline const uint8_t*
TMpeg2LanguageDescriptorHandle::TLanguageHandle::GetDataPtr() const throw ()
{
  return reinterpret_cast<const uint8_t*>(DataPtr);
}

inline uint32_t
TMpeg2LanguageDescriptorHandle::TLanguageHandle::GetSize() const throw ()
{
  return SIZEOF_TMPEG2LANGUAGEDESCRIPTORINFO;
}

inline uint32_t
TMpeg2LanguageDescriptorHandle::TLanguageHandle::GetLanguageCode() const
  throw ()
{
  return DataPtr->ISO_639_langugage_code;
}

inline uint8_t
TMpeg2LanguageDescriptorHandle::TLanguageHandle::GetAudioType() const throw ()
{
  return DataPtr->audio_type;
}

#endif
