/*
 *--------------------------------------------------------------------
 *
 * libH222/TMpeg2NptReferenceDescriptorHandle.h
 *
 * DSM-CC Normal Play Time Reference Descriptor.
 *
 * Copyright (c) 2002 Kreatel Communications AB. All rights reserved.
 *
 *--------------------------------------------------------------------
*/

#ifndef LIBH222_TMPEG2NPTREFERENCEDESCRIPTORHANDLE_H
#define LIBH222_TMPEG2NPTREFERENCEDESCRIPTORHANDLE_H

#include "media/libH222/TMpeg2DescriptorHandle.h"
#include "media/libH222/TMpeg2Util.h"

#include <endian.h>

#ifndef NDEBUG
#include <iostream>
#endif

#ifndef __LITTLE_ENDIAN
#error Big endian target environment not supported!
#endif





struct TMpeg2NptReferenceDescriptor
{
  unsigned content_id                   :7;
  unsigned post_discontinuity_indicator :1;
  
  unsigned stc_reference_highest        :1;
  unsigned                              :7; 
  
  unsigned stc_reference_higher         :8;
  unsigned stc_reference_high           :8;
  unsigned stc_reference_low            :8;
  unsigned stc_reference_lowest         :8;

  unsigned                              :8; 
  unsigned                              :8; 
  unsigned                              :8; 

  unsigned npt_reference_highest        :1;
  unsigned                              :7; 

  unsigned npt_reference_higher         :8;
  unsigned npt_reference_high           :8;
  unsigned npt_reference_low            :8;
  unsigned npt_reference_lowest         :8;

  unsigned scale_numerator              :16;

  unsigned scale_denominator            :16;
};

#define SIZEOF_TMPEG2NPTREFERENCEDESCRIPTOR 18


#define TAG_TMPEG2NPTREFERENCEDESCRIPTOR    23





class TMpeg2NptReferenceDescriptorHandle : public TMpeg2DescriptorHandle
{
protected:
  const TMpeg2NptReferenceDescriptor* GetDescriptorBody() const throw ();

public:
  TMpeg2NptReferenceDescriptorHandle(const uint8_t* dataPtr) throw ();
  
  uint32_t GetSize() const throw ();
  bool CheckSanity() const throw ();

  bool GetPostDiscontinuityIndicator() const throw ();
  uint8_t GetContentId() const throw ();
  uint64_t GetStcReference() const throw ();
  int64_t GetNptReference() const throw ();
  int16_t GetScaleNumerator() const throw ();
  uint16_t GetScaleDenominator() const throw ();

#ifndef NDEBUG
  void PrintInfo(std::ostream& s) const throw ();
#endif
};





inline TMpeg2NptReferenceDescriptorHandle::
TMpeg2NptReferenceDescriptorHandle(const uint8_t* buffer) throw ()
  : TMpeg2DescriptorHandle(buffer)
{
  
}

inline const TMpeg2NptReferenceDescriptor*
TMpeg2NptReferenceDescriptorHandle::GetDescriptorBody() const throw ()
{
  return reinterpret_cast<const TMpeg2NptReferenceDescriptor*>(GetBodyData());
}

inline uint32_t TMpeg2NptReferenceDescriptorHandle::GetSize() const throw ()
{
  return SIZEOF_TMPEG2NPTREFERENCEDESCRIPTOR;
}

inline bool TMpeg2NptReferenceDescriptorHandle::CheckSanity() const throw ()
{
  return (GetTag() == TAG_TMPEG2NPTREFERENCEDESCRIPTOR &&
          (TMpeg2DescriptorHandle::GetSize() ==
           SIZEOF_TMPEG2NPTREFERENCEDESCRIPTOR +
           SIZEOF_TMPEG2DESCRIPTORHEADER));
}

inline bool
TMpeg2NptReferenceDescriptorHandle::GetPostDiscontinuityIndicator() const
  throw ()
{
  return GetDescriptorBody()->post_discontinuity_indicator;
}

inline uint8_t
TMpeg2NptReferenceDescriptorHandle::GetContentId() const throw ()
{
  return GetDescriptorBody()->content_id;
}

inline uint64_t
TMpeg2NptReferenceDescriptorHandle::GetStcReference() const throw ()
{
  return TMpeg2Util::Make64(0x00, 0x00, 0x00,
                            GetDescriptorBody()->stc_reference_highest,
                            GetDescriptorBody()->stc_reference_higher,
                            GetDescriptorBody()->stc_reference_high,
                            GetDescriptorBody()->stc_reference_low,
                            GetDescriptorBody()->stc_reference_lowest);
}

inline int64_t
TMpeg2NptReferenceDescriptorHandle::GetNptReference() const throw ()
{
  uint8_t top = 0;
  if (GetDescriptorBody()->npt_reference_highest == 0x1) {
    
    top = 0xff;
  }
  return static_cast<int64_t>
    (TMpeg2Util::Make64(top, top, top, top,
                        GetDescriptorBody()->npt_reference_higher,
                        GetDescriptorBody()->npt_reference_high,
                        GetDescriptorBody()->npt_reference_low,
                        GetDescriptorBody()->npt_reference_lowest));
}

inline int16_t
TMpeg2NptReferenceDescriptorHandle::GetScaleNumerator() const throw ()
{
  return static_cast<int16_t>(TMpeg2Util::Reorder16(GetDescriptorBody()
                                                    ->scale_numerator));
}

inline uint16_t
TMpeg2NptReferenceDescriptorHandle::GetScaleDenominator() const throw ()
{
  return TMpeg2Util::Reorder16(GetDescriptorBody()->scale_denominator);
}

#endif
