/*
 *--------------------------------------------------------------------
 *
 * libH222/TMpeg2StreamModeDescriptorHandle.h
 *
 * DSM-CC Stream Mode Descriptor.
 *
 * Copyright (c) 2004 Kreatel Communications AB. All rights reserved.
 *
 *--------------------------------------------------------------------
*/

#ifndef LIBH222_TMPEG2STREAMMODEDESCRIPTORHANDLE_H
#define LIBH222_TMPEG2STREAMMODEDESCRIPTORHANDLE_H

#include "media/libH222/TMpeg2DescriptorHandle.h"
#include "media/libH222/TMpeg2Util.h"

#include <endian.h>

#ifndef NDEBUG
#include <iostream>
#endif

#ifndef __LITTLE_ENDIAN
#error Big endian target environment not supported!
#endif





struct TMpeg2StreamModeDescriptor
{
  unsigned stream_mode                  :8;
  unsigned                              :8; 
};

#define SIZEOF_TMPEG2STREAMMODEDESCRIPTOR 2


#define TAG_TMPEG2STREAMMODEDESCRIPTOR    25





class TMpeg2StreamModeDescriptorHandle : public TMpeg2DescriptorHandle
{
public:
  enum TStreamMode {
    STREAM_MODE_OPEN                       = 0,
    STREAM_MODE_PAUSE                      = 1,
    STREAM_MODE_TRANSPORT                  = 2,
    STREAM_MODE_TRANSPORT_PAUSE            = 3,
    STREAM_MODE_SEARCH_TRANSPORT           = 4,
    STREAM_MODE_SEARCH_TRANSPORT_PAUSE     = 5,
    STREAM_MODE_PAUSE_SEARCH_TRANSPORT     = 6,
    STREAM_MODE_END_OF_STREAM              = 7,
    STREAM_MODE_PRE_SEARCH_TRANSPORT       = 8,
    STREAM_MODE_PRE_SEARCH_TRANSPORT_PAUSE = 9
  };
protected:
  const TMpeg2StreamModeDescriptor* GetDescriptorBody() const throw ();

public:
  TMpeg2StreamModeDescriptorHandle(const uint8_t* dataPtr) throw ();
  
  uint32_t GetSize() const throw ();
  bool CheckSanity() const throw ();

  TStreamMode GetStreamMode() const throw ();

#ifndef NDEBUG
  void PrintInfo(std::ostream& s) const throw ();
#endif
};





inline TMpeg2StreamModeDescriptorHandle::
TMpeg2StreamModeDescriptorHandle(const uint8_t* buffer) throw ()
  : TMpeg2DescriptorHandle(buffer)
{
  
}

inline const TMpeg2StreamModeDescriptor*
TMpeg2StreamModeDescriptorHandle::GetDescriptorBody() const throw ()
{
  return reinterpret_cast<const TMpeg2StreamModeDescriptor*>(GetBodyData());
}

inline uint32_t TMpeg2StreamModeDescriptorHandle::GetSize() const throw ()
{
  return SIZEOF_TMPEG2STREAMMODEDESCRIPTOR;
}

inline bool TMpeg2StreamModeDescriptorHandle::CheckSanity() const throw ()
{
  return (GetTag() == TAG_TMPEG2STREAMMODEDESCRIPTOR &&
          (TMpeg2DescriptorHandle::GetSize() ==
           SIZEOF_TMPEG2STREAMMODEDESCRIPTOR +
           SIZEOF_TMPEG2DESCRIPTORHEADER));
}

inline TMpeg2StreamModeDescriptorHandle::TStreamMode
TMpeg2StreamModeDescriptorHandle::GetStreamMode() const throw ()
{
  return static_cast<TStreamMode>(GetDescriptorBody()->stream_mode);
}

#endif
