/*
 *--------------------------------------------------------------------
 *
 * libH222/TMpeg2SystemClock.h
 *
 * A class encapsulating the 90 kHz clock concept used in MPEG2.
 *
 * Copyright (c) 2004 Kreatel Communications AB. All Rights Reserved.
 *
 *--------------------------------------------------------------------
 */

#ifndef MEDIA_COMMON_TMPEG2SYSTEMCLOCK_H
#define MEDIA_COMMON_TMPEG2SYSTEMCLOCK_H

#include "media/libH222/TMpeg2Util.h"

#include <endian.h>
#include <stdint.h>

#ifndef __LITTLE_ENDIAN
#error Big endian target environment not supported!
#endif




struct TTimeStamp_3_15_15
{
  unsigned marker_bit_1               :1;
  unsigned base_30_32                 :3;
  unsigned padding                    :4;

  unsigned base_22_29                 :8;

  unsigned marker_bit_2               :1;
  unsigned base_15_21                 :7;

  unsigned base_7_14                  :8;

  unsigned marker_bit_3               :1;
  unsigned base_0_6                   :7;
};


struct TTimeStamp_3_15_15_e9
{
  unsigned base_28_29                 :2;
  unsigned marker_bit_1               :1;
  unsigned base_30_32                 :3;
  unsigned marker_code_01             :2;

  unsigned base_20_27                 :8;

  unsigned base_13_14                 :2;
  unsigned marker_bit_2               :1;
  unsigned base_15_19                 :5;

  unsigned base_5_12                  :8;

  unsigned extension_7_8              :2;
  unsigned marker_bit_3               :1;
  unsigned base_0_4                   :5;

  unsigned marker_bit_4               :1;
  unsigned extension_0_6              :7;
};

struct TTimeStamp_33_9
{
  unsigned base_25_32                 :8;

  unsigned base_17_24                 :8;

  unsigned base_9_16                  :8;

  unsigned base_1_8                   :8;

  unsigned base_0                     :1;
  unsigned reserved                   :6;
  unsigned extension_8                :1;

  unsigned extension_0_7              :8;
};


class TMpeg2SystemClock
{
public:
  
  
  
  
  enum { FREQUENCY = 90000 };  

protected:
  uint64_t Clock;  
  uint64_t Clock27; 

public:
  TMpeg2SystemClock() throw ();
  TMpeg2SystemClock(uint64_t base) throw ();
  TMpeg2SystemClock(uint64_t base, uint32_t extension) throw ();

  operator uint64_t() const throw ();
  uint64_t GetClock() const throw ();
  uint64_t GetClock27() const throw ();

  int GetDiff32(const TMpeg2SystemClock& previous) const throw ();

  void ReadTimeStamp_3_15_15(const uint8_t* data) throw ();
  void ReadTimeStamp_3_15_15_e9(const uint8_t* data) throw ();
  void ReadTimeStamp_33_e9(const uint8_t* data) throw ();

  bool operator<(const TMpeg2SystemClock& other) const throw ();
};


inline TMpeg2SystemClock::TMpeg2SystemClock() throw ()
{
  
}

inline TMpeg2SystemClock::TMpeg2SystemClock(uint64_t base) throw ()
  : Clock(base),
    Clock27(base * 300)
{
  
}

inline TMpeg2SystemClock::TMpeg2SystemClock(uint64_t base,
                                            uint32_t extension) throw ()
  : Clock(base),
    Clock27(base * 300 + extension)
{
  
}

inline TMpeg2SystemClock::operator uint64_t() const throw ()
{
  return GetClock();
}

inline uint64_t TMpeg2SystemClock::GetClock() const throw ()
{
  return Clock;
}

inline uint64_t TMpeg2SystemClock::GetClock27() const throw ()
{
  return Clock27;
}

inline int TMpeg2SystemClock::GetDiff32(const TMpeg2SystemClock& previous)
  const throw ()
{
  return static_cast<int>(Clock - previous.Clock);
}

inline bool
TMpeg2SystemClock::operator<(const TMpeg2SystemClock& other) const throw ()
{
  return Clock27 < other.Clock27;
}

#endif
