/*
 *--------------------------------------------------------------------
 *
 * libH222/TMpeg4AudioDescriptorHandle.h
 *
 * ISO/IEC 138181-1 MPEG-4 Audio descriptor.
 *
 * Copyright (c) 2010 Motorola, Inc. All Rights Reserved.
 *
 *--------------------------------------------------------------------
 */

#ifndef MEDIA_COMMON_TMPEG4AUDIODESCHANDLE_H
#define MEDIA_COMMON_TMPEG4AUDIODESCHANDLE_H


#include "media/libH222/TMpeg2DescriptorHandle.h"
#include "media/libH222/TMpeg2Util.h"
#include <string>



struct TMpeg4AudioDescriptorInfo
{
  unsigned profile_and_level             :8;
};

#define SIZEOF_TMPEG4AUDIODESCRIPTORBODY 1


class TMpeg4AudioDescriptorHandle : public TMpeg2DescriptorHandle
{
public:
  enum { UNDEFINED                               = 0x0f,
         MAIN_PROFILE_LEVEL_1                    = 0x10,
         MAIN_PROFILE_LEVEL_2                    = 0x11,
         MAIN_PROFILE_LEVEL_3                    = 0x12,
         MAIN_PROFILE_LEVEL_4                    = 0x13,
         SCALABLE_PROFILE_LEVEL_1                = 0x18,
         SCALABLE_PROFILE_LEVEL_2                = 0x19,
         SCALABLE_PROFILE_LEVEL_3                = 0x1a,
         SCALABLE_PROFILE_LEVEL_4                = 0x1b,
         SPEECH_PROFILE_LEVEL_1                  = 0x20,
         SPEECH_PROFILE_LEVEL_2                  = 0x21,
         SYNTHESIS_PROFILE_LEVEL_1               = 0x28,
         SYNTHESIS_PROFILE_LEVEL_2               = 0x29,
         SYNTHESIS_PROFILE_LEVEL_3               = 0x2A,
         HIGH_QUALITY_PROFILE_LEVEL_1            = 0x30,
         HIGH_QUALITY_PROFILE_LEVEL_2            = 0x31,
         HIGH_QUALITY_PROFILE_LEVEL_3            = 0x32,
         HIGH_QUALITY_PROFILE_LEVEL_4            = 0x33,
         HIGH_QUALITY_PROFILE_LEVEL_5            = 0x34,
         HIGH_QUALITY_PROFILE_LEVEL_6            = 0x35,
         HIGH_QUALITY_PROFILE_LEVEL_7            = 0x36,
         HIGH_QUALITY_PROFILE_LEVEL_8            = 0x37,
         LOW_DELAY_PROFILE_LEVEL_1               = 0x38,
         LOW_DELAY_PROFILE_LEVEL_2               = 0x39,
         LOW_DELAY_PROFILE_LEVEL_3               = 0x3a,
         LOW_DELAY_PROFILE_LEVEL_4               = 0x3b,
         LOW_DELAY_PROFILE_LEVEL_5               = 0x3c,
         LOW_DELAY_PROFILE_LEVEL_6               = 0x3d,
         LOW_DELAY_PROFILE_LEVEL_7               = 0x3e,
         LOW_DELAY_PROFILE_LEVEL_8               = 0x3f,
         NATURAL_PROFILE_LEVEL_1                 = 0x40,
         NATURAL_PROFILE_LEVEL_2                 = 0x41,
         NATURAL_PROFILE_LEVEL_3                 = 0x42,
         NATURAL_PROFILE_LEVEL_4                 = 0x43,
         MOBILE_INTERNETWORKING_PROFILE_LEVEL_1  = 0x48,
         MOBILE_INTERNETWORKING_PROFILE_LEVEL_2  = 0x49,
         MOBILE_INTERNETWORKING_PROFILE_LEVEL_3  = 0x4a,
         MOBILE_INTERNETWORKING_PROFILE_LEVEL_4  = 0x4b,
         MOBILE_INTERNETWORKING_PROFILE_LEVEL_5  = 0x4c,
         MOBILE_INTERNETWORKING_PROFILE_LEVEL_6  = 0x4d,
         AAC_PROFILE_LEVEL_1                     = 0x50,
         AAC_PROFILE_LEVEL_2                     = 0x51,
         AAC_PROFILE_LEVEL_4                     = 0x52,
         AAC_PROFILE_LEVEL_5                     = 0x53,
         HE_AAC_PROFILE_LEVEL_2                  = 0x58,
         HE_AAC_PROFILE_LEVEL_3                  = 0x59,
         HE_AAC_PROFILE_LEVEL_4                  = 0x5a,
         HE_AAC_PROFILE_LEVEL_5                  = 0x5b,
         HE_AAC_V2_PROFILE_LEVEL_2               = 0x60,
         HE_AAC_V2_PROFILE_LEVEL_3               = 0x61,
         HE_AAC_V2_PROFILE_LEVEL_4               = 0x62,
         HE_AAC_V2_PROFILE_LEVEL_5               = 0x63,
         USE_EXTENSION_DESCRIPTOR                = 0xff
 };

private:
  const TMpeg4AudioDescriptorInfo* DataPtr;
public:
  TMpeg4AudioDescriptorHandle() throw ();
  explicit TMpeg4AudioDescriptorHandle(const uint8_t* buffer) throw ();
  TMpeg4AudioDescriptorHandle(const TMpeg2DescriptorHandle &desc) throw ();

  uint8_t GetProfileAndLevel() const throw ();
};

inline TMpeg4AudioDescriptorHandle::TMpeg4AudioDescriptorHandle() throw ()
{
  
}

inline TMpeg4AudioDescriptorHandle::
TMpeg4AudioDescriptorHandle(const uint8_t* buffer) throw ()
  : TMpeg2DescriptorHandle(buffer)
{
  
}

inline TMpeg4AudioDescriptorHandle::
TMpeg4AudioDescriptorHandle(const TMpeg2DescriptorHandle &desc) throw ()
  : TMpeg2DescriptorHandle(desc)
{
  
}

inline uint8_t
TMpeg4AudioDescriptorHandle::GetProfileAndLevel() const throw ()
{
  const TMpeg4AudioDescriptorInfo* DataPtr =
        reinterpret_cast<const TMpeg4AudioDescriptorInfo*>(GetBodyData());
  return DataPtr->profile_and_level;
}

#endif
