/*
 *--------------------------------------------------------------------
 *
 * TPsPackHeaderHandle
 *
 * A class for accessing the program stream pack header.
 *
 * Copyright (c) 2004 Kreatel Communications AB. All Rights Reserved.
 *
 *--------------------------------------------------------------------
 */

#ifndef MEDIA_LIBH222_TPSPACKHEADERHANDLE_H
#define MEDIA_LIBH222_TPSPACKHEADERHANDLE_H

#include "TMpeg2Util.h"
#include "media/libH222/TMpeg2SystemClock.h"
#include <endian.h>
#include <stdint.h>
#include <cstdio>

#ifndef __LITTLE_ENDIAN
#error Big endian target environment not supported!
#endif

struct TPsPackHeader
{
  unsigned pack_start_code              :32;

  unsigned system_clock_ref_base28_29	:2;
  unsigned 				:1;
  unsigned system_clock_ref_base30_32	:3;
  unsigned id				:2;
  
  unsigned system_clock_ref_base20_27	:8;
  
  unsigned system_clock_ref_base13_14	:2;
  unsigned 				:1;
  unsigned system_clock_ref_base15_19	:5;
  
  unsigned system_clock_ref_base5_12	:8;

  unsigned system_clock_ref_ext_high	:2;
  unsigned				:1;
  unsigned system_clock_ref_base0_4	:5;

  unsigned				:1;
  unsigned system_clock_ref_ext_low	:7;

  unsigned program_mux_rate_high	:8;

  unsigned program_mux_rate_mid		:8;

  unsigned 				:2;
  unsigned program_mux_rate_low 	:6;
  
  unsigned pack_stuffing_length		:3;
  unsigned 				:5;
};

class TPsPackHeaderHandle
{
public:
  
  enum { HEADER_SIZE = 14 };
  enum { START_CODE = 0x000001ba };

protected:
  const TPsPackHeader* Header;
  
public:
  TPsPackHeaderHandle(const TPsPackHeader* data) throw ();
  TPsPackHeaderHandle(const uint8_t* data) throw ();

  uint32_t GetStartCode() const throw ();
  void GetScr(TMpeg2SystemClock& scr) const throw ();
  uint8_t GetPackStuffingLength() const throw();
  
  
  uint8_t GetTotalLength() const throw ();
  bool IsValid() const throw ();
};

inline
TPsPackHeaderHandle::TPsPackHeaderHandle(const TPsPackHeader* data) throw ()
  : Header(data)
{
  
}

inline 
TPsPackHeaderHandle::TPsPackHeaderHandle(const uint8_t* data) throw ()
  : Header(reinterpret_cast<const TPsPackHeader*>(data))
{
  
}

inline uint32_t TPsPackHeaderHandle::GetStartCode() const throw ()
{
  return TMpeg2Util::Reorder32(Header->pack_start_code);
}

inline void TPsPackHeaderHandle::GetScr(TMpeg2SystemClock& scr) const throw ()
{
  const uint8_t* data = reinterpret_cast<const uint8_t*>(Header);
  scr.ReadTimeStamp_3_15_15_e9(data + 4);
}

inline uint8_t TPsPackHeaderHandle::GetPackStuffingLength() const throw ()
{
  return Header->pack_stuffing_length;
}

inline uint8_t TPsPackHeaderHandle::GetTotalLength() const throw()
{
  return HEADER_SIZE + GetPackStuffingLength();  
}

inline bool TPsPackHeaderHandle::IsValid() const throw ()
{
  return Header != NULL
    && GetStartCode() == START_CODE
    && Header->id == 1;
}

#endif
