/*
 *--------------------------------------------------------------------
 *
 * TSsPackHeaderHandle
 *
 * A class for accessing the system stream pack header.
 *
 * Copyright (c) 2004 Kreatel Communications AB. All Rights Reserved.
 *
 *--------------------------------------------------------------------
 */

#ifndef MEDIA_LIBH222_TSSPACKHEADERHANDLE_H
#define MEDIA_LIBH222_TSSPACKHEADERHANDLE_H

#include "TMpeg2Util.h"
#include "media/libH222/TMpeg2SystemClock.h"

#include <endian.h>
#include <stdint.h>
#include <cstdio>

#ifndef __LITTLE_ENDIAN
#error Big endian target environment not supported!
#endif

struct TSsPackHeader
{
  unsigned pack_start_code              :32;

  unsigned 				:1;
  unsigned system_clock_ref_base30_32	:3;
  unsigned id				:4;

  unsigned system_clock_ref_base22_29	:8;

  unsigned 				:1;
  unsigned system_clock_ref_base15_21	:7;
  
  unsigned system_clock_ref_base7_14	:8;
  
  unsigned 				:1;
  unsigned system_clock_ref_base0_6	:7;

  unsigned				:1;
  unsigned mux_rate15_21         	:7;

  unsigned mux_rate7_14		        :8;

  unsigned				:1;
  unsigned mux_rate0_6         	        :7;
};


class TSsPackHeaderHandle
{
public:
  enum { HEADER_SIZE = 12 };
  enum { START_CODE = 0x000001ba };

protected:
  const TSsPackHeader* Header;
  
public:
  TSsPackHeaderHandle(const TSsPackHeader* data) throw ();
  TSsPackHeaderHandle(const uint8_t* data) throw ();
  
  void GetScr(TMpeg2SystemClock& scr) const throw ();
  uint32_t GetStartCode() const throw ();
  uint8_t GetTotalLength() const throw ();
  bool IsValid() const throw ();
};

inline
TSsPackHeaderHandle::TSsPackHeaderHandle(const TSsPackHeader* data) throw ()
  : Header(data)
{
  
}

inline 
TSsPackHeaderHandle::TSsPackHeaderHandle(const uint8_t* data) throw ()
  : Header(reinterpret_cast<const TSsPackHeader*>(data))
{
  
}

inline void TSsPackHeaderHandle::GetScr(TMpeg2SystemClock& scr) const throw ()
{
  const uint8_t* data = reinterpret_cast<const uint8_t*>(Header);
  scr.ReadTimeStamp_3_15_15(data + 4);
}

inline uint32_t TSsPackHeaderHandle::GetStartCode() const throw ()
{
  return TMpeg2Util::Reorder32(Header->pack_start_code);
}

inline uint8_t TSsPackHeaderHandle::GetTotalLength() const throw()
{
  return HEADER_SIZE;
}

inline bool TSsPackHeaderHandle::IsValid() const throw ()
{
  return (Header != NULL
          && GetStartCode() == START_CODE
          && Header->id == 2);
}

#endif
