/*
 *--------------------------------------------------------------------
 *
 * TSsSystemHeaderHandle.h
 *
 * A class for accessing the system streams system header.
 *
 * Copyright (c) 2004 Kreatel Communications AB. All Rights Reserved.
 *
 *--------------------------------------------------------------------
 */

#ifndef MEDIA_LIBH222_TSSHANDLES_H
#define MEDIA_LIBH222_TSSHANDLES_H

#include "TMpeg2Util.h"
#include <endian.h>
#include <stdint.h>
#include <cstdio>

#ifndef __LITTLE_ENDIAN
#error Big endian target environment not supported!
#endif

struct TSsSystemHeader
{
  unsigned system_start_code	        :32;
  unsigned header_length		:16;

  unsigned rate_bound_0_6		:7;
  unsigned 				:1;

  unsigned rate_bound_7_14		:8;

  unsigned 				:1;
  unsigned rate_bound_15_21		:7;
  
  unsigned CSPS_flag			:1;
  unsigned fixed_flag			:1;
  unsigned audio_bound			:6;
  
  unsigned video_bound			:5;
  unsigned 				:1;
  unsigned system_video_lock_flag	:1;
  unsigned system_audio_lock_flag	:1;

  unsigned reserved			:8;
};

class TSsSystemHeaderHandle
{
public:
  enum { HEADER_SIZE = 12 };
  enum { START_CODE = 0x000001bb };

protected:
  const TSsSystemHeader* Header;
  
public:
  TSsSystemHeaderHandle(const TSsSystemHeader* data) throw ();
  TSsSystemHeaderHandle(const uint8_t* data) throw ();

  uint32_t GetStartCode() const throw ();

  
  uint16_t GetHeaderLength() const throw ();

  uint32_t GetTotalLength() const throw ();

  bool IsValid() const throw ();
};

inline
TSsSystemHeaderHandle::TSsSystemHeaderHandle(const TSsSystemHeader* data)
  throw ()
  : Header(data)
{
  
}

inline 
TSsSystemHeaderHandle::TSsSystemHeaderHandle(const uint8_t* data) throw ()
  : Header(reinterpret_cast<const TSsSystemHeader*>(data))
{
  
}

inline uint32_t TSsSystemHeaderHandle::GetStartCode() const throw ()
{
  return TMpeg2Util::Reorder32(Header->system_start_code);
}

inline uint16_t TSsSystemHeaderHandle::GetHeaderLength() const throw () 
{
  return TMpeg2Util::Reorder16(Header->header_length);
}

inline uint32_t TSsSystemHeaderHandle::GetTotalLength() const throw ()
{
  return GetHeaderLength() + 6; 
}

inline bool TSsSystemHeaderHandle::IsValid() const throw ()
{
  return (Header != NULL && GetStartCode() == START_CODE);
}

#endif
