/*
 *--------------------------------------------------------------------
 *
 * libH222/TTsCaHandle.h
 *
 * A class for accessing conditional access (CA) section structures
 * used in MPEG2 transport streams. The class uses the bridge pattern
 * (also called the handle pattern) described in Design Patterns,
 * Gamme et. al., p151.
 *
 * Copyright (c) 2004 Kreatel Communications AB. All rights reserved.
 *
 *--------------------------------------------------------------------
 */

#ifndef MEDIA_COMMON_TTSCAHANDLE_H
#define MEDIA_COMMON_TTSCAHANDLE_H

#include "media/libH222/TTsSectionHandle.h"
#include "media/libH222/TMpeg2DescriptorHandle.h"

#ifndef __LITTLE_ENDIAN
#error Big endian target environment not supported!
#endif




class TTsCaHandle : public TTsSectionHandle
{
public:
  TTsCaHandle(const uint8_t* buffer) throw ();

  bool CheckSanity() const throw ();

  TMpeg2DescriptorHandle GetFirstDescriptor() const throw ();
  TMpeg2DescriptorHandle GetNextDescriptor(TMpeg2DescriptorHandle previous)
    const throw ();

#ifndef NDEBUG
  void PrintInfo(std::ostream& s) const throw ();
#endif
};


inline TTsCaHandle::TTsCaHandle(const uint8_t* buffer) throw ()
  : TTsSectionHandle(buffer)
{
  
}

#endif
