/*
 *--------------------------------------------------------------------
 *
 * THaiAudioFormat.h
 *
 * A class that handles the HAI audio format.
 * 
 * Copyright (c) 2004 Kreatel Communications AB. All Rights Reserved.
 *
 *--------------------------------------------------------------------
 */

#ifndef THAIAUDIOFORMAT_H
#define THAIAUDIOFORMAT_H

#include "media/streamer/IMediaDecoder.h"
#include "media/streamer/IStreamerTypes.h"
#include "media/streamer/IDebug.h"
#include "hal/IHaiMpegAudioDecoder.h"

class IMediaDecoder;
class IStreamInput;

class THaiAudioFormat
{
private:
  IDebug* Debug;

  THaiMediaPacketFormat PacketFormat;
  THaiMediaFormat CurrentDecoderFormat;
  IHaiMpegAudioDecoder_TConfig CurrentAudioConfig;
  std::string CurrentAudioMode;
      
  bool WriteFormat(IMediaDecoder* decoder,
                   THaiMediaFormat decoderFormat,
                   const IHaiMpegAudioDecoder_TConfig& config) throw ();
                   
public:
  THaiAudioFormat() throw ();
  THaiAudioFormat(THaiMediaPacketFormat packetFormat) throw ();
  void SetPacketFormat(THaiMediaPacketFormat packetFormat)
    throw ();
  void SetDebug(IDebug* debug) throw ();
  bool SetFormat(IMediaDecoder* decoder,
                 const IStreamerTypes::TAudioStreamInfo& streaminfo) throw ();
  void RestoreState(IMediaDecoder* mediaDecoder) throw ();
};

inline void
THaiAudioFormat::SetPacketFormat(THaiMediaPacketFormat packetFormat) throw ()
{
  PacketFormat = packetFormat;
}

inline void THaiAudioFormat::SetDebug(IDebug* debug) throw ()
{
  Debug = debug;
}

#endif
