/*
 *--------------------------------------------------------------------
 *
 * THaiStreamInputHandler.h
 *
 * A class that handles all payload transmission to the HAI stream
 * input.
 * 
 * Copyright (c) 2004 Kreatel Communications AB. All Rights Reserved.
 *
 *--------------------------------------------------------------------
 */

#ifndef THAIDECODERPAYLOADHANDLER_H
#define THAIDECODERPAYLOADHANDLER_H

#include "TMetadataStorage.h"

#include "media/streamer/TSegmentList.h"
#include "media/streamer/IPump.h"
#include "media/streamer/IDebug.h"
#include "media/streamer/IStreamInput.h"

#include <list>


class IHaiStreamInputObserver
{
public:
  virtual ~IHaiStreamInputObserver() {}
  virtual void OnPreConsumedMetadata(const std::string& key,
                                     const std::string& value) throw () = 0;
  virtual void OnConsumedMetadata(const std::string& key,
                                  const std::string& value) throw () = 0;
  virtual void OnFlushMetadata(const std::string& key,
                               const std::string& value) throw () = 0;
};

class IStreamInput;

class THaiStreamInputHandler
{
private:
  class TStorageEntry
  {
  public:
    uint32_t Offset;
    TMetadataStorage* MetadataStorage;

    TStorageEntry() throw ();
    ~TStorageEntry() throw ();
  };
  
  IHaiStreamInputObserver* Observer;
  IPump* Pump;
  IDebug* Debug;
  const uint8_t* Buffer;
  IStreamInput* StreamInput;
  uint32_t PendingCount;

  uint32_t LastPushedOffset;
  TMetadataStorage* CurrentMetadataStorage;
  std::list<TStorageEntry*> StorageList;
  bool PayloadRelease;

  uint32_t GetOffset(uint8_t* position) const throw ();
  void HandlePreConsumedMetadata(const TMetadata* md) throw ();
  void HandleConsumedMetadata(const TMetadata* md) throw ();
  void HandleConsumedMetadata(TMetadataStorage* mdS) throw ();
  void HandleFlushMetadata(const TMetadata* md) throw ();
  void ClearInternalMetadata() throw ();

public:
  THaiStreamInputHandler(IHaiStreamInputObserver* observer) throw ();
  ~THaiStreamInputHandler() throw ();

  void Initialize(IPump* pump, IDebug* debug,
                  IStreamInput* streamInput) throw ();
  void UsePumpBuffer() throw ();

  void Process(const TSegmentList& segments) throw ();
  void Flush(const TSegmentList& segments) throw ();
  bool IsPayloadConsumed() throw ();
  void Close() throw ();
  void SetPayloadReleaseMode(bool release) throw ();
};


inline THaiStreamInputHandler::TStorageEntry::TStorageEntry() throw ()
  : MetadataStorage(NULL)
{
  // Empty.
}

inline THaiStreamInputHandler::TStorageEntry::~TStorageEntry() throw ()
{
  if (MetadataStorage != NULL) {
    delete MetadataStorage;
  }
}

inline uint32_t THaiStreamInputHandler::GetOffset(uint8_t* position)
  const throw ()
{
  return position - Buffer;
}

#endif
