/*
 *--------------------------------------------------------------------
 *
 * TSinkElement.h
 *
 * A sink element that sends the stream data to HAL using the
 * HaiAudioDecoder and the HaiVideoDecoder interface indirectly
 * through IMediaDecoder.
 * 
 * Copyright (c) 2009 Motorola Inc. All Rights Reserved.
 * Copyright (c) 2011 Motorola Mobility, Inc. All rights reserved.
 *
 *--------------------------------------------------------------------
 */

#ifndef TSINKELEMENT_H
#define TSINKELEMENT_H

#include "THaiStreamInputHandler.h"
#include "media/streamer/IElement.h"
#include "media/streamer/IInputPad.h"
#include "media/streamer/ISink.h"
#include "media/streamer/IBlackboard.h"
#include "media/streamer/IHardware.h"
#include "media/streamer/IDebug.h"
#include "media/streamer/IStreamInput.h"
#include "media/streamer/IMediaDecoder.h"
#include "media/streamer/IPlaybackCommandObserver.h"

#include <map>

class TSinkElement : public IElement,
                     public IInputPad,
                     virtual public ISink,
                     public IPlaybackCommandObserver,
                     public IHaiStreamInputObserver,
                     public IStreamInputObserver,
                     public IMediaDecoderObserver
{
private:
  typedef std::map<int, std::string> TOptions;
  TOptions Options;

protected:
  IPump* Pump;
  IDebug* Debug;
  ISinkCallback* SinkCallback;
  IBlackboard* Blackboard;
  IMediaDecoder* Decoder;
  IStreamInput* StreamInput;
  THaiStreamInputHandler PayloadHandler;
  bool PayloadReleaseMode;
  bool IsBlocked;
  // The component flags can have three values: 
  // -1: not defined. 0: false.  1: true
  int AudioComponentFlag;
  int VideoComponentFlag;
  bool SyncEnabledFlag;

  void SetComponentStatus(const std::string& key, int8_t status) throw ();
  virtual void SetSynchronized(const bool& value) throw () = 0;
  virtual bool IsFormatSet() = 0;

  void SetOption(int id, const std::string& value) throw ();
  void SetPayloadReleaseMode(bool release) throw ();
  void RunIfReady();

public:
  TSinkElement() throw ();
  virtual ~TSinkElement() throw ();

  // IElement interface.
  virtual void Destroy() throw ();
  virtual void Initialize(IPump* pump, const char* dataFormat) throw ();
  virtual void* QueryInterface(const char* name) throw ();

  // IInputPad interface.
  virtual void Process(const TSegmentList& segments) throw ();
  virtual void Flush(const TSegmentList& segments) throw ();

  // ISink interface.
  virtual void Block() throw ();
  virtual void Unblock() throw ();

  // IPlaybackCommandObserver
  virtual void OnPlay(int32_t pace) throw ();
  virtual void OnPlayFromPosition(uint32_t position, int32_t pace) throw ();

  // IStreamInputObserver
  virtual void OnConsumed() throw ();

  // IMediaDecoderObserver
  virtual void OnRunning() throw ();
  virtual void OnEmpty() throw ();
  virtual void OnInfoChanged() throw ();
  virtual void OnNewPicture() throw ();
};

#endif
