/*
 *--------------------------------------------------------------------
 *
 * TMetaAudioStreamInfo.h
 *
 * Helper class for handling audio stream info in the meta data.
 *
 * Copyright (c) 2003 Kreatel Communications AB. All Rights Reserved.
 * Copyright (c) 2009 Motorola, Inc. All Rights Reserved.
 *
 *--------------------------------------------------------------------
 */

#ifndef TMETAAUDIOSTREAMINFO_H
#define TMETAAUDIOSTREAMINFO_H

#include "media/streamer/IStreamerTypes.h"
#include <string>

class TMetaAudioStreamInfo
{
private:
  IStreamerTypes::TAudioStreamInfo StreamInfo;

public:
  TMetaAudioStreamInfo();
  ~TMetaAudioStreamInfo();

  void Clear() throw ();

  void ParseXml(const std::string& xml);
  void GetXml(std::string& xml) const throw ();

  void SetEncoding(const std::string& encoding) throw ();
  const std::string& GetEncoding() throw ();
  void SetSampleRate(int sampleRate) throw ();
  void SetBitsPerSample(int bitsPerSample) throw ();
  void SetNumberOfChannels(int numberOfChannels) throw ();
  void SetBigEndian(bool bigEndian) throw ();
  void SetChannelFormat(IStreamerTypes::TAudioChannelFormat channelFormat)
    throw();
  void SetChannelMode(IStreamerTypes::TAudioChannelMode channelMode)
    throw();
  IStreamerTypes::TAudioChannelMode GetChannelMode() throw();
  void SetLanguage(const std::string& language) throw ();
  const std::string& GetLanguage() throw ();
  void SetSubLanguage(const std::string& subLanguage) throw ();
  void SetAudienceType(IStreamerTypes::TAudioAudienceType audienceType)
    throw ();
  void SetComponentTag(int32_t tag) throw ();
  void SetIsSupported(bool isSupported);

  void GetInfo(IStreamerTypes::TAudioStreamInfo& streamInfo) const throw ();
};


// ----------------------------------------------------------------------------
// Inlined functions
inline void
TMetaAudioStreamInfo::Clear() throw ()
{
  StreamInfo.Encoding.clear();
  StreamInfo.SampleRate = -1;
  StreamInfo.BitsPerSample = -1;
  StreamInfo.BigEndian = false;
  StreamInfo.ChannelFormat = IStreamerTypes::AUDIO_CHANNEL_FORMAT_UNKNOWN;
  StreamInfo.ChannelMode = IStreamerTypes::AUDIO_CHANNEL_MODE_NORMAL;
  StreamInfo.Language.clear();
  StreamInfo.SubLanguage.clear();
  StreamInfo.AudienceType = IStreamerTypes::AUDIO_AUDIENCE_TYPE_UNKNOWN;
  StreamInfo.ComponentTag = -1;
  StreamInfo.IsSupported = true;
}

inline void
TMetaAudioStreamInfo::SetNumberOfChannels(int numberOfChannels) throw () 
{
    StreamInfo.ChannelFormat =
      static_cast<IStreamerTypes::TAudioChannelFormat>(numberOfChannels);
}

inline void
TMetaAudioStreamInfo::SetEncoding(const std::string& encoding) throw ()
{
  StreamInfo.Encoding = encoding;
}

inline const std::string&
TMetaAudioStreamInfo::GetEncoding() throw ()
{
  return StreamInfo.Encoding;
}

inline void
TMetaAudioStreamInfo::SetSampleRate(int sampleRate) throw ()
{
  StreamInfo.SampleRate = sampleRate;
}

inline void TMetaAudioStreamInfo::SetBitsPerSample(int bitsPerSample) throw ()
{
  StreamInfo.BitsPerSample = bitsPerSample;
}

inline void TMetaAudioStreamInfo::SetBigEndian(bool bigEndian) throw ()
{
  StreamInfo.BigEndian = bigEndian;
}

inline void
TMetaAudioStreamInfo::
SetChannelFormat(IStreamerTypes::TAudioChannelFormat channelFormat) throw ()
{
  StreamInfo.ChannelFormat = channelFormat;
}

inline void
TMetaAudioStreamInfo::
SetChannelMode(IStreamerTypes::TAudioChannelMode channelMode) throw ()
{
  StreamInfo.ChannelMode = channelMode;
}

inline IStreamerTypes::TAudioChannelMode
TMetaAudioStreamInfo::GetChannelMode() throw ()
{
  return StreamInfo.ChannelMode;
}

inline void
TMetaAudioStreamInfo::SetLanguage(const std::string& language) throw ()
{
  StreamInfo.Language = language;
}

inline const std::string&
TMetaAudioStreamInfo::GetLanguage() throw ()
{
  return StreamInfo.Language;
}

inline void
TMetaAudioStreamInfo::SetSubLanguage(const std::string& subLanguage) throw ()
{
  StreamInfo.SubLanguage = subLanguage;
}

inline void
TMetaAudioStreamInfo::SetAudienceType(IStreamerTypes::TAudioAudienceType
                                      audienceType) throw ()
{
  StreamInfo.AudienceType = audienceType;
}

inline void
TMetaAudioStreamInfo::SetComponentTag(int32_t tag) throw ()
{
  StreamInfo.ComponentTag = tag;
}

inline void TMetaAudioStreamInfo::SetIsSupported(bool isSupported)
{
  StreamInfo.IsSupported = isSupported;
}

inline void
TMetaAudioStreamInfo::GetInfo(IStreamerTypes::TAudioStreamInfo& streamInfo)
  const throw ()
{
  streamInfo = StreamInfo;
}

#endif
