/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *--------------------------------------------------------------------------
 *
 * TMetaProgramSpecificInfo.h
 *
 * Helper class for handling PSI data in the meta data.
 *
 * Copyright (c) 2010 Motorola, Inc. All Rights Reserved.
 *
 *--------------------------------------------------------------------------
 */

#ifndef TMETAPROGRAMSPECIFICINFO_H
#define TMETAPROGRAMSPECIFICINFO_H

#include "base/NonCopyable.h"

#include <stdint.h>
#include <string>

class TTsPsiHandle;

class TMetaProgramSpecificInfo
{
private:
  NON_COPYABLE(TMetaProgramSpecificInfo);

  uint8_t* Data;
  uint32_t DataSize;

  static TTsPsiHandle GetPsi(
    const uint8_t* data, uint32_t dataSize, uint16_t& pid);

public:
  TMetaProgramSpecificInfo();
  ~TMetaProgramSpecificInfo();

  void Clear();

  void Append(uint16_t pid, const TTsPsiHandle& psi);

  /**
   * Fetches the first PSI (and the associated PID) from the stored data. If no
   * section can be found, TTsPsiHandle(NULL) is returned.
   */
  TTsPsiHandle GetFirstPsi(uint16_t& pid) const;

  /**
   * Fetches the next PSI (and the associated PID) from the stored data. The
   * PSI returned from GetFirstPsi() or a previous call to GetNextPsi() should
   * be passed as previous. Returns TTsPsiHandle(NULL) if no next section is
   * available.
   */
  TTsPsiHandle GetNextPsi(const TTsPsiHandle& previous, uint16_t& pid) const;

  void SetFromBase64EncodedString(const std::string& psi);
  std::string GetBase64EncodedString() const;
};

#endif
