/*
 *--------------------------------------------------------------------
 *
 * TMetaStreamList.h
 *
 * Helper class for handling stream lists in the meta data.
 * 
 * Copyright (c) 2003 Kreatel Communications AB. All Rights Reserved.
 *
 *--------------------------------------------------------------------
 */

#ifndef TMETASTREAMLIST_H
#define TMETASTREAMLIST_H

#include <string>
#include <vector>

class TMetaStreamList
{
public:
  struct TStream {
    int StreamId;
    int StreamIdExtension;
  };
  
  typedef std::vector<int> TStreamList;
  typedef std::vector<TStream> TStreamListExtended;

  // PID type.  Copy from common/media/libH222/TTsPmtHandle.h
  enum { TS_TYPE_11172_VIDEO = 0x01, 
         TS_TYPE_13818_VIDEO = 0x02,
         TS_TYPE_11172_AUDIO = 0x03,
         TS_TYPE_13818_AUDIO = 0x04,
         TS_TYPE_PRIVATE = 0x06,
         TS_TYPE_14496_2_VIDEO = 0x10,
         TS_TYPE_14496_3_AUDIO = 0x11,
         TS_TYPE_14496_10_VIDEO = 0x1b,
         TS_TYPE_USER_PRIVATE_0 = 0x80,
         TS_TYPE_ATSC_AC3 = 0x81 };
  
private:
  TStreamListExtended Streams;
  
public:
  TMetaStreamList();
  ~TMetaStreamList();

  void ParseXml(const std::string& xml);
  void GetXml(std::string& xml) const throw ();
  
  void AddStream(int streamId, int streamIdExtension = -1) throw ();
  void GetStreamList(TStreamList& streams) const throw ();
  void GetStreamList(TStreamListExtended& streams) const throw ();
  void SetStreamList(const TStreamList& streams) throw ();
  void SetStreamList(const TStreamListExtended& streams) throw ();
  void ClearStreamList() throw ();
  int GetStreamCount() const throw ();
};


// ----------------------------------------------------------------------------
// Inlined functions

inline void
TMetaStreamList::AddStream(int streamId, int streamIdExtension) throw ()
{
  TStream stream;
  stream.StreamId = streamId;
  if (streamIdExtension == -1) {
    streamIdExtension = streamId;
  }
  stream.StreamIdExtension = streamIdExtension;
  Streams.push_back(stream);
}

inline void 
TMetaStreamList::GetStreamList(TStreamList& streams) const throw ()
{
  streams.clear();
  for (TStreamListExtended::const_iterator i = Streams.begin(); 
       i != Streams.end(); ++i) {
    streams.push_back((*i).StreamId);
  }
}

inline void
TMetaStreamList::GetStreamList(TStreamListExtended& streams) const throw ()
{
  streams = Streams;
}

inline void 
TMetaStreamList::SetStreamList(const TStreamList& streams) throw ()
{
  Streams.clear();
  for (TStreamList::const_iterator i = streams.begin(); 
       i != streams.end(); ++i) {
    AddStream(*i);
  }
}

inline void 
TMetaStreamList::SetStreamList(const TStreamListExtended& streams) throw ()
{
  Streams = streams;
}


inline void
TMetaStreamList::ClearStreamList() throw ()
{
  Streams.clear();
}

inline int
TMetaStreamList::GetStreamCount() const throw ()
{
  return Streams.size();
}

#endif
