/*
 *--------------------------------------------------------------------
 *
 * TMetaSubtitleStreamInfo.h
 *
 * Helper class for handling subtitle stream info in the meta data.
 * 
 * Copyright (c) 2003 Kreatel Communications AB. All Rights Reserved.
 *
 *--------------------------------------------------------------------
 */

#ifndef TMETASUBTITLESTREAMINFO_H
#define TMETASUBTITLESTREAMINFO_H

#include "media/streamer/IStreamerTypes.h"
#include <string>

class TMetaSubtitleStreamInfo
{
private:
  IStreamerTypes::TSubtitleStreamInfo StreamInfo;
  
public:
  TMetaSubtitleStreamInfo();
  ~TMetaSubtitleStreamInfo();

  void ParseXml(const std::string& xml);
  void GetXml(std::string& xml) const throw ();
  
  void SetLanguage(const std::string& language) throw ();
  void SetType(IStreamerTypes::TSubtitleType type) throw ();
  void SetFormat(IStreamerTypes::TSubtitleFormat format) throw ();
  void SetAspectRatio(IStreamerTypes::TAspectRatio aspectRatio) throw ();
  void SetPageId(int32_t page) throw ();
  void SetAncillaryPageId(int32_t page) throw ();
  void SetPid(int32_t pid) throw ();
  void SetExtent(uint16_t width, uint16_t height) throw ();
  void SetCompression(const std::string& compression) throw ();

  void GetInfo(IStreamerTypes::TSubtitleStreamInfo& streamInfo) const throw ();

  bool operator==(const TMetaSubtitleStreamInfo& streamInfo);
};


// ----------------------------------------------------------------------------
// Inlined functions

inline void
TMetaSubtitleStreamInfo::SetLanguage(const std::string& language) throw ()
{
  StreamInfo.Language = language;
}

inline void
TMetaSubtitleStreamInfo::SetType(IStreamerTypes::TSubtitleType type)
  throw ()
{
  StreamInfo.Type = type;
}

inline void
TMetaSubtitleStreamInfo::SetFormat(IStreamerTypes::
                                   TSubtitleFormat format) throw ()
{
  StreamInfo.Format = format;
}

inline void
TMetaSubtitleStreamInfo::SetAspectRatio(IStreamerTypes::
                                        TAspectRatio aspectRatio) throw ()
{
  StreamInfo.AspectRatio = aspectRatio;
}

inline void
TMetaSubtitleStreamInfo::SetPageId(int32_t page) throw ()
{
  StreamInfo.PageId = page;
}

inline void
TMetaSubtitleStreamInfo::SetAncillaryPageId(int32_t page) throw ()
{
  StreamInfo.AncillaryPageId = page;
}

inline void
TMetaSubtitleStreamInfo::SetPid(int32_t pid) throw ()
{
  StreamInfo.Pid = pid;
}

inline void
TMetaSubtitleStreamInfo::SetExtent(uint16_t width, uint16_t height) throw ()
{
  StreamInfo.Width = width;
  StreamInfo.Height = height;
}

inline void
TMetaSubtitleStreamInfo::SetCompression(const std::string& compression)
  throw ()
{
  StreamInfo.Compression = compression;
}

inline void
TMetaSubtitleStreamInfo::GetInfo(IStreamerTypes::
                                 TSubtitleStreamInfo& streamInfo)
  const throw ()
{
  streamInfo = StreamInfo;
}

inline bool
TMetaSubtitleStreamInfo::operator==(const TMetaSubtitleStreamInfo& streamInfo)
{
  std::string xml1;
  std::string xml2;
  GetXml(xml1);
  streamInfo.GetXml(xml2);
  return (xml1 == xml2);
}

#endif
