/*
 *--------------------------------------------------------------------
 *
 * TMetaUdpDatagram.h
 *
 * Helper class for handling UDP datagram in the meta data.
 * 
 * Copyright (c) 2010 Motorola, Inc. All Rights Reserved.
 *
 *--------------------------------------------------------------------
 */

#ifndef TMETAUDPDATAGRAM_H
#define TMETAUDPDATAGRAM_H

#include "media/streamer/IStreamerTypes.h"
#include <string>
#include <vector>

class TMetaUdpDatagram
{
private:
  uint32_t Address;
  uint16_t Port;
  std::vector<uint8_t> Payload;

public:
  TMetaUdpDatagram();
  ~TMetaUdpDatagram();

  void ParseXml(const std::string& xml) throw ();
  void GetXml(std::string& xml) throw (); 

  void SetAddress(uint32_t address) throw ();
  uint32_t GetAddress() throw ();
  void SetPort(uint16_t port) throw ();
  uint16_t GetPort() throw ();

  void SetPayload(const uint8_t* buf, size_t buflen) throw ();
  const uint8_t* GetPayload() throw ();
  size_t GetPayloadSize() throw ();
};

inline void TMetaUdpDatagram::SetAddress(uint32_t address) throw ()
{ 
   Address = address; 
}

inline uint32_t TMetaUdpDatagram::GetAddress() throw ()
{ 
  return Address; 
}

inline void TMetaUdpDatagram::SetPort(uint16_t port) throw ()
{ 
  Port = port; 
}

inline uint16_t TMetaUdpDatagram::GetPort() throw ()
{
  return Port;
}

inline void TMetaUdpDatagram::SetPayload(const uint8_t* buf, size_t bufLen) throw ()
{
  Payload.assign(buf, buf + bufLen);
}

inline const uint8_t* TMetaUdpDatagram::GetPayload() throw ()
{
  return &Payload[0];
}

inline size_t TMetaUdpDatagram::GetPayloadSize() throw ()
{
  return Payload.size();
}

#endif
