/*
 *--------------------------------------------------------------------
 *
 * TMetaVideoStreamInfo.h
 *
 * Helper class for handling video stream info in the meta data.
 * 
 * Copyright (c) 2003 Kreatel Communications AB. All Rights Reserved.
 *
 *--------------------------------------------------------------------
 */

#ifndef TMETAVIDEOSTREAMINFO_H
#define TMETAVIDEOSTREAMINFO_H

#include "media/streamer/IStreamerTypes.h"
#include <string>

class TMetaVideoStreamInfo
{
private:
  IStreamerTypes::TVideoStreamInfo StreamInfo;
  
public:
  TMetaVideoStreamInfo();
  ~TMetaVideoStreamInfo();

  void ParseXml(const std::string& xml);
  void GetXml(std::string& xml) const throw ();
  
  void SetEncoding(const std::string& encoding) throw ();
  void SetStillPicture(const std::string& still) throw ();
  void SetComponentTag(const int32_t componentTag) throw ();

  void GetInfo(IStreamerTypes::TVideoStreamInfo& streamInfo) const throw ();
};


// ----------------------------------------------------------------------------
// Inlined functions

inline void
TMetaVideoStreamInfo::SetEncoding(const std::string& encoding) throw ()
{
  StreamInfo.Encoding = encoding;
}

inline void
TMetaVideoStreamInfo::SetStillPicture(const std::string& stillPicture) throw ()
{
  StreamInfo.StillPicture = stillPicture;
}

inline void
TMetaVideoStreamInfo::SetComponentTag(const int32_t componentTag) throw ()
{
  StreamInfo.ComponentTag = componentTag;
}

inline void
TMetaVideoStreamInfo::GetInfo(IStreamerTypes::TVideoStreamInfo& streamInfo)
  const throw ()
{
  streamInfo = StreamInfo;
}

#endif
