/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *---------------------------------------------------------------------------
 *
 * IDemuxer.h
 *
 * Demuxer interface for streamer elements.
 *
 * Copyright (c) 2009 Motorola, Inc. All Rights Reserved.
 *
 *---------------------------------------------------------------------------
 */

#ifndef STREAMER_IDEMUXER_H
#define STREAMER_IDEMUXER_H

#include "IElement.h"

#include "hai/IDemuxer.h"

#include "stdint.h"
#include <string>

class IDemuxerObserver
{
public:
  virtual ~IDemuxerObserver() { /* Empty */ }

  virtual void OnBufferFull() throw () = 0;
};


class IDemuxer
{
public:
  virtual ~IDemuxer() { /* Empty */ }

  virtual std::string GetResourceName() const throw () = 0;
  virtual std::string GetInstanceName() const throw () = 0;

  virtual THaiResult ConnectInput(const char* resourceName,
                                  const char* instanceName,
                                  int index) throw () = 0;
  virtual THaiResult GetSupportedFormats(uint32_t* formats) throw () = 0;
  virtual THaiResult GetFormat(THaiDemuxerFormat* format) throw () = 0;
  virtual THaiResult SetFormat(THaiDemuxerFormat format) throw () = 0;
  virtual THaiResult SetPcrPid(uint16_t pcrPid) throw () = 0;
  virtual THaiResult GetStc(uint64_t& stc) throw () = 0;
  virtual THaiResult CreateOutput(uint32_t* outputId) throw () = 0;
  virtual THaiResult DestroyOutput(uint32_t outputId) throw () = 0;
  virtual THaiResult OpenOutputPipe(uint32_t outputId,
                                    const char* socketName,
                                    THaiDemuxerOutputPipeType pipeType)
    throw () = 0;
  virtual THaiResult CloseOutputPipe(uint32_t outputId) throw () = 0;
  virtual THaiResult AddPacketFilter(uint32_t outputId,
                                     uint16_t pid) throw () = 0;
  virtual THaiResult RemovePacketFilter(uint32_t outputId,
                                        uint16_t pid) throw () = 0;
  virtual THaiResult AddSectionFilter(uint32_t outputId,
                                      uint16_t pid,
                                      const uint8_t* filter,
                                      const uint8_t* include,
                                      const uint8_t* exclude,
                                      uint8_t length,
                                      uint32_t* sectionId) throw () = 0;
  virtual THaiResult RemoveSectionFilter(uint32_t outputId,
                                         uint32_t sectionId) throw () = 0;
  virtual THaiResult SetDecryptionCipher(uint16_t pid, THaiDemuxerCipher cipher,
                                         THaiCryptoBlockMode blockMode,
                                         THaiCryptoResidueMode residueMode)
    throw () = 0;
  virtual THaiResult SetDecryptionKey(uint16_t pid, THaiCryptoKeyParity parity,
                                      bool encryptedKey,
                                      const uint8_t* key, const uint8_t* iv,
                                      uint32_t keyLength) throw () = 0;
  virtual THaiResult SetPrebufferTime(uint32_t duration) throw () = 0;

  virtual void AddObserver(IElement* element,
                           IDemuxerObserver* observer) throw () = 0;
  virtual void RemoveObserver(IElement* element,
                              IDemuxerObserver* observer) throw () = 0;
  virtual void Release() throw () = 0;
};

#endif
