/*
 *--------------------------------------------------------------------
 *
 * IDiscard.h
 *
 * If this interface is implemented by a root-element, streamer core
 * will use it to discard data which can't be flushed the normal way.
 * 
 * Copyright (c) 2006 Motorola Inc. All Rights Reserved.
 *
 *--------------------------------------------------------------------
 */

#ifndef PIPELINE_IDISCARD_H
#define PIPELINE_IDISCARD_H

/** \class IDiscard IDiscard.h "media/streamer/IDiscard.h"

Calling Flush in a source element will normally commit
min(BufferSize,CurrentlyHeldData). The rest of the data is left until
next process call gives the element more buffer space to commit data
to.

When this remaining data is invalid, such as after a state transition
in streamer, if a source element implements this interface it will be
used to tell the element to throw away this data.
*/

class IDiscard
{
public:
  virtual ~IDiscard() {}

  /** This function will ask the element to discard all available data
   *  which can't be commited with the IRoot::Flush.
   */
  virtual void DiscardUncommitable() throw () = 0;
};

#endif
