/*
*---------------------------------------------------------------------------
*
* IDiskReader.h
*
* Disk reader interface for streamer elements.
*
* Copyright (c) 2009 Motorola, Inc. All Rights Reserved.
* Copyright (c) 2011 Motorola Mobility, Inc. All rights reserved.
*
* This program is confidential and proprietary to Motorola Mobility, Inc and
* may not be copied, reproduced, disclosed to others, published or used, in
* whole or in part, without the expressed prior written permission of Motorola
* Mobility, Inc.
*
*---------------------------------------------------------------------------
*/

#ifndef STREAMER_IDISKREADER_H
#define STREAMER_IDISKREADER_H

#include "IElement.h"

#include "hai/IDiskReader.h"

#include "stdint.h"
#include <string>

class IDiskReaderObserver
{
public:
  virtual ~IDiskReaderObserver() { /* Empty */}

  virtual void OnEvent(THaiDiskReaderEvent event, int32_t error) throw () = 0;

  virtual void OnPositionChanged(uint32_t position, int32_t speed,
                                 THaiDiskReaderState state) throw () = 0;

  virtual void OnStreamChanged(uint16_t pid, THaiMediaFormat mediaFormat,
			       const uint8_t* data,
			       uint32_t dataSize) throw () = 0;

  virtual void OnKeyDataChanged(const uint8_t* token, uint32_t tokenSize,
                                const uint8_t* data, uint32_t dataSize)
    throw () = 0;

  virtual void OnEcmChanged(const uint8_t* ecm,
                               uint32_t ecmSize) throw () = 0;
};

class IDiskReader
{
public:
  virtual ~IDiskReader() { /* Empty */ }

  virtual std::string GetResourceName() const throw () = 0;
  virtual std::string GetInstanceName() const throw () = 0;

  virtual THaiResult CreateOutput(uint32_t* outputId) throw () = 0;
  virtual THaiResult SetAssetPath(const char* path) throw () = 0;
  virtual THaiResult SetPlaySpeed(int32_t speed) throw () = 0;
  virtual THaiResult SetPlaySpeedAndPosition(int32_t speed,
                                             uint32_t position) throw () = 0;
  virtual THaiResult SetBytePosition(uint64_t position) throw () = 0;
  virtual THaiResult SetPosition(uint32_t position) throw () = 0;
  virtual THaiResult AcknowledgeEcmActivated() throw () = 0;
  virtual THaiResult ConvertTimeToBytePosition(uint32_t timePosition, 
                                               uint64_t* bytePosition) 
                                               throw () = 0;
  virtual THaiResult ConvertByteToTimePosition(uint64_t bytePosition, 
                                               uint32_t* timePosition) 
                                               throw () = 0;
  virtual THaiResult GetBytePosition(uint64_t* position) throw () = 0;
  virtual THaiResult RequestPositionReport(uint32_t period) throw () = 0;

  virtual void AddObserver(IElement* element,
                           IDiskReaderObserver* observer) throw () = 0;
  virtual void RemoveObserver(IElement* element,
                              IDiskReaderObserver* observer) throw () = 0;
  virtual void Release() throw () = 0;
};

#endif
