/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
*---------------------------------------------------------------------------
*
* IDiskWriter.h
*
* Disk writer interface for streamer elements.
*
* Copyright (c) 2009 Motorola, Inc. All Rights Reserved.
* Copyright (c) 2011 Motorola Mobility, Inc. All rights reserved.
*
* This program is confidential and proprietary to Motorola Mobility, Inc and
* may not be copied, reproduced, disclosed to others, published or used, in
* whole or in part, without the expressed prior written permission of Motorola
* Mobility, Inc.
*
*---------------------------------------------------------------------------
*/

#ifndef STREAMER_IDISKWRITER_H
#define STREAMER_IDISKWRITER_H

#include "IElement.h"

#include "hai/IDiskWriter.h"

#include "stdint.h"
#include <string>

class IDiskWriterObserver
{
public:
  virtual ~IDiskWriterObserver() { /* Empty */ }

  virtual void OnEvent(THaiDiskWriterEvent event, int32_t error) throw () = 0;
};

class IDiskWriter
{
public:
  virtual ~IDiskWriter() { /* Empty */ }

  virtual std::string GetResourceName() const throw () = 0;
  virtual std::string GetInstanceName() const throw () = 0;

  virtual THaiResult ConnectInput(const char * resourceName,
                                  const char * instanceName,
                                  int index) throw () = 0;
  virtual bool IsReadyForEncryptionData() throw () = 0;
  virtual THaiResult Attach(const char* assetPath, uint32_t offset)
    throw () = 0;
  virtual THaiResult Detach(const char* assetPath) throw () = 0;
  virtual THaiResult Run() throw () = 0;
  virtual THaiResult Halt() throw () = 0;

  virtual THaiResult SetIndexType(uint16_t pid,
                                  const THaiDiskWriterIndexType indexType,
                                  const uint8_t* data, uint32_t size)
    throw () = 0;

  virtual THaiResult SetCryptoMode(uint16_t pid, THaiDiskWriterCryptoMode mode)
    throw () = 0;
  virtual THaiResult SetEncryptionData(const uint8_t* token,
                                       uint32_t tokenSize,
                                       THaiCryptoKeyParity parity,
                                       const uint8_t* data,
                                       uint32_t dataSize) throw () = 0;
  virtual void AddObserver(IElement* element,
                           IDiskWriterObserver* observer) throw () = 0;
  virtual void RemoveObserver(IElement* element,
                              IDiskWriterObserver* observer) throw () = 0;
  virtual void Release() throw () = 0;
};

#endif
