/*
 *--------------------------------------------------------------------
 *
 * IElement.h
 *
 * The main interface to all elements.
 * 
 * Copyright (c) 2004 Motorola Inc. All Rights Reserved.
 *
 *--------------------------------------------------------------------
 */

#ifndef PIPELINE_IELEMENT_H
#define PIPELINE_IELEMENT_H

#include "media/streamer/IPump.h"
#include "media/streamer/InterfaceNames.h"

#ifdef PIPELINE_NAMESPACE
namespace Pipeline {
#endif


/** \class IElement IElement.h "media/streamer/IElement.h"
    
    This is the main interface that must be implemented by all
    elements participating in the streamer pipeline.
*/

class IElement
{
protected:
  virtual ~IElement() throw () { }

public:
  /** This function should be implemented to delete the element.
   */
  virtual void Destroy() throw () = 0;


  /** Initializes the element by supplying the interface to the
      pipeline pump. The element may use the pump interface to query
      the environment for other interfaces. When this function returns
      the element must be fully operational.

      \param pump        Pointer to the streamer pipeline pump interface.
      \param dataFormat  The format of the incoming data in the segments or
                         the "" for the source element.
  */
  virtual void Initialize(IPump* pump,
                          const char* dataFormat) throw () = 0;


  /** This function returns a pointer to the named interface and the
      caller casts the returned pointer to the right interface.  The
      element must return NULL if it does not implement the named
      interface. This element must be able to handle calls to this
      function before the element has been initialized. The function
      must return the same pointer if it is called multiple times for
      the same interface name.

      \param interfaceName

              The name of the interface for which the caller requests
              a pointer. For example, the name could be
              "IBlackboardObserver" and the element must return the
              pointer to its blackboard observer interface if it is
              implement, or NULL it it does not.
  */
  virtual void* QueryInterface(const char* interfaceName) throw () = 0;
};


#ifdef PIPELINE_NAMESPACE
}
#endif

#endif
