/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
*---------------------------------------------------------------------------
*
* IFrontend.h
*
* Frontend interface for streamer elements.
*
* Copyright (c) 2009 Motorola, Inc. All Rights Reserved.
*
*---------------------------------------------------------------------------
*/

#ifndef STREAMER_IFRONTEND_H
#define STREAMER_IFRONTEND_H

#include "IElement.h"

#include "hai/IFrontend.h"

#include "stdint.h"
#include <string>

class IFrontendObserver
{
public:
  virtual ~IFrontendObserver() { /* Empty */ }

  virtual void OnStateChanged() throw () = 0;
  virtual void OnEmergency(bool emergency) throw () = 0;
};

class IFrontend
{
public:
  virtual ~IFrontend() { /* Empty */ }

  virtual std::string GetResourceName() const throw () = 0;
  virtual std::string GetInstanceName() const throw () = 0;

  virtual THaiResult GetInfo(THaiFrontend_Info* info) throw () = 0;
  virtual THaiResult TuneDescriptor(const char* descriptor) throw () = 0;
  virtual THaiResult GetTunerDescriptor(char** descriptor) throw () = 0;
  virtual THaiResult Flush() throw () = 0;

  virtual void AddObserver(IElement* element,
                           IFrontendObserver* observer) throw () = 0;
  virtual void RemoveObserver(IElement* element,
                              IFrontendObserver* observer) throw () = 0;
  virtual void Release() throw () = 0;
};

#endif
