/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *---------------------------------------------------------------------------
 *
 * IMediaDecoder.h
 *
 * MediaDecoder interface for streamer elements.
 *
 * Copyright (c) 2009 Motorola, Inc. All Rights Reserved.
 *
 *---------------------------------------------------------------------------
 */

#ifndef STREAMER_IMEDIADECODER_H
#define STREAMER_IMEDIADECODER_H

#include "IElement.h"

#include "hai/IMediaDecoder.h"

#include "stdint.h"
#include <string>

class IMediaDecoderObserver
{
public:
  virtual ~IMediaDecoderObserver() { /* Empty */ }

  virtual void OnEmpty() throw () = 0;
  virtual void OnRunning() throw () = 0;
  virtual void OnInfoChanged() throw () = 0;
  virtual void OnNewPicture() throw () = 0;
};

class IMediaDecoder
{
public:
  virtual ~IMediaDecoder() { /* Empty */ }

  virtual std::string GetResourceName() const throw () = 0;
  virtual std::string GetInstanceName() const throw () = 0;

  virtual THaiResult Reset() throw () = 0;
  virtual THaiResult ConnectInput(const char * resourceName,
                                  const char * instanceName, int index)
    throw () = 0;
  virtual THaiResult GetFormat(void* descriptor,
                               uint32_t size) throw () = 0;
  virtual THaiResult SetFormat(const void* descriptor,
                               uint32_t size) throw () = 0;
  virtual THaiResult GetInfo(void* info,
                             uint32_t size) throw () = 0;
  virtual THaiResult Halt() throw () = 0;
  virtual THaiResult Run() throw () = 0;
  virtual THaiResult GetState(IHaiMediaDecoder_TState* state) throw () = 0;
  virtual THaiResult SetOption(int id, const char* value) throw () = 0;
  virtual THaiResult GetOption(int id, int bufferSize, char* value)
    throw () = 0;

  virtual void AddObserver(IElement* element,
                           IMediaDecoderObserver* observer) throw () = 0;
  virtual void RemoveObserver(IElement* element,
                              IMediaDecoderObserver* observer) throw () = 0;
  virtual void Release() throw () = 0;
};

#endif
