/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *---------------------------------------------------------------------------
 *
 * IMediaTranscoder.h
 *
 * MediaTranscoder interface for streamer elements.
 *
 * Copyright (c) 2009 Motorola, Inc. All Rights Reserved.
 *
 *---------------------------------------------------------------------------
 */

#ifndef STREAMER_IMEDIATRANSCODER_H
#define STREAMER_IMEDIATRANSCODER_H

#include "IElement.h"

#include "hai/IMediaTranscoder.h"

#include "stdint.h"

#include <string>

class IMediaTranscoderObserver
{
public:
  virtual ~IMediaTranscoderObserver() { /* Empty */ }

  virtual void OnStateChanged(THaiMediaTranscoderState state) throw () = 0;
};

class IMediaTranscoder
{
public:
  virtual ~IMediaTranscoder() { /* Empty */ }

  virtual std::string GetResourceName() const throw () = 0;
  virtual std::string GetInstanceName() const throw () = 0;

  virtual THaiResult ConnectInput(const char* resourceName,
                                  const char* instanceName,
                                  uint32_t index) throw () = 0;
  virtual THaiResult Reset() throw () = 0;
  virtual THaiResult Halt() throw () = 0;
  virtual THaiResult Run() throw () = 0;
  virtual THaiResult GetConfig(THaiMediaTranscoderConfig& config) throw () = 0;
  virtual THaiResult SetConfig(const THaiMediaTranscoderConfig& config) throw () = 0;
  virtual THaiResult OpenOutputPipe(uint32_t outputId,
                                    const char* socketName) throw () = 0;
  virtual THaiResult CloseOutputPipe(uint32_t outputId) throw () = 0;
  virtual THaiResult CreateOutput(uint32_t* outputId) throw () = 0;
  virtual THaiResult DestroyOutput(uint32_t outputId) throw () = 0;

  virtual THaiResult SetPassthroughPids(const uint16_t* config,
                                        const uint32_t pidsSize) throw () = 0;

  virtual void AddObserver(IElement* element,
                           IMediaTranscoderObserver* observer) throw () = 0;
  virtual void RemoveObserver(IElement* element,
                              IMediaTranscoderObserver* observer) throw () = 0;
  virtual void Release() throw () = 0;
};

#endif
