/*
 *--------------------------------------------------------------------
 *
 * IOutputPad.h
 *
 * This is the interface to the postpone pad.
 *
 * Copyright (c) 2004 Motorola Inc. All Rights Reserved.
 *
 *--------------------------------------------------------------------
 */

#ifndef PIPELINE_IPOSTPONEPAD_H
#define PIPELINE_IPOSTPONEPAD_H

#include <stdint.h>

#ifdef PIPELINE_NAMESPACE
namespace Pipeline {
#endif


class TMetadata;


/** \class IPostponePad IPostponePad.h "media/streamer/IPostponePad.h"

    This is the interface used by an element to postpone segments to
    itself. The postponed segment will be merged with the next segment
    received from the element before it.

    The postpone pad can only be used by the element immediately
    following the source element.
*/

class IPostponePad
{
public:
  virtual ~IPostponePad() {}

  /** This function postpones a segment. It will be processed again
      later when it has been merged with another segment.

      A segment that is postponed must be the last part of the
      incoming segment to the elment, otherwise it is not possible to
      merge the segment with the next segment committed by the source
      element. Also, the postponed segment must be part of an incoming
      segment that has no framing, i.e. it must have a single frame
      with the whole frame in focus.
      
      \param position      Pointer to the starting position of the segment.
      \param frameCount    The number of frames in the segment.
      \param frameSize     The size of each frame.
  */
  virtual void PostponeSegment(uint8_t* position,
                               uint32_t frameCount,
                               uint32_t frameSize) throw () = 0;
};


#ifdef PIPELINE_NAMESPACE
}
#endif

#endif
