/*
 *--------------------------------------------------------------------
 *
 * IRoot.h
 *
 * This is the interface to root elements.
 * 
 * Copyright (c) 2005 Motorola Inc. All Rights Reserved.
 *
 *--------------------------------------------------------------------
 */

#ifndef PIPELINE_IROOT_H
#define PIPELINE_IROOT_H

#include "media/streamer/IStreamerTypes.h"

#include <stdint.h>

#ifdef PIPELINE_NAMESPACE
namespace Pipeline {
#endif


/** \class IRoot IRoot.h "media/streamer/IRoot.h"

    This is the interface that must be implemented by all root
    elements (including source elements) participating in a
    pipeline.
*/

class IRoot
{
public:
  virtual ~IRoot() {}

  /** The streamer core uses this function to ask the source element
      how much buffer space it requires in each call to the Process()
      function.
      
      \return           The size of the buffer to be supplied in each call to
                        the Process() function.
      \see Process

  */
  virtual uint32_t GetBufferRequirement() throw () = 0;


  /** This function is called to grant the element access to a piece
      of buffer space. The element may commit segments of data
      covering the buffer space immediately in the call to this
      function, or later on when handling an event.

      Process is not called again until all buffer space has been
      committed or released. Data must be committed from the beginning
      of the buffer. The element should commit data in the buffer
      until the remaining buffer space is too small for an additional
      chunk of data. Such unused buffer space must be released.

      \param buffer       A pointer to the available buffer space.
      \param size         The length of the buffer space.
  */
  virtual void Process(uint8_t* buffer, uint32_t size) throw () = 0;


  /** This function is called to force the element to commit all data
      that it has at its disposal at the time of the call. The
      remaining buffer space must be released.
  */
  virtual void Flush() throw () = 0;
};


#ifdef PIPELINE_NAMESPACE
}
#endif

#endif
