/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *---------------------------------------------------------------------------
 *
 * ISocketSink.h
 *
 * SocketSink interface for streamer elements.
 *
 * Copyright (c) 2009 Motorola, Inc. All Rights Reserved.
 *
 *---------------------------------------------------------------------------
 */

#ifndef STREAMER_ISOCKETSINK_H
#define STREAMER_ISOCKETSINK_H

#include "IElement.h"

#include "hai/ISocketSink.h"

#include "stdint.h"
#include <string>

class ISocketSinkObserver
{
public:
  virtual ~ISocketSinkObserver() { /* Empty */ }

  virtual void OnEvent(THaiSocketSinkEvent event,
                       int32_t code) throw () = 0;
};


class ISocketSink
{
public:
  virtual ~ISocketSink() { /* Empty */ }

  virtual std::string GetResourceName() const throw () = 0;
  virtual std::string GetInstanceName() const throw () = 0;

  virtual THaiResult ConnectInput(const char * resourceName,
                                  const char * instanceName,
                                  int index) throw () = 0;
  virtual THaiResult SetLocalAddress(const char* addr) throw () = 0;
  virtual THaiResult SetMode(THaiSocketSinkMode mode) throw () = 0;
  virtual THaiResult SetParameter(int id, int value) throw () = 0;
  virtual THaiResult Run() throw () = 0;
  virtual THaiResult Halt() throw () = 0;

  virtual void AddObserver(IElement* element,
                           ISocketSinkObserver* observer) throw () = 0;
  virtual void RemoveObserver(IElement* element,
                              ISocketSinkObserver* observer) throw () = 0;
  virtual void Release() throw () = 0;
};

#endif
