/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *---------------------------------------------------------------------------
 *
 * IStreamInput.h
 *
 * StreamInput interface for streamer elements.
 *
 * Copyright (c) 2009 Motorola, Inc. All Rights Reserved.
 * Copyright (c) 2011 Motorola Mobility, Inc. All rights reserved.
 *
 * This program is confidential and proprietary to Motorola Mobility, Inc and
 * may not be copied, reproduced, disclosed to others, published or used, in
 * whole or in part, without the expressed prior written permission of Motorola
 * Mobility, Inc.
 *
 *---------------------------------------------------------------------------
 */

#ifndef STREAMER_ISTREAMINPUT_H
#define STREAMER_ISTREAMINPUT_H

#include "IElement.h"

#include "hai/IStreamInput.h"

#include "stdint.h"
#include <string>

class IStreamInputObserver
{
public:
  virtual ~IStreamInputObserver() { /* Empty */ }

  virtual void OnConsumed() throw () = 0;
};

class IStreamInput
{
public:
  enum { INVALID_OFFSET = 0xffffffff };

  virtual ~IStreamInput() { /* Empty */ }

  virtual std::string GetResourceName() const throw () = 0;
  virtual std::string GetInstanceName() const throw () = 0;

  virtual THaiResult SetBuffer(uint32_t shmKey) throw () = 0;
  virtual THaiResult SetInputClocked(bool enable) throw () = 0;
  virtual THaiResult Push(uint32_t offset, uint32_t frameCount,
                          uint32_t frameSize, uint32_t focusOffset,
                          uint32_t focusSize, uint32_t flags) throw () = 0;
  virtual THaiResult Pop(uint32_t* offset) throw () = 0;
  virtual THaiResult Flush() throw () = 0;

  virtual void AddObserver(IElement* element,
                           IStreamInputObserver* observer) throw () = 0;
  virtual void RemoveObserver(IElement* element,
                              IStreamInputObserver* observer) throw () = 0;
  virtual void Release() throw () = 0;
};

#endif
