/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
*---------------------------------------------------------------------------
*
* ITimeStamp.h
*
* Timestamp interface for streamer elements.
*
* Copyright (c) 2009 Motorola, Inc. All Rights Reserved.
*
*---------------------------------------------------------------------------
*/

#ifndef STREAMER_ITIMESTAMP_H
#define STREAMER_ITIMESTAMP_H

#include "IElement.h"

#include "hai/ITimeStamp.h"

#include "stdint.h"
#include <string>

class ITimeStampObserver
{
public:
  virtual ~ITimeStampObserver() { /* Empty */ }

  virtual void OnTimeStamp(int token) throw () = 0;
};

class ITimeStamp
{
public:
  virtual ~ITimeStamp() { /* Empty */ }

  virtual std::string GetResourceName() const throw () = 0;
  virtual std::string GetInstanceName() const throw () = 0;

  virtual THaiResult Add(uint64_t pts, int token) throw () = 0;
  virtual THaiResult Clear() throw () = 0;
  virtual THaiResult GetStc(uint64_t* stc) throw () = 0;
  virtual THaiResult GetPts(uint64_t* pts) throw () = 0;
  virtual THaiResult AdjustStc(int ppm, int* actualPpm) throw () = 0;
  virtual THaiResult GetPcr(uint64_t* pcr) throw () = 0;

  virtual void AddObserver(IElement* element,
                           ITimeStampObserver* observer) throw () = 0;
  virtual void RemoveObserver(IElement* element,
                              ITimeStampObserver* observer) throw () = 0;
  virtual void Release() throw () = 0;
};

#endif
