// -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
//-----------------------------------------------------------------------------
//
// Definitions of statistics parameter names in the Blackboard.
//
// Copyright (c) 2006 Motorola Inc. All Rights Reserved.
//
//-----------------------------------------------------------------------------

// IMPORTANT!
// This file is parsed by a ./extractParameters.py to create documentation

#ifndef STATISTICSPARAMETERNAMES_H
#define STATISTICSPARAMETERNAMES_H

/**
 * Used for reseting all statistics.
 *
 * A value of 1 indicate that the statisics should be reset.
 * Values:    0, 1
 * Set by:    Any.
 */
#define STATISTICS_RESET "StatisticsReset"

/**
 * The URL to the stream.
 *
 * Values:    URL
 * Set by:    Statistics element.
 */
#define STATISTICS_URL "StatisticsUrl"

/**
 * The total playing time from open to now.
 *
 * Values:    0 - XXX ms
 * Set by:    Statistics element.
 */
#define STATISTICS_TIME_TOTAL "StatisticsTimeTotal"

/**
 * The total playing time in state playing.
 *
 * Values:    0 - XXX ms
 * Set by:    Statistics element.
 */
#define STATISTICS_TIME_PLAYING "StatisticsTimePlaying"

/**
 * The number of buffer overruns.
 *
 * Values:    0 - XXX
 * Set by:    Statistics element.
 */
#define STATISTICS_BUFFER_OVERRUN "StatisticsBufferOverrun"

/**
 * The number of buffer underruns.
 *
 * Values:    0 - XXX
 * Set by:    Statistics element.
 */
#define STATISTICS_BUFFER_UNDERRUN "StatisticsBufferUnderrun"

/**
 * The number of TS packets seen by the streamer.
 *
 * Values:    0 - XXX
 * Set by:    Statistics element.
 */
#define STATISTICS_TS_TOTAL_PACKETS "StatisticsTsTotalPackets"

/**
 * The number of duplicate TS packets seen by the streamer.
 *
 * Values:    0 - XXX
 * Set by:    Statistics element.
 */
#define STATISTICS_TS_DUPLICATE_PACKETS "StatisticsTsDuplicatePackets"

/**
 * The number of lost TS packets seen by the streamer.
 *
 * Values:    0 - XXX
 * Set by:    Statistics element.
 */
#define STATISTICS_TS_LOST_PACKETS "StatisticsTsLostPackets"

/**
 * The number of corrupted TS packets seen by the streamer.
 * Should probably always be 0.
 *
 * Values:    0 - XXX
 * Set by:    Statistics element.
 */
#define STATISTICS_TS_ERROR_PACKETS "StatisticsTsErrorPackets"

/**
 * The number of bytes seen by the streamer including TS headers.
 *
 * Values:    0 - XXX Bytes
 * Set by:    Statistics element.
 */
#define STATISTICS_TS_TOTAL_BYTES "StatisticsTsTotalBytes"

/**
 * The number of bytes seen by the streamer when playing.
 * This value will have the same value as STATISTICS_TS_TOTAL_BYTES.
 *
 * Values:    0 - XXX Bytes
 * Set by:    Statistics element.
 */
#define STATISTICS_TS_TOTAL_BYTES_PLAYED "StatisticsTsTotalBytesPlayed"

/**
 * The number of a/v unsync seen by the streamer when playing.
 * This value will have the same value as STATISTICS_TS_AV_OUT_OF_SYNCS.
 *
 * Values:    0 - XXX
 * Set by:    Statistics element.
 */
#define STATISTICS_TS_AV_OUT_OF_SYNCS "StatisticsTsAVOutOfSyncs"

#endif
