/*
 *--------------------------------------------------------------------
 *
 * unix-wrappers/TTimeVal.h
 *
 * A wrapper for the UNIX timeval structure, as specified in sys/time.h:
 *
 *  struct timeval {
 *    long tv_sec;   
 *    long tv_usec;  
 *  };
 *
 * This class is not complete; add what you think is needed.
 *
 * Copyright (c) 2002 Kreatel Communications AB. All Rights Reserved.
 *
 *--------------------------------------------------------------------
 */

#ifndef UNIXWRAPPERS_TTIMEVAL_H
#define UNIXWRAPPERS_TTIMEVAL_H

#include <sys/time.h>
#include <unistd.h>
#include <stdint.h>

class TTimeRange;

class TTimeVal
{
protected:
  struct timeval Timeval;

  void Set(uint32_t sec, uint32_t usec);

public:
  TTimeVal();
  TTimeVal(uint32_t sec, uint32_t usec);
  TTimeVal(const timeval& other);

  TTimeVal& operator =(const timeval& other);
  void Clear();

  operator timeval() const;
  uint32_t GetSeconds() const;
  uint32_t GetMicros() const;
  timespec GetTimeSpec() const;
  bool IsZero() const;

  bool operator ==(const TTimeVal& other) const;
  bool operator !=(const TTimeVal& other) const;
  bool operator <(const TTimeVal& other) const;
  bool operator >(const TTimeVal& other) const;

  TTimeVal& operator +=(const TTimeRange& other);

  TTimeRange operator -(const TTimeVal& other);
};


class TSystemTime : public TTimeVal
{
public:
  TSystemTime();
};


class TTimeRange : public TTimeVal
{
public:
  TTimeRange(uint32_t sec, uint32_t usec);
};


class TMillisecRange : public TTimeRange
{
public:
  TMillisecRange(uint32_t msec);
};




inline TTimeVal::TTimeVal()
{
  
}

inline TTimeVal::TTimeVal(uint32_t s, uint32_t us)
{
  Set(s, us);
}

inline TTimeVal::TTimeVal(const timeval& other)
{
  *this = other;
}

inline TTimeVal& TTimeVal::operator =(const timeval& other)
{
  Set(other.tv_sec, other.tv_usec);
  return *this;
}

inline void TTimeVal::Clear()
{
  Timeval.tv_sec = 0;
  Timeval.tv_usec = 0;
}

inline TTimeVal::operator timeval() const
{
  return Timeval;
}

inline uint32_t TTimeVal::GetSeconds() const
{
  return Timeval.tv_sec;
}

inline uint32_t TTimeVal::GetMicros() const
{
  return Timeval.tv_usec;
}

inline timespec TTimeVal::GetTimeSpec() const
{
  
  struct timespec ts = { Timeval.tv_sec, Timeval.tv_usec * 1000 };
  return ts;
}

inline bool TTimeVal::IsZero() const
{
  return Timeval.tv_sec == 0 && Timeval.tv_usec == 0;
}

inline bool TTimeVal::operator ==(const TTimeVal& other) const
{
  return timercmp(&Timeval, &other.Timeval, ==);
}

inline bool TTimeVal::operator !=(const TTimeVal& other) const
{
  return timercmp(&Timeval, &other.Timeval, !=);
}

inline bool TTimeVal::operator <(const TTimeVal& other) const
{
  return timercmp(&Timeval, &other.Timeval, <);
}

inline bool TTimeVal::operator >(const TTimeVal& other) const
{
  return timercmp(&Timeval, &other.Timeval, >);
}

inline TTimeVal& TTimeVal::operator +=(const TTimeRange& other)
{
  Set(Timeval.tv_sec + other.Timeval.tv_sec,
      Timeval.tv_usec + other.Timeval.tv_usec);
  return *this;
}

inline TSystemTime::TSystemTime()
{
  ::gettimeofday(&Timeval, NULL);
}

inline TTimeRange::TTimeRange(uint32_t s, uint32_t us)
  : TTimeVal(s, us)
{
  
}

inline TMillisecRange::TMillisecRange(uint32_t ms)
  : TTimeRange(ms / 1000, (ms % 1000) * 1000)
{
  
}

#endif
