







#ifndef UTILS_KFILESYSTEM_H
#define UTILS_KFILESYSTEM_H

#include <string>

#include <dirent.h>
#include <stdint.h>
#include <sys/types.h>

namespace KFileSystem
{
  class TDirIterator
  {
  private:
    DIR* Dir;
    struct dirent* DirEnt;
    std::string Path;

    void Next();

  public:
    TDirIterator();
    TDirIterator(const TDirIterator& other);
    TDirIterator(const std::string& path);
    ~TDirIterator();
    TDirIterator& operator++();
    TDirIterator operator++(int);
    bool operator==(const TDirIterator& other) const;
    bool operator!=(const TDirIterator& other) const;
    static const TDirIterator& End();
    std::string GetFullPath() const;
    std::string GetName() const;
  };

  std::string GetError();
  bool CreateDirectory(const std::string& path, mode_t mode = 0755);
  bool CreateSymbolicLink(const std::string& oldpath,
                          const std::string& newpath);
  bool ChangePermissions(const std::string& path, mode_t mode);
  bool GetPermissions(const std::string& path, mode_t& mode);
  mode_t SetCreationMask(mode_t mode);
  bool Move(const std::string& oldpath, const std::string& newpath);
  bool Remove(const std::string& path);
  bool RemoveDirectory(const std::string& path);
  bool RemoveDirectoryRecursive(const std::string& path);
  bool Exists(const std::string& path);
  bool IsDirectory(const std::string& path);
  bool IsRegular(const std::string& path);
  bool IsSymbolicLink(const std::string& path);
  bool IsFifo(const std::string& path);
  bool IsSocket(const std::string& path);
  bool IsSame(const std::string& path1, const std::string& path2);
  uint64_t GetSize(const std::string& path);
  bool RealPath(const std::string& path, std::string& realPath);
  bool ChangeDirectory(const std::string& path);
  std::string GetCurrentDirectory();
  std::string GetDirName(const std::string& path);
  std::string GetBaseName(const std::string& path);
  bool GetAvailableSpace(const std::string& path, uint64_t& avail);
  bool IsDirEmpty(const std::string& path);
}

#endif
