






#ifndef UTILS_TALIGNEDBUFFER_H
#define UTILS_TALIGNEDBUFFER_H

#include <cassert>
#include <cerrno>
#include <cstdlib>
#include <stdint.h>

class TAlignedBuffer
{
private:
  const size_t Alignment;
  void* Buffer;

public:
  inline TAlignedBuffer(size_t alignment);
  ~TAlignedBuffer() { Deallocate(); }

  inline bool Allocate(size_t size);
  inline void Deallocate();

  inline uint8_t* GetBuffer();
  operator uint8_t*() { return GetBuffer(); }
};

inline TAlignedBuffer::TAlignedBuffer(size_t alignment)
  : Alignment(alignment),
    Buffer(NULL)
{
  
}

inline bool TAlignedBuffer::Allocate(size_t size)
{
  assert(Buffer == NULL);

  int res = ::posix_memalign(&Buffer, Alignment, size);
  if (res == 0) {
    return true;
  }
  else {
    errno = res;
    return false;
  }
}

inline void TAlignedBuffer::Deallocate()
{
  if (Buffer != NULL) {
    ::free(Buffer);
    Buffer = NULL;
  }
}

inline uint8_t* TAlignedBuffer::GetBuffer()
{
  return reinterpret_cast<uint8_t*>(Buffer);
}

#endif
