







#ifndef UTILS_TBITSTREAM_H
#define UTILS_TBITSTREAM_H

#include "base/TException.h"

#include <stdint.h>

class TBitStream
{
private:
  const uint8_t* Bytes;
  uint32_t NumBytes;
  uint32_t ByteIndex;
  uint8_t BitIndex;

public:
  TBitStream(const uint8_t* bytes, uint32_t numBytes);

  void ByteAlign() throw (TException);

  uint32_t GetBits(uint8_t numBits) throw (TException);
  uint32_t PeekBits(uint8_t numBits) throw (TException);
  void AssertBits(uint8_t numBits, uint32_t value) throw (TException);

  uint32_t GetByteIndex() const throw ();
  void SkipBytes(uint32_t bytes) throw (TException);
  uint32_t BitsLeft() const throw ();
  const uint8_t* GetPointer() const throw ();
  uint16_t GetZeroBits(bool consumeFirstBitThatEqualsOne) throw (TException);
};

inline uint32_t TBitStream::GetByteIndex() const throw ()
{
  return ByteIndex;
};

inline uint32_t TBitStream::BitsLeft() const throw ()
{
  return (NumBytes - ByteIndex) * 8 - BitIndex;
}

inline const uint8_t* TBitStream::GetPointer() const throw ()
{
  return Bytes + GetByteIndex();
}

#endif
