






#ifndef UTILS_TBITRATECALCULATOR_H
#define UTILS_TBITRATECALCULATOR_H

#include "unix-wrappers/TTickCounter.h"

#include <sys/times.h>
#include <stdint.h>

template <typename T> class TBasicBitrateCalculator;

typedef TBasicBitrateCalculator<TTickCounter> TBitrateCalculator;

template <typename T>
class TBasicBitrateCalculator
{
private:
  uint64_t Bytes;
  uint32_t Byterate;
  uint32_t MeasureWindow;

  uint32_t Measure();

protected:
  T TickCounter;

public:
  inline TBasicBitrateCalculator(unsigned int measureWindow);

  inline void Reset(unsigned int measureWindow = 0);
  inline void Feed(unsigned int bytes) { Bytes += bytes; }

  
  inline void SetMeasureWindow(unsigned int measureWindow);

  
  inline uint32_t GetBitrate() { return Measure() * 8; }

  
  inline uint32_t GetByterate() { return Measure(); }
};

template <typename T>
inline uint32_t TBasicBitrateCalculator<T>::Measure()
{
  uint32_t timeSinceLastMeasure = TickCounter.Measure();
  if (timeSinceLastMeasure > MeasureWindow) {
    Byterate = (Bytes * 1000) / timeSinceLastMeasure;
    Bytes = 0;
    TickCounter.Reset();
  }
  return Byterate;
}

template <typename T>
inline
TBasicBitrateCalculator<T>::TBasicBitrateCalculator(uint32_t measureWindow)
  : Bytes(0),
    Byterate(0)
{
  SetMeasureWindow(measureWindow);
}

template <typename T>
inline void TBasicBitrateCalculator<T>::Reset(uint32_t measureWindow)
{
  Bytes = 0;
  Byterate = 0;
  TickCounter.Reset();
  if (measureWindow != 0) {
    SetMeasureWindow(measureWindow);
  }
}

template <typename T>
inline void
TBasicBitrateCalculator<T>::SetMeasureWindow(uint32_t measureWindow)
{
  MeasureWindow = measureWindow;
}

#endif
