






#ifndef UTILS_TDISKUSAGESTATUS_H
#define UTILS_TDISKUSAGESTATUS_H

#include <stdint.h>
#include <string>
#include <deque>

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>

class TDiskUsageStatus
{
private:
  class TFileHandle
  {
    int Fd;
    std::string Path;

  public:
    TFileHandle(int fd);
    TFileHandle(const std::string& path);
    int AcquireStat(struct stat64* fileStat) const;
    const std::string& GetPath() const;
  };

  typedef std::deque<TFileHandle> TFileHandleSeq;

  std::string FsPath;

  bool ReservedFilesFullyAllocated;
  uint64_t ReservedFilesAllocatedSize;
  uint64_t AllowedFreeDiskSize;
  double AllowedDiskUsageFraction;
  TFileHandleSeq ReservedFiles;

public:
  TDiskUsageStatus(const std::string& fsPath);

  void AddReservedFile(const std::string& path);
  void AddReservedFile(int fd);
  void ClearReservedFiles();
  void SetAllowedDiskUsageFraction(double allowedDiskUsageFraction);
  bool AreReservedFilesFullyAllocated() const;
  uint64_t GetReservedFilesAllocatedSize() const;
  uint64_t GetAllowedFreeDiskSize() const;
  void DecreaseAllowedFreeDiskSize(uint64_t written);
  void Update();

private:
  int AcquireFsStat(struct statvfs* fsStat) const;
};

#endif
