/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *---------------------------------------------------------------------------
 *
 * TFileIterator.h
 *
 * A class for iterating over the files located in a directory of the
 * file system.
 *
 * Copyright (c) 2003 Kreatel Communications AB. All Rights Reserved.
 *
 *---------------------------------------------------------------------------
 */

#ifndef UTILS_TFILEITERATOR_H
#define UTILS_TFILEITERATOR_H

#include <linux/stddef.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <dirent.h>


class TFileIterator
{
private:
  DIR* Dir;
  const char* DirName;
  struct dirent* Entry;
  struct stat Stat;
  bool StatValid;

  void UpdateStat();
  explicit TFileIterator();

public:
  TFileIterator(const char* dirname);
  ~TFileIterator();

  TFileIterator& operator ++();
  const char* operator *() const;
  bool operator ==(const TFileIterator& other) const;
  bool operator !=(const TFileIterator& other) const;

  bool IsDirectory();
  bool IsFile();
  bool IsEnd() const;
  uid_t GetOwner();
  TFileIterator GetSubdirBegin() const;
  TFileIterator GetSubdirEnd() const;
};


inline TFileIterator::TFileIterator()
  : Dir(NULL),
    DirName(NULL),
    Entry(NULL)
{
  
}

inline const char* TFileIterator::operator *() const
{
  return &(Entry->d_name[0]);
}

inline bool TFileIterator::operator ==(const TFileIterator& other) const
{
  return Dir == other.Dir;
}

inline bool TFileIterator::operator !=(const TFileIterator& other) const
{
  return !(*this == other);
}

inline bool TFileIterator::IsDirectory()
{
  UpdateStat();
  return S_ISDIR(Stat.st_mode);
}

inline bool TFileIterator::IsFile()
{
  UpdateStat();
  return S_ISREG(Stat.st_mode);
}

inline bool TFileIterator::IsEnd() const
{
  return Dir == NULL;
}

inline uid_t TFileIterator::GetOwner()
{
  UpdateStat();
  return Stat.st_uid;
}

inline TFileIterator TFileIterator::GetSubdirBegin() const
{
  return TFileIterator(Entry->d_name);
}

inline TFileIterator TFileIterator::GetSubdirEnd() const
{
  return TFileIterator();
}

#endif
