/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *---------------------------------------------------------------------------
 *
 * TFileMapper.h
 *
 * This class opens and maps a file read-only into the process
 * memory space.
 *
 * Copyright (c) 2003 Kreatel Communications AB. All Rights Reserved.
 *
 *---------------------------------------------------------------------------
 */

#ifndef UTILS_TFILEMAPPER_H
#define UTILS_TFILEMAPPER_H

#include "base/TException.h"

#include <stdint.h>

#include <string>

class TFileMapper
{
private:
  bool ApplyAdvisoryLock;
  int Fd;
  void* Buffer;
  size_t Size;
  
  void ThrowOpenError(const char* filename) const throw (TException);

  
  TFileMapper(const TFileMapper& other) throw ();
  TFileMapper& operator=(const TFileMapper& other);
  
public:
  TFileMapper(bool applyAdvisoryLock = false) throw ();
  TFileMapper(const char* filename, bool applyAdvisoryLock = false)
    throw (TException);
  ~TFileMapper();

  bool TryOpen(const char* filename) throw ();
  void Open(const char* filename) throw (TException);
  void Close() throw ();

  
  
  
  const char* GetBuffer() const;
  void GetString(std::string& str) const;

  uint32_t GetSize() const;
};

inline TFileMapper::~TFileMapper()
{
  Close();
}

inline const char* TFileMapper::GetBuffer() const
{
  return reinterpret_cast<const char*>(Buffer);
}

inline uint32_t TFileMapper::GetSize() const
{
  return Size;
}

#endif
