









#ifndef UTILS_TMTDDEVICE_H
#define UTILS_TMTDDEVICE_H

#include <mtd/mtd-user.h>
#include <vector>

class TMtdDeviceFile {
private:
  uint32_t Position;
  int Fd;

  class TEraseBlock
  {
  private:
    struct mtd_info_user MtdInfo;
    struct erase_info_user EraseInfo;
    std::vector<uint8_t> BlockBuffer;
    int BlockNumber;
    bool DirtyFlag;
    int Fd;
  public:
    TEraseBlock() throw ();
    ~TEraseBlock() throw ();
    bool Initialize(int fd) throw ();
    bool Erase(int num) throw ();
    bool Load(int num) throw ();
    bool Save() throw ();
    void Set(off_t start, void* src, size_t count) throw ();
    void Get(off_t start, void* des, size_t count) throw ();
    size_t GetTotalSize() const throw () {
      return MtdInfo.size;
    }
    size_t GetEraseSize() const throw () {
      return MtdInfo.erasesize;
    }
    bool IsDirty() const throw () {
      return DirtyFlag && BlockNumber >= 0;
    }
    int GetNumber() throw () {
      return BlockNumber;
    }
  } EraseBlock;

 public:
  TMtdDeviceFile() throw ();
  ~TMtdDeviceFile() throw ();

  bool SetFileDescriptor(int fd) throw ();
  bool Open(const char* label) throw ();
  off_t Seek(off_t position) throw ();
  ssize_t Write(const void* buffer, size_t count) throw ();
  ssize_t Read(void* buffer, size_t count) throw ();
  void Flush() throw ();
  void Close() throw ();
  off_t GetPosition() throw ();
  size_t GetSize() throw ();
};


#endif
