









#ifndef UTILS_TSHA1CHECKSUM_H
#define UTILS_TSHA1CHECKSUM_H

#include "secbin/secbincommon.h"

#include <stdint.h>

#include "base/NonCopyable.h"

class TPartialChecksum
{
  NON_COPYABLE(TPartialChecksum);
public:
  enum {
    CHECKSUM_SIZE = 20 
  };

private:
  uint32_t DataSize;
  uint32_t DataLimit;
  bool FinalizeFlag;

  uint8_t Checksum[CHECKSUM_SIZE];
  sha1_state_t Sha1State;
 public:
  TPartialChecksum() throw ();
  void Initialize(uint32_t limit = 0) throw ();
  bool Update(const uint8_t* block, uint32_t size) throw ();
  bool Finalize() throw ();
  bool Equals(const uint8_t* checksum) const throw ();
  bool Get(uint8_t* buffer, uint32_t& size) const throw ();

  
  bool IsComplete() const throw () {
    return FinalizeFlag || (DataSize != 0 && DataSize == DataLimit);
  }
  const uint8_t* Get() const throw () {
    return FinalizeFlag ? Checksum : 0;
  }
};


#endif
