







#ifndef UTILS_TRECT_H
#define UTILS_TRECT_H

#include <cstdio>
#include <iosfwd>
#include <vector>

class TRect
{
public:
  int X;
  int Y;
  int Width;
  int Height;
  
  TRect() throw ();
  TRect(const TRect& other) throw ();
  TRect(int x, int y, int width, int height) throw ();

  bool operator ==(const TRect& other) const;
  bool operator !=(const TRect& other) const;
  void Transpose(int deltaX, int deltaY);
  void ScaleArea(float scaleX, float scaleY);
  void ScalePosition(float scaleX, float scaleY);
  TRect Intersection(const TRect& other) const;
  TRect GetBoundingRect(const TRect& other) const;
  void Subtract(const TRect& other, std::vector<TRect>& result) const;
  bool IsEmpty() const;
  bool IsInside(const TRect& other) const;
  static TRect Transform(const TRect& to, const TRect& from, const TRect& in);
  static bool Clip(const TRect& dstClip, const TRect& src, const TRect& dst,
                   TRect& clippedSrc, TRect& clippedDst);

  int DebugPrint(char* buffer, size_t size) const throw ();
  friend std::ostream& operator <<(std::ostream& s, const TRect& rect);
};

inline TRect::TRect() throw ()
  : X(0),
    Y(0),
    Width(0),
    Height(0)
{
  
}

inline TRect::TRect(const TRect& other) throw ()
{
  *this = other;
}

inline TRect::TRect(int x, int y, int width, int height) throw ()
  : X(x),
    Y(y),
    Width(std::max(0, width)),
    Height(std::max(0, height))
{
  
}

inline bool TRect::operator ==(const TRect& other) const
{
  return X == other.X && Y == other.Y
    && Width == other.Width && Height == other.Height;
}

inline bool TRect::operator !=(const TRect& other) const
{
  return !(*this == other);
}

inline bool TRect::IsEmpty() const
{
  return Width == 0 || Height == 0;
}

inline int TRect::DebugPrint(char* buffer, size_t size) const throw ()
{
  return ::snprintf(buffer, size, "{%d, %d, %d, %d}", X, Y, Width, Height);
}

#endif
