











#ifndef UTILS_TSIMPLECACHE_H
#define UTILS_TSIMPLECACHE_H

#include <cassert>
#include <map>
#include <list>

template <class Key, class Item>
class TSimpleCache
{
private:
  const unsigned int SIZE;
  std::map<Key, Item> Cache;
  std::list<Key> Fifo;

public:
  typedef std::map<Key, Item> TCache;
  typedef std::list<Key> TKey;

  TSimpleCache(const unsigned int& cacheSize) :
    SIZE(cacheSize)
  {
  }

  ~TSimpleCache() {}

  bool Contains(const Key& id) const
  {
    return (Cache.find(id) != Cache.end());
  }

  void Remove (const Key& id) 
  {
    Cache.erase(id);
    Fifo.remove(id);
  }

  /**
     Get or add an item.
     Cache must not grow bigger than SIZE.
     Else remove oldest.
   */
  Item& operator[] (const Key& id)
  {
    bool contains = Contains(id);
    if (Fifo.size() >= SIZE && !contains) {
      Cache.erase(Fifo.front());
      Fifo.pop_front();
    }
    if (!contains) {
      Fifo.push_back(id);
    }
    return Cache[id];
  }

  size_t GetSize() const
  {
    assert(Cache.size() == Fifo.size());
    return Cache.size();
  }

  
  const TKey& GetKeys() const {
    return Fifo;
  }

  const TCache& GetCache() const {
    return Cache;
  }
  
  
  template <class Predicate>
  void RemoveIfItem(Predicate pred, const Item& otherItem) {
    typename TCache::iterator it;
    for (it = Cache.begin(); it != Cache.end();) {
      if (pred(it->second, otherItem)) {
        Fifo.remove(it->first);
        Cache.erase(it++);
      }
      else {
        ++it;
      }
    }
  }
};

#endif
