










#ifndef UTILS_TSTOPWATCH_H
#define UTILS_TSTOPWATCH_H

#include <sys/time.h>
#include <stdint.h>

#include <cstdlib>
#include <cstdio>
#include <cerrno>
#include <cstring>

#include "base/NonCopyable.h"

class TStopWatch
{
  NON_COPYABLE(TStopWatch);

private:
  struct timeval ReferenceTime;
  struct timeval StopTime;
  bool Running;

  void Update(struct timeval& tv) throw () {
    if (::gettimeofday(&tv, NULL) == -1) {
      ::fprintf(stderr, "Internal system error: %s\n", ::strerror(errno));
      ::abort();
    }
  }   

public:
  TStopWatch() throw ()
    : Running(false)
  {
    Reset();
  }

  void Reset() throw ()
  {
    Update(ReferenceTime);
    Running = true;
  }

  void Stop() throw ()
  {
    Update(StopTime);
    Running = false;
  }

  int64_t MicroSecondsElapsed()
  {
    struct timeval now;
    Update(now);
    int64_t elapsedTime = (1000000 * (now.tv_sec - ReferenceTime.tv_sec)) +
      (now.tv_usec - ReferenceTime.tv_usec);
    return elapsedTime;
  }

  int64_t MilliSecondsElapsed()
  {
    return MicroSecondsElapsed()/1000;
  }

  int64_t SecondsElapsed()
  {
    return MicroSecondsElapsed()/1000000;
  }

  uint32_t GetTime() throw ()
  {
    struct timeval now;
    if (Running) {
      Update(now);
    }
    else {
      now = StopTime;
    }
    uint32_t sdiff = now.tv_sec -  ReferenceTime.tv_sec;
    if (now.tv_usec > ReferenceTime.tv_usec
        && (now.tv_usec - ReferenceTime.tv_usec > 50000)) {
      ++sdiff;
    }
    return sdiff;
  }

  void GetTime(uint8_t& hours, uint8_t& minutes, uint8_t& seconds) throw ()
  {
    uint32_t difference = GetTime();
    hours = difference / 3600;
    minutes = difference / 60;
    seconds = difference % 60;
  }

};

#endif


  
