







#ifndef UTILS_TTIMESTATISTICS_H
#define UTILS_TTIMESTATISTICS_H

#define __STDC_FORMAT_MACROS

#include "debug/TTracker.h"

#include <inttypes.h>
#include <sys/time.h>
#include <limits>
#include <string>

class TTimeStatistics
{
private:
  const std::string Prefix;
  const uint32_t MaxCount;
  uint32_t Count;
  uint64_t Minimum;
  uint64_t Maximum;
  uint64_t Average;
  uint64_t BeginTime;
  uint64_t EndTime;

public:
  TTimeStatistics(const std::string& prefix, int maxCount);
  void Begin();
  void End();
  void Reset();
  void Print();

private:
  static uint64_t GetTotalMicroseconds();
};

TTimeStatistics::TTimeStatistics(const std::string& prefix, int maxCount)
  : Prefix(prefix),
    MaxCount(maxCount),
    Count(0),
    Minimum(std::numeric_limits<uint64_t>::max()),
    Maximum(0),
    Average(0)
{ }

void TTimeStatistics::Begin()
{
  BeginTime = GetTotalMicroseconds();
}

void TTimeStatistics::End()
{
  EndTime = GetTotalMicroseconds();
  uint64_t duration = EndTime - BeginTime;
  Minimum = std::min(Minimum, duration);
  Maximum = std::max(Maximum, duration);
  Average += duration;
  if (++Count >= MaxCount) {
    Print();
    Reset();
  }
}

void TTimeStatistics::Reset()
{
  Count = 0;
  Minimum = std::numeric_limits<uint64_t>::max();
  Maximum = 0;
  Average = 0;
}

void TTimeStatistics::Print()
{
  if (Count == 0) {
    return;
  }
  TTracker::GetProcessTracker()->Debug
    ("%s: min: %" PRIu64 " max: %" PRIu64 " avg: %" PRIu64 " (us) (%u samples)",
     Prefix.c_str(), Minimum, Maximum, Average / Count, Count);
}

uint64_t TTimeStatistics::GetTotalMicroseconds()
{
  timeval tv;
  int status = ::gettimeofday(&tv, NULL);
  assert(status == 0);
  return (static_cast<uint64_t>(tv.tv_sec) * 1000000LLU +
          static_cast<uint64_t>(tv.tv_usec));
}

#endif
