/* -*- Mode: C++; tab-width: 2; indent-tabs-mode: nil; c-basic-offset: 2 -*-
 *---------------------------------------------------------------------------
 *
 * xml/TXmlParser.h
 *
 * A small C++ wrapper for the expat XML parser.
 * More about expat can be found on http:
 * http:
 *
 * Copyright (c) 2002 Kreatel Communications AB
 *
 *---------------------------------------------------------------------------
 */

#ifndef XML_TXMLPARSER_H
#define XML_TXMLPARSER_H

#include "base/TException.h"

#include <expat.h>

#include <new>
#include <string>

class TXmlParser
{
private:
  
  class TXmlParserHandle
  {
  private:
    TXmlParser& Parent;
    XML_Parser Parser;
    std::string Encoding;

  public:
    TXmlParserHandle(TXmlParser& parent, const char* encoding) throw ();
    ~TXmlParserHandle() throw ();

    operator XML_Parser() const throw ();

    void Create() throw (std::bad_alloc);
    void Destroy() throw ();
  };

  friend class TXmlParserHandle;

  static void StartHandler(void* userData, const XML_Char* name,
                           const XML_Char** atts);
  static void EndHandler(void* userData, const XML_Char* name);
  static void CharacterDataHandler(void* userData, const XML_Char* s,
                                   int len);

  TXmlParserHandle Parser;

protected:
  std::string GetErrorMessage(const char *doc, int length) throw ();

  virtual void StartElementHandler(const XML_Char* name,
                                   const XML_Char** atts) throw ();
  virtual void EndElementHandler(const XML_Char* name) throw ();
  virtual void TextHandler(const XML_Char* s, int len) throw ();

public:
  TXmlParser() throw ();
  TXmlParser(const char* encoding) throw ();
  virtual ~TXmlParser() throw ();

  void Parse(const std::string& doc) throw (TException, std::bad_alloc);
  void Parse(const char* doc) throw (TException, std::bad_alloc);
  void Parse(const char* doc, int length) throw (TException, std::bad_alloc);
};

#endif 
