#!/usr/bin/python
#---------------------------------------------------------------------------
#
# fwlogclient
#
# A simple log client for the linux based firmware.
#
# Copyright (c) 2006 Kreatel Communications AB. All Rights Reserved.
# Copyright (c) 2008 Motorola, Inc. All Rights Reserved.
#
#---------------------------------------------------------------------------

import socket
import struct
import sys

VERSION = "1.4 (20100830)"

HEADER_LEN = 12

NR_IX = 0
TIME_IX = 1
IP_IX = 0
PORT_IX = 1

PORT = 19997

def error(msg):
    sys.stderr.write("Error: %s\n" % msg)

if len(sys.argv) > 1:
    try:
        PORT = int(sys.argv[1])
    except:
        print "Usage: %s [PORT]" % sys.argv[0]
        print "Listen for firmware log messages on PORT (default %d)" % PORT
        print ""
        print "fwlogclient %s (c) Motorola, Inc. 2008" % VERSION
        sys.exit(0)

try:
    s = socket.socket(socket.AF_INET, socket.SOCK_DGRAM, 0)
    s.bind(('0.0.0.0', PORT))
except socket.error:
    error('port %d is already used on this machine' % PORT)
    sys.exit(1)
    
client_info = {}

while True:
    try:
        data, client = s.recvfrom(1024)
        try:
            nr, sec, usec = struct.unpack('>III', data[:HEADER_LEN])
            t = float("%d.%06d" % (sec, usec))
            if not client in client_info:
                client_info[client] = (nr, t)
            else:
                if client_info[client][NR_IX] > nr:
                    client_info[client] = (nr, t)
                else:
                    client_info[client] = (nr, client_info[client][TIME_IX])
            print "%06d, %.03f, %s:%d: %s" % (nr, t - client_info[client][TIME_IX],
                                              client[IP_IX], client[PORT_IX],
                                              data[HEADER_LEN:])
            sys.stdout.flush()
        except:
            error("invalid log message recieved: [%s]" % repr(data))
    except:
        break
